\def\MathLogo{{\it Mathematica}}
\def\ML{{\it MathLink}}
@*Read portable network files into \MathLogo.
@c
@<Header@>@;
@<Read a PNG file@>@;
@<Write a PNG file@>@;
@<Write a PNG from a PBM string@>@;
@<Make a hexadecimal string@>@;
@<C-code for Mandelbrot image@>@;
@<Templates@>@;

@ Show that the \ML-program is running.
@<Templates@>=
@=:Evaluate:Print["mathpng MathLink program installed. \\[Copyright] Jens-Peer Kuska 1998."]@>@|

#ifdef __WIN32__
#  if MLVERSION < 3

int PASCAL WinMain(HANDLE hinstCurrent, HANDLE hinstPrevious,
                   LPSTR lpszCmdLine, int nCmdShow)

  { char buff[512];
    char FAR *argv[32];
    int argc;

	 if(!MLInitializeIcon(hinstCurrent,nCmdShow)) return 1;
	 argc=MLStringToArgv(lpszCmdLine,buff,argv,32);
	 return MLMain(argc,argv);
  }

#  else

int PASCAL WinMain(HANDLE hinstCurrent, HANDLE hinstPrevious,

		   LPSTR lpszCmdLine, int nCmdShow)

  {

    char buff[512];
    char FAR *buff_start=buff;
    char FAR *argv[32];
    char FAR * FAR * argv_end=argv+32;

    if(!MLInitializeIcon(hinstCurrent,nCmdShow)) return 1;
    MLScanString(argv,&argv_end,&lpszCmdLine,&buff_start);
    return MLMain(argv_end-argv,argv);
  }

#  endif

#else

int main(int argc, char *argv[]) {

  return MLMain(argc,argv);
 }

#endif


@ @<Output the image to \MathLogo@>=
  {
    int i,j;
    int row_bytes;
    png_bytep row;
    int pixel_no;

    row_bytes=png_get_rowbytes(png_ptr, info_ptr);
    MLPutFunction(stdlink,"List",5);
      MLPutFunction(stdlink,"List",height);
      for(i=height-1; i>-1; i--) {
        row=row_pointers[i];
	if(bit_depth<=8) 
	  @<Send 8 bit image row@>@;
	else {
	  if(!(color_type | PNG_COLOR_MASK_COLOR)) 
	    @<Send 16 bit gray row@>@;
	  else 
	    @<Send 16 bit color row@>@;
	 }     
       }
      MLPutInteger(stdlink,width);
      MLPutInteger(stdlink,height);
      MLPutInteger(stdlink,(bit_depth<8 ? 8 : bit_depth));
      switch(color_type) {
	case PNG_COLOR_TYPE_GRAY :
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorGray");
	    break;
	case PNG_COLOR_TYPE_PALETTE :
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorPalette");
	    break;
	case PNG_COLOR_TYPE_RGB :
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorRGB");
	    break;
	case PNG_COLOR_TYPE_RGB_ALPHA : 
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorRGBA");
	    break;
	case PNG_COLOR_TYPE_GRAY_ALPHA : 
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorGrayA");
	    break;
	default:
	    MLPutSymbol(stdlink,"PNGBitmap`PNGColorUnkown");
       }	
   }

@ @<Send 8 bit image row@>=
{
  MLPutFunction(stdlink,"List",row_bytes);
  for(j=0; j<row_bytes;j++)
    MLPutInteger(stdlink,(int) row[j]);
}

@ @<Send 16 bit gray row@>=
{
  pixel_no=row_bytes >> 1;
  MLPutFunction(stdlink,"List",pixel_no);
  for(j=0; j<row_bytes; j+=2)
    MLPutInteger(stdlink,(row[j] << 8)+row[j+1]);
}

@ @<Send 16 bit color row@>=
{
  pixel_no=row_bytes >> 1;
  MLPutFunction(stdlink,"List",pixel_no);
  for(j=0; j<row_bytes; j+=2)
    MLPutInteger(stdlink,(row[j] << 8)+row[j+1]);
}

@ Get the image manual from \MathLogo{}.
@<Get the image from \MathLogo@>=
  {
    int i,j,k;
    long row_bytes;
    long rb_alloc;
    png_bytep row;
    long imh;
    int *img_row;

    rb_alloc=png_get_rowbytes(png_ptr, info_ptr);
    row_pointers=calloc(height,sizeof(png_bytep));

    if(!MLCheckFunction(stdlink,"List",&imh) && imh!=height)
      @<Return in case of an error@>@;
    for(i=height-1; i>-1; i--) {
      MLGetIntegerList(stdlink,&img_row,&row_bytes);
      row_pointers[i]=calloc(rb_alloc,sizeof(png_byte));
      row=row_pointers[i];
      if(bit_depth==8) {
        for(j=0; j<row_bytes; j++)
          row[j]=(png_byte) img_row[j];
       }
      else if (bit_depth==16) {
	for(k=j=0; j<row_bytes; j++) {
	  row[k++]=(png_byte) (img_row[j]  >> 8);
	  row[k++]=(png_byte) (img_row[j]);
	  if(k>=rb_alloc) break;
	 }  
	}
      MLDisownIntegerList(stdlink,img_row,row_bytes);
     }
   }

@ @<Templates@>=
@=:Begin:@>@|
@=:Function: ReadPNGFile @>@|
@=:Pattern: PNGBitmap`RawReadPNGBitmap[pngfile_String] @>@|
@=:Arguments: {pngfile}                   @>@|
@=:ArgumentTypes: {String}                @>@|
@=:ReturnType: Manual                     @>@|
@=:End:@>@|

@ @<Templates@>=
@=:Evaluate:SetAttributes[PNGBitmap,HoldAll] @>@|

@ @<Templates@>+=
@=:Evaluate: Format[PNGBitmap[data_,w_,h_,d_]]:=                           @>@|
@=    SequenceForm["PNGBitmap","[","<<image data>>",",",w,",",h,",",d,"]"] @>@|


@ @<Read a PNG file@>=
void ReadPNGFile(const char *fname)
{
   char *gamma_str;
   char file_name[256];
   double screen_gamma,image_gamma;
   png_uint_32 row;
   png_structp png_ptr=NULL;
   png_infop info_ptr=NULL;
   png_uint_32 width, height;
   int bit_depth, color_type, interlace_type;
   png_bytep *row_pointers, trans=NULL;
   int num_trans=-1;
   png_color_16 *trans_values=NULL;
   const int ReadFile=1;
   FILE *fp;

   @<Remove double backslashs from file |fname|@>@;
   if ((fp = fopen(file_name, "rb")) == NULL)
     @<Return in case of an error@>@;
   @<Initialize reading of the |png_struct|@>
   @<Get the memory for the image information@>@;
   @<Set the error handling@>@;
   @<Initialise the input control@>@;
   /* The call to |png_read_info()| gives us all of the information from the
     PNG file before the first IDAT (image data chunk). */
   png_read_info(png_ptr, info_ptr);

   png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type,
       &interlace_type, NULL, NULL);
   png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans,&trans_values);
#ifdef NO_COLOR16
   /* tell libpng to strip 16 bit/color files down to 8 bits/color */
   png_set_strip_16(png_ptr); 
#endif
   /* expand paletted colors into true RGB triplets expand 
      images to the full 8 bits from 1, 2, or 4 bits/pixel */
   if (color_type == PNG_COLOR_TYPE_PALETTE || bit_depth<8)
      png_set_expand(png_ptr);

   /* expand paletted or RGB images with transparency to full alpha channels
     so the data will be available as RGBA quartets */
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
      png_set_expand(png_ptr);

   /* This is one way that applications share the same screen gamma value */
   if ((gamma_str = getenv("DISPLAY_GAMMA")) != NULL)
      screen_gamma = atof(gamma_str);
   /* If we don't have another value */
   else
      screen_gamma = 2.2;  /* A good guess for PC monitors */


   /* Tell libpng to handle the gamma conversion for you.  The second call
     is a good guess for PC generated images, but it should be configurable
     by the user at run time by the user.  It is strongly suggested that
     your application support gamma correction. */
   if (png_get_gAMA(png_ptr, info_ptr, &image_gamma))
      png_set_gamma(png_ptr, screen_gamma, image_gamma);
   else
      png_set_gamma(png_ptr, screen_gamma, 0.45);

   /* Add filler (or alpha) byte (before/after each RGB triplet) */
   if(!(color_type & PNG_COLOR_MASK_ALPHA) && num_trans<0)
      png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);

  /* optional call to gamma correct and add the background to the palette
     and update info structure.  REQUIRED if you are expecting libpng to
     update the palette for you (ie you selected such a transform above).*/
   png_read_update_info(png_ptr, info_ptr);

   @<Create the storrage for the image@>@;
   png_read_image(png_ptr, row_pointers);
   @<Output the image to \MathLogo@>@;
   /* read rest of file, and get additional chunks in |info_ptr| */
   png_read_end(png_ptr, info_ptr);

   /* clean up after the read, and free any memory allocated */
   png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
   @<Free the memory of the image@>@;
   /* close the file */
   fclose(fp);
   return;
}

@ @<Return in case of an error@>=
{
  if(fp) fclose(fp);
  if(png_ptr) {
    if(ReadFile)
      png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
    else
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
   }
  MLPutSymbol(stdlink,"$Failed");
  return;
}

@ Create and initialize the |png_struct| with the desired error handler
  functions.  If you want to use the default stderr and longjump method,
  you can supply NULL for the last three parameters.  We also supply the
  the compiler header file version, so that we know if the application
  was compiled with a compatible version of the library.

@<Initialize reading of the |png_struct|@>=
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
      (void *)NULL, NULL, NULL);
   if (png_ptr == NULL)
     @<Return in case of an error@>@;

@ Create and initialize the |png_struct| with the desired error handler
  functions.  If you want to use the default |stderr| and |longjump| method,
  you can supply |NULL| for the last three parameters.  We also check that
  the library version is compatible with the one used at compile time,
  in case we are using dynamically linked libraries.

@<Initialize the writing of |png_struct|@>=
   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
      (void *)NULL, NULL, NULL);
   if (png_ptr == NULL)
     @<Return in case of an error@>@;

@ Allocate/initialize the memory for image information.
@<Get the memory for the image information@>=
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL)
     @<Return in case of an error@>@;

@ Set error handling if you are using the |setjmp|/|longjmp| method (this is
  the normal method of doing things with libpng).  REQUIRED unless you
  set up your own error handlers in the |png_create_read_struct()| earlier.
@<Set the error handling@>=
   if (setjmp(png_ptr->jmpbuf))
     @<Return in case of an error@>@;

@ Set up the input control if you are using standard C streams
@<Initialise the input control@>=
   png_init_io(png_ptr, fp);


@ Allocate the memory to hold the image using the fields of |info_ptr|.
@<Create the storrage for the image@>=
   row_pointers=calloc(height,sizeof(png_bytep));
   for (row = 0; row < height; row++)
      row_pointers[row] = malloc(png_get_rowbytes(png_ptr, info_ptr));

@ @<Free the memory of the image@>=
if(row_pointers) {
  for(row=0; row < height; row++)
    if(row_pointers[row]) free(row_pointers[row]);
  free(row_pointers);
 }

@ Template for \MathLogo{}.

@<Templates@>+=
@=:Begin:@>@|
@=:Function: WritePNGFile                                                @>@|
@=:Pattern: PNGBitmap`RawWritePNGBitmap[pngfile_String,w_Integer,h_Integer, @>@|
@=                       bd_Integer,cno_Integer,gamma_Real,img_]         @>@|
@=:Arguments: {pngfile,w,h,bd,cno,gamma,img}                             @>@|
@=:ArgumentTypes: {String,Integer,Integer,Integer,Integer,Real,Manual}   @>@|
@=:ReturnType: Manual                                                    @>@|
@=:End:@>@|

@ @<Templates@>+=
@=:Evaluate: PNGBitmap`RawWritePNGBitmap[pngfile_String,                     @>@|
@=             PNGBitmap[img_,w_Integer,h_Integer,bd_Integer]]:=   @>@|
@=             PNGBitmap`WritePNGBitmap[pngfile,w,h,bd,4,2.2,img]            @>@|

@ @<Write a PNG file@>=
void WritePNGFile(const char *fname,
       int width,int height,int bit_depth,int png_fmt,double gamma)
{
   FILE *fp;
   char file_name[256];

   png_structp png_ptr;
   png_infop info_ptr;
   const int ReadFile=0;
   int row,png_color_type;
   png_color_8 sig_bit;
   png_bytep *row_pointers;

   @<Open the target file@>@;
   @<Initialize the writing of |png_struct|@>@;
   @<Get the memory for the image information@>@;
   @<Set the error handling@>@;
   @<Initialise the input control@>@;
   @<Set the image information@>@;
   /* Optional gamma chunk is strongly suggested 
    if you have any guess as to the correct gamma of the image. */
   png_set_gAMA(png_ptr, info_ptr, gamma);
   /* Write the file header information. */
   png_write_info(png_ptr, info_ptr);
   @<Setup the transformations@>@;
   @<Get the image from \MathLogo@>@;
   png_write_image(png_ptr, row_pointers);
   png_write_end(png_ptr, info_ptr);
   @<Free the memory of the image@>@;
   /* clean up after the write, and free any memory allocated */
   png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
   /* close the file */
   fclose(fp);
   MLPutString(stdlink,file_name);
   return;
}

@ @<Templates@>+=
@=:Begin:@>@|
@=:Function: WritePNGGraphics                                       @>@|
@=:Pattern: PNGBitmap`RawWritePNGGraphics[pngfile_String,           @>@|
@=                                        gamma_Real,pbm_String]    @>@|
@=:Arguments: {pngfile,gamma,pbm}                                   @>@|
@=:ArgumentTypes: {String,Real,Manual}                              @>@|
@=:ReturnType: Manual                                               @>@|
@=:End:@>@|

@ @<Write a PNG from a PBM string@>=
void WritePNGGraphics(const char *fname, double gamma)
{
   FILE *fp;
   char file_name[256];
   char * uni_str,*end_str;
   char *sp,*pbm,*image_data;
   long int j;
   unsigned long i;
   unsigned long len;
   const int png_fmt=3; /* fixed format for portable bitmap images */
   int width,height,bit_depth;
   png_structp png_ptr;
   png_infop info_ptr;
   const int ReadFile=0;
   int png_color_type;
   png_color_8 sig_bit;
   png_bytep *row_pointers;

   @<Convert last argument to plane ASCII@>@;
   @<Get the image dimensions form the pbm string@>@;
   @<Open the target file@>@;
   @<Initialize the writing of |png_struct|@>@;
   @<Get the memory for the image information@>@;
   @<Set the error handling@>@;
   @<Initialise the input control@>@;
   @<Set the image information@>@;
   /* Optional gamma chunk is strongly suggested
    if you have any guess as to the correct gamma of the image. */
   png_set_gAMA(png_ptr, info_ptr, gamma);
   /* Write the file header information. */
   png_write_info(png_ptr, info_ptr);
   @<Setup the transformations@>@;
   @<Set the row pointers from the positions of the |pbm| string@>@;
   png_write_image(png_ptr, row_pointers);
   png_write_end(png_ptr, info_ptr);
   free(row_pointers);
   /* clean up after the write, and free any memory allocated */
   png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
   /* close the file */
   fclose(fp);
   MLPutFunction(stdlink,"List",5);
     MLPutString(stdlink,file_name);
     MLPutInteger(stdlink,width);
     MLPutInteger(stdlink,height);
     MLPutInteger(stdlink,bit_depth);
     MLPutInteger(stdlink,strlen(image_data));
   free(image_data);
   return;
}

@ Since |MLGetString| returns unicode we make the
  explicit conversion. The |MLNextCharacter| function is used
  to copy the ASCII characters to |pbm|.
@<Convert last argument to plane ASCII@>=
   MLGetString(stdlink,(const char **) &uni_str);
   len=strlen(uni_str);
   end_str=uni_str+len;
   image_data=pbm=(char *)calloc(len,sizeof(char));
   for(j=i=0,sp=uni_str; i<len; i++) {
     pbm[j++]= (char) MLNextCharacter((const char **)&sp,(const char *)end_str);
     if(!pbm) break;
    }
   MLDisownString(stdlink,uni_str);

@ @<Get the image dimensions form the pbm string@>=
{

  while(*pbm!='\012') pbm++; /* skip over "P6\n" */
  sscanf(++pbm,"%d %d\n",&width,&height);
  while(*pbm!='\012') pbm++;
  sscanf(++pbm,"%d",&bit_depth);
  bit_depth=8/*log((double)(bit_depth+1))/M_LN2*/;
  while(*pbm!='\012') pbm++;
  pbm++;
}

@ @<Set the row pointers from the positions of the |pbm| string@>=
{   int i;

    png_get_rowbytes(png_ptr, info_ptr);
    row_pointers=calloc(height,sizeof(png_bytep));
    for(i=0; i<height; i++)
      row_pointers[i]=(png_bytep) (pbm+3*width*i);
}

@ Open the binray file for writing.
@<Open the target file@>=
   @<Remove double backslashs from file |fname|@>@;
   fp = fopen(file_name, "wb");
   if (fp == NULL) {
      MLPutSymbol(stdlink,"$Failed");
      return;
     }

@ Set the image information.  Width and height are up to $2^31$,
|bit_depth| is one of 1, 2, 4, 8, or 16, but valid values also depend on
the |color_type| selected. |color_type| is one of |PNG_COLOR_TYPE_GRAY|,
|PNG_COLOR_TYPE_GRAY_ALPHA|, |PNG_COLOR_TYPE_PALETTE|, |PNG_COLOR_TYPE_RGB|,
or |PNG_COLOR_TYPE_RGB_ALPHA|.  |interlace| is either |PNG_INTERLACE_NONE| or
|PNG_INTERLACE_ADAM7|, and the |compression_type| and |filter_type| {\it must}
currently be |PNG_COMPRESSION_TYPE_BASE| and |PNG_FILTER_TYPE_BASE|.
@<Set the image information@>=
   switch(png_fmt) {
     case 1 : png_color_type=PNG_COLOR_TYPE_GRAY;
              sig_bit.gray=(unsigned char)bit_depth;
              break;
     case 3 : png_color_type=PNG_COLOR_TYPE_RGB;
              sig_bit.red=sig_bit.green=sig_bit.blue=(unsigned char)bit_depth;
              break;
     case 4 : png_color_type=PNG_COLOR_TYPE_RGB_ALPHA;
              sig_bit.red=sig_bit.green=(unsigned char)bit_depth;
              sig_bit.blue=sig_bit.alpha=(unsigned char)bit_depth;
              break;
     default :png_color_type=PNG_COLOR_TYPE_RGB_ALPHA;
              sig_bit.red=sig_bit.green=(unsigned char)bit_depth;
              sig_bit.blue=sig_bit.alpha=(unsigned char)bit_depth;
              break;
    }
   png_set_IHDR(png_ptr, info_ptr,
                width, height, bit_depth,
                png_color_type,
                PNG_INTERLACE_NONE,
                PNG_COMPRESSION_TYPE_DEFAULT,
                PNG_FILTER_TYPE_BASE);
   png_set_sBIT(png_ptr,info_ptr,&sig_bit);

@ Set up the transformations you want.  Note that these are
  all optional.  Only call them if you want them.
@<Setup the transformations@>=
   /* pack pixels into bytes */
   png_set_packing(png_ptr);

@ Make a hexadecimal string from one row.

@<Templates@>+=
@=:Begin:@>@|
@=:Function: MakeHexString                                                @>@|
@=:Pattern: PNGBitmap`MakeHexRow[onerow_ /; VectorQ[onerow,IntegerQ]]            @>@|
@=:Arguments: {onerow}                                                 @>@|
@=:ArgumentTypes: {IntegerList}                                        @>@|
@=:ReturnType: Manual                                                  @>@|
@=:End:@>@|

@ Get a single image row and male one hex string for PostScript output.
@<Make a hexadecimal string@>=
void MakeHexString(int *row, long len)
{ char *hex, one[2];
  long i;

  hex=calloc(2*len+1,sizeof(char));
  *hex='\0';
  for(i=0L;i<len; i++) {
    sprintf(one,"%02X",(unsigned char) row[i]);
    strcat(hex,one);
   }
  MLPutString(stdlink,hex);
  free(hex);
}

@* Some common textures.

@ Mandelbrot image.

@<Templates@>+=
@=:Begin:@>@|
@=:Function: RawMandelJulia                                                      @>@|
@=:Pattern: PNGBitmap`RawMandelJulia[x1_Real,y1_Real,x2_Real,y2_Real,            @>@|
@=                                   cx_Real,cy_Real,                            @>@|
@=                                   b_Real,makeJulia_Integer,                   @>@|
@=                                   xsize_Integer,ysize_Integer,                @>@|
@=                                   maxit_Integer,makePot_Integer]              @>@|
@=:Arguments: {x1,y1,x2,y2,cx,cy,b,makeJulia,xsize,ysize,maxit,makePot}          @>@|
@=:ArgumentTypes: {Real,Real,Real,Real,Real,Real,Real,Integer,                   @>@|
@=                 Integer,Integer,Integer,Integer}                              @>@|
@=:ReturnType: Manual                                                            @>@|
@=:End:@>@|

@ @<Templates@>=
@=:Evaluate: PNGBitmap`RawMandelJulia[x1_,y1_,x2_,y2_,b_]:=                      @>@|
@=             PNGBitmap`RawMandelJulia[x1,y1,x2,y2,0.0,0.0,b,0,320,200,255,0]   @>@|

@ @<C-code for Mandelbrot image@>=
#define Image(mat,i,j) mat[i+j*xsize]
#if defined(__BORLANDC__)
#  define LONGDOUBLE long double
#  define LONGLOG    logl
#  define LONGSQRT   sqrtl
#  define TWO        (2.0L)
#  define HALF       (0.5L)
#elif defined(__SUNPRO_C)
#  define LONGDOUBLE long double
#  define LONGLOG    log
#  define LONGSQRT   sqrt
#  define TWO        (2.0L)
#  define HALF       (0.5L)

#else
#  define LONGDOUBLE double
#  define LONGLOG    log
#  define LONGSQRT   sqrt
#  define TWO        (2.0)
#  define HALF       (0.5)
#endif

void RawMandelJulia(double x1,double y1,
		   double x2,double y2,
         double cx,double cy,
		   double bound,
         int makeJulia,
		   int xsize, int ysize,
		   int maxiter,
         int makePot)
{ int *mat=NULL;
  double *dmat=NULL;
  LONGDOUBLE dx,dy,pot;
  int i,j,iter,k;
  register LONGDOUBLE ccx,ccy,xx,yy,x,y,xsq,ysq,xsave,lbound;

  if(makePot)
    dmat=calloc(xsize*ysize,sizeof(double));
  else
    mat=calloc(xsize*ysize,sizeof(int));
  lbound= (bound*bound);
  ccx=(LONGDOUBLE) cx;
  ccy=(LONGDOUBLE) cy;
  dx= ((x2-x1)/(xsize-1));
  dy= ((y2-y1)/(ysize-1));
  for(i=0; i<xsize; i++) {
    xx=(x1+dx*(i+HALF));
    if(!makeJulia) ccx=xx;
    for(j=0; j<ysize; j++) {
      iter=0;
      x=xx;
      yy=y=(y1+dy*(j+HALF));
      if(!makeJulia) ccy=yy;
      xsq=x*x;
      ysq=y*y;
      do {
	     xsave=x;
	     x=ccx+xsq-ysq;
	     y=ccy+TWO*y*xsave;
	     xsq=x*x;
	     ysq=y*y;
       }
      while(++iter < maxiter && (xsq+ysq)<lbound);
      if(!makePot)
        Image(mat,i,j)=iter;
      else {
        if(iter==maxiter)
          pot=0.0;
        else {
	  pot=LONGSQRT(xsq+ysq);
	  for(k=iter; k>0; k--)
	    pot=LONGSQRT(pot);
     pot=LONGLOG(pot);
	 }
        Image(dmat,i,j)=pot;
       }
     }
    }

   MLPutFunction(stdlink,"Partition",2);
     if(!makePot)
       MLPutIntegerList(stdlink,mat,xsize*ysize);
     else
       MLPutRealList(stdlink,dmat,xsize*ysize);
     MLPutInteger(stdlink,xsize);
   MLEndPacket(stdlink);
   if(!makePot)
     free(mat);
   else
     free(dmat);
}

@* System specific.
@ @<Remove double backslashs from file |fname|@>=
  {
    int i,j,n;

    n=strlen(fname);
    for(j=i=0; i<n; i++)
      if(fname[i]!='\\')
        file_name[j++]=fname[i];
      else if(fname[++i]=='\\')
        file_name[j++]='\\';
    file_name[j++]=fname[n]; @+ file_name[j+1]='\0';
#ifdef __WIN32__
    n=strlen(file_name);
    for(i=0; i<n; i++)
      if('/'==file_name[i]) file_name[i]='\\';
#endif
  }


@ @<Header@>=
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "png.h"
#include "mathlink.h"

@* Index.
