(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    397585,       6056]*)
(*NotebookOutlinePosition[    398484,       6085]*)
(*  CellTagsIndexPosition[    398440,       6081]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Portable Network Graphics with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " "
}], "Title"],

Cell["Jens-Peer Kuska", "Author"],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
  "The portable network graphic format is intended to replace the GIF format \
as well as to overcome some of its limitations.  Many applications support \
the format as xv3.1a, ImageMagic and GIMP in the UNIX world, as well as \
Picture Publisher 6 and Corel PhotoPaint 7. Ghostscript can create this \
format, and POVray can read and write PNG files.  The package and the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " program allow the reading of PNG files into the kernel of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " as well as writing this image format.  The Display[] function is modifyed \
to accept Display[file,anyGraphics,\"PNG\"]."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialisation and Tests", "Section"],

Cell[TextData[{
  "Load the package for Portabel Network Graphics. First add the directory to \
the ",
  StyleBox["$Path",
    FontFamily->"Courier"],
  " variable. Note that MS Windows systems need a different directory \
seperator. (Your directory names and paths may differ.)"
}], "Text"],

Cell[BoxData[
    \(\(AppendTo[$Path, "\<~/MathSourceSubmit/PNGBitmap\>"]; \)\)], "Input",
  CellLabel->"In[2]:="],

Cell["and load the package.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Get["\<PNGBitmap`\>"]\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \("mathpng MathLink program installed. \[Copyright] Jens-Peer Kuska \
1998."\)], "Print",
  GeneratedCell->False,
  CellAutoOverwrite->False]
}, Open  ]],

Cell["Look if the MathLink program is running", "Text"],

Cell[BoxData[
    \(Links[]\)], "Input",
  CellLabel->"In[3]:="],

Cell["Now we have available several functions to deal with images.", "Text"],

Cell[BoxData[
    \(\(?PNGBitmap`*\)\)], "Input",
  CellLabel->"In[3]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initial Tests", "Section"],

Cell[TextData[{
  "Here are all files from the PNG test suite. Get the file names. The \
\[Alpha]- channel is ignored when the pictures are displayed in ",
  StyleBox["Mathematica.",
    FontSlant->"Italic"],
  " For gray-\[Alpha] pictures the \[Alpha]-channel is stripped. (Your \
directory names and paths may differ.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(SetDirectory["\<~/MathSourceSubmit/PNGBitmap/Pictures\>"]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \("/export/home/jpk/MathSourceSubmit/PNGBitmap/Pictures"\)], "Output",
  CellLabel->"Out[8]="]
}, Open  ]],

Cell[BoxData[
    \(\(testfiles = 
      \(StringJoin[Directory[], $PathnameSeparator, #]\ &\)\ /@\ 
        FileNames["\<*.png\>", {"\<Pictures\>", "\<tsuite\>"}, 2]; \)\)], 
  "Input",
  CellLabel->"In[27]:="],

Cell["Read the images.", "Text"],

Cell[BoxData[
    \(\(testbm = ReadPNGBitmap\ /@\ testfiles; \)\)], "Input",
  CellLabel->"In[28]:="],

Cell["Link name and image together.", "Text"],

Cell[BoxData[
    \(\(\ \ 
    \(pics = Transpose[{testbm, 
            \(StringReplace[#, "\<tsuite/\>" -> "\<\>"]\ &\)\ /@\ testfiles}]
          \  /. \ {$Failed, _}\  :> \ Sequence[]; \)\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(\(testgr = 
      Map[\n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        Apply[\n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            Graphics[#1, PlotLabel -> #2]\ &, \n
            \t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ #\n
            \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]\ &, \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ pics\n
        \t\ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \)\)], "Input",
  CellLabel->"In[15]:="],

Cell["Show the pictures.", "Text"],

Cell[BoxData[
    \(\(\(Show\ [GraphicsArray[#]]\ &\)/@\ 
      Flatten[\({Partition[#, 6], {Take[#, \(-1\)]}}\ &\)[testgr], 1]; \)\)], 
  "Input",
  CellLabel->"In[1]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Read PNG files", "Section"],

Cell["\<\
The PNGBitmap[]-object is used to represent the image data. Here is \
a first example.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bm = 
      ReadPNGBitmap[Directory[]\ <>\ $PathnameSeparator\ <>"\<sample1n.png\>"]
        \)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    InterpretationBox[
      \("PNGBitmap"\[InvisibleSpace]"["\[InvisibleSpace]"<<image data>>"\
\[InvisibleSpace]","\[InvisibleSpace]256
        \[InvisibleSpace]","\[InvisibleSpace]192
        \[InvisibleSpace]","\[InvisibleSpace]8\[InvisibleSpace]"]"\),
      SequenceForm[ 
      "PNGBitmap", "[", "<<image data>>", ",", 256, ",", 192, ",", 8, "]"],
      Editable->False]], "Output",
  CellLabel->"Out[11]="]
}, Open  ]],

Cell[TextData[
"A matrix of Pixel[] entries is used to represent the image data. Color \
pictures are always expanded to Pixel[red,green, blue,\[Alpha]] structures. \
You may modify the \[Alpha]-channel, but you will not see it's effect in the \
PostScript output generated by Mathematica. The first argument of PNGBitmap[] \
holds the pixel data."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Short[First[bm]]\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    TagBox[\(\[LeftSkeleton]1\[RightSkeleton]\),
      Short]], "Output",
  CellLabel->"Out[12]//Short="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[Graphics[bm, ImageSize -> Reverse[Dimensions[First[bm]]]]]; 
    \)\)], "Input",
  CellLabel->"In[13]:="],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .75 
%%ImageSize: 256 192 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00372024 0.0178571 0.00372024 [
[ 0 0 0 0 ]
[ 1 .75 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .75 L
0 .75 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
.02381 .01786 translate
.95238 .71428 scale
768 string
256 192 8 [256 0 0 192 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
270048270048270048270048270048270048270048270048270048270048270048270048270048\
270048270048270048270048270048270048270048270048270048270048270048
270048270048270048270048270048270048270048270048270048270048270048270048270048\
270048270048270048270048270048270048270048270048270048270048270048
270048270048270048270048270048270048270048270048270048270048270048270048270048\
270048270048270048270048270048270048270048270048270048270048270048
270048270048270048270048270048270048270048270048270048270048270048270048270048\
270048270048270048270048270048270048270048270048270048270048270048
270048270048270048270048270048270048270048270048270048270048270048270048270048\
270048270048270048270048270048270048270048270048270048270048270048
27004F2A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A0055\
2A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A0055
2A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A00552A0055\
2A00552A00552A00552A00552C005A2C005A2C005A2C005A2C005A2C005A2C005A
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A\
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A\
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A\
2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A2C005A
2C005A2C005A2A00552A00552A0055270048270048270048270048270048270048270048270048\
270048270048270048
7600D77600D77600D77600D77600D77600D77600D77600D77600D77600D77600D77600E37600E3\
7600E37600E37600E37600E37600E37B00EA7B00EA7B00EA7B00EA7600E37600E3
7600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E3\
7600E37600E37600E37600E37600E37600E37600E37600E37B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7600E37600E37600E37600E37600E37600E37600E37600E37600E37600E3\
7B00EA7B00EA7B00EA7B00EA7B00EA7600E37600E37600E37600E37600E37600E3
7600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37600E3\
7600E37600E37600E37600E37600E37600E37600E37600E37600E37600E37B00EA
7B00EA7B00EA7B00EA7600E37600E37600E37600E37600E37600E37600E37B00EA7B00EA7600E3\
7600E37600E37600E37B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7600E37600E3
7B00EA7B00EA7B00F67B00F67B00F67B00F67B00F67B00F67B00F67B00F67B00F67B00F67B00F6\
7B00F67B00F67B00F67F00FD7F00FD7F00FD7F00FD7F00FD7B00F67B00F67B00F6
7B00F67B00F67B00F67B00F67B00F67F00FD7F00FD7F00FD7F00FD7B00F67B00F67B00F67B00F6\
7B00F67B00F67B00F68B00FD8B00FD8B00FD8B00FD8F00FF8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FD8B00FD8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF
8F00FF8B00FD8B00FD8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FD8B00FD8B00FD\
8B00FD8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF
8F00FF8F00FF7F00FD7F00FD7F00FD7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00DE7B00DE
8400EF8400EF8400EF8400EF8300EC8300EC8300EC8400EF8400EF8400EF8400EF8400EF8400EF\
8400EF8400FD8400FD8400FD8800FF8800FF8A00FF8A00FF8800FF8800FF8400FD
8400FD8400FD8400FD8400FD8400FD8400FD8400FD8400FD8800FF8800FF8800FF8400FD8400FD\
8400FD8400FD8400FD8400FD8400FD8400FD8400FD8800FF8A00FF8A00FF8A00FF
8A00FF8A00FF8800FF8800FF8400FD8400FD8400FD8400FD8400FD8400FD8400FD8400FD8800FF\
8800FF8A00FF8A00FF8800FF8800FF8800FF8400FD8400FD8400FD8400FD8400FD
8400FD8400FD8800FF8800FF8800FF8400FD8400FD8400FD8400FD8400FD8400FD8400FD8400FD\
8400FD8400FD8400FD8400FD8400FD8400FD8400FD8400FD8400FD8800FF8800FF
8800FF8800FF8800FF8800FF8400FD8400FD8400FD8400FD8400FD8800FF8800FF8800FF8800FF\
8800FF8400FD8800FF8800FF8A00FF8A00FF8A00FF8A00FF8A00FF8800FF8800FF
8400FD8400FD8800FF8800FF8800FF8800FF8800FF8800FF8800FF8800FF8800FF8800FF8800FF\
8800FF8800FF8A00FF8B00FF8B00FF8B00FF8B00FF8A00FF8A00FF8800FF8800FF
8800FF8800FF8800FF8800FF8800FF8800FF8A00FF8A00FF8A00FF8800FF8800FF8800FF8800FF\
8800FF8800FF8800FF8800FF8A00FF8A00FF9900FF9B00FF9D00FF9D00FF9D00FF
9B00FF9B00FF9900FF9900FF9900FF9900FF9900FF9900FF9800FF9900FF9900FF9900FF9900FF\
9900FF9900FF9900FF9900FF9900FF9900FF9900FF9900FF9900FF9900FF9900FF
9900FF9800FF9800FF9900FF9900FF9B00FF9B00FF9B00FF9900FF9900FF9800FF9800FF9800FF\
9900FF9900FF9B00FF9D00FF9D00FF9D00FF9B00FF9B00FF9900FF9900FF9900FF
9900FF9900FF9900FF9900FF9900FF9B00FF9B00FF9D00FF9F00FF9F00FF9F00FF9F00FF9F00FF\
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF
9F00FF9D00FF8F00FF8D00FF8D00FF8B00FF8B00FF8B00FF8B00FF8B00FF8B00FF8B00FF8B00FF\
8B00FB8B00FB8A00F8
7300D77F00E57F00E57F00D77F00E57F00E57F00D77F00E57F00E57F00E57F00E57F00E57F00E5\
7F00E57F00E57F00D77F00F18400E18800FF8800FF8800FF8800FF8400FD8300EC
8300EC8300F87F00E57F00E57F00F17F00E57F00E58300F88300F88300F88300F88300F87F00E5\
7F00F17F00F17F00E57F00F17F00F17F00F18300EC8400FD8800FF8A00FF8A00FF
8A00FF8800FF8400FD8300F88300EC7F00E57F00F17F00E57F00E57F00E57F00F18300EC8400EF\
8800FF8A00FF8800FF8400FD8400FD8300F88300EC7F00F17F00F17F00E57F00F1
7F00F18300EC8300EC8300F88300F88300EC7F00F17F00F17F00E57F00E57F00F17F00F17F00E5\
7F00E57F00F17F00F17F00E57F00E57F00F17F00F17F00F18300F88300F88400FD
8800FF8400FD8400FD8300EC8300EC7F00F17F00F17F00F18300EC8300F88400FD8400FD8300F8\
8300F88300F88300F88800FF8A00FF8B00FF8B00FF8A00FF8800FF8400EF8300F8
7F00F17F00F17F00F18300F87F00F17F00E57F00E57F00E57F00FD7F00FD7F00E58300F88300FF\
8300FF8300FF8800FF8A00FF8B00FF8B00FF8A00FF8800FF8400FF8300FF8300FF
8300FF8300FF8300F88300FF8400FF8400FF8800FF8800FF8800FF8400FF8300FF8300F88300F8\
8300F88300FF8300FF8300FF8400FD8800FF8B00FF8F00FF8F00FF9D00FF9D00FF
9B00FF9800FF8800FF9200FF9200FF9200FF9200FF8400FD8400FD9200FF9200FF9200FF8400FD\
9200FF9200FF9200FF9200FF9200FF9200FF9200FF8400FF9200FF8400FD9200FF
9200FF8400FD8400FD8400FD9800FF9900FF9900FF9900FF9600FF9200FF8400FD8300F88400FD\
8400FD8A00FF9900FF9D00FF9D00FF9B00FF9900FF9800FF9600FF9600FF9200FF
8400FD9200FF9200FF9600FF9600FF8B00FF8F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF\
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF
9F00FF9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF\
8F00FF8F00FF8F00FF
4C006D4A007440004A40004A40004A3C00473C004740004A40004A40004A40004A40004A40004A\
40004A40004A40004A40004A4F004F6C00BE7B00EA6F00C35E00994F007341004C
4A004A4A006A40004A40004A40004A40004A40004A40004A4C006D5600834C004C40004A40004A\
40004A40004A40004A40004A40005340004A41004C4F004F6C00B27F00F17F00F1
7F00F16F00B95E00A741004C40004A40004A40004A40004A40004A40004A40004A41004C4F007F\
6F00C37B00EA7B00D05E008345004F56007840004A40004A40004A40004A40004A
40004A4000534C00635600834C006D40004A40004A40004A40004A40004A40004A40004A40004A\
40004A40004A40004A40004A40004A40004A40004A40004A40004A41004C5C0096
7600E35C007F4F004F56008340004A40004A40004A40004A40004A4C00635C00965C008A4C0063\
41004C41004C45004F6100928300F88300F88300F88300F87B00DE4F006841004C
40004A40004A40004A4C007840004A40004A40004A40004A40004A40004A40004A40004A40004A\
40004A41004C5300837B00F67F00FD7F00FD7300CA6C00BE45005C41004C40004A
40004A40004A40004A41005656008345004F5E00837600ED5E00A745004F41004C40004A40004A\
40004A40004A40004A40004A41004C4700537600C38A00FF8A00FF8A00FF8800FF
8300FF61009F45004F41004C4C006341004C41004C41004C40004A41004C41004C41004C4C006D\
41004C41004C41004C41004C41004C41004C41004C41004C4F007345004F41004C
41004C41004C41004C41004C4700536500A48300F855006F47005341004C41004C40004A41004C\
41004C53006C7F00F19200FF9200FF8F00FF7300B253005345004F4F004F41004C
41004C41004C45004F4700534C005851006C8100F19F00FF9F00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF
9F00FF9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8F00FF8F00FF
33003A33003A33003A300037300037300037300037300037300037300037300037300037300037\
30003730003730003733003A35003E3900433C004739004335003E35003E33003A
33003A33003A30003730003730003730003730003733003A33003A33003A33003A33003A300037\
30003730003730003730003730003730003733003A35003E4300433C00473C0047
3C004739004335003E33003A33003A30003730003730003730003730003730003733003A35003E\
39004C3C004739004335003E35003E33003A33003A300037300037300037300037
30003733003A33003A33003A33003A3A003A300037300037300037300037300037300037300037\
30003730003730003730003730003730003730003730003733003A33003A35003E
39004335003E35003E33003A33003A30003730003730003733003A33003A35003E35003E33003A\
33003A33003A35003E47006553008A4C00634C006D40004A3C004735003E33003A
33003A33003A33003A33003A33003A300037300037300037300037300037300037300037300037\
30003733003A35003E3C004740004A40004A3C004743004335003E33003A300037
30003730003730003733003A33003A35003E35003E39004335003E33003A33003A300037300037\
30003730003730003730003733003A3900436300B07600E36F00C35E008341004C
40004A39004335003E33003A33003A33003A33003A33003A33003A33003A33003A33003A33003A\
33003A33003A33003A33003A33003A33003A33003A35003E390043390043390043
39004335003E35003E35003E3900435F009F6300A64F006F35003E33003A33003A30003733003A\
33003A35003E3C004740004A40004A40004A3C004739004335003E35003E390043
3900433900433C004740004A4700537800D59D00FF9F00FF9F00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF
9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8F00FF8F00FF
3000373000372C00332C00332C0033290030290030290030290030290030290030290030290030\
2900302900302900302C00332C00332C00332C00332C00332C00332C00332C0033
2C00332C00332900302900302900302900302900302900302C00332C00332C0033290030290030\
2900302900302900302900302900302900302C00332C0033300037300037300037
3000373000372C00332C00332900302900302900302900302900302900302900302C00332C0033\
2C00333000372C00332C00332C00332C00332C0033290030290030290030290030
2900302900302C00332C00332C00332C0033290030290030290030290030290030290030290030\
2900302900302900302900302900302900302900302900302900302C00332C0033
2C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033\
2C00332C00332C003330003733003A35003E33003A33003A33003A33003A33003A
3000373000372C00332C00332C00332900302900302900302900302900302900302C00332C0033\
2C00332C00332C00333000373000373000373000372C00332C0033290030290030
2900302900302900302C00332C00332C00332C00332C00332C00332C0033290030290030290030\
2900302900302900302900302C003330003735003E47006F3C004739004335003E
33003A3000372C00332C00332C0033290030290030290030290030290030290030290030290030\
2900302900302900302900302900302C003333003A39004340004A41004C41004C
40004A39004333003A3000373A004E33003A35003E33003A2C00332C0033290030290030290030\
2C00332C003330003730003733003A33003A33003A33003A35003E3900433C0047
40004A41004C45004F4700535A005A9900FF9F00FF9F00FF9F00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF
9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8F00FF8F00FF
33003A33003A33003A3000372C00332C0033290030290030290030290030290030290030290030\
290030290030290030290030290030290030290030290030290030290030290030
290030290030290030290030290030290030290030290030290030290030290030290030290030\
2900302900302900302900302900302900302900302900302C00332C00332C0033
2C00332C0033290030290030290030290030290030290030290030290030290030290030290030\
290030290030290030290030290030290030290030290030290030290030290030
290030290030290030290030290030290030290030290030290030290030290030290030290030\
2900302900302900302900302900302900302900302900302900302900302C0033
2C00332C00332C00332C00333000373000373000373000372C00332C0033290030290030290030\
2900302900302900302C003330003733003A33003A33003A35003E3900433C0047
3C004739004335003E3000372C00332900302900302900302900302900302C00332C00332C0033\
290030290030290030290030290030290030290030290030290030290030290030
290030290030290030290030290030290030290030290030290030290030290030290030290030\
2900302900302900302900302900302C003330003733003A33003A33003A33003A
33003A33003A3000372C00332C0033290030290030290030290030290030290030290030290030\
2900302900302900302900302C003330003735003E5600838B00FD9200FF8400FD
7F00F15C00A23C00473300433000373000372C00332C00332C0033290030290030290030290030\
2900302900302C00332C003330003733003A39004340004A41004C53006C6100AD
8300EC9200FF8800FF9800FF9B00FF9F00FF9F00FF9F00FF9D00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF
9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8F00FF8F00FF
41004C4C006D53007447005C35003E3000372C00332C00332C0033300037300037300037300037\
3000372C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033
2C00332C00332C00332C00332C00332C00332C00332C00332900302900302C00332C00332C0033\
2C00332C00332C00332C00332C00332C00332C00332C003330003733003A33003A
33003A3000372C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033\
3000372C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033
2C00332C00332C00332C00332C00332C00332C00332C00332C0033300037300037300037300037\
2C00332C00332C00332C00332C00332C00332C00332C00332C0033300037300037
30003733003A35003E3900433C004740004A3C00473C004735003E33003A3000372C00332C0033\
2C00332C00332C003330003735003E3900433C004741004C4C006D5E00996F00D0
7B00EA7600E353008A35003E3000372C003330003730003730003733003A33003A33003A300037\
3000372C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033
2C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C00332C0033\
2C00332C00332C00332C00332C00332C003330003733003A3900433C004740004A
40004A4A006A4F007A33003A3000372C00332C00332C00332C00332C00332C00332C00332C0033\
2C00332C00332C00332C003330003735003E40004A8300EC9900FF9D00FF9D00FF
9900FF8F00FF6800C339004333003A33003A33003A3000372C00332C00332900302900302C0033\
2C00332C003330003733003A3900434A005F5E00838F00FF9600FF9800FF9B00FF
9D00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9D00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF9F00FF
9F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8F00FF8F00FF
8300EC8300EC7F00E56C00CA4C006D3C004735003E3900433900433C00473C00473C00473C0047\
43004335003E33003A33003A30003730003730003733003A33003A33003A33003A
33003A33003A33003A33003A33003A33003A33003A30003730003730003733003A33003A33003A\
33003A33003A33003A33003A33003A33003A33003A33003A3900434F006F56008F
53007F3C004735003E33003A33003A33003A33003A33003A33003A33003A33003A35003E390043\
43005639004335003E35003E33003A33003A33003A35003E35003E35003E35003E
35003E33003A33003A33003A33003A33003A33003A35003E3C00473C004740004A47005C430056\
35003E33003A33003A33003A30003733003A33003A35003E3C00473C004740004A
40004A41004C4F007F6F00AD7F00F17F00F17F00F17B00EA6800B947004735003E33003A33003A\
33003A35003E35003E3C00475600836C00BE7F00F18300F88400FD8800FF8A00FF
8A00FF8400FD6F00C340004A3900433C00473C004740004A40004A40004A4C00635300744F007A\
39004335003E33003A33003A33003A35003E35003E35003E35003E35003E35003E
35003E35003E35003E35003E35003E35003E35003E35003E39004339004335003E35003E33003A\
33003A33003A33003A33003A33003A35003E3900434000535C00A27B00DE8300FF
8300FF7F00FD5E009941004C39004335003E33003A33003A33003A33003A33003A35003E35003E\
3900433900433900433900433C004741004C5500869800FF9F00FF9F00FF9F00FF
9F00FF9800FF58007341004C40004A40004A40004A3C004735003E33003A30003730003733003A\
35003E3900433C004740004A4700538300EC8A00FF9B00FF9F00FF9F00FF9F00FF
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF9F00FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8F00FF8F00FF
8B00FB8B00FB8A00F88800F47300CA53006C5C008A5E00A77B00D07300D77F00E57F00E57B00DE\
6C00BE4F007F4A005F47005C4700653C004F4700474A006A4A006A4A005F4A005F
4A006A4A005F4A005F4A005F4A006A4A005F4A005F4700653C004747005C47005C4700654A006A\
4A006A4A006A4A006A4A006A4A00744A005F4A004A4A006A4F00686F00D07F00F1
7F00F15E008F4C006D4C006D4A006A4A005F4A00744C006D4C00784C00634C00784F007F7600CA\
6C00BE5E00834F007F5600784C00784C00634C00785600834C0078560078560078
5600784C006D4A005F4A005F4A005F4A006A4C006D5C00966F00C37F00F17F00F17B00EA5E008F\
4C00634A006A4A006A4A006A4700474A004A5600784F00687B00EA7F00F18300F8
8300F88300F88400FD8800FF8A00FF8B00FF8B00FF8A00FF8400EF55007B4F00734C00784A006A\
4C006D5600784F00736100AD7600C38800FF8A00FF8B00FF8D00FF8D00FF8D00FF
8D00FF8B00FF7800BB5500555E008F6F00C37F00F18300F88300F88300F88300F88300F86F00AD\
53006C4C00634C00784C00784C004C5C00964F00685C00964F00735C008A4F007F
5C008A5C008A5C008A5C00965C008A4C00634C00785C00966C00A76C00A75C007F4C006D4C0063\
4C00784C00784C006D4C004C4C00784C006D53006C6500988400FF8A00FF8D00FF
8D00FF8B00FF8800FF7300CA5E008F4F007F4C006D4C00634C00634C00634C00635600785C008A\
5E00A75E00A76C00B25300767B00F65800738800FF8D00FF9100FF9F00FF9F00FF
9F00FF8D00FF5E005E6600A98300EC8F00FF8F00FF7F00E54F007340004A3C0047400053410056\
5C00965E00996500988300FF9600FF7D00D19D00FF9F00FF9F00FF9F00FF9F00FF
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF\
9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8F00FF8F00FF
8F00FF8F00FF8D00FF8B00FB8A00F87A00E67800C77A00E68800F48A00F88A00F88800F48800F4\
8300EC7F00D76F00B96F00B96C00BE6F00B96F00B96F00C37300D77300D77300CA
7300CA6F00C36F00C36F00C36F00C37300CA7300CA7300CA6F00AD61009F6F00C36F00C36F00C3\
6F00C36F00C36F00D06F00C36F00C36F00C36F00C37300CA8300DC7800E18800F4
8800FF8400FD7600D07600D07600DC7600D07600D07F00D77300D77F00D77300D78300DC8400EF\
8400FD8300DC8300EC8300DC8300DC7300D77F00D78300EC8300EC8300DC8300DC
8300EC7F00E57300CA7300CA7300CA7300CA7600C38400E18800FF8A00FF8A00FF8800FF8400FD\
7300CA7300D76F00C36F00C36F00C37300CA7600DC8400EF8A00FF8B00FF8B00FF
8A00FF8A00FF8B00FF8D00FF8D00FF8D00FF8D00FF8B00FF8A00FF8400EF7600DC7F00E57300D7\
7300D77F00E58300DC8400EF8A00FF8B00FF8D00FF8D00FF8D00FF8D00FF8D00FF
8D00FF8B00FF8A00FF7800E17800D58400FD8800FF8A00FF8A00FF8A00FF8A00FF8A00FF8800FF\
8400EF7800D58400E18400E18400EF8400FD8400FD8400FD8400EF8400EF8400EF
8400EF8400EF8400FD8400FF8300F88300DC7600D08400EF8400FF8400FF8400E18300DC7F00E5\
7300D77F00D77F00D78300EC8300DC8400EF8800FF8A00F88D00FF8F00FF8F00FF
8F00FF8D00FF8B00FF8A00FF8400FD8300EC8300EC8300F87F00D77300D77600DC8300F88400EF\
8400FD8400FD8800FF8800FF8800FF8A00FF8D00FF9100FF9100FF9100FF9100FF
9100FF9D00FF8B00FB8A00FF9900FF9900FF9900FF9800FF8400FD7600D07300CA7300CA7600D0\
8800E68A00FF9900FF9B00FF9D00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF\
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF\
8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8D00FF8D00FF8B00FB8D00FF8D00FF8D00FF8B00FB8A00F88800F4\
8400EF8300EC7F00E57F00E57F00E58300EC8400EF8800F48A00F88A00F88A00F8
8A00F88800F47800EF8400EF8800F48800F48800F48800F48800F48400EF8300EC8300EC8300EC\
8300F88300F88300F88300F88300F88300F88300F88300F88400FD8400FD8800FF
8800FF8800FF8800FF8A00FF8B00FF8B00FF8A00FF8A00FF8800FF8400FD8400FD8400FD8400FD\
8400FD8400FD8400FD8800FF8800FF8800FF8400FD8400FD8400FD8400FD8400FD
8400FD8400FD8400FD8400FD8800FF8A00FF8B00FF8B00FF8D00FF8B00FF8A00FF8800FF8400FD\
8400FD8300F88300F88300F88300F88400FD8A00FF8B00FF8D00FF8D00FF8B00FF
8B00FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF8800FF8400FD8400FD\
8400FD8400FD8400FD8800FF8800FF8A00FF8A00FF8B00FF8A00FF8A00FF8A00FF
8A00FF8A00FF8800FF8800FF8800FF8800FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF8A00FF\
8A00FF8B00FF8B00FF8B00FF8D00FF8B00FF8B00FF8B00FF8B00FF8B00FF8D00FF
8B00FF8A00FF8800FF8800FF8800FF8400FF8400FF8800FF8800FF8800FF8800FF8400FF8400FF\
8400FF8400FF8400FF8800FF8A00FF8B00FF8D00FF8F00FF8F00FF8F00FF8D00FF
8B00FF8B00FF8A00FF8800FF8800FF8800FF8800FF8800FF8A00FF8800FF8800FF8800FF8800FF\
8800FF8800FF8800FF8A00FF8A00FF8B00FF8D00FF9100FF9100FF9100FF9100FF
9100FF8F00FF8B00FF8B00FF8B00FF8B00FF9900FF9900FF9800FF9600FF9600FF9800FF9900FF\
9D00FF9F00FF9F00FF9D00FF9D00FF9B00FF9B00FF9B00FF8D00FF8D00FF8D00FF
9B00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9D00FF9F00FF9F00FF\
9F00FF9F00FF9F00FF9F00FF9F00FF9F00FF9D00FF9D00FF9D00FF9B00FF8D00FF
8D00FF8D00FF8D00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF\
8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FB5E00CC4E00B7\
4800AD4800AD5500AD5500AD5500AD5800B26600D08800F48B00FB8D00FF8D00FF
8B00FB7B00EA5C00BB6A00D58400EF8A00F88B00FB8B00FB8A00F86C00D85C00BB5A00B75800BE\
5800BE5800B25500AD5500AD5500AD5500AD5800B25800B25800B25800B25800B2
5800B25A00B77A00E68A00FF8D00FF8F00FF8D00FF7D00ED5E00BE5A00B75800B25800B25800B2\
5800B25800B25A00B76A00D58400EF5C00BB5A00C35A00C35800B26500CA5800B2
5800B25800BE7600DC5A00B76C00E68B00FF8D00FF8F00FF8D00FF5F00CE5C00C75A00C35A00C3\
5800BE5800BE5800BE5800BE5A00C35C00C77B00EA8D00FF8F00FF8D00FF5F00CE
5E00CC7800E15C00C75C00C75A00C35A00C35A00C35C00C75C00C77800EF5C00C75A00C35A00C3\
5800BE5800BE5800BE5A00C35A00C35A00C35A00C36A00D55A00C35A00C35A00C3
6A00E15A00C35A00D05A00C37800EF6A00E15C00C75A00C35A00C35A00C35A00C35C00C75E00CC\
7D00FB8D00FF8F00FF8F00FF8D00FF8B00FF7B00F87B00F88B00FF8D00FF8D00FF
8D00FF6D00EA5C00C75C00C75A00C35A00C35A00C35A00C35A00C35A00D05A00C35A00C35A00C3\
5A00C35A00C35A00C35C00C76D00DC8D00FF8F00FF9100FF8F00FF8D00FF6D00EA
5C00C75C00C75A00C35A00C35A00C35A00C35C00C75E00CC7B00F87A00F45C00C75A00D05A00C3\
5A00C35A00C35A00D05C00C77A00F45F00DC6F00E18F00FF9100FF9100FF9100FF
8F00FF7D00FB6D00DC5C00D56A00D55C00D55C00D55C00D55C00D55C00D55E00D86100DF9D00FF\
9F00FF9D00FF9B00FF8B00FF6D00DC5F00DC5E00D86C00D85E00D85E00D85E00D8
5E00D85E00D85E00D86D00EA6D00EA7B00F85E00D85E00D85E00D85E00D85F00DC6100DF6300E3\
9100FF9F00FF9F00FF9F00FF8F00FF6100DF6D00EA5E00D85E00D85E00D86C00D8
5E00D85E00CC6100D17F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF\
8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FB7A00E64C00B2\
4700A74500A24500A24500A24F00A25300A75800B26A00D58B00FB8D00FF8D00FF
8B00FB8800F45A00B75800B25A00B77800E18800F48B00FB8B00FB8800F45C00BB5800B25500AD\
5300A75300A75300A75300A75300A75300A75300A75300A75500AD5500AD5300A7
5300A75500AD5A00B77A00E68B00FF8F00FF8D00FF8B00FF6A00D55800B25300A75300A75300A7\
5300A75300A75500AD5800B27600DC6600D05800B25500AD5500AD5500AD5300A7
5300A75300A75500AD5800B25C00C77D00FB8D00FF8F00FF8D00FF6D00EA5C00C75800BE5500B9\
5500B95500B95300B25500B95500B95A00C36C00E68B00FF8D00FF8B00FF6C00D8
5A00C35800BE5800BE5500B95500B95500B95500B95500B95A00C37600EC6600D05800BE5800BE\
5500B95500B95300B25300B25500B95500B95500B95800BE5800BE5800BE5500B9
5500B95500B95500B95500B95500B95500C35500B95500B95500B95500B95500B95800BE5C00C7\
7B00F88D00FF8F00FF8F00FF8D00FF7B00F85E00D85C00C76C00E68A00FF8B00FF
8B00FF6C00E65A00C35800BE5500B95500B95500B95500B95500B95500B95500B95500B95500B9\
5500B95500B95500B95A00C36C00E68B00FF8F00FF8F00FF8F00FF7B00EA5C00C7
5800BE5500B95500B95500B95500B95800BE5A00C36C00E68A00FF6C00D85A00C35800BE5500B9\
5500B95500B95500B95800BE5A00C35C00D57D00FB8F00FF9100FF9100FF9100FF
8F00FF6D00DC5C00D55A00D05800CA5800CA5800CA5800CA5800CA5A00D05E00D87D00FF8D00FF\
8D00FF7D00FB5F00DC5C00D55C00D55A00D05A00D05A00D05A00D05A00D05A00D0
5800CA5800CA5800CA5800CA5A00D05A00D05800CA5800CA5800CA5800CA5A00D05E00D87D00FF\
9F00FF9F00FF9F00FF8F00FF6D00EA5C00D55800CA5800CA5800CA5800CA5A00D0
5A00C35C00C77B00F88D00FF8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00F1
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8A00F86A00D5\
4C00B24700A74500A24500A24500A24500A25500AD5A00B77B00EA8D00FF8D00FF
8B00FB7A00E65800B25300A75300A75500AD5A00C38800F48800F48800F47800E15800B25500AD\
5500AD5300A75300A74F00A24F00A25300A75300A75500AD5500AD5500AD5500AD
5300A75300A75500AD6A00D58A00FF8B00FF8B00FF8A00FF7800E15500AD5300A74F00A24F00A2\
4F00A24F00A25300A75800B26A00D57800E15A00B75800B25500AD5500AD5300A7
4F00A24F00A25300A75800B25C00BB7D00ED8F00FF8F00FF8D00FF7B00F85C00C75A00C35800BE\
5500B95300B25300B25300B25500B95A00C35E00CC8B00FF8D00FF8B00FF7B00EA
5C00C75800BE5500B95300B25300B25300B25300B25500B95A00C36A00E16A00E15A00C35800BE\
5800BE5500B95300B25300B25300B25500B95500B95800BE5800BE5800BE5500B9
5500B95500B95500B95500B95500B95500B95500B95500B95500B95300B25300B25500B95A00C3\
7B00EA8D00FF8F00FF8F00FF8B00FF5C00D55800BE5800BE5A00C36A00D58800FF
8800FF5C00C75A00C35800BE5500B95300B25300B25300B25300B25500B95800BE5800BE5500B9\
5500B95300B25300B25800BE7800EF8A00FF8D00FF8D00FF8B00FF6C00E65A00C3
5500B95300B25300B25300B25500B95800BE5C00C77B00F88A00FF5E00D85C00C75800BE5500B9\
5300B25300B25300B25500B95A00C35E00D88D00FF9100FF9100FF9100FF9100FF
8100FF6D00EA5C00D55800CA5500C35500C35300BE5500C35800CA5A00D06D00DC8B00FF8D00FF\
7D00FB5E00D85A00D05A00D05A00D05A00D05C00D55C00D55C00D55C00D55A00D0
5A00D05800CA5500C35500C35800CA5800CA5800CA5800CA5500C35500C35800CA7800EF9900FF\
9D00FF9F00FF9D00FF7D00FB5C00D55800CA5500C35500C35500C35800CA5A00C3
5C00C76C00E68B00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00F1
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8A00F8\
7800E16600D05800BE5500B94700A75300A76100C36600D07B00EA8D00FF8D00FF
8B00FB7A00E65500AD5300A74F00A25300A75500AD5800B26600D08400EF8400EF7800E16600D0\
5A00C36500CA5500AD5300A74F00A25300A76100C37600DC7800E17800E16600D0
5800B25500AD5500AD5800B27800E18800FF8800F48400EF7600EC6100C34F00AD4F00A24C0099\
4F00A24F00A25500B96600D06C00E68A00F88800F46C00E66A00D56600D05500B9
5300A75300A75500B97600EC7A00E68D00FF8F00FF9100FF8F00FF8D00FF7D00FB7A00E67800E1\
6600D05800BE5300B25300B26100D07600EC8800F48B00FF8B00FF8B00FF8A00F8
7800E17600DC6500CA5500B95300B25300B25300B26500D77800E16C00D88A00FF7B00EA7A00E6\
6600D05800BE5500B95300B25300B26100D07300D76600D07600EC7600EC6500D7
5500C36100C35800CA5800BE5800BE5800BE6600D06600D06600DC5800BE5500B95500B97600EC\
8A00FF8D00FF8F00FF8D00FF8A00FF5A00C35500B95300B25500B95800BE5A00C3
7800EF7800EF7800EF7800EF6500CA5500B95300B25300B25500B96500D77800E17800EF6A00D5\
5800BE5500B95300B25500B96500D77800EF7A00F47A00E67A00F48400FD7300E5
6100C35300B25300B25500B95A00D07A00F47D00FB8D00FF8D00FF8B00FF7B00F86A00E15800CA\
5500B95300B25300B25500B96600DC8A00FF8F00FF9100FF9100FF9100FF9100FF
8F00FF7F00FF7B00F85C00D55800CA5500C35300BE5300BE5800CA7800EF8A00FF8B00FF8B00FF\
6C00E66600D05800CA5A00D06A00E17B00F88B00FF8B00FF8B00FF8A00FF7B00F8
6A00E15A00D05800CA5800CA5A00D06A00E16600DC5800CA5800CA5500C35800CA6600DC8800FF\
8B00FF7D00FF8A00FF7800EF6600DC5500C35300BE5500C35800CA6600D07A00F4
6D00DC8B00FF8D00FF8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00F18D00EF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF\
8B00FB8A00F88800F48400EF6600D04C00B25800B28400EF8A00F88B00FB8D00FF
8B00FB7800E15500AD4F00A24F00A24F00A25300A75500AD5800B25A00B77800E18400EF8800F4\
8800F48800F47600DC5500AD5300A75300A77300D78800F48A00F88A00F88A00F8
7A00F45A00C35800B25800B25800B25800B25800B25800B25500AD5300A74F00A24C00994C0099\
4F00A25300A75800BE8800FF8B00FF8D00FF8D00FF8B00FF8B00FF8A00FF7800E1
5800BE5300A75500AD8300EC8A00FF8D00FF9100FF9100FF9100FF8F00FF8D00FF8D00FF8B00FF\
8A00FF6A00D55500B95500B95500B97600EC8400EF7A00F48A00F88800F47A00F4
7800EF8300F87F00F16F00D05300B25300B25500B96500D78800FF8B00FF8D00FF8D00FF8B00FF\
8A00FF7800EF5800BE5500B95300B25500B97F00E58300F88400FD8300F87F00F1
6500CA5800BE8300F88400FD7A00E65E00CC7B00EA8A00FF8800FF6A00D55A00C35800BE5A00D0\
8800F48D00FF8D00FF8B00FF6C00E65800BE5500B95300B25500B95500B95800BE
5800BE6600DC8400FD8800FF7800EF5800BE5300B25300B25500B98400FD8A00FF8B00FF8A00FF\
7A00E65A00C35500B95500B95500B95800BE5800BE5800BE5800BE5800BE5500B9
5300B25300B25500B95A00C37A00F48B00FF8F00FF8F00FF9100FF8F00FF8D00FF8A00FF7800EF\
5800BE5300B25300B25500B96600DC8B00FF8F00FF9100FF9100FF9100FF9100FF
9100FF8F00FF8D00FF8A00FF6A00D55800CA5500C35500C35800CA6A00E18A00FF8B00FF7B00F8\
5A00D05800CA5800CA7600EC8A00FF8B00FF8F00FF8F00FF9100FF8F00FF8D00FF
8B00FF7B00F85E00D85E00D85E00D89800FF8A00FF6C00E65A00D05800CA5800CA5800CA6600D0\
5C00D55C00D55A00D05800CA5500C35500C35500C35800CA5C00D57A00F48B00FF
8D00FF8D00FF8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00F18D00EF7F00EF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8D00FF8D00FF8B00FB8A00F87800EF5C00BB5C00C77A00F48A00F88A00F8
8A00F86A00D55500AD4F00A24C00994F00A24F00A25300B25800B25800B25A00B76A00C78800F4\
8800F48A00F88800F46600D05500AD5500AD5800BE8800F48B00FB8D00FF8D00FF
8B00FB8A00F87800E15A00B75800B25500AD5300A75300A74F00A24F00A24F00A24C00994C0099\
4F00A25500AD7800E18B00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8A00FF
6600D05500AD5500AD5A00B78800FF8D00FF8F00FF9100FF9100FF9100FF9100FF8F00FF8F00FF\
8B00FF7A00F45A00C35500B95500B95500B95800BE5800BE5A00C35A00C35800BE
5800BE5800BE5800BE5500B95300B25300B25300B25800BE7A00E68B00FF8D00FF8F00FF8F00FF\
8D00FF8A00FF6A00D55800BE5300B25300B25500B95800BE5800BE5800BE5800BE
5500B95800BE6A00E18A00FF8B00FF8B00FF8D00FF8D00FF8D00FF8A00FF6A00D55A00C35A00C3\
6A00D58A00FF8B00FF8A00FF6A00D55500B95500B95800BE7600EC5A00C35800BE
5800BE5800BE5A00D08400FD8400FD5800CA5500B95300B25800BE6A00E18B00FF8D00FF8D00FF\
8B00FF7A00F45A00C35500B95500B95300B25300B25300B25300B25300B25300B2
5300B25500B95800BE7A00E68B00FF8F00FF9100FF9100FF9100FF9100FF9100FF8D00FF7B00FF\
5A00D05500B95300B25500B95A00C38B00FF8F00FF9100FF9100FF9100FF9100FF
9100FF9100FF8F00FF8D00FF7A00F45800CA5500C35500C35800CA6A00D58A00FF8A00FF6C00D8\
5800CA5500C35800CA5A00D08A00FF8D00FF9100FF9100FF9100FF9100FF9100FF
8F00FF8F00FF8D00FF7D00FB8D00FF8D00FF9B00FF8B00FF6C00E65A00D05800CA5500C35500C3\
5500C35500C35500C35500C35500C35500C35800CA5A00D07A00F48B00FF8D00FF
8D00FF8D00FF8F00FF8F00FF8F00FF8F00FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00EF\
8D00EF7D00ED7D00ED
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB7A00F45C00C74F00BB6A00D58800F4
8800F47600DC5300A74F00A24C00994F00A25300A76500CA8300EC6600D05A00B75A00B75C00BB\
7800E18800F48800F48400EF5800B25500AD5800B26A00D58A00F88D00FF8F00FF
8D00FF8D00FF8A00F88800F45A00C35500AD5300A74F00A24F00A24F00A24F00A24C00994F00A2\
4F00A25800B27A00F48B00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FF
7A00F45800B25500AD5800B27800E18B00FF8F00FF8F00FF9100FF9100FF9100FF8F00FF8F00FF\
8D00FF8B00FF7800E15800BE5300B25300B25300B25300B25500B95500B95500B9
5500B95500B95500B95500B95300B25300B25300B25500B96A00E18A00FF8D00FF8F00FF8F00FF\
8F00FF8D00FF7A00F45A00C35500B95300B25300B25300B25500B95500B95500B9
5500B95800BE6A00D58A00FF8D00FF8D00FF8D00FF8F00FF8F00FF8D00FF8B00FF6A00D55A00C3\
5A00C36A00E18800FF7800E15800BE5500B95800BE7800E18800FF8800FF6A00D5
5A00C35800BE5800BE5A00C37300E56F00D05300B25300B25500B96A00E18B00FF8F00FF8F00FF\
8F00FF8B00FF7A00F45A00C35500B95300B25300B25300B25300B25300B25300B2
5300B25500B95C00D58B00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8B00FF\
6A00E15800BE5300B25500B95A00D07B00FF8F00FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF8D00FF7B00F85A00D05500C35500C35500C35A00D08800FF7A00FF5C00D5\
5800CA5500C35500C35A00D07A00F48B00FF8F00FF9100FF9100FF9100FF9100FF
9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8A00FF6C00E65A00D05800CA5500C3\
5500C35500C35500C35500C35500C35500C35A00D06C00E68B00FF8D00FF8D00FF
8B00FF8B00FF8B00FF8B00FF8B00FF8D00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00EF8D00EF\
7D00ED7D00ED7B00EA
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB8A00F86A00C74F00BB6A00D5
8300EC6100C34700A74C00994C00994C00995300A76500CA8800F48800F48800F45C00C75A00B7\
5A00B75C00BB7800EF8400EF6500CA5500AD5500AD5800BE8800F48B00FB8F00FF
8F00FF8F00FF8D00FF8B00FB8800F46A00D55800B25500AD5300B26C00CA4F00AD4F00A24F00A2\
5300A75800BE8800F48D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8B00FF
8A00FF6600DC5500AD5500AD5A00C37A00F48B00FF8D00FF8F00FF8F00FF8F00FF8D00FF8D00FF\
8B00FF8A00FF8800FF5800BE5500B95300B25300B25300B25500B95500B95500B9
5800BE5800BE5800BE5800BE5500B95500B95300B25500B95A00C37A00F48B00FF8D00FF8F00FF\
8F00FF8D00FF8B00FF7800E15800BE5300B25300B25300B25300B25500B95500B9
5500B95500B95A00C38800FF8B00FF8D00FF8D00FF8D00FF8F00FF8F00FF8D00FF8A00FF6A00D5\
5A00C35800BE6500D76500D75500C35500B95800BE7A00F48B00FF8D00FF8A00FF
6C00E65A00C35800BE5500B95500B95500C35300B25300B25500B95A00C38A00FF8D00FF8F00FF\
8F00FF8F00FF8B00FF7A00F45A00D05800BE5500B95300B25E00BE6C00D75300B2
5500B95800BE7A00F48D00FF9100FF9100FF8F00FF8D00FF8D00FF8F00FF8F00FF8F00FF8B00FF\
7A00F45800CA5500B95300B25800BE7A00E68B00FF8F00FF9100FF9100FF9100FF
9100FF8F00FF8F00FF8D00FF8A00FF5C00D55800CA5500C35500C35800CA7800EF8800FF6600DC\
5800CA5500C35500C35800CA5A00D06C00E68B00FF8D00FF8F00FF9100FF9100FF
9100FF9100FF9100FF9100FF8F00FF8F00FF7F00FF6F00EF8B00FF8B00FF6C00E65A00D05500C3\
5500C35300BE5E00CA6100D05500C35800CA6A00E18A00FF8B00FF8B00FF7B00F8
6A00D55C00C75A00C35C00C76C00E68A00F88D00FF8F00FF8F00FF8F00FF8D00EF8D00EF7D00ED\
7D00ED7B00EA7B00EA
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB8A00F86A00C74E00B7
6100C35E00BE4C00994100994000944C00995300A76600D08A00F88B00FB8B00FB8A00F87A00E6\
5C00BB5A00B75A00B77300D77F00E55500B95300A75500AD6A00D58A00F88D00FF
8D00FF8F00FF8F00FF8D00FF8B00FB8A00F87A00E65A00C35800B25300A74F00AD4F00A24F00A2\
5300A75A00C37B00F88D00FF8D00FF6100D16D00DC7A00F48A00FF8A00FF8A00FF
8800FF8300F85500B95300A75500AD6600D08800FF8A00FF8A00FF8A00FF8A00FF8A00FF7A00F4\
6C00D87A00E68400FD7300E55500AD4F00AD4F00AD5300BE7B00DE7300E57300D7
6600D07800EF7800EF7800EF7600EC6F00DE5300B25300B25500B96600D08800FF8A00FF8D00FF\
8D00FF8D00FF8B00FF8400EF5800BE5300B25300B25300B25E00CA6F00D07F00E5
7300E55500B95800BE7800EF8800FF7A00F46C00D85F00DC8D00FF8F00FF8F00FF8B00FF8800FF\
5800BE5500B95500B95500B95300B25500B95A00C37B00F88D00FF8F00FF8D00FF
8B00FF7A00F45C00C75800BE5500B95300B25300B25300B25300B25800BE7800EF8A00FF8D00FF\
8D00FF8F00FF8F00FF8B00FF7B00F86A00D55800BE5500B95300B25300B25300BE
5500B95C00D57D00FB8F00FF9100FF8F00FF7D00FB6D00DC5E00D87A00F48A00FF8A00FF8A00FF\
8400FD5800CA5500B95300B25500B95A00D08800FF8B00FF8B00FF8B00FF8B00FF
8B00FF7D00FB6F00EF8D00FF8A00FF7800EF5800CA5500C35500C35800CA6600DC8400FD6600DC\
5800CA5500C35500C35500C35800CA5A00D05C00D57B00F88B00FF8B00FF8D00FF
8D00FF8D00FF8D00FF8D00FF8B00FF8B00FF6D00EA6F00ED8D00FF8D00FF7D00FF6C00E65A00D0\
5800CA5500C35500C35500C35800CA5C00D57A00E68A00FF8A00FF6C00E65A00D0
5800BE5500B95500AD5500AD5A00B76C00CC8B00FB8D00FF8F00FF8D00FF8D00EF7D00ED7D00ED\
7B00EA7B00EA7B00EA
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB8800F47300D7
4700A74500A24000944000944000944100994700A76600D08A00F88D00FF8D00FF8D00FF8B00FB\
8800F46A00C75A00B75800B25500AD5E00B25300A75300A75800B28400EF8800F4
8B00FB8B00FB8D00FF8D00FF8D00FF8D00FF8B00FB8800F46A00C75800B25500AD5300A75300A7\
5500AD6A00D58A00FF8D00FF8B00FF6C00D85C00BB5A00B75A00B75A00B75A00C3
6600D06500CA6100B95300A75300A75500AD5800BE6600C35A00B75A00B75A00C35800B25800B2\
5800B25800BE6500CA6100C35300B25300A75300B25300B25500B95800BE7600DC
7800E18A00FF8B00FF8A00FF6A00D56500CA5500B95300B25300B25500B95A00C35C00C76C00E6\
8B00FB8B00FF7B00F87800E15800BE5300B25300B25300B25300B25500C36500CA
6F00DE5500B95500B95800BE5800BE5A00C35A00C35E00CC8B00FF8F00FF8F00FF8D00FF7A00F4\
5800BE5500B95300B25300B25300B25500B96A00D58B00FF8F00FF9100FF8F00FF
8F00FF8D00FF8A00FF6A00E15800BE5500B95300B25300B25300B25500B95800BE6A00D56C00E6\
7D00ED8D00FF8F00FF8F00FF8D00FF8A00FF6A00E15A00C35500B95500B95500B9
5A00C37A00F48D00FF9100FF9100FF8D00FF6D00DC5C00C75A00C35A00C35A00C35C00D55C00C7\
8300F85500B95300B25300B25300B25500B96600D06A00D56A00E16A00D55C00D5
5C00C75E00D87B00F88B00FF7A00E66600DC5800CA5500C35500C35500C35800CA6600DC6600D0\
5800CA5800CA5800CA5500C35500C35500C35800CA5A00D05C00D56C00D86C00E6
7B00F87B00FF7B00F86D00EA6C00D85E00D85F00DC7D00FB8100FF8F00FF8F00FF8B00FF6C00E6\
5A00D05800CA5500C35800CA5A00D07A00F48A00FF8A00FF6A00D55800BE5500B9
5300B24F00A25300A75500AD5A00B77A00E67D00ED8D00FF8D00EF8D00EF7D00ED7D00ED7B00EA\
7B00EA7B00DC7B00DC
8100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF7D00ED7800E17300D7
4F00AD4000944000943C008F3C008F4000944700A77600DC8A00F88F00FF8F00FF8F00FF8D00FF\
8B00FB8A00F87A00F45A00C35800B25300A75300A75300A75300A75800B26600D0
7800E17A00E67B00F88B00FB8D00FF8D00FF8D00FF8B00FB8A00F87A00E65A00B75500AD5500AD\
5800B27800EF8B00FB8D00FF8D00FF7B00F85C00BB5800B25500AD5500AD5500AD
5500AD5500AD5300A75300A75300A75300A75500AD5500AD5500AD5500AD5500AD5300A75300A7\
5300A75300A75500AD5300A75300A75300A75300A75300A75500B95500B95A00C3
6A00D58B00FF8D00FF8B00FF5C00C75800BE5500B95300B25300B25500B95500B95800BE5A00C3\
6C00D88A00FF7B00EA5C00C75800BE5300B25300B25300B25500B95500B95500B9
5500B95500B95500B95500B95500B95500B95800BE6A00D58B00FF8F00FF8F00FF8D00FF5E00D8\
5800BE5500B95300B25300B25500B95A00C37A00F48D00FF8F00FF9100FF9100FF
9100FF8F00FF8D00FF8A00FF6C00E65A00C35500B95300B25300B25500B95500B95800BE5C00C7\
6D00EA8D00FF8F00FF9100FF8F00FF8D00FF8B00FF7A00F45A00C35800BE5800BE
5C00C78B00FF8F00FF9100FF9100FF8F00FF5F00DC5A00C35800BE5500B95800BE5800BE5800BE\
5800BE5500B95500B95300B25500B95500B95800BE5800BE5800BE5800BE5800BE
5A00C36C00D88A00FF7B00FF5C00D55A00D05800CA5500C35500C35500C35800CA5A00D05A00D0\
6A00E17800EF5C00D55A00D05A00D05800CA5800CA5800CA5800CA5A00D05A00D0
5A00D05C00D55C00D55A00D05A00D05C00D56D00DC8D00FF8F00FF9100FF9100FF8F00FF8B00FF\
6C00E65A00D05800CA5A00D06C00E68B00FF8B00FF6C00E65A00C35500B95300B2
5300A75300A75300A75800B27800E17B00EA8B00FB8D00EF8B00ED7D00ED7D00ED7B00EA7B00EA\
7B00DC7A00D87A00D8
7F00FF8D00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FB5C00C74C00B2
4500A24000944000943C008F4000944000944700A77800E18B00FB8F00FF8F00FF8F00FF8F00FF\
8F00FF8D00FF8B00FB8800F46A00C75800B25500AD5300A75300A75300A75500AD
5500AD5800B25A00B76C00D88B00FB8D00FF8F00FF8F00FF8D00FF8B00FB8800F46A00E15A00B7\
5C00BB6D00DC8B00FB8F00FF8D00FF8B00FB8A00F86A00C75800B25800B25800B2
5800B25800B25800B25800B25800B25800B25800B25800B25500AD5500AD5500AD5500AD5500AD\
5500AD5500AD5800B25800B25800B25800B25800B25800B25800B25800BE5A00C3
5E00CC8B00FF8D00FF8D00FF6D00DC5C00C75A00C35800BE5800BE5800BE5800BE5800BE5A00C3\
5E00CC8A00FF7B00EA5C00C75A00C35800BE5800BE5800BE5800BE5800BE5800BE
5800BE5800BE5800BE5800BE5800BE5A00C35C00C76C00E68B00FF8F00FF8F00FF8D00FF5F00DC\
5C00C75A00C35A00C35A00C35A00C35C00C78B00FF8F00FF9100FF9100FF9100FF
9100FF9100FF8F00FF8D00FF8B00FF7A00F45C00C75A00C35800BE5800BE5A00C35A00C35C00C7\
6D00DC8D00FF8F00FF9100FF9100FF8F00FF8F00FF8B00FF7B00F85E00D85E00CC
6D00EA8D00FF9100FF9100FF9100FF8F00FF8B00FF5E00D85C00C75A00C35A00C35A00C35A00C3\
5A00C35A00C35800BE5800BE5800BE5A00C35A00C35A00C35A00C35A00C35A00C3
5C00C77B00EA8B00FF5F00DC5E00D85C00D55A00D05A00D05A00D05A00D05A00D05C00D55E00D8\
8A00FF8B00FF8B00FF8A00FF6C00E65C00D55A00D05A00D05A00D05A00D05A00D0
5A00D05C00D55C00D55C00D55E00D85F00DC7D00FF8F00FF9100FF9100FF9100FF9100FF8F00FF\
8B00FF6D00EA5E00D85E00D88B00FF8D00FF8B00FF5E00CC5A00C35500B95500AD
5300A75500AD5800B27800E17A00D88A00F88B00ED8B00ED7D00ED7B00EA7B00EA7B00EA7B00EA\
7A00D87A00D87A00D8
7F00EF7F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8A00F85A00C3
4800AD4500A24100994000944100994500A24800AD7800E18B00FB8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8D00FF8D00FF8A00F88800F45A00C35800B25500AD5300A75300A7
5500AD5500AD5800B25C00BB7B00F88D00FF8F00FF8F00FF8F00FF8D00FF8D00FF8B00FB8A00F8\
6D00DC8B00FB8D00FF8F00FF8F00FF8F00FF8D00FF8A00F87A00E65C00C76A00E1
7800E17800E17800E16A00D57800E17800E17800E16A00D56A00D56A00E16A00D56A00D56600DC\
5A00C35C00C77800EF7A00E67A00E67A00E67A00E67A00E66C00E67A00E66C00E6
6F00DF8D00FF8F00FF8F00FF8B00FB7B00EA7A00E67A00E67A00E67A00F46A00D55C00C76C00E6\
7B00EA8B00FB8B00FF7D00FB7B00EA7A00E67A00E67A00E66C00D87A00F47A00F4
7A00F46C00E66A00E16A00E17A00E67A00E67B00EA6F00ED7F00FF8F00FF8F00FF8F00FF8D00FF\
7D00FB8A00F88A00FF7B00EA7B00EA6F00DF8D00FF8F00FF9100FF9100FF9100FF
9100FF9100FF9100FF8F00FF8F00FF8D00FF8B00FF7B00EA7A00E67A00E67A00E67B00F87D00ED\
8D00FF8F00FF9100FF9100FF9100FF9100FF8F00FF8F00FF8D00FF8B00FF7D00FB
8D00FF8F00FF9100FF9100FF9100FF9100FF8F00FF8D00FF7D00FB7B00F88A00FF8A00FF6D00EA\
7B00EA7B00F87B00F87B00F87B00F87B00F87B00F87B00F88A00FF7B00F86D00DC
6D00EA8B00FF8D00FF7F00FF7D00FB7B00F87B00F87B00F87B00F86D00EA8A00FF7D00FF8D00FF\
8F00FF9100FF9100FF8F00FF8D00FF8B00FF7B00F86D00DC7B00F87B00F86D00EA
6D00EA7B00F88B00FF8B00FF8B00FF8D00FF8100FF9100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8D00FF6F00ED8B00FF8F00FF8F00FF8D00FF8B00FB5E00CC5A00C35A00C3
5A00C36A00D57A00E67B00EA8A00F88B00ED7D00ED7D00ED7B00EA7B00EA7B00EA7A00D87A00D8\
7A00D87A00D87A00D8
7D00ED7D00ED8D00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8B00FB7A00E6
6600C35800B24800AD4800AD4800AD4800AD5A00B77A00E68D00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB8A00F87A00E65A00C35A00B75800B2
5800BE5A00C35A00C36C00CC7D00ED8D00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8D00FF\
8D00FF8D00FF8F00FF8F00FF8F00FF8F00FF8F00FF8D00FF8B00FB8B00FB8B00FB
8B00FB8B00FF8B00FF8B00FF8B00FF8D00FF8B00FF8B00FF8B00FF8B00FF8B00FF8B00FF8B00FF\
8B00FF8B00FF8B00FF8B00FF8D00FF8D00FF8D00FF8B00FF8B00FF8D00FF8D00FF
8D00FF8F00FF9100FF9100FF8F00FF8D00FF8D00FF8D00FF8D00FF8D00FF8B00FF8B00FF8B00FF\
8D00FF8F00FF8F00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF
8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF8F00FF9100FF9100FF9100FF8F00FF\
8F00FF8D00FF8D00FF8D00FF8D00FF8D00FF8F00FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8D00FF8D00FF8D00FF8D00FF8D00FF8D00FF\
8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF
8F00FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF
8F00FF8F00FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8D00FF8D00FF8D00FF
8D00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8F00FF8F00FF8F00FF9100FF9100FF8F00FF8D00FF8B00FF8A00F88A00F8
8A00F88A00F88B00FB8B00FB8B00ED7D00ED7D00ED7B00EA7B00EA7B00DC7A00D87A00D87A00D8\
7A00D87A00D87A00D8
7B00EA7D00ED7D00FB8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF8F00FF8D00FF8B00FB
8A00F88800F48800F48800F48800F47A00E66D00DC8B00FB8F00FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8B00FB8A00F88A00F88800F4
8800F48A00F87B00EA8B00FB8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8D00FF8D00FF
8F00FF8D00FF8D00FF8D00EF7F00EF7D00ED7D00ED7B00EA7A00D87A00D87A00D87A00D87A00D8\
7A00CC7A00CC7A00CC
7D00ED7D00ED7F00EF8300FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF
8D00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8D00FF8D00FF
8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF\
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF\
8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF8F00FF
8F00FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF9100FF
9100FF9100F48300F48100F18100F18100F17F00EF7F00E17D00DF7B00DC7A00D87A00D87A00CC\
7800C77800C77800C7
7F00EF8100F18300F47B00E67B00DF7400D16F00C96F00C96F00C97400D17B00DF8800F68F00FF\
9100FF9100FF9100FF9100FF8F00FF8300EC7B00DF8600F19100FF9100FF9100FF
9100FF9100FF8A00FA7F00E67800D87400D17400D17400D16F00C96F00C96F00C96F00C96F00C9\
6F00C96F00C96F00C96F00C96F00C97400D17F00E68800F68F00FF9100FF9100FF
9100FF9100FF8800F67F00E67400D16F00C96F00C96F00C96F00C96F00C97800D87F00E68300EC\
7F00E67800D86F00C96F00C96F00C96F00C96F00C96F00C96F00C97800D88600F1
8F00FF9100FF9100FF9100FF8F00FF8600F17800D86F00C96F00D36F00D36F00D36F00D36F00D3\
6F00D37800E58300F88F00FF9100FF8F00FF8A00FF7F00F37400DE6F00D36F00D3
6F00D36C00CC6F00D36C00CC6F00D37B00EC8300F88300F87B00EC7400DE6F00D36F00D36F00D3\
6F00D36F00D36C00CC6C00CC6C00CC6C00CC6C00CC6C00CC6C00CC6C00CC6C00CC
7400DE7B00EC8600FD8A00FF8600FD7B00EC6F00D36F00D36F00D36F00D37400DE7F00F38A00FF\
9100FF9100FF9100FF9100FF9100FF8F00FF8F00FF9100FF9100FF9100FF9100FF
8F00FF8600FD7800E56F00D36F00D36F00D36F00D36F00D36F00D36C00CC6C00CC6C00CC6C00CC\
6F00D36F00D36F00D36F00D37B00EC8800FF8F00FF9100FF9100FF9100FF8D00FF
8300F87400DE6F00D36F00D36F00D36F00D36F00D37400DE7B00EC8600FD8800FF8600FD7F00FF\
7800EF7400E87400E87400E87400E87400E87400E87800EF8300FF8A00FF9100FF
9100FF9100FF9100FF8F00FF8800FF7B00F87800EF7400E87400E87400E87400E87400E87400E8\
7B00F88300FF8D00FF9100FF9100FF9100FF9100FF8F00FF8800FF7B00F86C00D7
6100C35C00B75C00B76600CE6F00DE7800EF7F00FF8600FF8A00FF8A00FF8600FF7B00F87B00F8\
7800E57800E57800E58300F88A00FF8F00FF9100FF9100FF9100FF9100FF8A00FA
8600F18300DC7800E37B00E67B00E67B00E67D00DC8100E38300E68100E37B00DC7A00CC7800C7\
7800C77600C37600C3
8300F48100F36800C531005E07001000000000000000000000000000000009001031005A6F00C9\
8D00FD9100FF9100FF8F00FF7B00DF3C006D39234C4C008A8300EC9100FF9100FF
9100FF8300EC5100941B0030020004000000000000000000000000000000000000000000000000\
00000000000000000000000000000000000009001031005A6F00C98D00FD9100FF
9100FF7F00E647007F120022000002000000000000000000000000000000020004120022270048\
1B003009001000000000000000000000000000000000000000000005000B270048
6C00C28D00FD9100FF8F00FF7800D8370065090010000000000000000000000000000000000000\
00000005000C2C00556C00CC8800FF8600FD5C00AE20003C020005000000000000
0000000000000000000000000000000900122300452C0055120023020005000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
00000210001E31005E4700862C00550B00150000000000000000000000000000021B00315C00AE\
8600FD9100FF9100FF8F00FF8300F86F00D36F00D38300F88F00FF9100FF8D00FF
6F00D331005E05000C000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000010001E4700867F00F38F00FF9100FF8A00FF6600C3
23004504000900000000000000000000000000000002000510001E31005E47008631005E120023\
05000C0200050200050200050200050200050400090B00173100636C00D78A00FF
9100FF9100FF8F00FF7800EF47008D15002C09001205000C05000C040009040009040009040009\
1000203100636F00DE8A00FF9100FF8F00FF8800FF6F00DE3C0078120025000000
02000405000C05000C0000000000000900122000403C00784C009947008D2C005515002A0B0015\
09001209001210001E27004C56009D8300EC9100FF9100FF8D00FD7400D1470078
20003C17002C17002C17002C17002A1C003120003937005F6100AB8100D57F00D57A00CC7800C7\
7600C36600B76500B2
8300F47600DC3A19530B0B0BAEAEB7F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFDEDEEA37234A090010\
6100B08D00FD9100FF8600F13C006D00000235403100000047007F8800F69100FF
8F00FF6600B90E0B104E4E53E5E5F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDEDEEA2A1B3A1200226F00C98D00FD
8A00FA56009D0000007D7D84F4F4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF8F8FF917FAD2A004E\
5A5A5EC0C0CAF8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECF84F455E
0B00156C00C28D00FD8600F1410076040404A4A4B5FBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFE5E5FD37304115002A6600C36100B914071947474EE5E5FDFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7DC391545200C297D7D8BE5E5FDFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F8F8FFAEAEC0170529020004201925AEAEC0F8F8FFFFFFFFFFFFFFFFFFFFF4F4FF747481050009\
6600C38F00FF9100FF8600FD51009B1E1B231407194700868300F88D00FF7B00EC
230045232325D0D0E6FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFF8F8FF88889800000051009B8A00FF8D00FF6C00CC0B0015
5A5A63E5E5FDFFFFFFFFFFFFFFFFFFFFFFFFE3FFF1D5EFE1819188100C120000021E17208A8A91\
C3C3D0D0D0DCD0D0DCD0D0DCD5D5E1D8D8E6D0D0DC9191990E0E0E310C557B00F8
8F00FF8F00FF7B00F82C005A0504076A7888A2B9D0A7BED7A7BED7ADC3DEB2CAE5B2CAE5ADC3DE\
6A7888020404370E5E7B00F88D00FF8300FF5100A40900121207197F91A4C2FFFF
D5FFFFC9FFF1C9FFF1D7FFFFC2DCF894A9C04C477A1E142A0900120B00153330537181918FA2B7\
94A9C094A9C07F91A423292E0B0C0E6600B98A00FA8A00FA6600B91000200C0E10
4766745881915378885378885881915378883C5663090E10191C226C00B48300D87A00CC7600C3\
6500B26100AD6100AD
8300F47300D822093C5E7B68FFFFFF8FB08A232A20151912151B15202C235A7F68F1FFFF4C227D\
10001C6F00C98A00FA6F00C9040009AEAEB7FFFFFF7F91A40200046C00C28F00FF
8D00FD5C00A6000000F6FFFFCEFFEC3547350B0C09000000000000000000040402020404000000\
0000000000000B0C09151912121B15202C2365A298E3FFFF2509412C00517F00E6
8300EC410076101214FFFFFF83B7961419120404020000000404020507052A4139B0FFFF9F48FF\
E6CAFFDFFFFF6F9D81202A200B0C09050705040402050705090E0B45685AF6FFFF
2A0B4A31005A7F00E67F00E62300414F5E63FAFFFF638B73202A200B0C09040402040402050705\
0C120E5A9186E6FFFF09001037006A41007B000000EFFFFFAEF3C9354735151912
0507050000000000000204041E2E279DFBECBBE6FF9B41DCFFFFFF99D7B02C3C2C151912050705\
000000000000000000000000000000000000000000000000000000000000121B15
415541D7FFC3FFFFF47B00EAA9D5FFC9FFFF5A8B7B2C3C2C1920171419144C7465FDFFFF1E1735\
3C00738800FF8F00FF7400DE1200232E2E314A51410000004C00927F00F36100B9
000000C3FFF8A6FFF839584F253023141912050705040402000000000000020404101B1725352C\
2533251920170C120C39584FF6FFFF3E374A31005E8300F88300F83C00731C0B23
FFFFFF6DAEA4304740202A201219120C120C202A2065986FF6FFFF99B0C75500A2C3FFB5B5FFFF\
5FA99D65916F65916A51745548684C415E4565986FE6FFFF78C9C910041E7400E8
8D00FF8A00FF5100A4000000ADFFB9C0FFFF8AD0B973C79283C7926FA97B53815E53815E8AD099\
FDFFFF51888819052A6F00DE7B00F847008D0000006A7888D8FFFFA7FFC9437455
101B1B0000020000000E1B1B3C6C6583D89DBBFFDFD5FFFFCAFFD89FCAD8C9FFFFAEFFFFA0FFC2\
99FFB999FFB9C9FFFFB9FFFF0000005100948300DC7400D11400252A2A3CB9FFFF
D0FFFFC0FFFFADFFF1A4FFE6A0FFDFA4FFE6BBFFFF81E8D70202046100A08300D86600B76500B2\
6100AD6100AD6100AD
8300F47D00EA400078050209CEFFFF5AA498000000090B0902040504050700000039686CE6FFFF\
10071B3C006D7800D8510094090010FFFFFFA2BE8DFFFFFF1E17312C00538600F1
8D00FD6F00C90900107491DAB2FFFF070E0E23302373866388A07665765629372E254045335C5F\
2A4C4F192E30070B0B00000000000000000202040483E6EFA2F3FF0200046C00C2
8600F15C00A6000000C3FFFF315C5C141E155F76566D8663556A4E2A454F09121420393CAEFFFF\
912AF4BBFFFF40747400000022271C4C5E4543533C27312E12222300000083E6EF
94D3FF0B00157400D18300EC31005A231733D3FFFF31585C0000002730234C5E454C5E45203737\
0E1B1C040709CCFFFF3A2C682000393C006D000000C7FFFF68BEBE040705090B07
3745315F7656556A4E253E41020204355F63C9FFFF8D00FFB0FFFF4883830000000B0E0B43533C\
6D86635F76563A5E6D335C5F335C5F335C5F335C5F335C5F254145192E30040505
0204022C3C2CB4DEA4AE6FEC8BAED166B5BC172A270000000405040B100E040709C5FFFF3A2C68\
31005E8600FD8A00FF5C00AE000002FFFFFFFFFFFF785F880900126100B94C0092
000002D5FFFF58A2A704070700000020291E485E45516A4E375348192E30070C0E000000000000\
0000000C100C14201E000002C5FFFF33224837006A8300F87F00F32300455E5371
C0FFFF1E37390000000C0E092330232C3A2C090B09203539D1FFFF8623EAA25CB9D0FFFF4F9491\
0E1B1B070C09020402000000070B07040504070E10B5FFFF5F68A915002A7B00EC
8F00FF8600FF31006329231ED5FFFF4F9194192C2E12231E151B14020402000000000000254347\
E6FFFF30154E2300486600CE4C0099000000788A9BD5FFFF8AE6A65E916A2C4A4F
0B1415000000000000000000040707142723304F394A7B5A538B653C684C315C5C2E5E4F3A5E43\
39653C4A867BC5FFFF94CAFF0200045C00AE8300DC450083000002A9FFFFBCFFFF
83EDED7DCC9276AE6D55A476407B5A63C08AE3FFFF5A7AB410001C6500B26F00C76600B76100AD\
5E00A75E00995E0099
7F00EF8300F47600DC29004C150227CAFFFF6F9D811519120B0C1412192220373C6FC3CCE6FFFF\
29454C2C0B4C6600B92C0051370063D3F8B704040253614892BBC70200046F00C9
8F00FF8800F64C008A0000009FC3FF8BFBFB000000738663B9DAA0738663090B0C000000000002\
0B14152543474883835E7A5E3940483048515EA7AED0FFFF99FFFF05050B6C00C2
8A00FA7B00DF200039472074A4FFFF04050483A0768DAE813C483C090C140407076FC3CCF4FFFF\
7A4FAD8A68F3DFFFFF539D9D00000037453137453105070B070C104C888DE3FFFF
76C9D819052A7800D88800F66600B902000478C5D7DAFFFF3A746300000043533C374531000202\
101B1E78D3DCF4FFFF151C2A31005A5C00A61B00303C454EE6FFFF7ADECC0B0E0B
2730236D86633C4C3A0000003C6C71D1FFFFC3FFFF4C008F96B9FFDCFFFF68B7BE090E0B22291E\
6D86634C5E450204050407091423251B30311B303120373C20393C355F634F9191
4E967B558A618FA97BC3EAC3AE8BC5C5FFFF92FFF4586A4822291E1019176FC3CCEFFFFF050409\
4C00928800FF8800FF3C0073220041FFFFFF94BE8DFFFFFF15002927004C4C0092
05000778C9C9DFFFFF73D7C01E1B12090B09516A4E485E4502020220393C6FBEBE7AE5E55AB0A2\
39392504040200000043787DEFFFFF0000025C00AE8A00FF8600FD37006A1C2223
EFFFFF92FFFF1E23190202002C3A2C1E291E090C0BAEFFFFB2FDFF2A0053454F5AD0FFFFCCFFFF\
63C7A9394F330C0C09040504141C150000004C888DE3FFFF0400074C00928A00FF
8F00FF8600FD3C00780C1215D5FFFFB9FFFF5AB0A24F5E402222170000000405040000008BEDED\
9FC3FF0000004C00995600AE0B00175F4C6DD5FFFFA0FFCCAEFFFFC0FFFFC7FFFF
C2FFFFB2FFFFA0FFFF73D0C0407B733365482E4C3522332517271C09100B12201C33533C557647\
638D5A92FDB9CAFFFF15052531005E7600DC7B00E651178B000000B4FFFFE6FFFF
B2FFD0A6B2987FA2AE61A476407B5A8FFFC9A2DCFF0000005800927300CA6A00BB6100AD5E0099\
5E00995E00995E0099
7D00ED7F00EF8300F47600DC29004C10001EE3C9FFECFFCE746CB994A0FFD5FFFF9FADFF413173\
0200044C008A6100B00B0015A76CE333402C313E2C020202D1FFC51B142C31005E
8800F68F00FF8300EC4100760000008F6CFA7163861417127D946D313A2C0C0E10718BD0C9FFFF\
ADBBFF926FFF7322C3AD41D5FFFFFF9191FF9FADFF40126D0000003C006D7F00E6
8F00FF8F00FF5C00A60000029100FF0B0C09556A4E4C5E450000002C3C58B4FFFF9BA9FF22093A\
00000000000019002E9643F47B8A6F000000090B09191C1B96B9FF9F78FF300C53
0200045100948800F69100FF8800F656009D05000B22003EAB71F153685A000000000000394048\
B0DAFF7837C710041C1500296C00C28300EC6600B91500290B00148363E69BB2C9
0000002730230000006A83C59845F8370E5E00000005000C000000291E488F6CFABB94D10E120C\
191E15000000584E6AA77FFFA27AFF9873FF926FFF8F6CFA8F6CFA8A68F38363E6
7B9FDECAFFBBB7FFA9FFFFF3D8ADF47B00EA6D0BC7D7ABF1F8FFD1DFDFF89227FD120022230045\
7800E58D00FF8300F820003C7F51ADA0D0990B0C097FA478AD8AC3000002470086
3C00730400091E003A8F6CFAE5A98F0E0E0B20291E04050453435E9100FF5A00AD4C008F8F83E3\
FAFAA686866A7891789829FF31005E20003C7F00F38F00FF8D00FF6F00D315002A
120022926FFFEC8A810B0C090C100C020402AB96CC741ECA0000021200230B0015020004411C6A\
98BEE5FFE6A46A5E43000000070B09101912A06AE32A00532300457F00F38F00FF
9100FF8F00FF6F00D31B003105000B664EB4B2FFA4ECD08F736648090C090000002C4531AB71F3\
1200232C005A6100C3310063120417FFFFFFA6FDB9AEFFDE9F6AE13100630B0015
05000C0E001E2A0056736AB7A7FFFFADFFC97DCC92487453273C2C1C3022315A356A9B6196D174\
A0DF7BD3FFED6F31B70000005C00AE8100F38300F46F00C7290048000000222245
535A4C735C4FA9E3C77ACC947ACC94BBFFFF0E00193700637100C96A00BB6100AD5E00995E0099\
5E00995C00965C008A
7B00EA7D00ED7F00EF8300F47300D81C0035300C53FFFFFFCAE6FF7B7BF631274F0000001B0030\
56009D7800D86100B0000000FFFFFF00000094B4814F6145536548869FB9040007
7400D18F00FF9100FF8600F141007605000BCA83FF0507054F61450E100C555F6D8D8DFF37195A\
00000209001005000B02000468735C47535F0B02120B00155100947F00E68F00FF
9100FF8F00FF7400D1020004C27BFF222A20303A2C191E151B2230A9ECFF6C48990000002C0051\
6600B96600B9270048120020E3C7FF19201700000078949B61489F020004270048
6600B98800F69100FF9100FF9100FF8A00FA6C00C220003922003EF6F6FF192017151912A9C0D8\
53158F0200044100767400D18D00FD9100FF8F00FF7800D84100760000008B25F1
637A5A0000002A33259F48FF0E001C20003C5C00AE6F00D36100B92C00550000006500C2AED0AE\
000000222A20C57FFF2A005300000010001E12002312002310001E0B0015090012
000000170B2783C27AC2FFB2DCFFDC150029000002230930D0D08AEAFFD01900301B00317400DE\
8D00FF8F00FF7800E505000CD5BBFD374735040504141912FFFFED1705292C0055
6600C36100B92C0055000002B265CA7A815E000000050705ECECFF2A005309001215002A000000\
8A7866FFFFABFFFFFFC79FE10400074700868800FF9100FF9100FF8A00FF6F00D3
27004C0E001CD848D82C3C2C0000004F6C4FBB78FD0000004700867400DE6F00D351009B1B0031\
020004C06A74E8A27F0B120C000000416148A76FED0000005C00AE8D00FF9100FF
9100FF9100FF8D00FF7800E5470086090012140522EAAD92DEA28B2231230407055E916A9F92FD\
0000005100A45600AE05000B917F6CCCFFE578B786F1FFFF27004E1200254C0099
5C00B75600AE37006A0B001505000B6FA266BBFFD896F3AE63A0736AA27899FFB9A6DCC2BBFF91\
BBFF91B9FFAB71A9780000025C00AE8100F38300E68300E67600D0530092200039
000000372A25BBFFAD92F4B0A7FFC978B0C70500096600B76D00C26500B25E00995E00995C0096\
5C00965C008A560083
7B00EA7B00EA7B00EA7D00ED8300F45C00AE000000ADBE99CAF1FD403A6D00000037005E7800CC\
8A00FA8800F6510094120020D7FFD7232A1EC5F1AB7D996D040402FAFFEA100C1C
3A006F8800F69100FF9100FF7B00DF0B0015AD6FEA33402C3A4731090C097A8B9D6565CA000000\
4100766F00C96C00C22C00510000001217140000003C006D7800D88A00FA8F00FF
9100FF8A00FA6600B9000000E6CAFF0B0C09303A2C0202025576B0A9ECFF0000024100768300EC\
8F00FF8F00FF6F00C9040009B776F8586C4F0507059DC9B54F40811500297800D8
8F00FF9100FF9100FF9100FF9100FF9100FF8F00FF6600B9000000D8BEFF637A5A3A4735C7E3FF\
0B00144100768300EC8F00FF9100FF9100FF9100FF9100FF8300EC31005A31005A
D7FFC30404024F6148C79FE10200046C00C28D00FD8F00FF8F00FF8300F841007B0E001CFAFAFF\
2A33254F6148C37DFF0000004C00927800E57F00F37800E56F00D36100B95C00AE
51009B20003C000002395333335A41000002230045020005533C33A6C38F0400073C00738600FD\
9100FF8F00FF6C00CC000000FFFFFF0E120C171E15000000C3E1A6A28FC2020005
6100B98600FD7800E520003C581C58ECF6B50000000E120CFDFDFF05000B4C00926F00D33C0073\
020004C7C783FFFFD8FFFFFF10041C31005E7F00F38D00FF8F00FF8D00FF8300F8
4C0092020004FFCAAD273325000000AEECAE47127A27004C7F00F38F00FF8F00FF8A00FF7400DE\
230045331E2CFFD399253023000000304735DFFBED0000006F00D38F00FF9100FF
9100FF9100FF9100FF8F00FF8600FD5C00AE0B0015715347FFC5A74C53430B120C65916AB7D0EC\
0000005100A43C0078050205FFFFA983C79278B786C3DFFB0000005C00B78800FF
8F00FF8F00FF8800FF7400DE3C00730000004C7147B5FFD1B0FFCCB0FFCCD7FFF1765EBE99D776\
B2F898B7FF9B94DE9F07050B3100587B00DA8300E68300E68300D87B00CC580092
100C0B2A221EBBFFAD8AE6A6C0FFE64C6F7D2300407100B96600A95E00995E00995C0096560083\
5600835600834A0074
7B00EA7B00EA7B00EA7D00ED8100F17800E31C00353E374AD3F4B437315C0202045C009B8A00EA\
9100F48A00EA37005E47127A819D6F4F6145D1FFB56D815E000000D3FFB76F8398
07020C7400D18F00FF9100FF8600F12700486C2C995365483A473114191283A08A5A5AB50B0410\
6C00C28A00FA8A00FA6600B910001C0202020000001B00304C008A6600B96F00C9
6C00C25C00A61B003031005ADEFFDE020202303E2C02040496D1FF6178B510001C7400D19100FF\
9100FF9100FF8300EC2000398856B784A478000000A9C7924F40812300418600F1
9100FF9100FF8F00FF8F00FF8F00FF9100FF9100FF7F00E6150029945FCA9BBE8D455540EDFFFD\
09020E56009D8F00FF9100FF9100FF8F00FF8F00FF9100FF8F00FF56009D0B0014
FFFFFF1519124F6148C79FE10400097800D89100FF9100FF9100FF8F00FF6C00C2000000FFFFFF\
4F61484F6148FFFFFF0000006C00CC8F00FF8F00FF8A00FF7400DE5F19A65F19A6
6600C36600C331005E05020904070427004C51009B27004C0704096D7D5C0C07102C00558300F8\
9100FF8F00FF5600A60E001CFFFFFF0000001217120204028A8A5CFFFFFF140522
31005E7F00F38600FD47008615051CFFF8B70B0C09050705FFFFE60900105600A68800FF7400DE\
1B0031373029F4F4A2FFFFEF785F8800000037006A5C00AE6600C36600C35600A6
1B0031220041FFFFBB0B0C09050705F8FFF81500294C00928D00FF9100FF9100FF9100FF8A00FF\
4C0092020004FFD79D48533C000000152017F4FFFF0000026600C38F00FF9100FF
9100FF9100FF9100FF9100FF9100FF7F00F331005E170C14FFC7A97B8A6F0B120C415E45E6FFFF\
0200043100631B003547272AEAFFFA4C745583C792B7D0EC0000026F00D38F00FF
9100FF9100FF9100FF8F00FF8300EC4C008A000002314A2E83D59883D59833395A10001E394539\
8FC778B2F898A9F89B5A865F0000004000716C00C07600C37600C36800AE3A0061
0000004F4037CCFFF67ACC94CCFFF62E2A4A37005A7300BB61009F5C0096560083560083560083\
4A007F4A007447006F
7B00EA7B00EA7B00EA7B00EA7D00ED8100F14000780B040ED1FFB5534C8400000056079D7F00ED\
8300F48300DC120020B273F11B201788A676AED398313C310000007F9878FFFFF4
07050B47007F8A00FA9100FF8D00FD3700654C1484738B633A4731232A1E76968A6565CA09070E\
6F00C98F00FF9100FF7F00E631005A0000002327250E0017040007000000000000
00000004000731005A9F2AFF31402C303E2C273123152027B5FDFF1E1B3031005A8600F19100FF\
9100FF9100FF8A00FA31005A532276A9D7990000006D8B636A6AD31500298300EC
9100FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF8800F62C00515A2581BEECAE313C2CECFFFF\
1505254C008A8D00FD9100FF9100FF9100FF9100FF9100FF9100FF6600B9000002
FFFFFF2A33253A4735E1FFEF0000007400D19100FF9100FF8F00FF9100FF7B00DF090010BE98D7\
6D86633A4735FFFFFF0000026600C38F00FF8F00FF8600FD4C009223202A2C1C3C
5600A67B00EC7400DE51009B5537786C00CC7800E551009B0000005A533C2E2E310B00156F00D3\
8F00FF8A00FF3C007331005ECCFFC30000000C100C121214A69268FFFFF19B9BAB
0200056100B98300F85C00AE020004FFFFC7141912000000F3FFE81900304C00928A00FF8800FF\
51009B000000AEAE73FFFFD8FFFFFF631BAD150029020004000002000002040007
27004AA427DC867A4E000000141912FFFFFF0000026600C38F00FF9100FF9100FF9100FF8F00FF\
6600C3000000FFC38F6F7453000000040705FFFFFF0E001C5600A68F00FF9100FF
9100FF9100FF9100FF9100FF9100FF8A00FF4C0092000000F1B2989D9D68101912223123FAFFFF\
15002915002A07020B7D454CECFFD13C5E4578B786B2CAE50200057400DE9100FF
9100FF9100FF9100FF9100FF9100FF8600F151009410041B0000000000000E001B17002C000000\
405A37A4E58BB5FB99B2FFBE41535804000502000407000E07000E020004020002
372A25A9B79B92F4B061A476D5FFFF120E1C4E007D6F00A961009F56008353007F4A007F4A0074\
47006F47006F47006F
7B00EA7B00EA7B00EA7B00EA7D00ED8300F45C00AE000000AED3985F789100000241126F7B00E6\
8100F36800C5000000FFFFFF000000AED39894B481090C0C0C10194E6F8BFFFFFF
4545610E051B7800D88F00FF8F00FF47007F31005A96B783313E2C2931235A73686D6DDC040407\
6C00C28F00FF9100FF8D00FD56009D000000819B81E1FFADFFFFFFFFFFFFFFFFFF
FFFFFFFDFDFFC9C9D331372C1419126D8B63192015394865B5FBFF0000025100948D00FD9100FF\
8F00FF9100FF8F00FF47007F270047D7FFC3000000455840988BF10900107B00DF
9100FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF8D00FD3C006D370E5EE1FFCE222A20D7FFCE\
31194F31005A8300EC8F00FF8F00FF8F00FF8F00FF8F00FF8A00FA6600B9000000
FFFFFF3A47352A3325F3FFFF0000026C00C29100FF9100FF8F00FF9100FF8300EC1B00308F5CC2\
9BBE8D222A20FFFFFF1500274700868300F88600FD6100B910001E070709020002
2C00557B00EC8A00FF8600FD8600FD8A00FF8D00FF7400DE12002330231E68866300000051009B\
8800FF7F00F320003C7F51AD74986F0C100C090B091C172081538AFDE3A0FFFFFD
15091E27004C7400DE6100B9000000FFE6AD202A20000000C0F8B73C106837006A8800FF8F00FF\
7B00EC230045251720FBFBA7FFFFDCECFFD1F6F6FFFDFDFFFFFFFFFFFFFFFAFAFF
D8D8EF5E5E4A0000000C100C2E3C2CD8BEFF0200057800E59100FF9100FF9100FF9100FF8F00FF\
7400DE090012DE9F749B8B61000000000000B0FFC3300C533C00738A00FF9100FF
9100FF9100FF9100FF9100FF9100FF8D00FF6100B9020002CA947FCCB581151E15101912D1FFDE\
4C336D0200050000009B555EE5FBCC35513C669D73BBD5EF0000006F00D39100FF
9100FF8F00FF8F00FF8F00FF8F00FF9100FF7D00EA6100B55A299453009B6800B96800B92C004F\
0202026F9B5EB5FB99BBFFADD7FFFFBED8F4B7D0ECB7D0ECBBD5EFBED8F4B5DFB5
B9FFAB91D799437151538B65D8FFFF0000025F008F6C00A25C008A4A00744A007447006F47006F\
43006A43005F43005F
7B00DC7B00EA7B00EA7B00EA7D00ED8300F46F00D10B00145A5A5E8AA2AB0E0C192017336F00D1\
7D00EA4A008B17002ACEF6CE232A1EB9E1A0657A5600000033486C719DECFFFFFF
CAFAD700000056009D8D00FD8F00FF51009422003EBCE5A4293123313E2C51685E7373E6020204\
6600B98F00FF9100FF9100FF7800D810001C665A7AFFFFFF73AE9829413915221C
0B141002040402020240553C98C28B83A6760405046A94DE8FC7FF0200046C00C28F00FF9100FF\
8F00FF9100FF8F00FF5C00A60E001BF8FFF805070531402CC2DCF80000027400D1
9100FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF8F00FF5100941E0037FFFFE80E120C84A978\
5E47990000003C006D56009D5C00A65C00A65C00A65C00A65100942300410B0014
CA83FF2A33250E120CECFFFF0B00145C00A68F00FF9100FF8F00FF9100FF8A00FA31005A4C206D\
BEECAE0E120CBEECAE5A00A4040009370065370065090010190030AEAEC063636D
0B00157400DE8F00FF9100FF9100FF9100FF9100FF8300F837006A05040763735307050B2C0055\
7B00EC6F00D305000CC39BDC2E3C2C2E3A2C2E3A2C040507555A94D1AE7BFFFFD8
A2B49100000251009B5C00AE000002FFB0A23747350000008AB083784EA22300458600FD9100FF\
8A00FF5C00AE000000C0C07FE5E5B4536C4F101B17040707020404040707020404
0000000B100E40533C20291E5E7A5A8F5CC215002A8300F89100FF9100FF9100FF9100FF9100FF\
7B00EC15002A985A53ECAB8102040200000083B083784EA220003C8600FD9100FF
9100FF9100FF9100FF9100FF9100FF8F00FF6F00D30E021299715FECAE94223123090C099DE1A6\
7F53B4000002000002AD8F65C9C984253A2A43684CD8FFF40000026600B99100FF
8F00FF8D00FF8D00FF7F00EF7D00ED7F00EF8300F48300F48100E38100E58300E68100E56800B9\
120520202E1B9FDE86B9FF9DAEF8A08AE6A6589D9139655E315851487F745FA99D
68B7A93C65482239295A986DD1FFFB0000006A00A05A009B4A007447006F47006F43005F43005F\
43005F3E005A3E0051
7A00D87B00DC7B00EA7B00EA7B00EA8100F17B00E6230043292330B5D199272341100C196100B5\
6F00D120003C784EA25C714F657A56B9E1A03A47310C1019719DECB0F6FFD8ADF4
FFFFFF1E1E2A2304357300D87F00ED53009B150027E1FFC3141912293123485C537676ED020204\
6600B98F00FF9100FF9100FF8800F63C006D10041CFFFFFF61A4A4192E30070E0E
000000020202294A4A98C298C7FFB578996D00000096D1FF5A6FA71500297B00DF9100FF9100FF\
8F00FF9100FF9100FF6600B9020004FDFDFF151B122A3525CAE6FF0000006C00C2
8F00FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF8F00FF5C00A60B0014FFFFFF0507050E120C\
786CCC6D00C722003E1200200E001B0E001B0E001B0E001B1200202A004E7F00E5
8A58B9000000000000D7FFCC22093A4C008A8D00FD9100FF8F00FF9100FF8F00FF47007F22003E\
ECFFD7040402222A208F41EC6A00C037006331005A6500B7D8ADF4FFFFFFF1FFD7
0000006600C38F00FF9100FF9100FF9100FF9100FF8D00FF5C00AE020002514A372E2E3110001E\
6100B95C00AE000000FFFFFF0507055A765668866309101022333A915C99FFEAA6
FFFFFD1E0C2A1B0031470086040009D0737F6D815E000000688663945FCA15002A8300F89100FF\
8F00FF7B00EC230045331E2CFFFFA948533C000000070C0E020204000000000002
0912143C4743688663171E15ABDEA441107131005E8A00FF9100FF9100FF9100FF9100FF9100FF\
8300F82C005547272AFFC39B0C100C020202486148B992D00900127B00EC9100FF
9100FF9100FF9100FF9100FF9100FF9100FF7B00EC1B00315E453AFFCC9630413104070565916A\
BED8F41200220C070EB4946ADEC58B223123293A2AE6FFFF1200205100948F00FF
8100FF7D00ED7D00ED7D00ED7B00EA7B00EA7B00DC7D00DF7F00E17D00DF7D00DF7F00E18100D5\
4A007A000000618853B9FF9D96D17F4A7B5A101B1B020405070E1020393C4C888D
5C6876405A3A2741307ACC94B9FFDE0C001263009D53008347006F43005F43005F3E005A3E005A\
3E00513A004E3A004E
7A00D87A00D87A00D87B00EA7B00EA7D00ED8100F3400078070209D1FFB54741710000004C078A\
5800A605000BCCA2E60B0E0994B481AED3981215122E4161ADF1FF9BA9FF6300BB
DCFFDC667A8100000253009B7B00E65800A607000EEFFFEF040504222A1E4856487F7FFD000000\
6600B98F00FF9100FF9100FF9100FF6F00C905000B88888F9FBBD8BBFFFFBBFFFF
A0ECFF537AA20000006D8B63B0E1A04C614510171CB7FFFF1E1B3031005A8800F69100FF8F00FF\
8F00FF9100FF9100FF6F00C9000000FFFFFF222C20222C20C7E3FF0400076100B0
8F00FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF6C00C2000002FDFDFF000000192015\
040402789178CEF6CEE8FFE8EFFFEFEFFFEFEFFFEFEFFFEFE8FFE8C3EAC3637663
000000374731191E158FB78345147631005A8A00FA9100FF8F00FF9100FF8F00FF5C00A60B0014\
FFFFFF0000000C100C050705637A5AB4DEA4B4DEA47A986F313C2C586C4FFFFFFF
0B00145C00A68F00FF9100FF9100FF9100FF9100FF8F00FF7400DE15002A291E194E5E4E000000\
3C007337006A0E001CFFFFFF02020291BC8B91B9A948837A0000004E336FD8B27F
FFFFD8ADBE9900000020003C05000CAE5F6A9BA278000000536C4FB273F10900127B00EC9100FF\
9100FF8D00FF5600A6000000D7D78FB9B9922231251C2C315381918AD3EF7ABCD5
273C45020200354531020202F3FFE81900304C00928D00FF9100FF9100FF9100FF9100FF9100FF\
8A00FF470086100510FFD8AD252C20020202253325E3FFF10000006F00D38F00FF
9100FF9100FF9100FF9100FF9100FF9100FF8300F82C00552E191BFFD89F555E4C04070539513C\
E6FFFF2A005315051CC28F7AD8C084293A2A101912B9FFC34C336D2700487D00EA
8100F17B00EA7B00EA7B00EA7A00D87A00D87A00D87A00D87A00D87800C77800C77A00CC8100D7\
7600C327043C1417149FDE86B0F6967AAE714A815E5A8B9D92E1FDC0FFFFC2DCFA
83665A7AAE714A7B5AA7FFC9668A781B002C65009F4F007F43005F3E005A3E005A3A00553A004E\
370048300048300040
7A00D87A00D87A00D87A00D87B00EA7D00ED8300F45C00AE000000A0C28B5A71880202042C0C4C\
370066040007FAFAFF020202AED39888A6760000005A7FBEB9FFFF472E65140025
5847659FBC980B0B102C04436C00CC5800A6000002F6F6FF000000222A1E3A45318888FF000000\
5800B08100FF8100FF8F00FF9100FF8800F641007600000074946D4017713A006A
4000748A00F85A486619201583A676222A1E405171B2F8FF0000025100948D00FD9100FF8F00FF\
8F00FF8F00FF9100FF7800D8000002FFFFFF31402C0E120CECFFFF0B091456009D
8F00FF9100FF8F00FF8F00FF8F00FF8F00FF9100FF9100FF7400D1000000FFFFFF0404025F7A56\
556D4E10191700000000000000000000000000000000000000000002020214201E
4E7D76A6D398556D4E4558408F6F982000398600F19100FF8F00FF9100FF9100FF6C00C2000002\
FDFDFF0000006D8663638B7327403A070C0B0202020405050405040B0C09FFFFEF
1E003747007F8D00FD9100FF9100FF9100FF9100FF9100FF8600FD37006A0504076373530C0710\
1200231200233A0071B5ECAE171E15B4E8ABCAFFEA99FFFF25413E22333AA25CB9
FFF3ABFFFFFF4A3A53000000000002782778CAD39B0404022E3C2CFBFBFF0000027400DE9100FF\
9100FF8F00FF7F00F32300452E1B27FFFFADD7F6B5BB94D18800FF5500A26A00CA
A629DC2E2A1B0C100C040402FDFDFF0200046100B98F00FF9100FF9100FF9100FF9100FF9100FF\
8F00FF5C00AE000000FFD1A74F4C35020202090C09EAFFFF05000B5C00AE8F00FF
9100FF9100FF9100FF9100FF9100FF9100FF8800FF41007B0C070BFFD1997B7B610407051C291E\
ECFFF845007D190523B78873ECD08F304131090C0976A97BADC3DE05000B6F00D1
8300F47B00DC7A00D87A00D87A00D87A00D87A00D87800C77800C77600C37600C36600B76A00BB\
7300CA510091020204537447B7FF9BA9EC8FC3FFDC9148E65600834C0771735688
D57FB7A0F6B561A476C3FFEC3A4F4535004C63008D4A006A3E005A3A004E3A004E300048300048\
2C003C2C003C2C003C
7A00D87A00D87A00D87A00D87A00D87B00EA8300F47300D80E001B4E4E538AA2AB0E0C1909070E\
0E001B2C0053C0E6B7293123C5F1AB657A560405078AC2FF96D0FF040207020004
05020B86A081454F5A0200044A008B4E0092000000FFFFFF0000002731232A35258B8BFF000000\
5800A68100F37F00EF7D00ED7F00FF8300FF6C00D81C0039000000040405000000
0000023A006AC2C2CC040504556D4E040504638AD08FC7FF0400096C00C28F00FF9100FF8F00FF\
8F00FF8F00FF9100FF7F00E60B0015B273F14F6548050705C9FFE81B142C410076
8D00FD9100FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF7B00DF05000BCCA2E6151B125F7A56\
6D8B630C1415000000040707040707040707040707020404000000000000172C2E
569FA4A9E1A06D8B63192017BE98D70B00157F00E69100FF8F00FF9100FF9100FF7800D8000000\
FFFFFF05070578996D76A6881C3335000000020404040707050C0B29392AFFFFF8
3C27533700658A00FA9100FF8F00FF9100FF9100FF9100FF8D00FD5C00A600000055513A3E3E45\
0000000000009963D168866340533CCAFFC0ADFFFFC3FFFF6DC0B40000005F19A6
EAC28AF4FFBBE6FFE6170529000002682268F4E8A7050705202A20FFFFFF0000006C00CC8F00FF\
9100FF9100FF8D00FF5C00AE0000009B8A5EFFFFD8FFFFF147127A020004090010
9825CA6D815E000000202A20FFFFFF0000006F00D39100FF9100FF9100FF9100FF9100FF9100FF\
8F00FF6F00D3020004FFB9927F7355000000000000DEFFDE1E003A4700868D00FF
9100FF9100FF9100FF9100FF9100FF9100FF8D00FF5600A6000000FFBE8B9B8B61040705090C09\
C2FFD76A1CB719002E918158FFEDA74C5A3A090C09304531FFFFFF07000E4E008B
8300E67D00DF7A00D87800D57800C77800C77800C77600C37600C36500BE6500B26100AD6500B2\
6D00C26D00B22C104E0B0E0B8AC074BBFF9FBBFFADA2E6961B002E09020E3A2331
C376A9A7FFC96AB07FD3FFFD120E1C4700665C007847005C3A004E3A004E3000403000402C003C\
2C003C290030290030
7800D57800D57A00D87A00D87A00D87A00D87F00E17D00EA29004C201925ABC5912A2748000000\
000000A76CE3485840657A56C5F1AB3A4731141C2AA9ECFF6188CC05000B2C0053
0B001437304191AD8B0B0B102304353A006F000000FFFFFF000000303E2C222C208D8DFF000000\
5800A68300F47F00EF7B00EA7D00ED8100F17F00ED4E009D0B0410000000040007
0400093A006ACCF4C30C100C43553C00000096D1FF5A6FA71500297B00DF9100FF9100FF8F00FF\
8F00FF8F00FF9100FF8300EC1B0030945FCA7A9D6F040402AED0AE3A2E5E31005A
8800F69100FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF8600F11500299963D1455840374731\
222A1E10191C91C3FFC0FFFFC7FFFFC9FFFFC9FFFFC7FFFFBBFDFF8AB9FF223141
090E0E638F6D6D8B63050705ABB9FF0000027400D19100FF9100FF8F00FF9100FF7F00E6090010\
B776F819201743553C222A1E1B233591C3FFC0FFFFC5FFFFADFFFFB7FFA9FFFFFF
301E412C00518600F19100FF8F00FF8F00FF8F00FF9100FF9100FF7800D8120022291E196D836D\
190030220041D8ADF41920175A7656D5FFCAD7FFFFD7FFFFB4FFFF192E2A405171
A043E3FFF4ADFFFFFF785F881900305F1781FFECAE0B0C090B0C09FFFFE605000B5C00AE8F00FF\
9100FF9100FF9100FF7F00F32C00551B1017D7D78FFFFFCC715A81040007090010
EC838F5E7A5A0000004C6148AD6FEA0B00157B00EC9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF7B00EC10001EBC6F66B5A67A00000002020299D0996A438F27004C8600FD
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF6C00CC020004DE9F74D79B74090C09000000\
7FB78D8F5ECA1B0033514731FFF1AB92835C101712101912B0FDB9665EA217002A
7B00DA7F00E17A00CC7800C77600C37600C36600C37300BE6500B26100AD6100AD61009F61009F\
6A00AD7300BB5C00960C0914394E30A4E58BB7FF9BB2FFBE4C477A140723513147
C298CE99FFB973C08AD8FFFF00000058007355006F39005635004839004C35004835004833003A\
2C0033290030290030
6500CA7600D07800D57A00D87A00D87A00D87D00DF8100E54A008B020004B9E1A0534C8412101E\
270447D8ADF4040402A0C28BAED39812151233486CBBFFFF2327402300436100B5
3A006F000002A0C28B454F5A0000021C0035020004FFFFFF000000303E2C151B129191FF000002\
53009B8100F37F00EF7B00EA7B00EA7F00EF8300F46C00CC20003C000000000000
0400077F22DC7A9D6F222A1E273123141919B7FFFF1E1B303700658A00FA9100FF8F00FF8F00FF\
8F00FF8F00FF9100FF8A00FA31005A5A25819BC58D0000008FA97B5A5AB5200039
8600F19100FF8F00FF8F00FF8F00FF8F00FF8F00FF9100FF8A00FA2C00515A25817A9D6F141912\
000000A491C36A00C01200200200040000000000000000020E001B5A00A4BB6AD5
15140E2E3A2A6D8B63000000EDFFFD0000026600B98F00FF9100FF9100FF9100FF8600F1200039\
8F5CC24F65481419120404028B25F34F00910B00140900101E0237715878B5BE88
150C1C2700488300EC9100FF9100FF9100FF9100FF9100FF9100FF8600F13700650504078BA076\
7148997300D0D1D1E800000086AE81E6FFDCBEFFFFE1FFFFD0FFFF6DC0B4040707
7F22DED8BB83FFFFABFDFFFD5817996A09C0FBE5A90B0C09000000F3FFE81900304C00928D00FF\
9100FF9100FF9100FF8D00FF6100B9020005746647FDFDC7E6FFC33C10683A0071
FFCAAD2733250000008AB083784EA22300458600FD9100FF9100FF9100FF9100FF9100FF9100FF\
9100FF8300F827004C5A3137FFBB940507050202026386639963D11200237F00F3
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF7400D10B0015A97858F3AE7F121714000000\
486C4FDFC3FF1B0033231429E3CA8FDEC58B303C2C090C0948684CFFFFFF020002
5800928300D87A00CC6500BE6500B26500B26100AD6100AD6100AD5E009961009F6500A46A00AD\
6C00A26F00A77100AB35005C0000006F9B5EB2F898B7FFA9B9FFD06C517F9F5F8A
E6CAAD7ACC9473C08AD1FFFB00000253006F47005E4100564C006551005F55006351005F4C0058\
3C004733003A290030
4F00985600B06F00C37600D07800D57A00D87B00DC8300E66100AB00000286A0817391AE453E76\
7322C3B0B0B9141912B9E1A088A676000000638AD0B0F4FF0504074500837800E3
6800C50E001B37304191AD8B07050B040007050009FFFFFF040402303E2C0E120C9191FF040409\
4E00928100F17F00EF7B00EA7B00EA7D00ED8100F17D00EA450083040204121515
450568C781FF151B12374731141912405171B2F8FF0000004E009D8100FF9100FF8F00FF8F00FF\
8D00FF8D00FF8F00FF8F00FF47007F2A004ED7FFC3000000637F5A7373E610001C
7F00E69100FF9100FF8F00FF8F00FF9100FF9100FF9100FF8D00FD4100762A004EB4E5A4040504\
040402C2DCFA1200202700486100B06C00C26C00C26100B0370065000002FFC2A4
56533531372783A676090C09D8FFFB15022047007F8800F69100FF9100FF9100FF8A00FA370065\
4C14849BC58D000000222C20C77FFF00000037006551009431005A0000020E0C0B
0000002C00517F00E68F00FF9100FF9100FF9100FF9100FF9100FF8F00FF5C00A6000000989F71\
F8F8FF8600F1485648253023B4E8ABF3FFE6DCFFFFEAFFFFEAFFFFB4FFFF0E1B19
5C76A4AB29E5FFF4ADFFFFEFD0B7F88A00FFF4A784050705000000B5ECAE3C106837006A8A00FF\
9100FF9100FF9100FF9100FF7F00F32C005510090ED0D08AFFFFCECAB2F18800FF
F4A784050705000000D7FFCE2A005341007B8A00FF9100FF8F00FF8F00FF8F00FF9100FF9100FF\
9100FF8800FF3C0073190E1CFFD3A720231B020202354735D8BEFF0200057400DE
9100FF9100FF9100FF9100FF9100FF9100FF8F00FF7800D81B00305E453AFFCC96222C20000000\
223325F3FFFF1B00330400056D5A40E6FFF4687B551019121C291EC2FFCC5F5899
1400227B00CC6F00D16500B26100AD6100AD6100AD5E00A75E00995E00996500A46D00B27300AD\
7100A97300AD6500AD58008B19092C202E1B99D783B9FF9DCAFFD8DFDFEDD883BB
C7AE945A9D7373C08AC3FFEC0700095500714F006A56007347005323002917001B230029450051\
560065470053270030
05000B33006A4F00A26C00BE7300CA7800D57A00D88100E37600D010001E4E4E53B0D1DC7B6FD0\
7155B933402C5A6D4ED1FFB5738B630405078AC2FF8FC7FF0000006100B58100F3
7D00EA40007800000294B4813C454E02000210001EE6CAFF0E120C303E2C050705A7D3FD0B0B15\
47057F7F00ED8100F17D00ED7B00EA7B00EA7D00ED8300F46500BC10002030352A
846AD58A8A920405044C6145020202719DEC83B7FF05000B6500C97F00FD8300FF8300FF8300FF\
8100FF7F00FF8100FF8300FF5C00A6120020FFFFEF0507054558409B8FF805020B
7800D89100FF9100FF8F00FF9100FF9100FF9100FF8F00FF8A00FA510094150027F4FFDE000000\
040402DAFFFF05000B56009D8A00FA8F00FF9100FF8D00FD6F00C9040009EAAD92
5F5C3C3A402E98C28B2731238DA98D45373E1410226F0BA78800F68F00FF9100FF8A00FA47007F\
270047D7FFC30000002A3525D1A7ED0000026C00C28800F67B00DF47007F100B12
1015204100767B00DF8800F68300EC8600F18D00FD9100FF9100FF9100FF7400D10900108F6A5A\
FFFFFF353A5E0202027BA076DEFFD3EAFFFFE6FFFFEFFFFFEFFFFFD7FFFF53948B
192A309227FDD7B981FFFFADFFFFFF9929FFB46A610404020202028AB083784EA22300458600FD\
9100FF9100FF9100FF9100FF8D00FF6100B9050009746647FFFFA9FFFFFF8423E5
5F4F39000000040402FFFFFF0B00155600A68A00FF8D00FF8300F86F00D37400DE8600FD8F00FF\
9100FF8A00FF51009B020004FFD8AD353C2C020202121912F4FFFF0000006600C3
9100FF9100FF8F00FF9100FF9100FF8800F66F00DF5C00AE20003C271517FFD89F484C35000000\
0B120CF1FFFD1E00370000020B070CBEE399B9DE962C4130101912517455FFFFFF
0B00103C006C7300CA6A00BB6100AD5E00995C00965C00965E008F6100926D00A45F00A6560783\
5A4E37511E9163009B65009F4700660202025A7D4CB2F898B9FF9DD0FFFAB28BC0
8BA46A3C684C7AD59BADFFDC0C000E51005F53005F27002C00000068A0769FFFFF68A076020202\
2705434700550B0017
00000000000023003A43008A5600B06F00C37800D57D00DF7F00DF2C004F201925F6FFF69183F4\
6D8AA6000000A0C28BD1FFB53A4731141C2AA9ECFF6188CC0E001B6F00D18300F4
8300F46C00CC0B001447474A7DA076171227230041CA83FF192017303E2C000000C0DAF615152C\
3C12667D00EA8100F17D00ED7B00EA7B00EA7D00ED8300F47800E32C0053292330
C9E5FF2027203747314C6145020404A0DFFF474C7B1C00356800C57300D86C00D86F00D17D00FA\
8300FF8100FF7F00FF8300FF6100C2020004FAFAFF151B123A4A35C3DFFB000000
7400D19100FF8F00FF8F00FF9100FF8A00FA7B00DF7400D17400D14C008A090010F1F1FD000000\
000000DFFFFF0E041956009D8F00FF9100FF9100FF9100FF7800D805000BEAAD92
5653353A3C25B0E1A0556D4E4F6548765E6C00000041333A55408A7B0BBB8800F68600F14C008A\
120020F8FFF8040402151B12FFFFFF0000006F00C98F00FF8F00FF7B00DF510794
55689D5C00A66C00C256009D3700654E206F6F00C98F00FF9100FF9100FF7F00E61B0030714361\
E1FFD70000005A7A56CAFFC0F3FFE6E5FFFFE6FFFFEFFFFFF1FFFFEDFFFFA4FFFF
020202747DCCAB29E3FFF1ABFFFFEAC2C2D7512C31000000090B09536C4F9963D11200237F00F3\
9100FF9100FF9100FF9100FF8F00FF7F00F32C0055150B17ECEC9BE5FBCC5E5540
0000000C100C141912FFFFFF0000006600C38A00FF7F00F341007B1B151C1505233C00737800E5\
8D00FF8A00FF6600C3000000FFC79F685E45020202000000FFFFFF0E001B5600A6
8F00FF8F00FF8F00FF8300FF7800E34E0092452C5E310E531C0035090209FFCE987F7355000000\
000000C9FFDE37105C0B0012040007455A4FE6FFBB839B6A222C201C29209DE1A6
ADC3DE00000055008A7300BB6500A45E008F5600835600835C008A66009B63009D47056A000000\
04050415122358007F63008F63008F2905480E140C96D07FBBFF9FB2FFC09BB776
434F332745318AEFAE7399880C000E40004A23002907090BBEFFFFCCFFFF9BFFFFC0FFFFC7FFEF\
151E140E1414000000
0000000000000000001000153C00715300A96F00C37A00D88300E64A00830B040EFFFFFF8F84E5\
2A3135313E2CD1FFB5B9E1A012151233486CBBFFFF292C4829004C7D00EA8100F1
8100F17F00ED3A006F05020798C28B536171431978C781FF151B12273123000000B7D0EC1E1E3C\
310E537B00E68100F17D00ED7B00EA7B00EA7B00EA7D00ED8300F44E0092040007
FFFFFF050705556D4E3747311B2022B9FFFF15102531005E6800C5682A92352A3C1C0035400078\
6F00D18300FF8300FF8300FF6500C9000000FFFFFF222C20222C20CAE6FF000000
5800B08100FF8300FF8300FF8600F15100941B00301C172031005A31005A040007FFFFFF050705\
000000CEF6CE270B4147007F8D00FD9100FF9100FF9100FF7800D805000BF1B298
4C48303C3A25B0E1A078996D151B128F8FC90907071512143C30375C48536C35AB6F00BC410076\
090010F6F6FF0B0E09050705FFFFE605000B5C00A68D00FD9100FF8D00FD7B00DF
6F00C96100B03700650000021010120000003C006D8600F19100FF9100FF7B00DF1200226C1B91\
99A073121912B4F1ABF3FFE6FFFFF6E5FFFFE6FFFFF1FFFFF4FFFFEFFFFFD7FFFF
3A6861487A889100FFBE7ACCFFFFA9BEE1A619120C090907171E152E3C2CCCA2E60400097400DE\
8F00FF9100FF9100FF9100FF8F00FF8300F84C0092000000A49163E6E698313A2A
090B07171E15374735C79FE105000C7400DE8600FD5C00AE05020745483363636D05020B1B0031\
6600C38300F86600C3040009FDAD8A8A7B5C000000000000B7F8B7370E5E3C006D
7D00FA8300FF8300FF7B00E645008302000400000009050C17002A000000FAB283B48361000000\
0000008AD099664E78090012200039000000A2C283D0F8A945684A151C15304535
D1FFE891919907000C5F008F6F00A96500984F008A4C007853008361009951007F040204746647\
99ECAD05040743005863007F53007D4A006F09090C4C6A41B5FB99A9EC8F6AA278
274130223A2AA7FFD35F81710500051B001B00000091F1AEC0F6DA73B0834A735363925CB0FFCC\
99E6A6000000000000
0000000000000000000000000200053700684F00A26F00C38100E36800B9000000FFFFFD617A94\
00000088A676EAFFCA94B4810000005A7FBEB0F4FF05050C4500838100F37F00EF
7D00ED8300F46800C50700105E6853C7FDBB6A4FAD8F84E5050705374731000000B2CCE6222245\
310E537B00E68100F17D00ED7B00EA7B00EA7B00EA7D00ED8300F46F00D1040007
D5BBFD2A35252731230C100C3A517AADEFFF0000004E00926800C52C00530405043E3E4105020B\
1400255800A67600DC7800E353009B000000FFFFFF1920170E120CCCF3FF140E20
3100656C00D87600EC6500C92C005A0000005A5A5E63736D0000000900100E001BFFFFFF050705\
090C0965916F4835741B00307B00DF8F00FF9100FF9100FF7400D1000000FFC3A7
2E2C1C534F33C7FFB5A6D398000000B2CCE63A2E350000001B141731272C3C303741333A150525\
19002AD5BBFD050705000000C3EAC33700632700486F00C98300EC8600F17B00DF
6C00C231005A000002886C99FFFFFF5A5A631500297F00E68F00FF8D00FD56009D000000A7B5FF\
30302040553CD5FFCAFAFFEDFFFFF6BEFFFFE6FFFFF1FFFFF4FFFFF4FFFFEAFFFF
88F1E10204049B76FF830CECFFF1ABB9CCA60E100B222319354531050705FFFFFF0400074C0092\
7F00F38A00FF8800FF8300F87B00EC6600C341007B05000C37233AFFFFAD4C5343
040504090B0974986F8856B71B00317B00EC8300F841007B090707EAEAB9FFFFFFE6FFFF2A0B4A\
09001241007B41007B000002F18F83A9735C0202020C100C5E7A5A9963D10E001B
6500BC7800E36F00D14000780000008D664C8BBC8B000002140023000000DE9F74C08A65020402\
020202416148CCA2E6000000270045120020191C17BEFFCC9BE8A7314C35151C15
415E48ECFFFF45376F07000E51007F65009F5C00925500865A008363008F3A0E4F371E22FBDF99\
F4FFFF454F5A27003A55006F47005E5500713907610204028AC074B5FD997AAE71
30513C30513CBBFFEC4C3C430000000000004F453AC2FFB2BBA06F73AB7A584E436CA073A0DF88\
B0D8B02E4C37000000
00000000000000000000000000000000000029004C4C00996600D07B00DA07000ED7D7E34F5E63\
293123DEFFC0F4FFD3738B6302020484B9FF8FC7FF0000005C00AE8300F47D00ED
7D00ED8100F17D00EA3700660B040EFDFFE69FBBD87A618B00000043553C000000AEC7DF27274E\
2C0C4C7800E38100F17D00ED7B00EA7B00EA7B00EA7B00EA7F00EF7F00ED2C0053
471C65CAFFB700000000000076A6F883B7FF0900106500BC7300D829004C251E2AFFFFFFFFFFFF\
5817990400070B00141C0035070010270048B273F30404020202027FA2945C4596
0000020B00150E001C000000230047C79FE1FFFFFFF4FFFF665EA2140027430088A96CE5000000\
303E2C0E120CAD9FFF0B00142C00516F00C98800F68D00FD6600B9000002FFFFD7
05050476734AE8FFD3D3FFC0303E2C4F5E408A6D7B271E230000000404040B090B040404040404\
6A535F8A6D9B020202222A1E3A4A359929FF0E001B05000B1B0030270048270048
0B00150E001BB776F8F3FFE8FFFFF8737DCA10001C7B00DF8D00FD6F00C90B0015632CA492FFF3\
02020291C28BE6FFDCFFFFF6FAFFEDD0FFFFEAFFFFF1FFFFF4FFFFF4FFFFEFFFFF
C3FFFF1E353173B0C78A00F8B776C3F8DC9B22291E27291E688B630C100CA6B59255009B000002\
31005A47007F4C009241007B370E532510351B14200E0C07040007C7B078C2C281
0B0C09040402D7FFCE3C106837006A8600FD8600FD41007B0E0C0BF8F8A4D7E1A0C0F8B7FFFFFF\
631BAD09001000000019002EEF8492845A48090B072E3A2C0B0C09D5A9F117002A
0E001B2000390B001207000E9453A9FFFFFFFFFFFF0900120B0012000000FF9F92A07355000000\
141E15020402D8FFE52E004C0200040E0019000000537A4CCAFFD88ACC92253727
101712416148ECFFFF637F8800000029003C56007B61008D5F008A3C004E000000D77683E5FBCC\
D1FFEC78AB6F1C002753006F40005348005551005F151214394E30B5FB9996D17F
53815E53916ACCFFFF4F40480B0907331E2CBEBECAC9FFD7AE966863946A4F8B8173C08AB7FFC3\
C7AD7833563E000000
0000000000000000000000000000000000000000001B00314F00A27300E61C0031AD8AC33C4A35\
738B63FDFFDCEAFFCA47553C141C2AA9ECFF6188CC10001C6F00D18300F47D00ED
7B00EA7D00ED8300F46100B5020004ADBE99F6FFE62C333C0C100C5F7A560202029BB2C937376D\
230B3C7600DC8300F47D00ED7B00EA7B00EA7B00EA7B00EA7D00ED8300F46100B5
000000FFFFF16D8B6341534CADF1FF41477320003C7600DC7F00ED4500830C0215FFFFFFCAFFB7\
D7FFC3E3C7FFB273F17A31AD7820D19D29FF454C3C19201543553C0000007A71C3
A947EFA76CE3AD6FEAC981FFE5FBCC6D8B6331402C687B83B2DCFF7F00FF9829FF3C4135303E2C\
98C28B2731235A71559D29FF27004E02000440007F6800D147007F120020FFFFBB
020200AEBC88FDFFE6FDFFE698C28B0000007171789173835F4C561E171B0B0909271E23765E6C\
C0A98F0B0E0B4C614583A67614191265656FC781FFB776F88F5CC25F27884F0091
8322E3DFC5FF688B63141B12A0D7997A83D51200227800D87800D827004831005AC0FFFF000000\
71AD94E6FFDCFFFFF6FFFFF6DEFFFFE1FFFFEDFFFFF1FFFFF4FFFFF4FFFFF1FFFF
E1FFFF76D0C2101C1B9B47FF7B0BDEF4CA915A5E4317201586B4815A7A56050705B48FCC8322E3\
300C480B040E0000000000000000000000000000000B0710040009270E30FFFFAD
7B8A6F536C4FFFFFFF0B001551009B8A00FF8A00FF5C00AE000000D0D08AC3C3993A45312E3C2C\
ABDEA4DFC5FFC27BFFAD48F49B58B20E0C0935453173946D171E154F5F4FAB47F3
8337BB7330A4AD6FEAC9E5FFCEF6CEAEECAEFFFFFF1E003A02000233005FFA8A9827231B141E15\
4E76562030231C2A20DAF6E65A2C9109000C0000000400055A8453CAFFD87AB583
19302C07120E304535B0FFC3E3FFFF403366000002040004040005000002884C99F3FFD8669D73\
AEFFC399ECAD1C002251005F3900433900435300533A003A00000083B76FB5FD99
9BB97D92FDB9D8FFFF2C054C2E271B8A7866B9E3B9BBFFD17A6C5C39533C355A417A7A83C5FDDF\
91567D000200000000
0000000000000000000000000000000000000000000000001E00277100D72000408F3ACA232C20\
A0C28BFFFFE6C5F1AB12151233486CBBFFFF292C482300437B00E68100F17D00ED
7B00EA7D00ED8100F17B00E629004C3C2753FFFFFF191C2227312383A6761419128B9D925555AB\
0C05156800C58300F47F00EF7D00ED7B00EA7B00EA7B00EA7D00ED8100F17D00EA
29004C3A1953FFFFFFC5F3FFB7FFFF0C09174000788100F18300F46C00CC040007CAB2F184A978\
04040205070531402C637F5A637F5A151B1214191283A676A6D3984C6145000000
2A35254558403A4A35151B12000000040504000000191E2E6D815E7A9D6F1920171419128DB481\
D3FFC098C28B14191231402CFFFFF8CAFDC20000003A0076230048452C5EBEF3AE
273123DFFFCAFFFFFDFFFFFDF1FFDC83A676090C0931402CFFFFE8A2B0965F4C56FDFFE1F4FFDE\
0E120C303E2CB0E1A0D3FFC083A67614191205070531402C58714F7A9D6F7FA978
4C65480507050C100C0405048AB7835827912000396C00C2410076090010FFFFFF0E120C73996D\
E6FFDCFFFFFDFFFFFFFFFFFDE5FFFFE1FFFFEFFFFFF4FFFFF4FFFFF4FFFFF4FFFF
EFFFFFBBFFFF0E1B1983C9E18B00FBC77FD5A6926802020286B4819FD398486145000000688B63\
FFFFF8F8FFEA816F4E413927332C1E201C14000000170920270048000002918158
FDFDA7F8FFECD7D7ED0000026F00D38F00FF9100FF7F00E6200039453C33FAFAA6353727000000\
0000000B0C092E3C2C2E3C2C040402171E1573946D9FCC9873946D090B09192017
68866374986F486148202A201219126F986FE6FFF43C006A812A81FFFFFF486148040504567656\
81AE8163946D141E15152017D1FFE8D0FFE6090C0B05000B000000395333D7FFFF
7DF4E122413C050C0B09120C39554CA6EFB7FAFFFFC3DFFBBBD5EFC9E3FFC0FFD87FBE8D6DA491\
CCFFFF334F3A3700404E004E3300333000304500455500551C152E2C3E25B0F496
B5FD99D8FFF498FFC20B000B29231E8FB08FB2FFA673AB7A354139121C1C1E35319BC0CACCB07A\
1E2E201B2733000000
0000000000000000000000000000000000000000000000000000000C0019000000CA83D8000000\
C5F1ABFFFFEDAED3980000005A7FBEB2F8FF05050C45007A8100E37F00EF7B00EA
7B00EA7B00EA7D00ED8300F45800A6000002FFFFFF252E2943553CB0E1A04C61453948418A8AFF\
0404073700667300D88300F47F00EF7D00ED7B00EA7B00EA7B00EA7D00ED8300F4
6800C505000B617DADBEFFFF6891DA0200026100B58300F48300F47F00ED3700662A0B4AFFFFEF\
0407070E19190E1919020202000000070C0C1C33333E7171457D7D30585C172C2E
070C0C040505040505070C0C070C0C0000000E120C33405F437171192E30000000172C2E3E7174\
4F9194457D83234041000000315C5FDFFFFF231B401700301400293A056A7ADEDE
294A4CA9FFFFCAFFFFCAFFFFC3FFFFA9FFFF78DCE3234041122223BBFFFF8B6F7D9FCAD85FAEB5\
00000037656866BBC06FC9D05EADB23E71742340411220220E191B070C0C040505
070C0C122022172C2E000000C5FFFF2A0B4A27004847007F00000098A4FF48654C30585CB0FFFF\
D5FFFFD5FFFFD5FFFFD0FFFFC7FFFFEAFFFFEFFFFFF4FFFFF4FFFFF4FFFFF4FFFF
F1FFFFE1FFFF6DC0B4101C1B7B23D19F2AF4C3AE7A070C0C294A4F5EADB94F919B294A4F000002\
315C63C7ECF8C7AD788874515F533A332C1E0000002C0B4356009D31005A020004
AEAE73B2FFCA251E2A2C00518600F19100FF9100FF8F00FF4E009D000000DEDE92B9B992101912\
000000000002000000000000000002091214192C30254045192C30091214000000
0204050204050000020000021C3535A0FFCCB7D0EC4F0778E1FFADA4FFFF101B1E040507192C30\
2540451E33390912140407098BEDFFD7FFFF0000003900514700661900230B1010
7FF3F3ADFFFF488B8112201C0407070E1B1B315C5C68B7A992FFEFB5FFFFC5FFFFD1FFFFD8FFFF\
5AAEA60B00125500554000402C002C2900293300334F004F410041020202537447
B9DC9671AE8100000009000900000078AB6FC0FFCC43816A0E171009120C437151C7FFB7416548\
19232E000000000000
000000000000000000000000000000000000000000000000000000000000273C47929273293123\
F4FFD3FFFFE688A6760000007DB0FF96D0FF0000006100AB8300E67D00DF7B00EA
7B00EA7B00EA7B00EA8300F47600DC1000209173A484A978303E2CBCF1AB8DB4810000007A71C3\
735CB907000E1C00356100B58100F18100F17D00ED7B00EA7B00EA7B00EA7F00EF
8100F153009B05000B0B12150002022C04517600DC8300F47F00EF8300F46C00CC05000BB099D1\
9BF8FF2541471E3739315C5C407474407474396868315C5C315C5C315C5C396868
40747440747840747848838858A2A783ECD89BFBB4B5FDFFA698FFB2FFFF68BEC3407478315C5F\
2A4C4F2A4C4F39686C4F91949BFFFFE3FFFF1E2E4017003014002953206683C792
1C33358AF8FFB0FFFFB9FFFFB0FFFFA9FFFF8AF8FF457D83040707B9FFFF846A788888BEC0FFFF\
39686C070E0E000000000000000000020404040707040707070E0E0B14140E1B1C
1222231E373939686C9BFFFFDCFFFF0000022700480900104F17888BECFF10151099FFFFD5FFFF\
DFFFFFDFFFFFDFFFFFDCFFFFEDFFFFEFFFFFF1FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFEFFFFFC3FFFF25413E354145BC9B6FB5966C407174020405020204040507000002040709\
48838DD8B27FCEB27BA68F637868483A31230000004100767800D86F00C9230041
000000090E0E0B00156100C28100FF8100F17F00EF8300F47800E329004C1C1915F8FFDCE3FFFF\
B5FFFFAEFFFFAEFFFFAEFFFFAEFFFFA6FFFF9BFFFF9BFFFF9BFFFFA6FFFFAEFFFF
B5FFFFB5FFFFAEFFFFB2FFFFB2FFFF8AFFFD14072317121988E1F1F4FFFFCCFFFFA6FFFF92FBFF\
8BEDFF92FBFFA6FFFFD8FFFFE6FFFF12101E29003C63008F63008D6300813A0058
05000B223033BEFFFFDFFFFFB2FFFF9BFFFFAEFFFFD1FFFFEDFFFFEDFFFF7A98B7272C45020202\
19042A4F004F4C004C3300332500252500252C002C3C003C5600563C156C020202
000202090B103A003A4C004C141B1B070C09B0FFF48FFFF45AA49871CCCCF1FFFF45737B0E171B\
000000000000000000
00000000000000000000000000000000000000000088AEBB020202FFFFFDB0D79900000094B481\
FDFFDCF4FFD35A6D4E05090C9BD8FF6F9BE80B02146F00C78300E67B00DC7A00D8
7A00D87B00EA7B00EA7D00ED8300F44E0092090012FFFFFF00000083A676B0E1A04C6145040402\
8BAB94FFFFFF564C680700106100B58300F47F00EF7B00EA7B00EA7B00EA7D00ED
8100F17F00ED5C00AE37105E4720766800C58300F47F00EF7D00ED7F00EF8100F34E0092000000\
81BCF8CCFFFF9F78FF8F6CFA7F5FDE7A5CD57A5CD57F5FDE7F5FDE7F5FDE7A5CD5
7A5CD57A5CD57A5CD57356CA664EB45A439F37315C3045561E02374131735F48A97A5CD57F5FDE\
7F5FDE7F5FDE7F5FDE7356CA5A439F1510250B04103A00761700305A3366C9D083
0C0E0C7B9476A9C9A0B4D7ABB4D7ABA9C9A086A0812E372C1B2014A6D0FA5A48513A2E357899A6\
A9D5FFABD7FFA7D3FDA7D3FDA7D3FDA7D3FDA7D3FDA7D3FDA7D3FDA9D5FFABD7FF
ADBBFFA77DFF926FFF5A439F05020B10001C0B001522003EC0FFFF00020099FFFFE6FFFFF1FFFF\
F4FFFFF4FFFFF1FFFFF1FFFFF1FFFFF1FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFF1FFFFE6FFFF91FFF1070C0C48402CA68A61AE91668B73517863476F5C41786347927A56\
CCA978F4CA91E6BE88CAA776997F5A3A302207040756009D8A00FA8D00FD7400D1
47057F411E6D5C00AE7F00ED8100F17D00ED7B00EA7D00ED8300F46C00C02C0B430000000B0014\
14002310001C10001B10001B10001B0B00170B00170B00150B00150B0015090010
0900100B00140E0017150B2305090B05020927003A2C004309021000000009021010051E10051E\
10051E10051E0B04150400050502093C005665008355006F4A005F45006851007A
53006F3700480C0012000000090910171B2A1B1E3010141E0402050200021700173700374C004C\
560056470047330033290029250025250025250025300030470047560056745C68
58209B5500554C004C2A002A00000019222200000071D3D3B0FFFF81DAF1020202000000000000\
000000000000000000
0000000000000000000000000000000000000000004E7A4E27392CFFFFFF202C231E2A2376A688\
98D3AE76A688141E1B202C41B5FDFF2E31512900487B00DA7F00E17B00DC7A00D8
7A00D87A00D87B00DC7B00EA8100F17800E317002C714899C3FFE100000045614F567A651E2A23\
0000005A7F68FFFFFF3A2C6817002C7300D88300F47D00ED7B00EA7B00EA7B00EA
7D00ED8100F18100F37D00EA7B00E68300F48100F17D00ED7B00EA7D00ED8100F17F00ED53009B\
1B072C0400070B001414002520003C20003C20003C20003C20003C20003C20003C
20003C20003C20003C20003C23004329004C29004C31194F31005E31005E29004C23004320003C\
20003C20003C20003C23004329004C4000786100B56500BC2300435A3366DCD381
020200534F3376734A837D51837D51837D516865411B1B103A3727AB81C93A2E35090707050405\
120E101E171B221B1E221B1E221B1E221B1E221B1E221B1E221B1E1E171B191417
0E0B0C040205000000090010200039150029090010ADBBFF07100E88FBE1EAFFFFF4FFFFF4FFFF\
F4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFF4FFFFEFFFFFD7FFFF5CA499000000836C4CDCB783EAC28AEDC58DEDC58DEDC58DF1C78F\
F4CA91EDC38BE1BB84CAA776A4885F2A23191407196500BC8300F48300F48300F4
7F00ED7D00EA8300F48100E37B00DC7A00D87A00D87A00D87D00DF8300D87800C96800AE5C0099\
5500A05800A758009B5C00AE5C00A25C00A25F00A95F009B5F009B5F009B6300A0
6300A06300945C008B5800845C008B5F00A465009F65009F63009B5A008F56007B56007B56007B\
56007B58007F5A007651007B56008351007B45006835004833004333004335003E
41004C4E005A5500555500554E004E4700474500454A004A4F004F5600565300534C004C450045\
3900393000302900292500252500252500252500252900293300334100414C004C
4700472A0033040007000000000000000000000000000000000000000000000000000000000000\
000000000000000000
00000000000000000000000000000000000000000000000005090974ECC7C0FFFF31585C070E0E\
0204040B121431585CB2FFFFADFFFF0000004800928300E67D00DF7A00D87A00D8
7A00D87A00D87A00D87A00D87D00DF8300F46500BC0400077356CAB9FFFF2A4A4C070E0E040707\
122022487F83C0FFFFA9FFFF0000005C00AE8300F47F00EF7B00EA7B00EA7B00EA
7B00EA7D00ED7F00EF8100F18100F17F00EF7D00ED7B00EA7B00EA7B00EA7D00ED8300F48100F3\
7600DC7300D87600DC7B00E67D00EA7D00EA7D00EA7D00EA7D00EA7D00EA7D00EA
7D00EA7D00EA7D00EA7D00EA7D00EA7F00ED7D00EA7D00EA7F00ED7F00ED7F00ED7D00EA7D00EA\
7D00EA7D00EA7D00EA7D00EA7F00ED8100F38100F37800E329004C5A3366EAE18A
0000003C3A25534F335E5A3A5E5A3A5E5A3A534F331B1B1017170EE6E6985A4851040404000000\
040404050505050505050505050505050505050505050505050505050505090707
0B090B151214231C20231B3A2002300000028F41EC3C71655CAB99E6FFFFF4FFFFF1FFFFF1FFFF\
F1FFFFF1FFFFF1FFFFF3FFFFF3FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFF4FFFFF4FFFFEAFFFFADFFFF192C2C393122E5A98FFFC9ABFFCAADFFCAADFFCAADFFCAAD\
FFC7A9EAC28AE6BE88D7B27F997F5A19150E270B3C7300D88300F47D00ED7D00DF
7F00E17F00E17D00DF7A00D87800D57800C77800C77600C36A00C76C00CC6F00D17100C96F00D5\
6F00C36D00C26D00C26D00B26C00B06C00B06C00B06A00AD6A009F66009B66009B
5A009B5A009B5A008F5C009F5C009258008B5300834F007353007655007B55007B55006F53006C\
47006C47006C45005C4100563C004F3500483000372C003329003025002C290029
2C002C3000373500353C003C4100414500454500454500454000403900393300333000302C002C\
290029250025250025250025250025250025250025230023250025250025230023
100015020004000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
0000000000000000000000000000000000000000000000000000000000000912126FB4CE8FFFFF\
98FFFF92FFFF7FF4E8417D780000003C12666D00DC6F00DF7A00D87800D57800D5
7A00D87A00D87A00D87A00D87B00DC7D00DF8100E56100B50700102E2351ADEAFFC9FFFFCAFFFF\
CAFFFFC7FFFFA7E3FF374A6D0C10176100B58300F47F00EF7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7D00ED7D00ED7D00ED7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED7F00EF\
8100F18100F18100F18100F17F00EF7F00EF7F00EF7F00EF7F00EF7F00EF7F00EF
7F00EF7F00EF7F00EF7F00EF7F00EF8100F18100F18100F18100F17F00EF7F00EF7F00EF7F00EF\
7F00EF7F00EF7F00EF7F00EF7F00EF7F00EF8300F47B00E62300436A3C78DCD381
00000035311E534F33534F33534F335E5A3A534F333C3C2500000091915AD7B981221B1E000000\
120E101B141720191C20191C20191C20191C20191C20191C20191C20191C20191C
231C20272023272023120E1000000056418F71BED325453ED7FFFFF4FFFFF1FFFFEDFFFFEDFFFF\
EDFFFFEDFFFFF1FFFFF1FFFFF1FFFFF3FFFFF3FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFF4FFFFF4FFFFEFFFFFD7FFFF5CA499000000926C5CFDBBA0FFCAADFFCAADFFCAADFFCAAD\
FFCAADFFC7A9FFC3A7FDBB9FA278660402023A00687D00DC7F00E17B00DC7A00D8
7A00D87A00CC7800C77600C36600C36500BE6500BE6500BE6500B26500B26500B26100AD6100AD\
61009F61009F5E00995E00995C00965C008A5C008A4C00835600834C00834A0074
4A007447006F4A00744A007447006543005F43005F3E00513E005135005135005133004E330043\
33004E3000403000403000372C003C290030250025250025250025250025250025
2500252500252500252900292900292C002C2C002C290029290029250025250025250025250025\
250025250025250025250025250025230023230023200020190019170017070007
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000003300437300D86A00D56100C36500CA6600D07800D5
7800D57800D57A00D87A00D87A00D87B00DC7F00E18300E66C00CC3700660E001B000002000000\
0000000000000400071C00354E00927B00E68300F47D00ED7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7D00ED7D00ED7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7F00EF7D00EA1C0035AD7F6CB9B06C
00000035311E48472A48472E534F33534F335E5A3A5E5E3A2323170B0B07ECE1A08A68A2090707\
05050520191C231C20231C20231C20231C20231C20231C20231C20231C20231C20
231C2020191C0B090B020202654F5C81D3E6091410C3FFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF1FFFFF1FFFFF3FFFFF3FFFFF4FFFF
F4FFFFF4FFFFF4FFFFF4FFFFEAFFFFADFFFF192C2C37271CE1A076FFCC96FFD59BFFD89FFFCA9F\
FFCA9FFFCA9FFFCA9FFFBEA28F6A5A0000005300928300E67B00DC7800C77600C3
7600C37600C36500BE6500BE6100B96100AD6100AD5E00A75E00A75E00A75E00995C00965C0096\
5C009656008F56008356008353007F4A007F4A007F47006F47006F43006A43006A
43006A3E005A3E005A3A00553A00553A00553A004E3700483000482C00432C00432C003C290037\
29003729003729003025002C25002C25002C250025250025250025250025250025
250025250025250025250025250025250025250025250025250025250025250025250025250025\
2500252500252500252500252300232000202000201900190E0012050005000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000000000005000C3300614C00995600B06100C36500CA
6600D07800D57800D57800D57A00D87A00D87B00DC7D00DF8300E68100E37800D57300D86F00D1\
6F00D16F00D17300D87800E38100F18300F47F00EF7D00ED7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA8100F17800E3100020CA947F9D965C
0000002A291948472A48472A48472E534F335E5A3A6868415353330E0E09404027FAFAA645373E\
000000100C0E20191C231C20231C20231C20231C20231C20231C20231C20231C20
231C20120E100000004C3C43AEDFC7000000A4FFFFF1FFFFF3FFFFF1FFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF1FFFFF1FFFFF3FFFF
F3FFFFF4FFFFF4FFFFF4FFFFF1FFFFDCFFFF6DC0B40000007F5A41FBB484FFCC96FFD098FFD399\
FFD59BFFD79DFFD89FFFCC9673513C0E02125F00B48100D76A00C76500BE6500BE
6500B26100AD6100AD5E00A75E00995E00995C00965C00965C009656008356008F5600834C0083\
53007F4A007F4A007447006F47006F47006F43005F43005F3E005A3E005A3A0055
3A004E3A004E33004E3000483000483000402C003C2C003C290030290030290030290030250025\
25002525002525002C250025250025250025250025250025250025250025250025
250025250025250025250025250025230023230023230023230023230023230023230023230023\
2300232300232300232300232000201C001C1400140C000C000002000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000000000000000000000020003740007853009F5E00BE
6500CA6600D06600D07800D57800D57800D57A00D87A00D87B00DC7D00DF7F00E18100E38100E3\
8100E38100F18100F18100F17F00EF7D00ED7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA8100F17800E3100020D19983918B55
0202002A291948472A48472A48472E534F335E5A3A68684168684148482E00000091915AD5B77F\
1E171B0000001B1417231C20231C20231C20231C20231C20231C20231C20231C20
151214020202221B1E9191CA12201E6DC9B4EAFFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFF
F1FFFFF1FFFFF3FFFFF3FFFFF4FFFFEDFFFFADFFFF20302A271C15C08A65FFC08BFFC992FFCC96\
FFD098FFD399FFD79DFFC7924833252304356D00D06C00BE6500B26100AD6100AD
5E00995E00A75E00995C00965C00965600835600834C00834A007F4A007F4A007F47006F47006F\
47006F47006F43006A43006A43005F3E005A3E005A3A004E33004E300048300048
3000482C003C2C003C2C003C290030290030290030290030250025250025250025250025250025\
250025250025250025250025230023230023230023230023230023230023230023
230023230023230023230023230023230023230023240023240023240023240023240023230023\
2300232300232000201D001C1800170F0012080009010002010000010000010000
010000010000010000010000010000010000010000010000010000010000010000010000010000\
010000010000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000000001000153700614C0091
5C00AD6100C36500CA6600D06600D07800D57800D57800D57A00D87A00D87B00DC7B00DC7B00DC\
7B00DC7B00DC7B00DC7B00DC7B00DC7B00DC7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA8100F17600DC0B0014E3A78F767345
05040235311E48472A48472A48472A534F335E5A3A68654176764A6868412A2A1B0B0B07ECE1A0\
83639909070709070720191C231C20231C20231C20231C20231C20231C2020191C
0907070B09097F657353916A335F55D7FFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFF1FFFFF1FFFFF1FFFFF3FFFFF4FFFFE1FFFF6DC0B40000007A5343F4A784FFC098FFC992\
FFC992FFCC96FFD79DFFBE8B1C151233005C7300CA6600A961009F5C00965C0096
5C00965600835600834C008353007F4A007F4A007447006F47006F43006A43006A43005F43005F\
43005F3E005A3E005A3A004E33004E33004E3700482C003C2C003C2C003C29003E
29003729003029003025002C25002C250025250025250025260025260025260025240023240023\
240023230023230023230023230023240023240023240023240023240023240023
2300232300232300232400232100201C00201D00201D00201D00201D00201D00201D00201D0020\
1D00201B001C1700191600170E000E020002030000030000030000030000020000
020000020000020000020000020000020000020000020000020000020000020000020000020000\
020000020000020000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000000000000000000000000000000000002000533005A
4800885600A66100B96500CA6500CA6600D06600D07800D57800D57800D57A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87B00DC7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8100F17300D805000BF8B79B5F5C39
05040235311E48472A48472A48472A48472A534F335E5A3A76764A76764A5353330E0E09404027\
FAFAA645373E000000100C0E20191C231C20231C20231C20231C20231C20100C0E
0000005A485196C2AB070E0CB4FFFFF4FFFFF3FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF3FFFFF1FFFFB4FFFF20302A2A1C17BE8368FFB48FFFC098\
FFC39BFFC9A0FFD8ADE89F7F0000004E007D7100B961009F56008F560083560083
4A007F4A007F4A007F47007A47006F47006F43006A43006A43006A3E005A3E005A3A00553A0055\
3A00553A00553700483000482C00432C004329003729003729003725003325002C
25002C25002C230029230023230023230023240023240023240023250023250023250023240023\
2400232400232400232100201C00201C00201D00201D00201D00201D00201C0020
1C00201C00201C00201C00201D00201D00201B001C1C011C1C011C1D011C1C011C1C011C1C011C\
1B001C1700191301140E010C040103040101050101050101050101050101050101
040101040101040101040101040101040101040101040101040101040101040101040101040101\
040101040101040101
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000020004
20003743008153009F5E00B26100B96500BE6600D06600D06600D07800D57800D57800D57A00D8\
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8
7A00D87A00D87A00D87A00D87B00DC7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8300F46F00D1000000FFFFD5403C25
0B090535311E48472A48472A48472A48472A534F335E5A3A68684183835176764A48482E000000\
91915AD5B77F1E171B0000001B1417231C20231C20231C20231C20151214000000
31272CA6B29802050488FBE1EDFFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF4FFFFE1FFFF6DC9B40000006D4A3AE1997AFFAE8B\
FFB48FFFC39BFFD8ADB0785F07020B6500985E00A24F008A4A007F47007A47006F
47006F43006A43005F43005F3E005A3E005A3E005A3A004E3A004E33004E3000483000482C0043\
2C003C2C003C2C003C29003025002C25002C25002C2300231C00232300231C0023
1C00231B00201B00201B00201B00201C00201C00201D00201D00201D00201E01201D00201D0020\
1D00201D00201D00201D00201D00201B001C1B001C1C011C1C011C1B001C1B001C
1B001C1A001C1A001C1B001C1B001C1C011C1E011C1F011C20011C20011C20011C1F011C1B0119\
1901171301130D010A0701030801010902020A02020A02020A0202090202090202
090202090202090202080101080101070101070101060101060101070101070101080101080101\
080101070101050101
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000001C00313C00714F00985C00AD6100B96500BE6500CA6600D06600D06600D06600D07800D5\
7800D57A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87B00DC7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8300F46800C5000000FFFFCA252315
15140C35311E3C3A233C3A233C3A233C3A2348472A534F3368684176764A76764A5E5E3A232317\
0B0B07F4E8A783639905040509070720191C231C20231C201B1417040404151014
8B6F7D315851539B8BE1FFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF3FFFFF1FFFFB4FFFF232A2319120E9B7155E1A47A\
F4B284FFCC98FFE3AB6D4A3A23043563009B5500864A007443005F43005F3E005A
3E005A3E005A3A00553A004E33004E3000483000483700483000402C003C290037290037290030\
25002C25002C25002C2300231C00231C00231C00231B00201B00201B00201B0020
1B00201B00201B002019001C1A001C1B001C1B001C1C011C1D011C1E011C1F011C1F011C1E011C\
1D011C1C011C1C011C1C011C1D011C1E011C1F011C1F011C1F011C1E011C1D011C
1C011C1B001C1B001C1B001C1D011C1F011C20011C22021D23021D24021D24021D1F021A190215\
15020F0E02060A02020C02020D02020E03030F03030F03030E03030E03030D0202
0D02020D02020D02020D02020C02020C02020B02020A02020A02020B02020C02020C02020C0202\
0C02020B0202090202
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000E00143000614800885600A65E00B26100B96500BE6500BE6500CA6600D06600D0\
7800D57800D57800D57800D57A00D87A00D87A00D87A00D87A00D87A00D87A00D8
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8\
7A00D87B00DC7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8300F46100B5020004FFFFC71B1B10
14140C29291931311E31311E31311E31311E3A3A234F4F305A5E3A65684173764A65684147482E\
090B05484C30FDFDA745373E000000100C0E20191C20191C090707050405735C68
73A4CC19302ACAFFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF4FFFFE1FFFF6DC9B40000005A392AB07155\
D79B74FFC794FFDFA7301C1935005563008F4C006D3E005A3A00553A004E33004E
3000483000483000482C003C2C003C29003E29003729003729003025002C25002C25002C1C0023\
1C00231C00231B00201B00201B00201B00201B002019001C19001C19001C19001C
19001C19001C19001C19001C1A001C1B001C1D011C1F011C20011C22021D22021D22021D21021D\
20011C1F011C1F011C1F011C1F011C20011C22021D22021D22021D22021D20011C
1F011C1D011C1C011C1D011C1F011C21021D24021D26031D29031D26031A25031A1F031519030E\
0E03040E03030F0303110303130303140404150404140404140404140404140404
1303031404041404041303031203031003030F03030E03030E03030E0303100303120303130303\
1203031003030D0202
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
00000000000000000002000529005343008153009F5E00B26100B96500BE6500BE6500BE6500CA\
6600D06600D06600D07800D57800D57800D57A00D87A00D87A00D87A00D87A00D8
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87B00DC7B00EA7B00EA7B00EA7B00EA7B00EA
7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8300F453009B07000EFFFFC00E0E09
14140C29291931311E31311E31311E31311E31311E47472A4F53335A5E3A6568416568415A5E3A\
313520000000969D61D1B57D1E171B000000120E10100C0E0000004C3C43AEDFC7
000000A4FFFFF1FFFFF3FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF1FFFFF3FFFFF1FFFFB4FFFF2C352C0C0705714835\
CC8361FFCC98FFCA980000004E006F5C00844300563000483000482C003C2C003C
29003729003729003725002C25002C25002C25002C25002C2300292300231C00231B00201B0020\
1B00201B00201B002019001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C19001C19001C1B001C1C011C1F011C21021D24021D26031D28031D27031D26031D\
24021D22021D22021D22021D22021D25021D26031D28031D28031D26031D23021D
21021D1F011C1E011C1E011C20011C24021D28031D2B041D2B041B2B04192404141E040E140406\
1303031303031404041604041904041B05051B05051B05051B05051A0505190404
1904041A05051A05051A0505180404160404140404130303130303140404150404170404190404\
180404150404120303
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000001C00314000784F00985C00AD6100B96100B96500BE6500BE\
6500BE6500BE6600D06600D06600D07800D57800D57800D57800D57A00D87A00D8
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8
7B00DC7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA7B00EA\
7B00EA7B00EA7B00EA7B00EA7B00EA7D00ED8300F4450083231429FFFFB20B0B07
14140C29291929291929291929291929291931311E3A3A234F53305A5E3A6568416568415A5E3A\
4F5333191B100E1009F4E8A7836399090707020202000000271E239191CA12201E
6DC9B4EAFFFFF4FFFFF1FFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF\
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFF
EDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFEDFFFFF4FFFFE1FFFF7DC9BC02020039231B\
C07A5CFFD09BFDA27A07020B53006D45005C3300432C003C29003725003325002C
25002C25002C2300231C00231C00231C00231C00231B00201B00201B00201B00201B002019001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C19001C1A001C1B001C1E011C21021D25021D28031D2B041D2E041D2D041D2B041D\
28031D26031D26031D26031D27031D2A031D2C041D2E041D2E041D2A041D27031D
24021D21021D1F011C1F011C22021D26031D2B041D2C051B2B051729061222050C1B0507190404\
1804041804041A05051C05052006062206062306062206062006061F06061F0606
1F06062006062106062106061E05051B05051904041704041704041804041B05051D05051F0606\
1E05051B0505160404
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000001000153C00714C00915600A65E00B26100B96500BE\
6500BE6500BE6500BE6500BE6600D06600D06600D06600D07800D57800D57800D5
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8
7A00D87A00D87A00D87A00D87A00D87A00D87A00E67A00E67A00E67A00D87A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87B00DC8300E6400071371E3EFFFFA9050504
14140C29291929291929291929291929291931311E3A3A234F53305A5E37656841656841656841\
5A5E3A47482E040502484C30FDFDA74F40480E0B0C1E171B846A7853916A2A4F47
D0FFFFF4FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFF\
F1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFF
F1FFFFF1FFFFF1FFFFF1FFFFF1FFFFF1FFFFEDFFFFEDFFFFF3FFFFF1FFFFBBFFFF313A370C0705\
A4684EFFE3A98856411E043556006539004329003025002C2300231C00231C0023
1C00231B00201B00201B00201B00201B00201B00201B00201B002019001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C19001C1A001C1C011C1F011C24021D29031D2D041D31051E33051E33051E30051D\
2D041D2A041D2A031D2A031D2C041D2E041D32051E34051E33051E30051D2B041D
27031D22021D20011C20011C23021D28031D2A041B2B05172C06102407082306062006061D0505\
1C05051C05051E0505210606240707270707260707240707230606230606220606
2306062407072607072507072306061F06061B05051A05051A05051B05051E0505220606240707\
2306061F06061A0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000C00102C005A48008853009F5E00B26100B9\
6100B96100B96500BE6500BE6500BE6500BE6500BE6600D06600D06600D07800D5
7800D57800D57800D57A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D8\
7A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87A00D87800D57800D5
7A00D87A00D87A00D87A00D87A00D87800D57800D57800D57800D57800D57800D57800D57800D5\
7800D57800D57800D57800D57800D57B00DC8300E63A00613C2245FFFF9B040402
14140C29291929291929291929291929291931311E3A3A2347482A5A5E3765684165684173764A\
73764A5A5E3A3A3C250000008B915AD8BB836D5663846A7881A692040705ADFFFF
F1FFFFF4FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFF\
F3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFF
F4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF3FFFFF3FFFFF4FFFFE1FFFF6AADA0000000\
985F48FFE6AD2012143700404E005A2700302500251B00201B00201B00201B0020
1B00201B00201B002019001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C19001C1B001C1E011C22021D26031D2C041D32051E36061E39061E38061E35061E\
31051E2F041D2E041D2E041D31051E34051E38061E39071E39061E35061E30051D
2A031D25021D22021D22021D21021A250318260515290610250708280707260707230606210606\
2006062006062106062407072907072B0808290707260707240707230606230606
2407072607072907072907072607072206061E05051C05051C05051D0505210606250707280707\
2707072306061B0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000200052000374000784F00985C00AD\
5E00B26100B96100B96100B96100B96100B96500BE6500BE6500BE6500BE6500BE
6600C36600D07600D07800D57800D57800D57800D57800D57800D57800D57800D57800D57800D5\
7800D57800D57800D57800D57800D57800D57800D57800D57800C77800C77800C7
7800C77800C77800D57800D57800D57600C36600C36600C36600C36600C36600C36500BE6500BE\
6500BE6500BE6500BE6500BE6500BE6C00CC7300CA3000554C2A56F6F696040402
14140C29291929291929291931311E31311E3A3A2347472A4F53305A5E3765683C73764A7D8351\
7D83517D835173764A313520000000847F5A8B68A268715F0205046DC9B4E1FFFF
F4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFFF4FFFF\
F4FFFFF4FFFFF4FFFFF4FFFFF4FFFFEFFFFFE6FFFFE1FFFFDCFFFFDCFFFFDCFFFF
DCFFFFDCFFFFDCFFFFDCFFFFDCFFFFE1FFFFE6FFFFEDFFFFF1FFFFF4FFFFEAFFFF99FFFF14120C\
7B4F3AFFDFA700000040004C3900452000251B00201B00201B002019001C19001C
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C1A001C1B001C1F011C24021D2A031D31051E36061E3C071E3E081E3D071E39071E\
36061E34051E32051E33051E35061E39071E3D071E3E081E3D071E39061E33051E
2C041D26031D22021D1F021A2103181F041022050D2306082807072A0808290707250707230606\
2206062306062306062707072B08082C08082A0808260707240707230606230606
2407072707072B08082B08082807072306061F06061D05051D05051F06062306062707072A0808\
2907072407071D0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000015002A33006A4C0088\
5600995C00A25E00A75E00A75E00A76100B96100B96100B96100B96100B96100B9
6100B96500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6600C3\
6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE6500BE
6500BE6500BE6500BE6500BE6500B26500B26500B26100B96100B96100B96100AD6100AD6100AD\
6100AD6100AD6100AD6100AD6500B26A00BB7300CA270045714361D3D381000000
2323153A3A233A3A233A3A233A3A2347472A4F4F305A5A377376457D834C949B5AA0A761AEB571\
B9C278C5CE81C5CE81B9C2785A5E3A0000001E1E17090C0B25413EADFFFFE1FFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFF1FFFFC7FFFF78E1C95FB4A058A694519988519988
51998851998851998851998858A69458A69468C2AD8AFFE8BBFFFFD7FFFFDCFFFFB4FFFF273A35\
654030FFD09B0C040E4700553000391C002319001C19001C19001C19001C19001C
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C19001C1A001C1C011C21021D26031D2D041D34051E3A071E3F081E41081F40081E3D071E\
39071E37061E36061E37061E3A071E3E081E41081F41081F3F081E3B071E35061E
2E041D27031D1F021A1B02151A03101A04081C05052306062A08082B08082B0808270707240707\
2306062306062507072907072C08082D08082A0808260707230606230606230606
2407072807072B08082C08082A08082407072006061E05051E05051F06062306062807072B0808\
2B08082507071E0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000C00102C0053\
3A007847008F4C00994F00A25C00A25C00A25E00A75E00A75E00A75E00A75E00A7
5E00A75E00A75E00A75E00A75E00A76100AD6100AD6100AD6100AD6100AD6100AD6100B96100B9\
6100B96100B96100AD6100AD6100AD6100AD6100AD6100AD6100AD6100AD6100AD
6100AD6100AD6100AD6100AD5E00A75E00A75E00A75E00A76100AD6100AD6100AD6100AD6100AD\
6100AD61009F6500A46500A46A00AD6F00B56C00B00E0019C7B0788B8B55020200
3A3A235A5A375A5A3765653C73734588885394945AA0A061C5C578DEE888F4FF94FDFF99FFFFE3\
FFFFEAFFFFF8FDFFFFF1FFFFC2FFFF3A5F5800000000000037452ABCE891FFFFCA
FFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDCFFFFDC\
FFFFDCFFFFDCFFFFDCFFFFCAD3FFA4518863102314020502000000000000000000
0000000000000000000000000000000000000205020C12192E474F66B4BB91FFF888FBEA254540\
5E5540FFB0831000124700552A00331C002319001C19001C19001C19001C19001C
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C
19001C1A001C1B001C1E011C22021D29031D30051D37061E3D071E42081F43081F41081F3E081E\
3C071E3A071E39071E3A071E3D071E41081F44091F44091F40081F3C071E35061E
2B041B2303181C021517020D1003051504041C05052306062A08082C08082B0808280707250707\
2407072407072607072A08082C08082C0808290707250707220606210606210606
2306062707072B08082C08082A08082407072006061E05051E05051F06062306062907072C0808\
2B08082507071E0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000020004\
1C00313300613E00744700834A008A4C008F4C008F4C008F4C008F4C008F4C008F
4C008F4C008F4C008F4F00965C00965C00A25C00A25C00A25C00965C00965C00965C00965C0096\
5C00965C00965C00965C00965C00A25C00A25C00A25C00965C00965C00965E0099
5E00995E00995E00995E00995E00995E009961009F61009F6500A46600A96600A96C00A25F00A4\
6F00B76500AD6500AD6300AB6C00A453008F29004807000CFFE69F15150C3A3A23
94945AB9B971B9B971DEDE88EAF498F1FFD1FFFFE3FFFFFDF8FFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFDCFFFF2945400C0E101B22290000005F7648D3FFA4
FDFFC3FFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCAFFFFCA\
FFFFCAFFFFCAFFFFCAE8FFB4A6CC81234025000000050B050C1C0E102514152E19
152E19152E19152E19152E19152E19152E19102317090C1400000002040517222C233A41070E0C\
659886C76D780B000C4800563700412700302000251C00231B002019001C19001C
19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C1A001C1B001C1B001C1B001C
1B001C1B001C1D011C1F011C25021D2B041D33051E3A071E40081F44091F44091F42081F40081E\
3D071E3C071E3C071E3E081E42081F46091F480A1F48091F44091F40081E36071C
2E051921041417030B0E03041303031A05052206062A08083109093309093209092D08082A0808\
2807072907072B08082E08083009093009092B0808270707230606230606230606
2607072B08082F08083009092C08082607072106061F06061E05051F06062306062907072C0808\
2B08082507071E0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000014002329004737005F3E006C43007347007A47007A47007A47007A47007A
47007A47008347007A47007A4A007F4A007F4A007F4A007F4A007F4A007F4A008A4A008A4C008F\
56008F4C00834A007F4A007F4A007F4A007F4A007F4A007F4C00835C00965C0096
5E00996500A45800985A009B5C009F5F00A46300A96500AD6500AD6100A75C00A05800984A0081\
39005A29004115002205000B000000040005290041732ACA454C3C303929CAFFCA
EAFFFFF8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFADFFFF000000556373768AA048553A02020283A065
DFFFADF1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3FDFFC3FDFFC3DFFFADA6CC81334F3A000200050B05152E19224C2927562E
27562E27562E27562E2C5F332C5F3327562E274C4C2C405C1B253710152007090E000000070B0E\
5FA4B5A77DC30200042000274500534500533A004730003925002C2000251C0023
1B002019001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C19001C\
19001C19001C19001C19001C19001C19001C19001C1D011C1F011C20011C20011C
20011C21021D22021D23021D26031D2E041D38061E40081F47091F490A1F480A1F46091F44091F\
3F081E3D071E3E081E42081F480A1F4F0B1F530C20540C20530C204D0B1D420A19
35081325060A1904041704041B05052407072F08083A0A0A420C0C440C0C420C0C3D0B0B3A0A0A\
380A0A390A0A3B0B0B3F0B0B400C0C3C0B0B360A0A3009092D08082D0808300909
350A0A3B0B0B400C0C3E0B0B370A0A2E08082607072106061E05051F06062306062807072B0808\
2B08082407071D0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000B000E1C00312900453700583A005E3300553E00633E00633E0063
3E00633E00633E00633E006C3E006C3E006C3E006C3E00633E00633E006343006A43006A430073\
4300734300734300734300734300734300734300734300734300734A007F550086
61009965009F5F00985A009B53008F4700713500551E00310C001502000200000007000C1B002A\
4C4C6A6D6D998888BEF4FFFFD8FFFF8AE6E64C7D7100000068AB99EAFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFF8FFFFF8FFFFF3FFFFF3FFFF\
F3FFFFF3FFFFFFFFFFFFFFFF416D6540404399B5D39FBBD88BA4C02C352C27301E
A6CC81E8FFB4FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3FDFFC3F1FFBBDFFFADBCE8916D8653101C150000000C1C0E224C292C5F33
316C3A316C3A316C3A2C5F33316C3A2C5F3333535F3148683148683145652C3C5825314A202A40\
455E8BFFFFC5DCFFFF3A3A530000000B000C2E00374500534500533A0047300039
25002C2000251B002019001C150019150019150019150019150019150019150019150019150019\
1500191500191500191500191500191500191700191E021A23031A25031A26031A
27041A28041A28041A25031A26031A2D051B3A071C460A1D4C0B1D4D0B1D4C0B1D490A1D450A1D\
3F081C3A071C3C081C40091C480A1D510C1E580D1E5C0E1F5C0F1D550F174B0D14
390B0C2B09082006061F06062607073209093F0C0B4C0F0E56110F57121051100E4A0F0D460D0C\
450D0C480E0D4A0E0D4E0F0E4E0F0E480E0D3D0C0B350A09330A09350A093C0C0B
430D0C4A0E0D4E0F0E4B0F0E410D0C3409092907072206062006062006062306062907072B0808\
2B08082407071D0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000020004170023200037250040250040290045290045290045
29004529004529004529004529004529004529004529004529004530004C330053330053330053\
33005333005333005333005333005333005333005333005337005847006F610099
5600881B00270000000200041200192A104C5A5A7F7F7FB2BBE6FFEAFFFFA7FFFF83E5D54C7D71\
1E312C020504000200152320416D6171B9A6ADFFFFEAFFFFFFFFFFFFFFFFFFFFFF
FDFFFFFDFFFFF8FFFFF8FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFECFFFFECFFFFECFFFF\
ECFFFFF8FFFFFFFFFFCAFFFF000000848F7A92ADC77F96AD9BB7D5769276040402
5F7648C7F699F1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3F1FFBBE8FFB4D3FFA4A0DA8183A0654353330B100B020402152E1927562E
2C5F33316C3A316C3A2C5F332C5F332C5F33394F74394F74394F7440557F455E8B394C712C3C58\
2C373A4C402C7A866592FFFFD8FFFFC7FFFF4F4F7102000409000B2E0037450053
4500533A00472A00332000251B0020150019140017100014100014100014100014100014100014\
1000141000141000141000141000141000141301142003162B06172F0717300817
3108173208173008172A06162605162E06173E09194D0D1A550F1B550F1B530F1B4F0E1A490D1A\
3F0B193808183908183E0919460A19520E1A5C12196214196315185E1615541611
47130D360F0A2B0B08290A08330B09420F0C50120F5C1611651913661A135C181153150F50150F\
5315105815115A16115D16115D171155151048120D3C0F0B380E0B3E0F0C48120E
52130F5A16115D17115B16114D130F3E100C2E0B092509072307062206062507072A08082C0808\
2B08082407071D0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000020C00140E001710001B10001B10001B10001B
10001B10001B10001B10001B10001B10001B10001B10001B10001B10001B14001E14001E14001E\
14001E14001E14001E14001E14001E14001E14001E14001E1900272A0043480073
1C25276D868DD7FFFFA4FFFF6AAE9D31534A121E1B000000070E0C29453E5E9B8B8FECD3B7FFFF\
D1FFFFF1FFFFFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFF8FFFFF3FFFF
F3FFFFF3FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF\
F3FFFFFFFFFFFFFFFF5E9B91453C2A9FBBD86F83985E6F81788FA69BB7D54C5E51
0C100998BC76DFFFADF1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3F1FFBBE8FFB4C7F6996A997147654A435333191E120000000C1C0E224C29
2C5F33316C3A2C5F33316C3A2C5F332A5345314868394F74405A83455E8B455E8B394C7125314A\
1C17100505040000000000000002020002022547479BFFFFCAFFFF555578040004
07000923002E3700473300412500301900201000150B000E09000C07000B070009070009070009\
0700090700090700090700090700090700090E010A1F040D300A0F3A0D103E0F12
3E0F123E0F12390D102E0B0E25060D2C070E400B115410145C14155C15155914155513144C1113\
3F0E0F34090E33080E38090F400B104E0F125B1413671916691B16651D135F1C12
551A1041140C340F09320D093F0F0C50130F5E19126B1D14732116702115631E12571B10561910\
5D1A11641C13671D136B1E146A1E14621B1251170F41120C3D100B46120D541610
5F1912671C136C1E14671D1358191046140D330E0A290A082508072306062707072B08082C0808\
2B08082407071C0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
050909FFFFFF121E1B3A5F5599FBE1CAFFFFE3FFFFF8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\
FFFFFFFFFFFFFFFFFFFFFFFFFDFFFFF8FFFFF3FFFFF3FFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF\
F8FFFFFFFFFFDCFFFF04050499845C8FA9C358687847535F5E6F8192ADC78BAED1
15191237452AB0DA88E8FFB4FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3F1FFBBDFFFADA6CC813A5A411023140C17100C10090000000B170C1C4022
2C5F33316C3A316C3A2C5F332C5F332A534F31486831486840557F455E8B455E8B31456520272A\
100C0900000000000015120C4C533E65B9B96DC9C9488888091414101E1E8BFFFF
BBFFFF487F7F000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000200000C02022306063B0E0B47130D4E150E
4E150E4E150E45140D34100A280A082E0808440D0C5B1410671913681A13641A135D191152160F\
41120C310B092C0808320909390A0A4B0F0D5E16126D1D157120157022156B2214
6020134B190E3B130B3B110B4C130E5D19116C1E147723167E2718792717682314591E115A1C11\
651E136F21157323167724167624166C2214581C1146160D41120C4D140E5F1A12
6C1F14732316782416732416621E134D180F37100A2C0B092808072607072807072B08082C0808\
2907072306061B0505
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000C3FFFF050907B7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFDFFFFFDFFFFF8FFFF\
F8FFFFF8FFFFF3FFFFF3FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFF\
FFFFFFFFFFFF71B9AD23231C9BB7D56F839847535F3C475147535F6F83989FBBD8
6C8A7A0000006D8653D3FFA4F1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3F1FFBBD3FFA483A06517251B0000000000000000000204020C1C0E1C4022
27562E316C3A2C5F332C5F332C5F33314868314868394F7440557F455E8B394C7125314A1C1710\
0505040000000000000B0905352C1E838F6AB4FFFFD7FFFFD0FFFFB4FFFF65B9B9
122323101E1E9BFFFFBBFFFF3C6A6A000000000000000000000000000000000000000000000000\
000000000000000000000000000000040101140404300A084B130E591A10601E12
621E12601E12531B103F140C310D09390B0A50100E671813741F167621167121156920145A1D12\
46160D350E0A2D09082F0808360A0A4B0F0D6218127620167A2517792817752817
6A2615531E1043170D42140C5617106A1E14772416802818862C1A7E2C186A26145A20115C1D12\
6920147825177D29188029188029187627165F201249180E44140D541610681D13
7624167D28188029187E29186B2415521B103B130B2D0B092A08072707072907072B08082B0808\
270707200606190404
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000040505AEFFFF4A7A6DFFFFFFFFFFFFFDFFFFF8FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFF\
F3FFFFF3FFFFF3FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF8FFFF\
FFFFFFEAFFFF10191583714E96B0CC586878414C583C47513C4751536171869FB9
96B0CC41533C14170E98BC76DFFFADF1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3F1FFBBD3FFA46D8653070E090204020C1C0E0C1C0E0E1C121B301C224C29
2C5F33316C3A316C3A2C5F33275335394F74314868394F7440557F455E8B31456520272A100C09\
0402020402020000000000000B0905483C2A91DAC2D7FFFFEFFFFFF4FFFFF4FFFF
E6FFFFBBFFFF6DC9C90E1C1C142525A6FFFFB9FFFF335A5A000000000000000000000000000000\
0000000000000000000000000100000801011D05053D0E0B581810662013702415
7326166D25155C211246190D3A120B440F0C5B1410701D157E25188028187D2818722615602112\
491A0E38110A2D0A082B0808310909470F0D6219127823177E2A187D2C187A2C18
702A16582312471B0E48160D5D1A117122157B2817822D1986301A7B2E18662814532011581D11\
6B22147C2817812D19852D19852E197A2B186224134A1A0E43140C5517106D1F14
7B2717822D19862E1A822D196F2916561E103D140C2E0C092B0908280707290707290707280707\
2407071D0505160404
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000005581912A4740A2FFF1FFFFFFFFFFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFF\
F3FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFF\
FFFFFF8FECDC17120C96B0CC788FA64C5A683C47513C47513C4751414C585E6F81
98B2CE84A7C90E120C435333BCE891E8FFB4FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
FDFFC3E8FFB4C7F6995F764802050207100719351C1C40221C4022224C2927562E
2C5F332C5F332C5F332C5F33305C4C314868394F74405A8340557F394C7125314A1C1710050504\
0402020000000000000000000000000E0C075345309BE8CEDCFFFFF4FFFFF1FFFF
F1FFFFF3FFFFF4FFFFE6FFFFBBFFFF65B9B90E1C1C142525A6FFFFB9FFFF335A5A000000000000\
0000000000000000000000000200000E030327070748120E611D126F2616792A17
7A2C18722B165F26124B1D0E44140C52130F651813762116812919862D1A812E19742B17602513\
4B1C0E3A130B2D0B092607072807073E0D0B5A1911712315762B17762D17732E16
6B2C15552511441B0D46180E5E1C11722516792C177D2F187E31187130175A2812451E0D4B1A0E\
6120137529167A2E187E30187F3019772E175F2613441B0E3D140C4F170F6A2114
7629167C2E188030187F30196D2B155420113A150B2E0C092A0807270707270707260707240707\
2006061A0505130303
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000D5FFFF000000E3FFFFFFFFFFFDFFFFF3FFFFF3FFFFF3FFFFF3FFFFF3FFFF\
F3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF8FFFFFFFFFF\
F8FFFF1C2C259B5A5599B5D15E6F81414C583C47513C47513C47513C47514C5A68
748A9F9FBBD8637F7100000078945CDFFFADF1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB4BCE891556A410000000C1C0E224C2927562E27562E2C5F332C5F33
316C3A2C5F332C5F332C5F3333535F314868394F7440557F40557F31456520272A100C09040202\
00000000000000000000000000000000000015120C5E4E37A4F6DAE1FFFFF4FFFF
EDFFFFEAFFFFEAFFFFEDFFFFF3FFFFF4FFFFE6FFFFBBFFFF65B9B90E1C1C142525A6FFFFB9FFFF\
335A5A000000000000000000050101150404340B0953150F6721147228167A2E18
7C3018712F165D28124D1F0F4E180F5E18126F1E15782517812C1986311A813219722E165D2812\
491F0F3A130B2E0D09240706220606380C0A5518106D2315732C16732F16713016
6A2D15552711441C0E45180D5F1D11702716762D17782F17773117692F155026103A1B0B42180D\
5A1E117129167930187D31197E3219762F175E2813431B0D3A130B4E160F692114
752B177A30187F32197E31196D2D1553221139150B2B0A082707072407072306062206061F0606\
1B05051504040F0303
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000DCFFFF1C302AFDFFFFFFFFFFF8FFFFF3FFFFF3FFFFF3FFFFF3FFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFFFFFFFF\
A2FFF80B090792ADC9788FA647535F3C47513C47513C47513C47513C4751414C58
586878869FB992ADC9354035222919A6CC81DFFFADFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB4B0DA883C533000000010251427562E2C5F332C5F332C5F33316C3A
316C3A2C5F332C5F3327562E314868314868394F7440557F394C7125314A1C1710050504040202\
00000000000000000000000000000000000000000015120C5E4E37C2FFFDEAFFFF
F4FFFFEAFFFFE6FFFFE6FFFFE6FFFFEAFFFFEDFFFFF3FFFFF4FFFFE6FFFFBBFFFF65B9B90E1C1C\
142525A6FFFFB9FFFF2F53530A02021F06063F0E0B5B1A116B2415732B16793018
7832176C2F155829124E200F551B106A1D147722167A28177E2D188332197C33196C2F15562812\
451E0E39150B2D0D082007051B050531090950150F6A2214702A16712D166F2D15
682B15522510411A0D43170D5D1C116E2615732C16752E17732E16642C144A220F32160A39110A\
561A106E2515772D177A2F187B2F18742D175C261240190C37100A4C130E671F13
752A177A2E187E30197D2F186B2B1552201035120A2709072306061F06061C05051B0505190404\
1404041003030C0202
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000090E1292EFD768AB99FFFFFFFFFFFFF3FFFFF3FFFFECFFFFECFFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF8FFFFFFFFFFF8FFFF\
2239338663557BB2C243616A37404A30394137404A37404A3C47513C47513C4751
414C5863748899B5D17D9B7D040402556A41C7F699F1FFBBFDFFC3FDFFC3FDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB4A6CC81273929000000152E1927562E316C3A316C3A316C3A316C3A
2C5F33316C3A2C5F33305C4C314868314868314868394C712C3C5820272A100C09040202000000\
0000000000000000000000000000000000000000000000001B15106D684AB4FFFF
EDFFFFF4FFFFEAFFFFE6FFFFE3FFFFE3FFFFE3FFFFE6FFFFEAFFFFEDFFFFF3FFFFF4FFFFE6FFFF\
BBFFFF65B9B90E1C1C1D3131B2F1F1B7D8D84D1713621E126D2615712D16743017
6E3016622F135127104C1F0F5C1D127221157C27177B2B18792E187C3118753317632E134F2610\
3F1C0C36140A2D0E081E08061504042B08084B140E6621146D29156F2C156E2C15
682B1550240F3E1A0D41170D5B1C116C2615712C16722D16702D16612913461F0E2C1208350E0A\
52170F6C2315752C17792E18792E18722C165A25123D180B350F0A4B120E671E13
7429167A2E187E2F197D2F196A2A14501F1033120A2308061D0505190404160404140404120303\
0F03030C0202080101
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000006598AD1E312CB7FFFFFFFFFFF8FFFFECFFFFECFFFFECFFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFFFFFFFF8FECDC\
151510D0A6EA5C838F2C404523313727313730394130394137404A3C47513C4751
3C47514C5A68788FA69DB9D753685A090B078DAE6DDFFFADF1FFBBFDFFC3FDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB498BC76273929000000152E1927562E316C3A316C3A316C3A316C3A
316C3A2C5F3327562E2A534F314868314868394F7431456525314A1C1710050504040202000000\
000000000000000000000000000000000000000000000000000000231E15787351
BBFFFFEDFFFFF3FFFFEAFFFFE6FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFFE6FFFFEAFFFFEDFFFF\
F3FFFFF4FFFFE6FFFFB5FAFA517272412220E6AFA86824166D29156F2D156E2F16
642F14562B1149240F4B1E0F611F127825167F2C187A2E18752F177230176A3115582C1245230E\
38190B32130A2C0F091F0A0613050326070747130D6520136D29156E2C156C2C15
662A144E240F3D190C3F160C591B116A26156F2C16712D166F2C155F2813441E0D291108320C09\
4F160F6A2314722B16772D17762E176E2B155625113A180B310E0948120E651E13
742916792E187D2F197C2E18692A144E1F1031120A1E07051804041404041003030E03030C0202\
0A0202070101050101
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000D8FFFF070E0CF8FFFFFFFFFFF3FFFFECFFFFECFFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF8FFFFFFFFFFF1FFFF101C19\
454E7A78ADBB3A535A1E2C301B272A1E2C302731372A313A30394137404A3C4751
3C4751414C585868788FA9C3849BB4191E1537452AB0DA88E8FFB4FDFFC3FDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB4B0DA882A41300000000C1C0E27562E2C5F33316C3A316C3A316C3A
2C5F332C5F332C5F332E4C5631486831486831486825314A1B2022100C09040202000000000000\
000000000000000000000000000000000000000000000000000000020200231E15
8F8353BBFFFFEFFFFFF3FFFFEAFFFFE6FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFF\
E6FFFFEAFFFFF1FEFEF3F5F5C8E1E1461815DEA89F6A25156E2A156C2D15652C14
582B1249260F401F0D4B1C0E6520147A28177F2F187931186F2F16682E155E2D134E28103E1F0D\
34170B3012092C1009200B0613060424070646130D6320136B29156D2C156B2B15
652A144D240F3B190C3D160C571B116A25146F2B156F2D166E2C155E2813421E0D271108300C09\
4E150E692214722B16752D17742E176D2B1554241036170B2F0E0945110D651E13
7328167A2E187D2F197A2E18672A144C1E0F2D11091C07051404040E03030C0202090202080101\
060101040101030000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000040505A6FFFF53887DFFFFFFFFFFFFF3FFFFECFFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFFFFFFFF84D8C92A2C20\
D7ABF34E6F7827393C1B272A1720231B272A1B272A1E2C302A313A30394137404A
37404A3C475147535F6F83989DB9D77394830000006D8653D3FFA4F1FFBBFDFFC3FDFFC3FDFFC3\
F1FFBBE8FFB4BCE89163864F070E0902050419351C27562E2C5F33316C3A316C3A
2C5F332C5F332C5F333148683148683148682C3C58202A401C1710050504040202000000000000\
000000000000000000000000000000000000000000000000000000000000050402
2A23199B8D5AC3FFFFEFFFFFF3FFFFEAFFFFE6FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFF\
E3FFFFE6FFFFEDFCFCEEEEEE83ABAA715C57936E666C27156E2C156A2C155D2A13
4B260F3D210C3C1C0C4E1B0F6821147A2B187D3119773217692F155C2B1250281042230D351B0B\
2F14092D11092A1009230C0617080524070645130D6320136B29156D2C156B2B15
642A144B230F39180B3B150B561B116825146E2B156F2D166E2C155E2813411E0D2610072F0C09\
4E150E692214722B16762D17752E176E2B1554251138170B300E0946110D651E13
7328167A2E187C2E18792E18642914471D0E2810081706040F03030A0202060101040101040101\
040101030000020000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000004F788831534CADFFFFFFFFFFF8FFFFECFFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF8FFFFFFFFFFF8FFFF101C19253A43\
73A498354C531B272A1720231720231720231B272A1B272A1E2C302A313A303941
37404A37404A3C47515361718399B296B0CC41533C14170E98BC76DFFFADF1FFBBFDFFC3FDFFC3\
FDFFC3F1FFBBD3FFA498BC76374F3A0002020710071C402227562E2C5F332C5F33
2C5F332C5F33305C4C3148683148682C405C25314A1E201B0B0907040202000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
050402352C1E8D9B74CAFFFFF1FFFFF3FFFFEAFFFFE6FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFF\
EAFFFFEDFDFDF3F6F6A2E4E43F1C1AB5ACA56A22146F2A166F2C156A2F17876757
634B3A341C0B3B180B541A106D23157A2C187B3218763317723F27714E3B644A395B4638554136\
533D36523C36523C363B221C210A062D0A084A140E77372BAE867BC9AFA7BEA197
8152404D25114E31255E3F3775453C844D3F8952408A53408952407C4F3E4F2C1C301A12543734\
6E403A844B3F8B52418F53419054418B534071443142190C3D110B5314106C1F14
7829167C2E187C2E18752D17602813431D0D240F071405030C0202060101040101020000020000\
010000010000010000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000ABFFFF020402F8FFFFFFFFFFF3FFFF\
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFFFFFFFFA2FFF8141712FFB5A7\
51747F2331371720231720231720231720231720231720231B272A1E2C30273137
30394130394137404A414C585E6F8196B0CC84A7C90E120C435333BCE891E8FFB4FDFFC3FDFFC3\
F1FFBBF1FFBBE8FFB4BCE89183A0652C3A230000000B170C224C292C5F33316C3A
316C3A2C5F332E4C563148682C405C25354C202A4017120C050504040202000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000050402352C1EA6B588D0FFFFF1FFFFF1FFFFEAFFFFE3FFFFE3FFFFE3FFFFE3FFFFE5FFFF\
FAFEFFF8FAFBB8ECEC2B0808C5BDBA641C13742616762C17722D16713822D9D0CC
B5ABA4371D103F160C5C1C117426167D2E187C3218753217794B358D7467D2CDC9F0EFEFF0EFEF\
F1EFEFD3CCCB7D686351342E2D0E08350D0A5E2B24D6C3C0FAF8F8FEFEFFFBFAFB
EDE6E48C706365483E8B716CE3D8D7F4EFEFF5F0EFF5F0EFDAC6C19A71635F372840231A735754\
C3AEACF3EDEDF5F0EFF5F0EFEFE7E5B28B7C875B474F1D0F4B150E621812772216
7D2C187D2F19792E186E2C155826113C1B0C200E060F0503090202040101020000010000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000066B5A98FECDCFFFFFFF8FFFF\
F3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFFFFFFF8FFFF2239335C63558484BB\
3A535A1B272A1720231720231720231720231720231720231720231B272A1E2C30
2731372A313A3039413C47514C5A68748A9F9FBBD8637F7100000078945CDFFFADF1FFBBFDFFC3\
F1FFBBE8FFB4DFFFADC7F699A6CC816D8653151E12000000152E1927562E2C5F33
316C3A2C5F333148682C405C2C405C202C411519140B0907040202000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000B09053C3123A6B588D0FFFFF4FFFFF1FFFFEAFFFFE3FFFFE3FFFFE3FFFFF0FEFF\
FEFEFFF5FDFD382929D7D3D248120D621E136D28156E2C15692D15886150F0EEED
E9E6E54C372E3C140C561B116C2615732E167131166A30155A2C1243230E67584EEBEAEAFEFEFF\
EDEBEB6D565036140C34120A32110A37100A90706BF7F5F5FEFEFFEFE9E8AA897C
A88C80CCC1BC58372B481A109B716AF7F4F4FEFEFFEFE5E39659466B2A1450211034140A3B0F0B\
67312BE0D1D0FEFEFFFEFEFFB9958878341B642A14552110531A10651D13752616
792E18772F17702E16642B144F241034180A1B0D060D0402070101040101010000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000C0A98F3A584CFFFFFFFFFFFF\
F3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFDFFFFFFFFFF84D8C9202720FFB7A95A747F\
27393C1B272A1720231720231720231720231720231720231720231720231B272A
1B272A1E2C302A313A3039413C4751536171869FB992ADC9354035222919A6CC81DFFFADF1FFBB\
E8FFB4DFFFADABE88A98CC7898BC7683A0654353330002000B170C224C292C5F33
2C5F332C5C3C3148682C405C25354C151E2C100C09040202000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000B0905483C2AB0C291D7FFFFF4FFFFEDFFFFEAFFFFE6FFFFEAFFFFF9FEFF\
FEFEFFF9F9FADFF1F132100E45150D5B2112642A14642D1466351DC1B2AAFEFEFF
F6F5F580746E37130B4F1B0F612613652D14642F145D2F134E2910381F0B3C2F23E0DEDDFEFEFF\
E2DEDE492F2633140A33140A33140A36130BAA9691FEFEFFFEFEFFBEA89E62321A
572B1474584958392D46190D7E493EF5EFEEFEFEFFE5D5D1823C246B2D1550241037170B3C120B\
551A10D0BAB6FEFEFFFEFEFF9D6A58763117652D14562411521F106020126C2915
6E2E166A2E15622C1356281242200D2B1609150B050A0302050101030000010000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000040505FFD7A0373527FDFFFFFFFFFF\
F8FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFFFFFFEAFFFF04070551749656639937474C\
222C301B272A1B272A172023172023172023172023172023172023172023172023
1B272A1B272A1E2C302A313A303941414C5863748899B5D17D9B7D040402556A41C7F699E8FFB4\
E8FFB4BCE8916A997137613A3E5A41556A414C5E3A10171002040219351C27562E
2C5F332C53533148682C405C1B25370C0E0B050504000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000B0905483C2A91D1A0D7FFFFF4FFFFF1FFFFEDFFFFF4FFFFF8FEFF\
FEFEFFFEFEFFD9D6D84528253E140C501F0F582611582A1266412CE2DCD9FEFEFF
FEFEFFAFA8A636171045190D542311582912562A1150281041240E2E1A09352C22DFDDDDFEFEFF\
E2DEDE442E252D13092E130931140933140AA69590FEFEFFFEFEFF9C8376502710
4A250F40200D3A190B431A0D7B483CF4EFEEFEFEFFE4D6D17F3C23692F154F241035180B3B130B\
54190FCFB9B4FEFEFFFEFEFF9D6B59753117642D145525114E2010592111612813
612B135C2B1355281147220E341C0B1F1107100803070201040101020000010000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000020202FFFFC3684835583C2EFDFFFDFFFFFF\
FFFFFFF8FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFFDFFFFFFFFFF71B9AD5E7A5E273333273933141C1E\
1920231E272A1E272A1B272A172023172023172023172023172023172023172023
1720231720231B272A1E2C3027313737404A47535F788FA69DB9D75E7465090B078DAE6DD3FFA4\
DFFFADA6CC813E5A4109150B050B050C17101E29170C1009000000102514224C29
2C5F3333535F314868202C41101722050504000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000B09055345309BE8CEDCFFFFF4FFFFF4FFFFE9FEFFFAFBFC\
FEFEFFD7DBDCA29A9A897A7835130A431B0D49220E4A240F856F62FAF9FAFEFEFF
D4D1D0CFCBCA4A332E3A160B451E0E49230F47230E41210D331C0A2415072F2820DEDDDDFEFEFF\
E1DEDE3F2B242711082911082B12082D1309A2948FFEFEFFFEFEFF95807542210D
3D1E0C351B0B32170A3C170B75463BF4EFEEFEFEFFE3D5D1783C22632D144E241034180A39120A\
531A10CFBAB5FEFEFFFEFEFF9C6B59723017622C13522510491F0F502010542410
5227104D251044220E361C0B261507160C040B0602040201030000010000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000020404FFDCA6FBAE83000000583C2EFFB588FDFFFF\
FFFFFFFFFFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFF3FFFFFFFFFFF1FFFF04070578ADA019252C0C1415101719\
1720231E272A1E272A1E272A1E272A1B272A172023172023172023172023172023
1720231720231720231B272A1E2C3027313737404A5361718BA4BE88A0BB22291E303A25B0DA88\
C7F6998DAE6D1E2E220000000204020000000000000202020000000C1C0E1C4022
27562E3148682C405C1B25370C0E0B000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000E0C075345309BE8CED0FFFFD0FFFFB0CBCCFEFEFF\
FCFCFE9D999A3C2F2DBCB6B63F2A2334170B391B0B3F2413B2A9A3FEFEFFF4F3F3
746B66999290796C692B120835180B381C0C361C0B301A0A2616081B11062C2620DEDDDDFEFEFF\
E0DEDE3A2A24210E06220F07230F072410079D928FFEFEFFFEFEFF8A7C73311A0A
2E19092A16082B130934150A6E453AF3EFEEFEFEFFE0D4D06F39205C2C1348230F31180A36120A\
511A10CFBAB5FEFEFFFEFEFF9869586F30165D2C124B230F3F1D0C421C0D441F0D
42200D3B1E0C311A0A2615071910060D0803060402030000020000010000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000050707FFD39FFFB78A07050419120C583C2EBB815FFFFDC7\
FFFFFFFFFFFFFDFFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFECFFFFFDFFFFFFFFFF84D8C945685A2531310E170E070B0C0C1415\
141C1E1720231B272A1E272A1E272A1E272A1B272A1B272A172023172023172023
1720231720231720231720231B272A1B272A2331373C47516374889BB7D5718B710000005F7648\
A6CC8178945C101C14020402102514102514071007040704090C091025141C4022
25483C2C405C202C41101722050504000000000000020200020200000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000E0C075345306AA08F5C8686E9EDEEFEFEFF\
EBEBEC3F3A391308078F89877F7773211006261508574A40EAE9E9FEFEFFE0DFDF
362D2867605DC0BDBC2F221B231207261507231407201306191005120C0428231FDCDCDCFEFEFF\
DFDEDE312721180B05170C05160C05150C0495908DFEFEFFFEFEFF807770211307
211207201107210F062B1108634138F2EFEFFEFEFFDED5D061351E50271040200D2C160934110A\
4F190FCFBAB5FEFEFFFEFEFF956857672D14542A113D200C2B18092C17092E190A
2C19092717081F1306170F050D0A03070501030100020000010000010000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000090C0CFFC091FFC3920E0907120C0933231B4A31257D5641EAA078\
FFFFE3FFFFFFFFFFFFF8FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFF3FFFFFFFFFFFDFFFF101C1994C7AE1C2C33090E10070B0C101719\
1720231720231720231720231B272E1E272A1E272A1B272E172023172023172023
1720231720231720231720231720231B272A1B272A233137414C58788FA69BB7D5485C430C1009\
6D8653556A41050B07050B051C4022224C2919351C10251415271719351C224C29
25484525354C1B25370C0E0B000000000000000000020200020200020200020200020200000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000B09050B0905656D6DFEFEFFFEFEFF\
B3B3B306050404020033302EB4B3B326241D110F0576756FFEFEFFFAFAFB969593
0D0A05211E1BB8B7B73F3D390D0B030F0D040F0D040D0B020C090208070122201DDCDBDCFEFEFF\
DEDDDD26231F0A08020907020A07020B0903928F8CFEFEFFFEFEFF7C746F1D0F06
1C0D061A0C05180B051B0D06533D35F0EEEEFEFEFFD9D2CE4C2E1A3C210C321B0A25140731110A\
50190FD0B9B5FEFEFFFEFEFF956756622B134B270F2F1C09191406151005171205
161105140F04100D040C0A03070501030200010000010000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000986F53FFDEA7140C09120C0933231B402C204A3125583C2E91634A\
F8AB81F1FFFFFFFFFFFFFFFFF8FFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFECFFFFF8FFFFFFFFFFA2FFF81E2E2729373722273C0C1415101719141C1E\
1720231720231720231720231B272A1B272A1E272A1E272A1E272A172023172023
1720231720231720231720231720231720231720231B272A27393C53617192ADC784A7C9191E15\
191E12303A250002020B170C224C2927562E27562E224C29224C2927562E27562E
2C405C202C41101722050504000000000000020200020200020200020200020200020200020200\
020200000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000A0A09E6E2D3FEFEFFF9F9FA\
6D6D6E0000000101000707058A8A896F6F6D1D1D19CACAC9FEFEFFEDEDEE3B3B38
0304010506037474738989880506010607010607020606010505010304011E1E1DDBDBDCFEFEFF\
DCDCDD21211E040401040501040401070502918D8CFEFEFFFEFEFF807270230C07
210A061C0A05140904120903443832EFEEEEFEFEFFD5D1CE3B28172E1A09281709201107301009\
511A10D0BAB5FEFEFFFEFEFF956857612A1347240E2818080E0D03080802080902
080902070802060601040401020200000100000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000006D4F3AFFFFC02217120C09052C1E17402C20402C20402C204A3125654533\
AD7658FFC091FFFFFFFFFFFFFDFFFFF3FFFFECFFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFF3FFFFFFFFFFFFFFFF416D65557163C376A93A4C532A393C27393C233137\
1E2C301B272A1720231720231720231720231720231B272A1E272A1E272A172023
1720231720231720231720231720231720231720231720231E2C302C40456374889DB9D7739483\
0507040202020000000C1C0E27562E2C5F332C5F332C5F332C5F332C5F332C5F33
2C405C1B25370C0E0B000000000000000000020200020200020200020200020200020200020200\
020200020200020200020200000000000000000000000000000000000000000000
0000000402020402020402020505040505040402020505040B0907474339FAF6F1FEFEFFECEDEE\
303030000000000000000000313131B0B0B16C6C6CF7F7F8FEFEFFC2C2C2151515
0001000001003E3E3EA8A8A91515140101000101000001000001000001001C1C1CDBDBDCFEFEFF\
DCDCDD1F1F1F000100000100000100040201928C8CFEFEFFFEFEFF8473712B0C08
2A0C08230B071608040B06023A3430EEEDEDFEFEFFD1CFCD2A1D121F12071E10061B0D062D0D08\
4C180ECEBAB6FEFEFFFEFEFF9166555C2A1243220E2514070A0802020200020200
020200020200010200010100000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000005F4533FFFFC937251C0202002C1E17402C20402C20402C20402C204A31254A3125\
714E3AC98B68FFFFD0FFFFFFFFFFFFF8FFFFF3FFFFECFFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFECFFFFF8FFFFFFFFFFEAFFFF070E0C2337407155B986ADB97FA2AD6F8F98486878\
354C532C40452331371E2C301B272A1720231720231720231720231E272A1E272A
1B272A1720231720231720231720231720231720231720231720231E2C30354C536D8F999BB7D5\
78917812190E071007152E1927562E2C5F33316C3A316C3A316C3A2C5F33274C4C
25354C101722050504000000000000020200020200020200020200020200020200020200020200\
020200000000000000000000000000000000000000000000000000000000000000
0402020505040B090717120C1C17102E251B2E251B352C204035259E9281FEFEFFFEFEFFB9B9B9\
252F2F0000000000000000000606068C8C8CC6C6C7FEFEFFFBFBFC8D8D8E000000
000000000000121212B3B3B33434340000000000000000000000000000001D1D1DDCDCDDFEFEFF\
DDDDDE1E1E1E000000000000000000040101938C8CFEFEFFFEFEFF8775722F1009
2E0F09270E07170A0409050234312FEDECEDFEFEFFCFCDCD1E1710130B04140A04180A04280D08\
44160DCBBAB5FEFEFFFEFEFF8A66545228113D200C201307070702000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000005F4533FFFFC937251C0202002C1E17402C20402C20402C20402C20402C20402C204A3125\
583C2E7D5641EAA078FFFFFFFFFFFFFFFFFFF8FFFFF3FFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFF3FFFFFDFFFFFFFFFFADFFFF0000008F554EB5E1ECA2D1B994BBDF99C5D39BC7D5\
7B9FAD567D8843616A354C5327393C1E2C301E2C301B272A1B272A1B272A1B272E
1E272A1720231720231720231720231720231720231720231720231720231E2C303E5A617FA4B2\
9BC3EC83B06827562E27562E2C5F33316C3A316C3A316C3A316C3A2C5F3329414C
202C410C0E0B000000000000000000020200020200020200020200000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000050504
1C19104037255F5339866F4E9D835CB9996CBEB583CAC08AD7C492F7F1E1FEFEFFFDFEFD95A48E\
1420122937370000000000000000005A5A5AFEFEFFFEFEFFE5E5E6353536000000
0000000000000101017676778D8D8D0303030000000000000000000000001D1D1DDCDCDDFEFEFF\
DDDDDE1E1E1E000000000000000000040101938D8DFEFEFFFEFEFF88767231130A
2F12092810081A0C05090602312F2EECECEDFEFEFFCECDCD15130F0A06020D0703130804230B07\
3A130BC7B9B5FEFEFFFEFEFF81635347240E341D0B1D1206070701000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
4C3729FFFFCC412C220202002C1E1733231B402C20402C20402C20402C20402C20402C20402C20\
4A31256545339F6D51FFC091FDFFFFFFFFFFFFFFFFF3FFFFECFFFFECFFFFECFFFF
ECFFFFECFFFFF8FFFFFFFFFFFDFFFF4A7A710202041B2320122220070E0C091412253C415C83A4\
8BAED18DB5C57FB7C773A4B2567D8843616A354C532C40452331372331371E2C30
1B272A1920231720251720231720231720231720231720231720231720231B272A27393C4E6F78\
88B0BEDCFFEC5AC56A3C8648316C3A2C5F33316C3A2C5F332C5F3327562E25354C
15202E050504000000000000000000000000000000000000000000000000000000000000050504\
100E0917140C2520152E271B352E20403725352E201C1910100E09100E092E2A20
685E45A6A683D1D18BD0F8A9B5FD8D83E69183E69183E69193E79EEDF9EFFEFEFFF0FBEB518057\
000000355535000000000000000000676768FEFEFFFEFEFFA8A8A8060606000000
000000000000000000414142C7C7C82121210000000000000000000000001D1D1DDCDCDDFEFEFF\
DDDDDE1F1F1F000000000000000000040101938D8DFEFEFFFEFEFF877671301209
2E1209271108190C050806022E2E2EECECEDFEFEFFE3E3E440403F0504020804020F06031D0906\
35170FC9BEBBFEFEFFF8F7F772594B3A1E0C2B1809180F05060601000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000201C14
FFFFD3412C220000002C1E1733231B402C20402C20402C20402C20402C20402C20402C20402C20\
402C204A3125714E3ABB815FFFF4C0FFFFFFFFFFFFFDFFFFF3FFFFECFFFFECFFFF
ECFFFFF3FFFFFDFFFFFFFFFFB7FFFF14140E0B05050000000C19193361613361611B3131000202\
040707274741558A9D76C2CC7DB2C283BBCA76A9B7638F9951747F43616A31474C
27393C2331371E2C331B272E1B272A1720231720231720231720231720231720231E2C302C4045\
567D888DB7C7C0FFD85AC56A3776402C5F332C5F332C5F332C5F332A534525354C
1E201B100E0905050404040204040200000000000000000000000004040217140C4037255F5339\
917855B4AB7AB5A67AC3C399CCCCA0C9C984A4C3849B9B7A766845564C315E5E4A
589B616AD58673D8836AC7785EB26C56A663539F5F4F985CA8CCADFEFEFEFEFEFFD0E9D47DAE5F\
1E37220407044E7A4E000000121212C1C1C2FEFEFFF6F6F7686869000000000000
0000000000000000000C0C0C9898985757580000000000000C0C0C101010515152ECECEDFEFEFF\
DDDDDE1E1E1E000000000000000000040101928C8CFEFEFFFEFEFF8576712B1108
291108231007170C050906022E2E2EECECEDFEFEFFE5E5E6A3A3A4403F3E0D0B0A0C04021B0C09\
817571F2F1F1FEFEFFD5D0CE3D28182B1809201307120C04050501000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000100E09FFFFD8
6848350000002C1E1733231B402C20402C20402C20402C20402C20402C20402C20402C20402C20\
402C204A3125583C2E7D5641DC9871FFFFD7FFFFFFFFFFFFF8FFFFF3FFFFECFFFF
ECFFFFF3FFFFFFFFFFFFFFFF68AB9F0000000202020910104A8B8B78E1E181F1F171D3D358A6A6\
417A7A203E3E050C0C0000001C353543817A6CB0B97FC0D881B9C97FB7C76F9FAD
5C838F486871354C532C404523313A1E2C301B272A1B272A1720231720231720231B272A1E2C30\
354C535F8A94ADE6CA9BFFA7479D55316C3A316C3A316C3A2C5F33356863394F74
5F5339564C314C412C564C314C402C2520150B0B0700000000000014120C4C4C3C8BA671A0C7A0\
B5FD8D83E69183E6917DDF8B7BDC8A6DCE7B6FD57F83E691ECECBBCAC08AD0B57A
DEDEAEB9FF8F66C376539F5F4888514888514C91566EAA77E8F2E7FEFEFEFEFEFFD4EED964BA72\
40814F070C072C5331639D63585858F6F6F7FEFEFFF5F5F6606060000000000000
000000000000000000010101787879B9B9B91A1A1A0000003838388F8F8FCFCFD0FEFEFFFEFEFF\
DDDDDE1E1E1E0000000202021616161B18189B9898FEFEFFFEFEFF8A837F33261E
33261E2C221A130D060705012F2F2EEDEDEEFEFEFFCCCCCD383838ADADAD9796977674748E8C8B\
EFEEEFFEFEFFF8F7F8948D882215081D1206170E050C0903030401000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000090507FFFFD8744F3C
00000020151033231B402C20402C20402C20402C20402C20402C20402C20402C20402C20402C20\
402C20402C204A312565453391634AF8AB81FFFFFFFFFFFFFFFFFFF8FFFFF3FFFF
F3FFFFF8FFFFFFFFFFDCFFFF35403100000000020233616171D3D391FFFF98FFFF91FFFF88FFFF\
81F1F178E1E15FB4B44A8B8B254747091010000000091212304F4F4E968F76C2CC
7BB0C083BBCA76A9B75F8A944E6F783A535F2C40452331371E2C301E2C301B272A1720231B272A\
2331373E5A6173A4B2B5FFD381E68A3C8648316C3A3776403C86485A94A99BC0CA
E1CC96F3DEA2C0E59BC0E59B96D174A6A6836C5E4025201517140C352C205A7F4773CC7F71D781\
61B96F539F5F4C915648885143814E4888514F985C61B96F71D781ADF186F3DEA2
FFF3A2DCFFB281E38F66C37665BE736AC77895DD97D4F3D0F0FAEFF0FAF2F0FAF3EDFEF9BAFEE1\
64AFAF00020269979F80919AB9B9B9E5E5E6E5E5E6E5E5E6B0B0B0353536000000
000000000000242424616162C0C0C1E2E2E3989898303030030303202020555555B8B8B8F1F1F2\
DDDDDE1E1E1E0000000E0E0E8B8B8BCECDCEECECEDFEFEFFFEFEFFDBDBDBC3C2C1
C3C2C1A3A19F1715100504012F2F2EEDEDEEFEFEFFCCCCCD0D0D0D2C2C2D99999AE5E5E6FCFCFD\
FEFEFFF7F7F89695921C19100F0C030E0B030B0903070601020200000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000020202FFFFD8835A43000000
20151033231B402C20402C20402C20402C20402C20402C20402C20402C20402C20402C20402C20\
402C20402C20402C204A31256545339F6D51FFC091FDFFFFFFFFFFFFFFFFF3FFFF
F8FFFFFFFFFFFFFFFFA2FFF80B0E140509092547475FB4B488FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF91FFFF88FFFF78E1E168C2C25199993361611B3131050C0C000000
172C2C45737B6CB0B983C3DC83BBCA7AB0BE6896A251747F3E5A6131474C27393C1E2C301E2C30\
1E2C302C404551747F8BC3D1B7FFC35FD07141914E3C864855B963A2E8FF7FDF8D
71D7816DCE7B6AC7786AC7786DCE7B83E691C7ECA0B9996C564C314C412CA99463A9D0A97BDC8A\
539F5F4078483C714340784843814E4888514F985C61B96F6FD17D6DD07D65B26F
719D5676A65C58A7655FB46C58BC6561C39260A47F68A79A72A7A788C9C9BEFFFFEAFFFFA0FFFF\
080A0A19272C0D0D0D272727272727262626262626262626272727171717000000
000000000000101010272727272727262626262626151515000000000000000000111111585858\
7C7C7C1010100000000000001313136B6B6BE9E9EAFEFEFFFEFEFF8585842A2A26
2A2A262424210707040202002E2E2EEDEDEEFEFEFFCCCCCD0D0D0D0000000808083A3A3A8A8A8A\
8F8F8F78787712120E050601050601050501040501030301010100000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000070405FFFFC3CC8D6A000000120C09
2C1E1733231B402C20402C20402C20402C20402C20402C20402C20402C20402C20402C20402C20\
402C20402C20402C20402C204A3125654533BB815FFFFFC0FFFFFFFFFFFFFDFFFF
FDFFFFFFFFFFFFFFFF9FBC9835302325474751999981F1F198FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF98FFFF98FFFF98FFFF91FFFF88FFFF81F1F178E1E15FB4B44A8B8B
2547470910100000000E1B1B395C655FB5B57BBBD181B7C77DB5C36F9FAD567D8843616A354C53\
2C40452C40453E5A616896A2ADE6CA84FF9B5FD07155B96381F6CC9DFFFF66C376
4C91564078484078484078484888515CAD687FE18DADD5ADC0AE8176684586744EE1CC96B9FF8F\
6AC7785EB26C5EB26C65BE736CCC7A71D78173D8836DD07D5CAE684A94612C5A43
142017091010070C0C0407090000020000000000000407092A4A5392FFFFE6FFFF3C7373040505\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000505056A6A6AEAEAEBFEFEFF6C6C6B010200
0102000102000101000001002D2D2DEDEDEEFEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000100010100010100010100010100010100010100000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000222C2C3A2C20FFFFD3744F3C040202000000
050402120C0919120C2C1E172C1E1733231B33231B402C20402C20402C20402C20402C20402C20\
402C20402C20402C20402C204A3125583C2E714E3AC98B68FFFFE3FFFFFFFFFFFF
FFFFFFFFFFFFF1FFFF986F53416C6C51999978E1E191FFFF98FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF91FFFF88FFFF
78E1E168C2C25199992E5555102020000000070E0E30554F4E94947AC9D37DB2C281B9C976A9B7\
5F8A9451747F4E6F785C838F78ADBB8FFFD57AFF8F76FF8AADFFFF6FDF8B4F985C
37683E2E5835335F3A3C714343814E56A66366C37673D88368B9746A6A534C412C5E5E4A53925C\
63BB6F66C37663BB6F66CE9951A47A417F7F33655C1C3030070E10000000091014
1C313930555F37616D3E6D7A30555F12202300000040717DD0FFFF83D5F4000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000060606707071F4F4F56C6C6C000000
0000000000000000000000002D2D2EEDEDEEFEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000FFB28B000000FFE6ADFFFFD0FFC392AE785A
5C40302217120E09070000000202000C090519120C2015102015102C1E172C1E1733231B33231B\
402C20402C20402C20402C20402C20402C204A31257D5641DAA078F1FFFFFFFFFF
FFFFFFFFFFFFFAFFFA99A9885FB4B471D3D388FFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF98FFFF98FFFF9FFFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF
91FFFF88FFFF81F1F178E1E15FB4B44A8B8B336161172A2A0205050204041C3535558B9876C2CC\
88CCE683BBCA7BB2C273A4B27AB0BE91CAD88FFFA988FFA09DFFFF6AC7784F985C
4888514C915656A66365BE736DCE7B73D88371D5815FB46C376A40102217020402000000040707\
0E17170E1717070E0E0000000202020E191B1C313530555C457A835EA6B266B4C0
78D3E378D3E35EA6B2376168070C0E234047B9FFFF99FAFF020405000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000171717C2C2C36C6C6C000000
0000000000000000000000002F2F2FEDEDEEFEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000B100B04070745454899B574
F8FFDCFFFFD8FFFFBBCAC08AA26F534C3527140C09000000000000000000020200050402120C09\
19120C2C1E172C1E1733231B33231B33231B33231B3A2C20765641F6D188F1FFFF
FFFFFFFFFFFFDCFDF181F1F178E1E188FFFF91FFFF98FFFF98FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF98FFFF98FFFF98FFFF91FFFF91FFFF88FFFF88FFFF81F1F181F1F1
78E1E178E1E171D3D368C2C258A6A658A6A6519999519999417A7A2E55550C1919000000122020\
437F7F7ACCCC88FFFF99FDFF83BBCA81B9C986FFC9A0FFFD73E6916DCE7B6AC778
6DCE7B73D8836DD07D78EFB461C3924E94943A7168172727000000091012233E41376168376168\
376168376168457A835699A466B4C078D3E381E1F18AF1FF91FFFF99FFFF8AE8FF
78CCE34F8694121E220B1214A4FFFFA4FFFF05090B000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000000006262635D5D5D000000
0000000000000000000000002D2D2EEDEDEEFEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0B0E0E0000000C1415416A7183FDF1ABFFFFF6FFFFC2F6E18AB0A07A74534541302220170C0907\
0000000000000504020C09050C09050C09050B0905100C09291E17765641E3DAA4
E3FFFFEAFFFF9FFFFF91FFFF88FFFF91FFFF91FFFF91FFFF91FFFF98FFFF98FFFF98FFFF98FFFF\
98FFFF98FFFF98FFFF91FFFF81F1F178E1E168C2C25FB4B4519999417A7A336161
2E55552547471B3131172A2A0C19190C1919172A2A2547473361612E55550C1919000000091212\
274A4A539D9D8AFFFFBBFFFFCCFFFFC0FFFF92F1FD92FFC98FFFD791FFD89FFFFF
B2FFFFA0FFFF68B7C331585F0E1B1C0000000000000E191B29474C457A8366B4C078D3E381E1F1\
8AF1FF8AF1FF91F6FF99FFFF99FFFFA2FFFFA2FFFF99FFFF91F6FF78CCE34F8694
1C30350000005FA4B5BBFFFF1B2731000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000101010555555\
5555551010100000000000000000000000000000000000000707070F0F0F000000
0000000000000808080808085C5C5CF2F2F3FEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000070C07020202253E41539F9F8AFFFDABFFFFA6FFFF8BFFFF\
61BCB43A736D29332E1015140B0B0705050404040200000000000017120C765641
B0E1B0B7FFFD98FFFF91FFFF88FFFF88FFFF88FFFF91FFFF88FFFF91FFFF88FFFF88FFFF88FFFF\
88FFFF81F1F178E1E171D3D36AA08F47654E27392C101712000000000000090C09
0E17171727271C3933294F4833533C33533C202E22090B0900000000000010151033533C4C9973\
61C3926CD88F78F1A06FED7F71F3816DEA7D73E6988BFFC08AB2FA9FFFFF9FFFFF
C0FFFF96F6FF68B0C339656C122023040707000000040505121E2229454C3E6A745694A466AEC0\
6FBCD078CCE381DAF18AE8FF91F6FF91F6FF8AE8FF6FBCD0457683121E22020404
5898A7C0FFFF5C83980000001B2323000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000686869F7F7F8\
F7F7F86A6A6A000000000000000000000000000000000000000000000000000000
00000000000050505089898ADEDEDFFEFEFFFEFEFFCCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000C12120405040407072A474A\
6CD1C79FFFFFADFFFF98FFFF81F6EC61BCB4488B842A534F172C2A020505070909
4E655AB0D199D1E3A4D1E3A4C5D79BC5D79BC5D79BD7CE9BD7CE9BD7CE9BDAB978DAB978DAB978\
BCA068AE945F76654153472E1E1910000000100E093A372766784E719D5665C984
5FCC6C65D8736AE3786DEA7D6FED7F6FED7F66DC7684B9668D946A9B946A8AC26C6CE67B73F483\
71F1816FED7D6DEA7B6DEA7B73F4836DEA7D7FFFAE7AFFA48FECF891BBFF8BB5FB
8DB7FF91BBFF8DB5FD7FA4E676C2CC5F9DA6558A9D406871253C41122023040707000000040505\
0E171B1C303530535C3E6A7445768345768330535C070B0C0E191C7AD0E6CAFFFF
4E6F81020202202339664EA9304855000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000767677FBFBFD\
FBFBFD777778000000000000000000000000000000000000000000000000000000
0000000000000808082C2C2D767677D2D2D3F9F9FACCCCCD0D0D0D000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000C150C000000142022417B7B7FF4E8A4FFFFABFFFF94FFFF5CB0A90E1B19
0E1C198DB48DE8C581E8C581DAB978DAB978CCAE71CCAE71BCA068BCA068BCA068BCA068AE945F\
86734A3A312005040204040235312066784E84B96676EC9B71F38173F4836DEA7B
6AE17866DC7665D77361D06F61D06F61D06F6AE17873F483B7FF8FE3FFB773F48366DC765ECA6C\
56BB6353B45F56BB6365D77373F48388DFEC5CC083489D5F71F6848FD7F384ADEF
6D8FC76A8AC06D8FC77FA4E584ADEF8BB5FB91BBFF91BBFF8DB5FD86ADF1789BD86CB0B9558A9D\
406871253C41192A30070C0E020404070C0E2A474FA4FFFFC3FFFF273941040205
151E1E293535000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000262626989898\
989898262626000000000000000000000000000000000000000000000000000000
0000000000000000000000000000001F1F1F7B7B7B858586080808000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000E1414050B05020404304E5392FFFF84FFF4
00000065B2ADF1E8AEF6D188DAB978BCA068AE945F947D5186734A86734AA08858A08858947D51\
453A25000000252C1C6A94516FDF9471F38171F1816AE17861D06F56BB6353B45F
4FAD5C4CA4564CA4564CA4564899514FAD5C53B45F5CC36865D77365D7735CC3684FAD5C4CA456\
4899514CA4565CC3686DEA7B7BF6A473BCCC25513120452A489D5F7FFFC08BC3FB
6A8AC04F68914C61884C61884F6891556F9B5F7DAE6583B76A8AC07396D07FA4E588B0F48FB9FF\
91BBFF8DB5FD86ADF17BA0DF84D8E592F1FDB2FFFF17222C050707000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000558655
2239395CABABFDFFDCFFEA98DAB978AE945F86734A453A251C19101E191053472E766541766541\
2523150405043778406CE67B6FEF7F61D06F56BB634FAD5C4CA456489951489951
43914E43914E43914E43914E43914E43914E4899514CA4564CA4564CA4564CA45648995143914E\
48995153B45F66DC7671F1817DCCEA2A5A3C07120B07120B20452A68E1928DD3ED
7FB2E55A74A4455A7D4053743A4C6A3A4C6A405374405374405374455A7D4C61884F68915A74A4\
6583B76A8AC07396D07699D77396D04C6188000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
40787476D8D8D0FFFFFFFFD0E8C581A088583A312000000010120C0B0C0700000029231753472E\
30291B000000557B4E6FED7F6AE17853B45F4CA45648995143914E43914E43914E
43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E489951\
4FAD5C61D06F73F48376E1E1304851050C0700000000000007120B3068437AFFA7
91CAFF6D99C74F71914053743A4C6A3747633747633747633747633747633747633A4C6A405374\
405374405374455A7D455A7D31405A05070B223140000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000335A5A
1B3131A4FFFFEAFFFFFFFFFFFFDE9186734A02020035402945924E4E9D68434F330000001E1910\
30291B0202004C452CA0EC946DEA7B53B45F48995143914E43914E43914E43914E
43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E43914E4CA456\
5CC3686FEF7F6FDF943C587D00000002050400020200000002040212291B55B778
96DAF684BBEF5F88AE4C6A884053743A4C6A374763374763374763374763374763374763374763\
37476337476337476331405A1E27370000003C5671000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000000000000000000000000000000000000000083D7E3
000000D0FFFFF4FFFFEAFFFFFFF49F5E4F3305050476AE6D6FED7F73F4836FDF9463684A000000\
100E090B090519150EAECE8471F18156BB6348995143914E43914E43914E43914E
48995148995148995148995143914E43914E43914E43914E43914E43914E43914E48995153B45F\
6CE67A6DEA7D5C8ABE0204041520152030200B170E000202000000050C07255135
71F68491CAFF76A7D75A7FA44C6A88405374405374374763374763374763374763374763374763\
37476337476337476331405A19202E000000476684000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000D8FFFF
1E3939EAFFFFF4FFFFF1FFFFFDFFDC3A312035372766DC766FED7D66DC7671F18173E698536340\
0705040000002A2317AECE8471F18156BB6348995143914E43914E43914E489951
4CA4564FAD5C4CA4564CA45648995143914E43914E43914E43914E43914E4899514FAD5C65D773\
73F4838AC3F417223109100B4A6D4A537A532541300710090000000000000E1E14
489D668BE6F38BC3FB6D99C755789B4C6A8845617D4053743A4C6A374763374763374763374763\
37476337476337476331405A19202E0000004A6C8A000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000B2FFFF
5CABABF4FFFFF1FFFFF4FFFFC3FFFF30291B53634071F18161D06F53B45F5ECA6C6FEF7F66DC76\
787F5A3933226C6340A0EC946DEA7B53B45F48995143914E43914E43914E4CA456
53B45F68D08A5CC36853B45F4CA45643914E43914E43914E43914E43914E4CA4565CC3686FEF7F\
84D8E53C587D00000037533771A6717AB47A476D47122719020504000000020704
20452C5FD0718FD5F183B7EC658FB74F71914C6A8845617D4053743A4C6A374763374763374763\
37476337476337476331405A151C270204045C68A2000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000000000000000405075EAEAE
99FFFFF4FFFFEDFFFFF4FFFFD0FFFF1B271E376D4873F4835CC3684CA4564FAD5C5CC36871F181\
73E698B7C08ACED79971F18166DC764FAD5C43914E43914E43914E4899514FAD5C
5ECA6CADFD9FD8FFB061D06F4FAD5C48995143914E43914E43914E48995156BB636CE67A7BF6A4\
5C8ABE070B100C15154E764E74AE7484C284689968254130071009000000000000
0B191035744C7FFFAE91CAFF76A7D75A7FA44F71914C6A8845617D405374405374374763374763\
3747633747633747632A394E12172005070B304A56000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000437878071010
CAFFFFF1FFFFEAFFFFF1FFFFE1FFFF335C58234C2973F4835CC3684899514899514FAD5C61D06F\
73F483C0FFB0FFFFD071F1815CC3684CA45643914E43914E43914E48995153B45F
6AE178BBFFADBBFFAD6DEA7B56BB634CA45643914E43914E4899514FAD5C65D77373F4838AC3F4\
1C2A3C0204053A553A5A865A58997171A6716899683E5E3E0C1C12000202000000
02040212291B55B77896DAF688BEF46A94C055789B4F71914F719145617D45617D4053743A4C6A\
3747633747633747632A394E0E141B0B0E15314E5A000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000D1FFFF0E1C1C
EAFFFFEDFFFFE6FFFFEDFFFFEFFFFF6DC9C90C1B126AE37865D7734CA45643914E48995153B45F\
65D773B7FFA9C7FFB766DC7653B45F48995143914E43914E43914E4CA4565CC368
71F181E6FFB0FFFFC983FFAD61D06F4CA4564899514899514CA4565CC36871F18184D8E53C587D\
0000001B2C2C5E8B5E6AA06A4A815E4171534E764E4161411B2E22050B07000000
000000050C0725513571EC9991CAFF7BADDE5F88AE55789B4F71914F71914C6A8845617D405374\
3A4C6A3747633747632A394E0E141B0B0E15334F5C000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000CCFFFF488888
F1FFFFEAFFFFE3FFFFEAFFFFF4FFFFB4FFFF020505489B536FEF7F56BB634899514899514CA456\
56BB635ECA6C6FDE925CC3684CA45648995143914E43914E43914E4FAD5C66DC76
71F181D7E1A0EAD78AB5FD8D6AE1784FAD5C48995148995156BB636DEA7B7BF6A46394CC090E15\
090C12537A537AB47A74B474417153234F332541302E452E203020071009000202
0000000204020E1E14489D668BE6F38BC3FB6D99C75A7FA455789B4F71914F71914C6A8845617D\
4053744053743747632A394E0E141B0B0E1535515F000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
00000000000000000000000000000000000000000000000000000078DCDC88FBFB
F4FFFFE6FFFFE3FFFFE6FFFFF1FFFFE6FFFF417A7A1B35276DEA7D61D06F4CA45643914E489951\
4CA4564FAD5C4FAD5C4CA45648995143914E43914E43914E48995153B45F6DEA7B
A0EC949D8F5C918455A0DF7B6DEA7B56BB634CA45653B45F66DC7671F38181C0FF223147000000\
2540407AB47A99E1998FD38F4E764E152E1E0C1C1217271C1B291B0C1710000202
00000000000002070420452C65DC768FD5F183B7EC658FB755789B4F71914F71914F71914F7191\
45617D45617D40537431405A1217200B0E1535515F000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000000002A4C4C1B3131C3FFFF
F3FFFFE6FFFFE3FFFFE3FFFFEAFFFFF4FFFFADFFFF0000004EA6586FEF7F53B45F48995143914E\
48995148995148995148995143914E43914E43914E43914E48995156BB6371F181
B5D78A5A53355A53358ACC8173F4835ECA6C56BB6365D77373F4837AE8E83C587D0204050C141C\
4E765699E199ADFFAD99E1995A865A152E1E071009050B0709100B0B100B000202
0000000000000000000B191035744C7FFFAE91CAFF76A7D75A7FA455789B4F71914F71914F7191\
4F71914C6A8845617D3747631217200B0E1535515F000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000101010101010101010101010101010000000000009BFFFF020404E1FFFF
EDFFFFE3FFFFE3FFFFE3FFFFE6FFFFF1FFFFE1FFFF417A7A1B35276FED7F61D06F4CA45643914E\
43914E43914E43914E43914E43914E43914E43914E43914E4CA4565ECA6C73F483
8BA46A1C1B121C1B1263925C71F1816AE17866EC7671FF8173E6985A84BC0E151E000204254040\
7AB47AA2F1A2B7FFB7A2F1A26AA06A203929050B07000202000202000202000202
00000000000000000002070412291B55B77896DAF688BEF46A94C055789B4F71914F71914F7191\
4F71914F71914C6A883A536A151C270B0E1535515F000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000010101020202030303030303030303030303030303030303\
020202010101010101000000000000000000000000000000000000000000000000
000000010101020202030303030303030303030303030303020202010101000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000010101020202030303040404\
050505060606060606060606060606050505040404030303D4FFFF346060EFFFFF
EAFFFFE3FFFFE3FFFFE3FFFFE3FFFFEAFFFFF4FFFFADFFFF0000004EA6586FEF7F53B45F489951\
43914E43914E43914E43914E43914E43914E43914E4899514FAD5C65D7736FED7F
5E6F4804040200000045413058C96573FF8373FF8378F1A06CA0DE1C2A3C000000101923476A4E\
88D38FA4FFADB7FFB7ADFFAD7AB47A2E4F3A071009000000000000000000000000
000000000000000000000000050C0725513571EC9991CAFF7BADDE5F88AE55789B4F71914F7191\
4F71914F71914F7191405A74151E270B101535535F000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000202020303030606060808080909090A0A0A0A0A0A0A0A0A090909\
060606040404030303010101000000000000000000000000000000000000000000
010101030303050505070707080808090909090909070707060606030303020202010101000000\
000000000000000000000000000000010101030303030303030303030303030303
0303030101010000000000000000000000000000000101010303030404040606060808080A0A0A\
0B0B0B0C0C0C0C0C0C0C0C0C0C0C0C0B0B0B0A0A0A09090995FFFF79D9D9F4FFFF
E6FFFFE3FFFFE3FFFFE3FFFFE3FFFFE6FFFFF1FFFFE6FFFF4E88881B35276DEA7D61D06F4CA456\
43914E43914E43914E4899514899514899514899514CA45656BB636FED7D6FDF94
4541300000000B09050404024C5A3A55C2615CD16873A4CC273A53020405020407203737639463\
91E191AEFFB7B7FFC2A4FFAD8FD38F476A470C1C12000000000000000000000000
0000000000000000000000000204020E1E14489D668BE6F38BC3FB6D99C75A7FA455789B4F7191\
4F71914F71914F7191405A74151E27121B22293737000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000303030505050909090D0D0D1010101212121212121212120F0F0F\
0B0B0B070707040404020202000000000000000000000000000000000000000000
0202020505050808080B0B0B0D0D0D0E0E0E0E0E0E0C0C0C090909060606030303010101000000\
000000000000000000000000010101030303060606080808080808080808070707
0606060303030000000000000000000000000202020404040707070A0A0A0D0D0D0F0F0F111111\
1111111010100F0F0F0F0F0F0F0F0F0F0F0F1010102B4343315151B8FFFFF3FFFF
E7FFFFE3FFFFE3FFFFE3FFFFE3FFFFE3FFFFE6FFFFFFFFFFC0FFFF0000004EA6586FEF7F56BB63\
4CA4564899514899514899514CA4564CA4564CA45653C05F61DE6F73FF838AC37D
17150E0E0E092C291B0B09050705043540292C5A43273A53070B100000021019233E5E4574AE74\
91E191A4FFADAEFFB7AEFFB791E1994E764E122719020504000000000000000000
0000000000000000000000000000000207041B3A2568D8928DD3ED83B7EC658FB755789B4F7191\
4F71914F71914F7191405A74121920202C39000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000101010303030606060C0C0C1212121616161919191A1A1A181818151515\
0F0F0F090909040404020202000000000000000000000000000000000000000000
0303030606060A0A0A0E0E0E1111111212121212120F0F0F0C0C0C070707030303010101000000\
0000000000000000000000000303030707070C0C0C0E0E0E0E0E0E0E0E0E0C0C0C
0909090505050101010000000101010303030606060A0A0A0E0E0E121212151515151515151515\
1414141212121010100F0F0F1010101212121414148AD4DF151515DAFFFFF2FFFF
E4FFFFE4FFFFE3FFFFE3FFFFE3FFFFDCFFFFF1FFFFFAFFFFF8FFFF5A9B9B1B35276DEA7D65D773\
4FAD5C4CA4564CA4564FB75C53C05F53C05F53C05F5CD0686AEF7871FF817B8A6F
0000002C291B4C472E3731200B0905000000070B0E04050900000005090E2037374E764E74AE74\
91E191A4FFADAEFFB7AEFFB791E199639968203929050B07000000000000000000
00000000000000000000000000000000000007120B35744C7AE8E891CAFF73A2D05A7FA455789B\
4F71914F71914F7191405A74121920233141000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000101010303030707070E0E0E1616161C1C1C2020202020201E1E1E1A1A1A\
1212120B0B0B050505020202000000000000000000000000000000000000000000
0303030606060C0C0C1212121515151515151515151212120F0F0F090909030303010101000000\
0000000000000000000202020606060C0C0C121212151515161616151515121212
0E0E0E0707070202020101010303030606060C0C0C1111111515151919191B1B1B1B1B1B181818\
151515121212111111101010111111141414181818DCFFFF364E4EEFFFFFEFFFFF
E5FFFFE4FFFFE4FFFFE3FFFFDCFFFFECFFFFE6FFFFF1FFFFFFFFFFC0FFFF0000004EA65871F181\
5ED76C53C05F53C05F53C05F56C76356C76356C7635ED76C6CF47A6CF67B536340
0000002C291B564F33403A251C19100002020000000000000002041019233E5E456394637DBC7D\
91DA91A4FFADAEFFB7AEFFB7A4FFAD6AA6712E4F3A071009000000000000000000
00000000000000000000000000000000000002040212291B4EA96F86C9E388BEF46A94C05A7FA4\
4F71914F71914F7191405A740E151B273948000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000303030808081010101A1A1A2222222626262727272525251F1F1F\
1616160C0C0C050505020202000000000000000000000000000000000000000000
0303030707070E0E0E141414171717181818181818151515101010090909030303010101000000\
0000000000000202020505050B0B0B1313131B1B1B1E1E1E1E1E1E1D1D1D181818
1212120909090303030303030606060C0C0C1212121818181D1D1D2020202020201E1E1E1A1A1A\
1515151212121010100F0F0F1010101515151B1B1BB5FFFF79C2C2F5FFFFECFFFF
E6FFFFE5FFFFE4FFFFDCFFFFECFFFFE6FFFFE6FFFFECFFFFFDFFFFF8FFFF4E88881730206CF67B\
6CF47A5CD06856C76356C76356C76356C7635CD06861DE6F6FFF7F5FD86C3C4135
020200403A25564F33403A2515191902040700000000000205090E2037374E764E6AA06A7DBC7D\
91DA91A4FFA4AEFFB7AEFFB7A4FFAD88D38F476A470C1C12000000000000000000
00000000000000000000000000000000000000000002070425513592F4B091CAFF7FB2E55F88AE\
55789B4F71914F71913A536A0E151B2C4051000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000303030808081212121D1D1D2626262A2A2A2B2B2B2A2A2A232323\
1818180D0D0D050505020202000000000000000000000000000000000000000000
0303030707070F0F0F1616161919191A1A1A1919191717171212120A0A0A040404010101000000\
0000000202020505050A0A0A1212121B1B1B2424242727272727272323231C1C1C
1515150B0B0B0505050505050B0B0B1212121A1A1A2020202424242525252323231F1F1F191919\
1414141010100E0E0E0C0C0C0E0E0E141414272E275D8F8FB2FFFFF6FFFFE9FFFF
E6FFFFE5FFFFDEFFFFECFFFFE6FFFFE6FFFFE6FFFFE6FFFFF1FFFFFFFFFFB7FFFF00000053A66D\
73FF8365E5735CD06856C76356C76356C7635CD06865E57373FF8384C37A1C1910
0E0E09403A25403A252A291B090C1400000200000000000005090E273A2C4E764E6AA06A7DBC7D\
88CC889BF19BA4FFADAEFFB7A4FFAD88D38F5A865A152E1E020504000000000000
0000000000000000000000000000000000000000000000000B19103A815388DFEC8DC7FF6D99C7\
5A7FA44F71914C6A883A536A0B101431455A000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000101010707071212121E1E1E2828282D2D2D2D2D2D2C2C2C252525\
1919190C0C0C040404000000000000000000000000000000000000000000000000
0101010606060F0F0F1515151919191A1A1A191919171717121212090909030303000000000000\
0101010404040909091010101818182323232C2C2C2F2F2F2D2D2D2727271F1F1F
1616160C0C0C0606060909091010101919192121212727272A2A2A2929292424241D1D1D151515\
0F0F0F0A0A0A0808080707070909091010105B8D8D262828D8FFFFF3FFFFEAFFFF
E7FFFFDFFFFFEDFFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFECFFFFFAFFFFF4FFFF477A7A173020\
6AF1786CF47A5ED76C56C76356C76356C7635ED76C6AEF7871FF819BAB8B0C0B07
1C19103731202C291B0C1419000204000000040509090E15020404020202273A274E764E639463\
7DBC7D91DA91A4FFADAEFFB7AEFFB791E199639968203929050B07000000000000
0000000000000000000000000000000000000000000000000207041731207ACC948DD3ED83B7EC\
658FB755789B4C6A883A536A090C10314853000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000010101010101
0202020202020202020202020202020808081212121E1E1E2828282C2C2C2C2C2C2A2A2A242424\
1818180B0B0B030303000000000000010101010101020202020202020202020202
0202020505050D0D0D1414141616161717171717171515150F0F0F070707020202000000010101\
0303030808080F0F0F1616161F1F1F2929293131313333332F2F2F282828202020
1717170E0E0E0909090E0E0E1818182424242B2B2B3232323434343030302828281D1D1D141414\
0B0B0B0606060202020202020404040C0C0CDBFFFF303C3CEDFFFFF0FFFFE8FFFF
E1FFFFEEFFFFE7FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFF1FFFFFFFFFFB7FFFF000000\
459D4E73FF8365E5735CD06856C76356C7635ED76C6AEF7871FF819BAB8B0C0B07
0E0E092523171519190204070000001722315A84BCA0ECFFC9FFDE7FBE7F223322000000141E14\
3E5E3E6AA06A88D38F9BF1A2AEFFB79BF1A26AA6712E4F3A071009000000000000
000000000000000000000000000000000000000000000000000000050C073068437AE8E891CAFF\
73A2D05A7FA44F7191374C630405073A5561000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000010101030303050505080808
0B0B0B0D0D0D0F0F0F0F0F0F0E0E0E1111111717172222222929292C2C2C2B2B2B292929222222\
1616160A0A0A0202020101010303030505050808080A0A0A0C0C0C0D0D0D0D0D0D
0B0B0B0C0C0C1212121515151717171515151515151313130E0E0E060606010101000000060606\
0B0B0B1111111919192121212828282F2F2F3434343434342E2E2E2727271E1E1E
1717171111111111111919192727273333333A3A3A3D3D3D3C3C3C3636362C2C2C202020171717\
1010100B0B0B0707070505050404040B0B0BD1FFFF609898F6FFFFEDFFFFE8FFFF
E1FFFFEEFFFFE7FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFECFFFFFDFFFFF8FFFF4E8888\
152C2265E6736FFB7D61DE6F5CD06856C7635CD06865E57373FF838FD384403A25
0000000909070204070000001C2A3C92D7FFD1FFFF293A4C0B1215475E53C7FFEFE6FFE6659865\
0B120B0405042E472E63996888D38F88D38F74B47A3E5E3E0B170E000000000000
00000000000000000000000000000000000000000000000000000002040212291B6AB78688CCE6\
84BBEF658FB74F7191374C6302020439516A000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000480000840000B00000B50000B50000B50000AD0000
8100003B0000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000101010303030707070C0C0C
1212121717171919191919191818181A1A1A1D1D1D2525252A2A2A2C2C2C2C2C2C2A2A2A232323\
1818180C0C0C0303030202020303030606060B0B0B101010121212131313131313
1111111010101414141616161717171515151515151313130E0E0E0606060101010505050E0E0E\
1515151D1D1D2525252C2C2C3030303434343333333232322C2C2C2424241E1E1E
1A1A1A1919191C1C1C2727273434343E3E3E4343434242423C3C3C3333332929292020201A1A1A\
1717171313131010100D0D0D0B0B0B0F0F0F85DFDFA0FFFFF6FFFFEAFFFFE2FFFF
EFFFFFE9FFFFE7FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFF1FFFFFFFFFFC0FFFF\
070B07377F4071FF816AEF785ED76C5CD0685CD06861DE6F6FFB7D6AF1786F8355
15150C0000000000021C2A3C99E1FFB9FFFF000000000000000000000000000000000000619F71\
FFFFFFC9FFC9486C480000001420153E61414E7A532E452E0B170E000202000000
000000000000000000000000000000000000000000000000000000000000050C072A5E3C8AE1A0\
91CAFF76A7D755789B31455A0000003C5671000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000030000100000940000F90000FF0000F60000800000
0C0000020000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000020202030303080808101010
1818181F1F1F2222222323232222222121212222222727272A2A2A2D2D2D2D2D2D2B2B2B262626\
1B1B1B0F0F0F0808080707070808080B0B0B111111161616181818181818181818
1616161414141515151616161717171515151515151313130F0F0F0707070505050D0D0D1A1A1A\
2323232C2C2C3232323636363636363434343030302D2D2D2727272222221F1F1F
2121212424242929293535354040404646464848484343433B3B3B3030302525251F1F1F1D1D1D\
1D1D1D1C1C1C1919191616161313133F61612C3C3CCBFFFFF3FFFFE8FFFFEFFFFF
EAFFFFE9FFFFE7FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFE6FFFFECFFFFF1FFFFFFFFFFECFFFF\
477A7A1E201B65C98473FF8366EC765ED76C5ED76C61DE6F6DF87B71FF8153A66D
25374C0E151E273A5399E1FFB2FFFF000000000000000000000000000000000000000000000000\
0000000204026FB481FFFFFF9DF3A6223523000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000E1E14489D66\
8FD0EC8AC2F8658FB7374C630000003C5671000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000002A0000E80000FF0000DA00001C0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000020202030303080808131313
1E1E1E2828282B2B2B2C2C2C2A2A2A2828282626262828282A2A2A2E2E2E2E2E2E2D2D2D292929\
1F1F1F1515150F0F0F0D0D0D0F0F0F1212121919191F1F1F1F1F1F1F1F1F1C1C1C
1A1A1A1818181616161616161717171515151515151313130F0F0F0909090D0D0D181818292929\
3333333A3A3A3F3F3F4040403A3A3A3232322B2B2B272727242424222222252525
2929292E2E2E3535353F3F3F4949494B4B4B4848484343433939392E2E2E2424241E1E1E202020\
2525252525252222221E1E1E1C1C1CD1FFFF242B2BEAFFFFEEFFFFEFFFFFEAFFFF
EAFFFFE9FFFFE8FFFFE7FFFFE6FFFFE6FFFFECFFFFF1FFFFF6FFFFFFFFFFFFFFFFF4FFFFAEFFFF\
203939202C23B7C08A6FFD7F71FF816CF47A6AEF786DF87B73FF8378F1A056AE83
3C587D43638DA7F4FFADFDFF000000000000000000000000000000000000000000000000000000\
000000000000000000000000152025A6FDBBFFFFFFB0FFB953916A20452C12291B
050C0702040200000000000000000000000000000000000000000000000000000002070420452C\
8AE6A68FC9FF7BADDE405A740000023C5671000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000270000E70000FF0000D900001A0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000010101030303090909161616
2323233030303434343434343333333030302B2B2B2A2A2A2A2A2A2E2E2E2F2F2F2F2F2F2A2A2A\
212121181818121212131313151515191919242424282828292929262626232323
2020201C1C1C1616161616161717171515151515151414141111111111111717172626263A3A3A\
4545454A4A4A4A4A4A4646463A3A3A2F2F2F2525252323232222222626262C2C2C
3333333A3A3A4040404848484E4E4E4D4D4D4848484242423838382A2A2A2020201E1E1E242424\
2C2C2C2D2D2D2B2B2B282828262626D0FFFF5B8D8DF6FFFFF8FFFFEAFFFFEAFFFF
EAFFFFE9FFFFEDFFFFF1FFFFF6FFFFFFFFFFFFFFFFFFFFFFECFFFFB7FFFF63ABAB0B1414152525\
8AEFEFBBFFFFFFFFC0B2FFA66FFD7F71FF8171FF817DFAA676EC9B5CB97A6A9BCC
618FCCBBFFFFBBE6C7000200000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000001B2E309FF4B4D8FFFF94FFD1
5CC7832A5E3C0E1E1402070400000000000000000000000000000000000000000000000007120B\
3A815384D8E58BC3FB4F71910202043C5671000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000270000E70000FF0000DA00001A0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000010101030303090909181818
2828283737373B3B3B3C3C3C3A3A3A3636362F2F2F2C2C2C2A2A2A2E2E2E2F2F2F2F2F2F2A2A2A\
2222221919191515151616161B1B1B2020202C2C2C3232323232322F2F2F2B2B2B
2323231E1E1E1717171616161717171515151515151616161717171A1A1A2424243636364B4B4B\
5555555757575252524848483939392B2B2B2121212020202626262D2D2D353535
3C3C3C4242424949494E4E4E5050504D4D4D4848484040403535352727271C1C1C1F1F1F282828\
3333333535353333333131312F2F2F97F2F29AFBFBFFFFFFF3FFFFEFFFFFEFFFFF
F3FFFFFBFFFFFFFFFFFFFFFFF4FFFFC7FFFF7DD8D82039390002025A9B9BC0FFFFA4FFFF335333\
070B0C000000B7A473FFFFCAA9EF9B7DB97471A668839963838A615A84AE7AB4FF
C9FFFFADD5B7020202000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000000000000000202041B2E30
8AFFF4D8FFFF81FFB543915E1B3A2507120B020402000000000000000000000000000000020402\
17312073C7928DD3ED6A94C0040507405C76000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000270000E70000FF0000D900001A0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000010101070707181818
2A2A2A3B3B3B3F3F3F4040403E3E3E3939393030302B2B2B2929292D2D2D2E2E2E2F2F2F2A2A2A\
2222221919191515151717171C1C1C2323233232323939393838383535352F2F2F
2727272020201616161414141616161515151616161B1B1B1F1F1F2626263434344646465A5A5A\
6060605E5E5E5454544646463636362727271C1C1C2121212A2A2A3434343E3E3E
4545454B4B4B4E4E4E4F4F4F4E4E4E4747474040403737372B2B2B1D1D1D131313171717252525\
3535353A3A3A393939383838546C6C445555C5FFFFFFFFFFFBFFFFFFFFFFFFFFFF
FCFFFFE1FFFFA1FFFF3C6363020404396363A4FFFFD8FFFF5F91A405090B000000000000000000\
000000000000000000FFFFA7F6E191484C352523152723173948416A9BDCD0FFFF
88A991050B05000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000477383D0FFFFCCFFFF6DEF9B35744C173120050C07000000000000000000000000\
050C0730684396F3AE83B7EC090C104E588A000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000280000E70000FF0000DA00001A0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000101010606060808080909090909090D0D0D1C1C1C
2E2E2E3E3E3E4141414040403C3C3C3737372C2C2C2727272626262C2C2C2E2E2E2F2F2F2A2A2A\
2323231D1D1D1C1C1C1F1F1F2525252C2C2C3B3B3B4040403F3F3F3A3A3A303030
2727271F1F1F1414141212121515151515151B1B1B2222222C2C2C363636444444595959686868\
6767675F5F5F5252524141413030302323231B1B1B2424243030303E3E3E474747
4F4F4F5555555959595C5C5C5A5A5A5353534D4D4D4141413131311C1C1C0D0D0D1010101F1F1F\
3232323838383A3A3A3A3A3AA9FFFF2F2F2FC9FFFFF5FFFFEEFFFFC8FFFF7CC1C1
2934341F282885DEDED1FFFF84D7E3172517000000000000000000000000000000000000000000\
0000000000000000003A3323FFFFC33933220C0B07191B145A84BCD3FFFF537B58
0709090000000000000000002300002F00002F00002F00002F00002F00002F00002F00002F0000\
2F00002F00002F00002F00002F00002F0000110000000000000000000000000000
0000000000000000000000001D0406787688E2D0D0CED0D080A28A4D422B350F093003021A0402\
050C0720452C83E1A6A5D0EA6D0F14AE16199600003D00000700001300002F0000
2F00002F00002F00002F00002F00002F00001F00000200002500002F00002F00002F00002F0000\
2F00002F00002F00000B00000C00002F00002F00002F00002F00002F00002F0000
2F00002400000100000000000A00004600009C0000A100008A00002B0000020000000000080000\
720000A000005D0000130000000000000000270000E70000FF0000DA0000200000
3A0000940000A00000A000006800001800000000000000000000000000000000000000003B0000\
4100003900008E0000A10000A00000A100007500001D0000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000606061414142020202A2A2A3636363F3F3F4D4D4D
5959595C5C5C5656564F4F4F4343433636362B2B2B2727272626262C2C2C2E2E2E2E2E2E2A2A2A\
2424242727272E2E2E3A3A3A4B4B4B5757576161615E5E5E5757574A4A4A383838
2929292020201414141212121515151919192222222E2E2E3939394747475F5F5F767676767676\
6868685B5B5B4B4B4B3939392929291F1F1F1C1C1C2929293939394C4C4C5E5E5E
6D6D6D7676767B7B7B7D7D7D7777776F6F6F6D6D6D6666665959594646462E2E2E232323262626\
3131313838383A3A3A3A3A3AD0FFFF3A4646435D5D344B4B1919195B8B8BC0FFFF
D9FFFF415E410A0A0A050505020202000000000000000000000000000000000000000000000000\
000000000000000000050707FFFFB2D8C9838B815392D7FFF4FFFF37513A0C1212
0000000000000000000000003B00005100006B0000E10000EF0000EF0000EF0000E50000C20000\
C00000C10000C10000C00000DE0000EF0000BC00009E00005A0000120000000000
0000000000000000000000002C0000610000CE0000F00101F80E0EFE1010EE3535946A5A556140\
43915E61D58B9BEFC4E65350F70101FF0000FD0000EB0000780000290000590000
C00000EF0000EF0000EF0000D80000660000340000030000400000770000E50000EF0000EF0000\
EE0000AB0000500000130000150000510000B10000EF0000EF0000EF0000E40000
7100003E00000100000A00008B0000F00000FE0000FF0000FB0000E000005A0000100000710000\
F40000FF0000F50000AC00001C0000000000280000E70000FF0000F00000B30000
C100006B00006100006F0000D20000C700006300000600000000000000000000000000006D0000\
F10000F00000E90000A30000600000680000B80000C80000720000080000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000707071818182727273F3F3F6969698B8B8B999999
9292927A7A7A6060605353534444443737372B2B2B2727272626262B2B2B2D2D2D2C2C2C272727\
2121212626263030304646466F6F6F8E8E8E8C8C8C7575756161615353533F3F3F
2E2E2E2323231515151313131717172020202C2C2C3B3B3B4C4C4C656565888888979797818181\
6464645252524141413131312525251E1E1E1F1F1F3232324E4E4E6E6E6E8C8C8C
9B9B9B9898988B8B8B7B7B7B6B6B6B6262626363636969696F6F6F7474746E6E6E6060604B4B4B\
3E3E3E3939393A3A3A3A3A3A8CC8C8FBFFFFE3FFFFDAFFFF7EC5CE1B2022131313
1111110D0D0D090909050505020202000000000000000000000000000000000000000000000000\
000000000000000000000000040402FFFDA6FDFFFF4C6F7D000000151E1E000000
000000000000000000000000000000000000040000A50000FE0000FF0000DD0000560000130000\
110000100000110000100000460000780000D30000FC0000F40000A900001A0000
000000000000000000000000000000020000510000EE0000FF0000EB00005D212299FBFBE1FFFF\
C7FFFF73E6ADA0AA6FF40906FF0000F304057B00006C0000BC00003A0000010000
390000E30000FF0000F60000620000030000000000000000000000080000830000FF0000FF0000\
CF00001F0000000000000000000000000000210000D80000FF0000FF00007A0000
050000000000000000490000F40000FF0000FF0000CA0000680000790000C90000A20000B60000\
FF0000FF0000D000007C00008900000C0000270000E70000FF0000FF0000D70000
410000010000000000020000450000DA0000ED00005C00000000000000000000000000006F0000\
FB0000AB0000410000090000000000010000190000BF0000ED0000410000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000303030909090F0F0F262626636363A5A5A5BCBCBC
9E9E9E6D6D6D4B4B4B4444443F3F3F3A3A3A3131312D2D2D2C2C2C2F2F2F2F2F2F2D2D2D262626\
1C1C1C1515151515152929296262629494948C8C8C6666665353534C4C4C3F3F3F
3131312424241717171515151D1D1D2A2A2A3939394A4A4A6565658C8C8CAEAEAEAAAAAA818181\
5B5B5B4747473636362828282020201E1E1E2727274A4A4A787878A0A0A0B2B2B2
A8A8A88686866363634B4B4B3D3D3D3A3A3A3C3C3C4141414B4B4B6767678D8D8D9F9F9F828282\
5555553C3C3C3A3A3A87B38755706E2A2A2A1A1A1A0E0E0E0E0E0E0F0F0F0E0E0E
0C0C0C0A0A0A070707040404010101000000000000000000000000000000000000000000000000\
000000000000000000000000222C2C454835090C0C000000293737000000000000
000000000000000000000000000000000000000000430000FA0000FF0000C70000070000000000\
000000000000000000000000000000030000200000AB0000FC0000FB0000A90000
1A0000000000000000000000000000000000230000E30000FF0000DE00001E00000000000E1519\
152215000000760000FF0000FF0000D02B380F0000010000310000340000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000A80000FE0000FF0000CB00002F0000000000030000210000A30000E90000\
FF0000FF00006D0000040000310000080000270000E70000FF0000E90000460000
020000000000000000000000020000730000F90000C200002600000000000000000000006F0000\
DB00002A0000000000000000000000000000010000670000FD0000A90000080000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000E0E0E4A4A4A9C9C9CBABABA
9292925858583D3D3D3E3E3E3F3F3F3E3E3E3737373636363535353939393838383636362C2C2C\
1F1F1F0F0F0F0606061111114949498484847B7B7B5555554848484646463F3F3F
3333332626261818181A1A1A2626263535354747475F5F5F858585ADADADC2C2C2ACACAC777777\
4D4D4D3A3A3A2B2B2B2121211D1D1D2323233D3D3D757575A8A8A8BCBCBCADADAD
8585855959593838382929292424242828282B2B2B2B2B2B2B2B2B484848898989B7B7B7A0A0A0\
6666664141413C3C3C3B3B3B3737372828281717170B0B0B0A0A0A0B0B0B0B0B0B
0A0A0A090909060606030303000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000001B0000BE0000FF0000FB0000
A200000C0000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000B10000FF0000FF00005D00000000000000000000000000000C0000A60000\
FF0000FF0000650000000000000000000000280000E70000FF0000D900001A0000
000000000000000000000000000000250000E00000FC00007F00000000000000000000006B0000\
8700000300000000000000000000000000000000005F0000FF0000EB00002E0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000B0B0B4545459A9A9ABABABA
8E8E8E5454543C3C3C3F3F3F4040404040403838383737373939393F3F3F4040403E3E3E363636\
2727271717170D0D0D1313134646467F7F7F7878785353534848484646463E3E3E
3333332727271B1B1B1F1F1F3030304343435B5B5B7E7E7EA6A6A6C1C1C1C6C6C6A7A7A76D6D6D\
4040402E2E2E2424241E1E1E1D1D1D323232666666A4A4A4C0C0C0B2B2B2898989
6161614141412A2A2A2020201D1D1D242424272727292929272727434343878787B9B9B9A7A7A7\
6D6D6D4747474141414040403A3A3A2B2B2B1A1A1A1010100D0D0D0C0C0C0C0C0C
0C0C0C090909060606020202000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000000000005B0000F40000FF0000
DB0000230000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000B10000FF0000FF00005300000000000000000000000000000000009F0000\
FF0000FF0000650000000000000000000000270000E70000FF0000D900001A0000
0000000000000000000000000000000C0000A50000FF0000BC00001400000000000000004C0000\
2A0000000000000000000000000000000000160000A70000FF0000F10000320000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000B0B0B4545459A9A9ABABABA
8E8E8E5454543C3C3C3F3F3F4040403F3F3F3939393838383C3C3C4343434646464646463F3F3F\
3232322323231717171B1B1B4A4A4A8181817878785353534747474444443C3C3C
3232322A2A2A2121212A2A2A3C3C3C535353787878A2A2A2BCBCBCBFBFBFB9B9B99E9E9E636363\
3434342525251F1F1F1C1C1C2323234E4E4E919191BFBFBFBDBDBD9797976A6A6A
5050503A3A3A2626261717171515151E1E1E232323262626282828444444898989B9B9B9A7A7A7\
6A6A6A4444443C3C3C3B3B3B3636362828281D1D1D151515131313121212121212
1010100C0C0C070707020202000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000000000002D0000EC0000FF0000
FB0000640000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
0000000000000000006C0000FA0000FF00006600000100000000000000000000000000009F0000\
FF0000FF0000660000000000000000000000280000E70000FF0000DA00001A0000
0000000000000000000000000000000000007A0000FF0000E80000340000000000000000080000\
0300000000000000000000000200002600009F0000FA0000FF0000E600002A0000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000B0B0B4545459A9A9ABABABA
8E8E8E5454543C3C3C3F3F3F4040404040403B3B3B3A3A3A3C3C3C4646464C4C4C4E4E4E4A4A4A\
3E3E3E313131262626272727515151838383787878525252444444404040393939
3232322E2E2E2B2B2B3636364B4B4B6D6D6D999999BBBBBBBBBBBBA3A3A39C9C9C8F8F8F595959\
2B2B2B2020201D1D1D1E1E1E333333707070AEAEAEC5C5C5AEAEAE7D7D7D5A5A5A
4D4D4D3A3A3A2323231313131010101818181E1E1E2222222525254444448A8A8ABABABAA6A6A6\
6868684040403838383636363232322828282121211A1A1A1A1A1A181818171717
1414140F0F0F090909030303000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000000000002C0000EC0000FF0000
FF00006C0000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000190000BE0000FD0000C600001400000000000000000000000000009F0000\
FF0000FF0000650000000000000000000000280000E70000FF0000D900001A0000
0000000000000000000000000000000000007B0000FF0000FC0000800000000000000000000000\
0000000000000000000F0000540000D80000FA0000FF0000FC00009A0000040000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000000000000000000000000B0B0B4545459A9A9ABABABA
8E8E8E5454543C3C3C3F3F3F4141414242423B3B3B3A3A3A3C3C3C4747474F4F4F545454525252\
4949493E3E3E3434343434345858588585857676764D4D4D3F3F3F3B3B3B363636
3434343535353636364545456565658F8F8FB4B4B4BFBFBFA2A2A27979797E7E7E838383515151\
2626261E1E1E1D1D1D2424244A4A4A8F8F8FBDBDBDBEBEBE9999996B6B6B555555
4D4D4D3B3B3B2424241111110C0C0C1212121818181C1C1C2222224343438A8A8AB9B9B9A5A5A5\
6767673C3C3C3434343333332F2F2F2828282424241F1F1F1D1D1D1B1B1B191919
151515101010090909030303000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000000000000000000000680000F60000FF0000
F900005F0000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000000000250000B70000ED00009400002100000100000000000000009F0000\
FF0000FF0000650000000000000000000000270000E70000FF0000D900001A0000
0000000000000000000000000000000000007B0000FF0000FF0000890000000000000000000000\
0000000F00004D0000AE0000EF0000FF0000FF0000F10000B00000220000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
0000000000000000000000000000000000000000000000000000000000000000002222002F2F00\
2F2F002F2F002F2F002F2F002F2F002F2F002F2F003838095050419A9A9ABABABA
8E8E8E5454543C3C3C3E3E3E4141414141413A3A3A4D4D315D5D2E6666376E6E40757546777748\
71714369693B5D5D363F3F3B5C5C5C868686747474484848383838363636343434
3838383E3E3E4040405D5D58ABAB60B8B8A2BFBFBFA8A8A87676765353536C6C6C7C7C7C4C4C4C\
2222221C1C1C1D1D1D2C2C2C6C6C5BB6B688D7D77FE2E242D2D232C3C323BEBE1E
BABA1D71712B4B4B1E1A1A110808080C0C0C1212121616161C1C1C3F3F3F888888B9B9B9A4A4A4\
6464643838382F2F2F2E2E2E2B2B2B2626262323232020201E1E1E1B1B1B181818
1515150F0F0F090909030303000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000000000000000270000C80000FF0000FF0000
CB00001B0000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
000000000000000000000000000000120000590000B50000C600004A00001B00000100009F0000\
FF0000FF0000650000000000000000000000280000E70000FF0000D900001A0000
0000000000000000000000000000000000007B0000FF0000FF0000890000000000000000000000\
1700009D0000EE0000FF0000FF0000FC0000DC0000670000110000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
0000000000000000000000000000000000000000000000000000000000000000003C3C00515100\
6A6A00E1E100EFEF00EFEF00EFEF00B6B60053530058580857573E9A9A9ABABABA
8E8E8E5454543B3B3B3C3C3C3E3E3E3B3B3B3232325454246F6F1E868622DEDE0CF4F405EFEF08\
AEAE298686367A7A354A4A42606060868686707070424242323232333333363636
3C3C3C45454553534EADAD49EFEF1FCACAA5B0B0B07D7D7D4747473838386363637979794B4B4B\
2121211C1C1C33331BA2A218DBDB1EF9F90CF2F224DFDF36A7A746959535929231
A8A824D3D30DE7E703B4B4057F7F032626060C0C0C1010101616163B3B3B878787B7B7B7A2A2A2\
5F5F5F3333332929292828282525252323232121211F1F1F1D1D1D181818151515
1212120C0C0C060606010101000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000060000310000BC0000FC0000FF0000F90000
6B0000020000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000460000
0000000000000000000000000000000000000000001300005A0000A80000C40000760000B10000\
FF0000FF0000660000000000000000000000270000E70000FF0000D900001A0000
0000000000000000000000000000000F0000AE0000FF0000FF00008A00000000000000000B0000\
990000FA0000FF0000FC0000DC00008600002C0000050000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
030300707000FCFC00FFFF00DCDC002929000000000B0B0B4545459A9A9ABABABA
9191915D5D5D4949494D4D4D4D4D4D4949493C3C3C2C2C2C2121212929287C7C24F4F403BABA21\
5D5D4F5151514D4D4D4949496262628585856D6D6D3D3D3D2F2F2F3232323A3A3A
444444595954A8A844F7F70CF8F814C1C19D8A8A8A5050502727272A2A2A5D5D5D7777774A4A4A\
232320626213C3C308FCFC01F2F20CD9D965C1C1A09393885E5E5E4B4B4B4B4B4B
4B4B464A4A386F6F1BDEDE02F9F900D4D4012727050A0A0A101010383838848484B6B6B6A0A0A0\
5B5B5B2C2C2C2222222121212020201F1F1F1F1F1F1C1C1C181818151515121212
0E0E0E090909040404010101000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000001400001F00006E0000E00000FD0000FF0000DB0000750000
140000000000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008D0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
0000000000000000000000001500001F00000A00000000000000001400005B00008A0000E30000\
FF0000FF0000650000000000000000000000280000E70000FF0000D900001A0000
0000000000000000000000000000001D0000DD0000FF0000FB00007D0000000000000000430000\
EB0000FF0000EA00008700002C0000050000000000000000090000050000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000B0B0B4545459A9A9ABCBCBC
9C9C9C7575756C6C6C7171717373737777776E6E6E5959593F3F3F31313154542BEEEE0599992C\
5050505050504E4E4E4A4A4A6262628484846A6A6A3838382C2C2C3434343D3D3D
4F4F4F88885FF0F015FFFF00F7F713A0A07C5858582D2D2D1616162020205959597676764B4B49\
6A6A16E7E703FFFF00ECEC06B6B65DBDBDBBB3B3B3828282505050434343454545
4444443B3B3B31312ACCCC05FFFF00FAFA003D3D020606060F0F0F3C3C3C8A8A8AB7B7B7A0A0A0\
5B5B5B2828281A1A1A1A1A1A1919191919191919191414141212120F0F0F0C0C0C
090909060606030303000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000E200008100007E0000\
7E00007E00007D0000BC0000DF0000F30000FF0000E80000890000410000020000
000000000000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F00000300000000000000000000000000000000000500000FF0000FF0000\
B40000000000000000000000000000000000000000BE0000FF0000FF0000450000
0000000000000000001B0000BD0000DF0000830000040000000000000000000000060000A90000\
FF0000FF0000650000000000000000000000280000E70000FF0000DE0000240000
0000000000000000000000000000004D0000EE0000FF0000E700003000000000000000006F0000\
FF0000EA00006600000500000000000000000000000100005200002B0000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000B0B0B4545459A9A9ABEBEBE
A3A3A37E7E7E6E6E6E7171717676768484849090908F8F8F7F7F7F63636365653DEEEE06969629\
4D4D4D4C4C4C4A4A4A4545455D5D5D8282826666663333332B2B2B373737464646
787858DFDF2DFEFE02FFFF00F5F5107575512E2E2E1515150B0B0B191919575757787875858531\
E6E603FFFF00E9E90388882D92928FBCBCBCB1B1B17A7A7A4646463A3A3A3E3E3E
3F3F3F39393932322CCDCD06FFFF00FAFA004141071313132828285B5B5B999999B7B7B7A6A6A6\
7373733F3F3F2727271F1F1F1C1C1C1919191515150D0D0D0B0B0B090909060606
050505030303010101000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000E40000870000830000\
830000830000830000960000DD0000F40000FF0000E70000730000120000000000
000000000000000000000000000000000000230000E30000FF0000DE00001E0000000000000000\
000000000000760000FF0000FF00008E0000000000000000000000000000000000
110000D10000FF0000F700007000000500000000000000000000000400008A0000FF0000FF0000\
D600002A0000000000000000000000000000070000C50000FF0000FF0000460000
000000000000000000370000F70000FF0000CA00000A00000000000000000000000000009E0000\
FF0000FF0000650000000000000000000000270000E70000FF0000F60000940000
170000000000000000000000190000AF0000FF0000FF0000B700001200000000000000006E0000\
FF00009F0000050000000000000000000000000000150000B70000310000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000B0B0B4545459A9A9ABCBCBC
9999996767674D4D4D4C4C4C4F4F4F5959596A6A6A888888A3A3A39E9E9E8D8D65EFEF07969629\
4747474747474141413C3C3C5757577F7F7F6464643232322D2D2D3E3E3E707050
DADA2AFEFE02FEFE04F9F90AEEEE0B50502B1515150A0A0A030303171717575757878767DEDE0D\
FFFF00FCFC00A0A00E4848458F8F8FBCBCBCAFAFAF7575753A3A3A2E2E2E363636
39393936363633332DCDCD07FFFF00FAFA004949102626263F3F3F6262628080808B8B8B888888\
7474745353533939392626261E1E1E181818111111070707050505030303020202
010101010101000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000060000210000730000E30000FD0000F80000930000120000
0000000000000000000000000F00001000005D0000F00000FF0000DD00001E0000000000000000\
000000000000760000FF0000FF00008D00000000000000000000000B0000120000
440000E60000FF0000F80000AE0000650000110000000000010000380000E20000FF0000FC0000\
A300009D00003000000600000000000600004B0000F60000FF0000E20000260000
0000000000000000001D0000D50000FF0000C900000A0000000000000000000000160000C50000\
FF0000FA00005C00000000000F00001B0000650000F20000FF0000DE0000A00000
A800004700001B00004A0000AE0000FA0000FF0000F10000580000000000000000000000450000\
EC0000930000000000000000000000000000010000600000EA0000310000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000B0B0B4545459B9B9BBABABA
9090905757573C3C3C3E3E3E3D3D3D3A3A3A3A3A3A5C5C5C989898B9B9B9B4B48BF3F3099A9A2D\
4343433E3E3E3737373333335252527E7E7E666666383838353535555551C2C23E
FDFD02FEFE04E1E140C1C132E9E906383813090909020202000000161616595957B3B342FCFC01\
FFFF00E5E5044C4C1C424242878787B8B8B8AEAEAE7171713030302121212A2A2A
30303032323234342ECECE08FFFF00FAFA004B4B111E1E1E2929293232323A3A3A3E3E3E454545\
4A4A4A4343433535352727271E1E1E1818180E0E0E020202000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000060000450000E80000FF0000F800008D0000
0900000000000000000000004500008F0000D80000FF0000FF0000DE00001E0000000000030000\
160000190000840000FF0000FF0000990000190000190000170000350000900000
C70000FF0000FF0000F000004300008B0000B20000750000790000D10000FF0000FF0000F20000\
340000390000BA00008D00007500008F0000E30000FF0000F80000810000010000
000000000000000000000000720000F30000E200003800000100000000000400006C0000F60000\
FF0000CD00001C00000000004A0000B90000E40000FF0000FF0000D900003B0000
B20000E90000DB0000ED0000FF0000FF0000FC0000AE00001200000000000000000000000C0000\
9D0000B900002600000400000000000A0000430000D50000F00000310000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000C0C0C4747479D9D9DBBBBBB
9191915858583F3F3F4141414040403A3A3A2D2D2D3E3E3E7A7A7AB5B5B5C7C79FF6F60CA3A335\
3D3D3D3636362F2F2F2C2C2C5050508080806969693E3E3E464643A6A646F6F610
FFFF00F4F41D9696709A9A23E7E7022D2D080303030000000000001818186D6D4FE6E617FFFF00\
FFFF00AFAF0E2A2A263A3A3A747474ADADADB2B2B27878783131311A1A1A202020
2727272B2B2B32322CCFCF09FFFF00FAFA004D4D12161616161616151515161616191919202020\
2B2B2B2D2D2D2929292424241E1E1E1717170D0D0D010101000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000600009D0000FD0000FF0000E70000
3A00000000000000000000000500002100005C0000BC0000F30000DE00001E00000000000C0000\
850000CC0000EB0000FF0000FF0000E30000C10000C10000B100002300001F0000
4E0000AF0000EF0000F000003000001200007C0000D50000FA0000FF0000FE0000E90000800000\
0800000000003A0000D10000ED0000FF0000FF0000F800008A00000A0000000000
0000000000000000000000000800006A0000C80000C90000760000720000860000EB0000EF0000\
AE0000470000020000000000060000220000600000BF0000F40000D900001A0000
220000B00000F30000FF0000FF0000F20000AF0000210000000000000000000000000000000000\
1800009E0000C900008300007200009A0000DE0000E60000E10000310000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000C0C0C4747479D9D9DBBBBBB
8E8E8E5353533939393A3A3A3A3A3A3434342828282C2C2C5F5F5FA8A8A8CECEA5F8F80EAAAA3D\
3A3A3A2C2C2C2525252323234B4B4B7E7E7E6969694949459A9A3BF4F40EFFFF00
F7F716B3B35F50504D888811E5E501272704000000000000000000181818999937FCFC02FFFF00\
F6F6017474152323232D2D2D5B5B5B9B9B9BB5B5B58888883B3B3B161616181818
1E1E1E2323232C2C25CDCD08FFFF00FAFA004C4C12151515141414131313141414181818212121\
2929292828282525252222221C1C1C1616160C0C0C010101000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000003E0000F00000FF0000FF0000
6900000000000000000000000000000000000000001400005D00007D0000110000000000000000\
120000620000E60000FF0000FF00009F0000270000260000230000060000000000
0000000F00004E00008700001B00000000000100002300007A00008F00008C00003C0000080000\
0000000000000000001D00004200008F00008F00007500000A0000000000000000
0000000000000000000000000000000000001A00006C00008F00008F00008F00008F00004E0000\
0F00000000000000000000000000000000000000001500006000007B00000E0000
0000001400005C00008F00008F00005C0000140000000000000000000000000000000000000000\
0000001000005200008F00008F00008F00006700002600004500001B0000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000C0C0C4747479D9D9DBABABA
8C8C8C4C4C4C3030303232323232322E2E2E252525252525545454A1A1A1CFCFA6F8F80FACAC3F\
3434342121211B1B1B1B1B1B4646467D7D7D6E6E6E78784FEDED13FFFF00FAFA0C
BCBC645A5A572424247D7D07E3E3002525010000000000000000001B1B18A8A830FFFF00FFFF00\
E5E50340401A1F1F1F222222434343848484B1B1B19898984C4C4C1616160F0F0F
1616161B1B1B25251ECDCD07FFFF00FAFA004D4D131616161515151414141515151F1F1F303030\
3434342828282121211E1E1E191919121212090909000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000000002D0000ED0000FF0000FF0000
6A0000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000400005D0000E60000FF00008E0000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000606000000000D0D0D4747479C9C9CB9B9B9
898989454545282828292929292929262626202020252525565656A2A2A2CFCFA5F8F80FA8A83C\
2B2B2B1717171111111414144343437F7F7F909073D7D72CFDFD02FDFD03DBDB39
6C6C5E2D2D2D101010797903E4E400252500000000000000000000292916DBDB13FFFF00FFFF00\
ADAD0B1E1E1A1B1B1B1A1A1A2D2D2D646464A2A2A2A6A6A66767672222220A0A0A
0E0E0E1212121C1C16CBCB06FFFF00FAFA004D4D131616161515151515151717172F2F2F4F4F4F\
4444442727271C1C1C1818181313130D0D0D060606000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000000000000000360000EE0000FF0000FB0000
630000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000400005F0000EF00008E0000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FAFA00FFFF00CDCD003939007C7C00868606A4A423CECE4EDCDC5B
A4A46451513820201F1F1F1F2020201E1E1E1A1A1A292929646464A8A8A8CBCBA3F7F70E9F9F32\
1E1E1E0D0D0D0909090F0F0F414141969678DFDF34FEFE03FDFD03DDDD3775755A
2F2F2F131313070707787801E3E3002424000000000000000000002B2B16E3E30EFFFF00FFFF00\
A3A30A1515151616161515151A1A1A3E3E3E818181A9A9A98888883F3F3F0E0E0E
0707070B0B0B1D1D0ED0D003FFFF00FAFA004C4C121515151515151414141D1D1D4949496F6F6F\
5050502222221515151111110D0D0D080808040404000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
0000000000000000000000000000000000000300008F0000FC0000FF0000D30000
210000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000F0000B000008E0000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FAFA00FFFF00F1F100878700808000868606A4A423E6E627FBFB08
F5F50BD7D70B7C7C0C2525151717171515151A1A1A3939397C7C7CB3B3B3C0C097F4F40B8D8D21\
0D0D0D0505050404040C0C0C444441CACA45FEFE03FFFF00F1F11981815B2D2D2D
101010070707030303777700E3E3002424000000000000000000002B2B16E3E30EFFFF00FFFF00\
A2A2081010101010101010100E0E0E1E1E1E4F4F4F8B8B8B999999666666232323
0606060E0E055F5F06F8F800FFFF00FCFC008989091A1A131313131717173131316D6D6D848484\
4D4D4D1717170C0C0C090909060606040404020202000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000003A0000FA0000FF0000C70000070000000000\
000000000000000000000000000000030000570000E60000FF0000ED0000590000
030000000000000000000000000000000000000000000000000000000000000000000000000000\
0000000000000000000000004D0000780000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00CDCD000F0F000000000C0C0C464646A3A38ED1D171
E3E31CFCFC00F8F8009393041919090F0F0F2626265B5B5B999999AEAEAE9F9F76EFEF0679790D\
0202020000000202021616128C8C32F4F410FFFF00F5F513A4A44D3737330E0E0E
030303020202000000777700E3E3002424000000000000000000002A2A16E3E30EFFFF00FFFF00\
9E9E050808080808080808080707070909091E1E1E505050848484878787666647
6F6F127F7F03CBCB00E2E200E2E200E3E301D8D8027C7C0751510D252525575757909090858585\
3B3B3B0B0B0B030303020202010101010101010101000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000006A0000FC0000FF0000CA00000B0000000000\
0000000000000000000A00001F0000710000E60000FF0000E600008800000C0000
000000000000000000000000000000000000000000130000570000560000110000000000000000\
000000000000000000000000040000120000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C6000707010303031515155454549F9F9FB6B6B6
A3A366F0F005FFFF00F8F8006D6D0D2929295454548A8A8AA2A2A28888886A6A42EBEB026E6E02\
0202020707071616137B7B23F0F00CFFFF00FAFA0BAAAA4F3B3B380E0E0E010101
000000000000000000777700E3E300232300000000020202050505323220DFDF15FFFF00FFFF00\
9F9F060303030101010000000000000000000404041B1B1B4E4E4E7E7E7E818177
67674743432231311027270624240324240328280630300F3A3A224C4C4C7F7F7F9C9C9C757575\
272727010101000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000A00000E0000200000D00000FF0000FF0000F400008200005D0000\
5D00005D00005E0000A70000D80000F30000F00000BF0000540000030000000000
000000000000000000000000000000000000000000740000F90000F800006C0000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C7C7011616101B1B1B3434346C6C6C9E9E9EAAAAAA
93938CC6C623FEFE01FFFF00D3D3116B6B567777778383837272724646463F3F16E9E9006E6E02\
1010102424245F5F36EAEA0EFFFF00FEFE03D5D52F535344121212010101000000
000000000000000000777700E3E3002323000606061212121F1F1F424240B8B840FFFF00FFFF00\
B3B3111C1C190E0E0E0303030000000000000000000404041B1B1B4646466B6B6B
7070705D5D5D4646463333332B2B2B2A2A2A3030304040405656566D6D6D7979797A7A7A5C5C5C\
1E1E1E000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000005400007200007E0000A30000A30000A30000A30000A30000A40000\
A40000A30000A30000A30000A400005B0000310000120000000000000000000000
000000000000000000000000000000000000000000810000FD0000FD0000790000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C8C80220201A2C2C2C3E3E3E5858586969696C6C6C
666666838342F7F704FFFF00FAFA028D8D394D4D4D3C3C3C2828281212122A2A03E9E900707003\
1A1A1A48482EC1C119FCFC01FDFD02CCCC215B5B3E1B1B1B030303000000000000
000000000000000000777700E3E3002525010C0C0C202020303030444444878745F4F407FFFF00\
E7E7074949241818180606060000000000000000000000000404041212122C2C2C
4848485757575B5B5B5959595555555454545757575B5B5B545454454545363636373737353535\
151515000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000002A0000990000970000260000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C7C70113130D1414141818181D1D1D1E1E1E1E1E1E
1E1E1E484818F3F302FFFF00FFFF00717111121212090909030303000000282800E9E9006E6E02\
24240BAFAF08FBFB00FFFF00DCDC053E3E17101010040404000000000000000000
000000000000000000777700E3E3002424000606060F0F0F1515151A1A1A2D2D1CD1D106FFFF00\
F0F00245450F0B0B0B030303000000000000000000000000000000000000060606
1111111E1E1E2A2A2A3434343737374D4D304949312E2E2E1F1F1F0F0F0F0707070B0B0B111111\
080808000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C600060600000000000000000000000000000000
000000303000F1F100FFFF00FFFF00616100000000000000000000000000282800E9E900707000\
868600FBFB00FFFF00E6E6005C5C00030300000000000000000000000000000000
000000000000000000777700E3E300242400000000000000000000000000020200727200FBFB00\
FCFC00909000020200000000000000000000000000000000000000000000000000
0000000000000303030707070A0A097D7D053F3F06050505010101000000000000010101020202\
010101000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C600060600000000000000000000000000000000
000000616100F6F600FFFF00FBFB00585800000000000000000000000000282800E9E900BABA00\
E2E200FFFF00E7E7005C5C00030300000000000000000000000000000000000000
000000000000000000777700E3E300242400000000000000000000000000000000202000D2D200\
FFFF00E1E1002E2E00000000000000000000000000000000000000000000000000
000000000000000000000000212100D8D8003B3B00000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FBFB00FFFF00C6C600060600000000000000000000000000000000
020200AAAA00FFFF00FFFF00CFCF001C1C00000000000000000000000000282800E9E900FFFF00\
FFFF00FCFC007C7C00030300000000000000000000000000000000000000000000
000000000000000000777700E3E300232300000000000000000000000000000000020200555500\
E5E500FBFB008F8F00020200000000000000000000000000000000000000000000
000000000000000000070700808000FAFA003B3B00000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
0000003B3B00FAFA00FFFF00C6C600060600000000000000000000000000020200
5B5B00E9E900FFFF00E3E300505000010100000000000000000000000000303000EBEB00FFFF00\
FCFC00B0B0001E1E00000000000000000000000000000000000000000000000000
000000000000000000777700E3E300232300000000000000000000000000000000000000020200\
555500E6E600E6E600575700020200000000000000000000000000000000000000
000000000000000000494900F5F500DFDF00222200000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000424200FBFB00FFFF00C9C9000909000000000000000000000909005A5A00
E9E900FFFF00E5E500525200010100000000000000000000000000161600A3A300FBFB00FFFF00\
B7B7001F1F00000000000000000000000000000000000000000000000000000000
0000000000000000007C7C00E6E6002B2B00000000000000000000000000000000000000000000\
020200575700E6E600E6E600575700080800000000000000000000000000000000
000000070700505000C6C600FFFF00B3B300000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000909000E0E00\
202000B9B900FEFE00FFFF00F3F3008282005C5C005C5C005C5C00A9A900E5E500
F0F000BDBD004F4F000101000000000000000808001C1C00575700A9A900FBFB00FFFF00EAEA00\
4C4C00000000000000000000000000000000000000000000000000000000000000
0C0C000E0E00404000D5D500FAFA009B9B001919000E0E00070700000000000000000000000000\
000000020200505000BFBF00E3E300A5A5005959001D1D000E0E000E0E00101000
4C4C00828200E9E900D8D800B8B800B1B100000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000545400717100\
7E7E00A3A300A3A300A3A300A3A300A3A300A3A300A3A300A3A300A3A300585800
3030001010000000000000000000000000004444007A7A00A0A000A3A300A3A300A3A3005A5A00\
0303000000000000000000000000000000000000000000000000000000000D0D00
6363007171008A8A00A3A300A3A3009E9E00797900727200414100000000000000000000000000\
0000000000000000001111005A5A00A3A300BDBD00CBCB00C1C100C1C100C2C200
CECE00A0A0004848002020001111007F7F00000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000111100313100313100313100313100
212100000000000000000000000000181800000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000000000\
000000000000000000
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[13]:=",
  ImageSize->{256, 192},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000400000`1000`40O003h00Oogoo0Goo003oOol1
Ool00?moo`5oo`00ogoo0Goo003oOol1Ool00?moo`5oo`001WoolP0027oo0006Oomb40TV50Y=50/3
50X:40T8Ool000Ioo`Xh6PLh70@l7AHh70Dl7@Xh70@l7A`h70@l7@Hh708l7@@h70Hl7@03>1`l7C`M
00hl7P@l7`Pl7P@l7`Hl7Ti47`<l7`Pl7@8l6`Qoo`001Woo3D0M0d0O1T@O240O0d@O2D0O1d@O240O
1T@O1d0O0d@O440O1T@O140O1D@O00=07d@OA1l01T@O0T0O9d@OBd`O3D@O27oo0006Ool013PJ?1`l
73`J0S`L00<l6S`L?1`01C`L00<l6S`N@1`014@O00I07d0M@1e07c`L?1h2?1`5@1l013`L?1hl7S`L
0c`N00=07D0OA1l00d@O0T0O00=07C`L?1h00c`L00<l7T0M@1d00d@O0T0O00A07C`N?1hl708l7P90
7@907`06@1dl7S`N?1`l7C`N0S`L0S`N0S`L0c`N0d0O00E47d0O@1m07D0N008l7P03@1e07d0O00E0
7`I47`06@1hl7S`N?1i07c`N0c`L0S`O00<l740O@1l00T0O1D@O2D0O0d@O240O14@O14`O00A47dPO
B1m87`907`=87`03@1m87dPO00E87`04@1m87d0OB1l3@1l4C1l2B1l4@1l00d@OC1m<7`03C1l3B1l0
0d0OB1m87`02B1l2A1lBC1l8B1l5A1l8Ool000Ioo`0390dT3R09008P2@8L20TP2@0990Td5c`M=1P/
4b@>80TT2B@=00DP2@0390dX42@900HP2@0580XP2B0990Th6P02?1h00c@G;1@P2@0880T01R@?=1Pl
7C@E80TX3`HP2@0480XT32P@90dA80T01B`B>1`/3b@9:100120900@T32`B;14T30<P2@03<1907d0O
00907`03?1/P2b09008P2@0390lP2B0900TP2@06:10l7c`O>1Td5b0;1B0900HP2RP@80T/43PM;1@8
80T00b0:>1Q47`03A1l01D0O<1<P2B0990`01R0900<T3B0980T01R0900<T3R0980T00b0900DP2S0D
@1lX3B0:00DP2@08:0dl7TPOB1lh5RP:80TT2@@P2@0480XT2bP=@1hBC1l;B1l2A1l8Ool000Ioo`<H
1``H1P8H1`<L20DH1`@H1PDH1`LH1P8H1`@L20<H1`HH1P8H1`0370TL21P700<H1`DH1P@H1`0370LH
1QP600`H1P<H1`0470PH1aP760L360H860L01R0<:14T32@=80TL20HH1`TH1P8H1`0470PP2A`880P2
60L460H460L00a`860LH1`0660H01QP770P`5SPL=1P/408P2@0370PH1aP7010H1`<L20<H1`0470P/
4c0D90d360L01AP660LH1aP770P00R090Q`80QP711`800<P2B0:?1X04T`O34PO0T@O27oo0006Ool2
60HY50H460H<50H00aP650HD1P0/50H00aP660LH1`0460L260H>50H360HE50H00aP660LP3@0270P2
60L00aP650HD1P0>50H00aP770PP2@0280T011`860LH1Q`90aP71a@60QP611P70Q`80b0900<P2R`;
C1l04T`O34PO0T@O27oo0006Ool360L00aP650HD1P1I50H360H950H00aP660LH1`0260L370P00aP7
60HD1P0T50H00aP660LH1`0460L00aP650HD1P0<50H02AP660LX44@OB1m07b`D70PH200260H;50H0
2AP670PP2B09:0d`5D0MB1m47`0EC1l<B1l2A1l8Ool000Ioo`07<1<`5C0E:18L2QP750H01AP700<L
1a@650H071@600@L220<80/H1`XD1P0360LL21P7010D1P0460LL1a`870P850H260L270L00a`870/X
3`03;1<00b@?70LH1P0550H02aP780/X42`C<1@`5S`K@1hl7R`E70P00aP70Q`700DL21`980XL2QP7
014D1P8H1`PD1P0460LL22@>;182<1@0130F:14L21P71a@61AP700<L22@<B1h00d`O00=87c0C70P0
0a`800<H1a@650H011@60QP700@L22P?>1Q87a]<7`]87`=47`Qoo`001Woo0d@O00I47SPI:0d/4B`D
>1X2?1`023`K=1LT3b@;80/P31`980P290d290/012@=90/T2b@=0R@;00DP31`880/P2b0<00DT3@05
90hT2b@990d/5002?1h02B`A90dT3B@=90/T3R@=90lT300290l00cPI=1L/400290l012@<90lX42@?
0bP?00@T3B@;90/T2`8T3@06;18d63`N?1d/4B@<0b@=00HP22@9:0lT3C`M?1h4@1l4A1l0140M:0lT
3R@?0R@=00@X3b@><1Dh60M47`05?1LX2R`A=1Pl7P05@1l02c@E:0dT32@?90`/4R@=;18T3R`A90l0
0b`A00D/4R`A90`T3b`B008d500;:0hT32@?90lT3B@990lT3BP=<1=07`05A1l00cPI;14T3P0490`0
22P?;14/52`D=1HX3S`N;0h2A1l01dPOC1m<7d@O;0/`5D0M00947`04?1`T3R0970P280X01B`B;1<`
4d0OA1d074`O34PO0d@O27oo0006Ool5A1l00c`L?1Pl7002A1l2A1h00d0M?1Xd5`04=1L00c@H>1Xh
6P02>1T3=1P3>1T00c@E<1<d6004=1P00c@J=1Pd6002=1P01CPI@1a47T@O@1l00SPJ00lh6cPJ>1Xl
6SPJ?1Xh6T0K@1e07d0K@1a06cPJ?1X00T0M0T0K00=07C`L>1T00cPI00<h640LA1l00T@O00=07cPI
>1X00c@H00<h6CPK@1d02d@O00=07CPK?1`00SPJ00<l740K@1d02D@O00<l73`J@1l01d@O00A07C`J
@1a07@=07`E07@=07`03@1/h6T0M00907`03@1`l73PJ008l6P03@1e06d0M00U47`08@1m07D0O?1Xh
6SPK@1m07@907`E47`A87`03C1m47d@O00A<7`07@1lh6SPI>1Th6T@LA1l07d`O2dPO14@O27oo0006
Ool;A1l00d@N@1e07@03?1`2@1d00d@NA1m47`02A1l00d@N@1e47P04A1h4@1d9@1l:A1l6@1l3A1l9
@1l7A1l7@1l?A1l5@1lSA1l2@1l4A1l4@1lKA1l4B1l5A1l3C1l2B1l9C1l3A1lFC1l5A1l7B1l4A1l8
Ool000Ioo`Y47`04;1TT5R@E91D3:1D00b`F<1Y47P04A1l00c`M<1U07@04A1l00c@K;1L/5P02;1L0
0b`F:1DX5@02:1D6;1H00c`LA1m47`02A1l00c`M;1L/5P05;1H00c0H@1d/5`02;1P022`F<1T/5R`F
;1Lh6b`F=1`4A1l00b`I;1P/6004;1L2;1P013`MA1m47d@O0R`I00<l72`H;1P012`H00@l7B`H;1P/
60</5`@/6009=1X/62`H=1`/62`J;1Pl7C@L00H/6003;1Tl7d@O00=47`8l7`A47`03=1d/62`H00D/
6003;1X/62`H00@/6004=1]47d@OB1l2A1l00c@M;1P/6004;1P01B`I?1ll7R`H;1X00b`H00H/6R`H
?1h/6c@LA1l3B1l013`O=1//6S@J1B`J00</6c0KC1l00d`O00A47c0K;1/d6`H/6`8d7@03?1l/6b`K
00</6`03<1/l7T`O009<7`03A1l`6c@M00</6`06=1//6b`I<1Xl7d@O1DPO1D@O27oo0006Ool:A1l0
0c`L91HP500381@012@D:1@/5S@J14@O00E47R`F;1Hl74@N00947`04A1h/5b`F:1D8:1@01RPE:1@X
52PE;1Hl70A47`03=1X/5RPD00@X5004:1Hh6c0J;1H3:1D3:1@012PE;1H/63`O0d@O00<`6b`G:1L0
0RPG00DX5RPG:1L/63@L00=47`04=1//62`G;1L4:1L00b`H>1d`6P02;1L2:1L2:1H3:1L2;1L6:1L0
0bPH:1LX5`03:1L00b`G;1Pl7`03A1l013`O;1//63@L0d@O00<d72`H;1L02bPG00</63@LA1l00d@O
00<l7B`H;1L012PG00H/63@LA1ld6b`H;1L4:1L01B`G;1P/6S`OA1l00dPO00<d6b`J;1X01B`I00</
6R`K?1l00T@O00<l7b`K;1X01R`J12`I0R`J12`I00</6R`KA1l00T`O00=47c@M;1X012`I00@/6R`H
;1Pl7`947`A87`E47`03A1ioogoo00Ioo`001Woo2d@O00<d6R@F81@0120D00<X5B`F?1d00d@O00Dl
72PE:1@X5B`H00=47P04?1`/5RPE:1D2:1@291@2:1@3:1D2:1@00bPE=1Y47`03A1l00c`L:1DX5004
91@00bPE=1Xl7002;1H2:1D01bPD91@T52PD;1H/5c`M00=47`04=1`/62`G:1L3:1H00bPG;1P/6@03
A1l013`M;1P/5bPG0bPF00DX5b`H=1`d72`H008/5`04:1LX5RPF:1H2:1L2;1L::1L2:1H00bPG;1Pl
7@03A1l01B`J;1L/5b`H=1X00T@O0R`H00</5bPG:1H00bPF00@X5b`G:1LX5`8X5P03;1Ll7D@O00=4
7`03=1`/62PG00<X5P08:1L/63`OA1l/6b`H;1LX5`<X5P04:1L/62`KA1l3B1l014@O=1d/6R`I0RPH
00DX5bPH;1T/6S@K00947`03?1l/6b`J00P/6P03;1TX62PH00@/6@8X6003;1Tl7D`O009<7`03?1l/
6R`I00<X6004;1T/62`H=1`2A1l5B1l5A1l00d@NOomoo`06Ool000Ioo`a47`09?1``6R`G:1LP52PD
<1P`6S`M00=47`07?1`X5B@D:1@X5B`F<1X00T0M00dl730J;1P`6BPE:1@T52PD<1Ph6c`L<1X/5P02
:1D02R`F?1a47d@N@1dh7C0H91DT52@C0R@D00P/6C@LA1m47S@L=1X`6RPG0RPD00<X5cPM?1`00T@O
00=87d@O@1l00S`L00L`6R`G:1HX5S0J>1e47P04A1l013`L>1/`6BPG0RPF00T`6S`L=1]47c`M?1``
6R`G:1L00RPF00P`6SPJ<1Xh7C@L:1P`62`I0b`G0S0J00D`6b`G:1LX5cPM00A47`06;1PX5bPF:1L/
5b`H13`M01@`6BPG:1HX5RPG<1Xl7C@J;1LX5bPF:1L`6S`M?1hl73`N@1lh730H0RPF00<X5b`J?1l0
0d@O00@l7c@L;1TX5`8X5P04:1L`6d@OA1l4B1l2?1l00b`J;1TX6002:1L00b`I?1e47`02A1l01C@L
<1X/6B`J>1h014@O00<l7c@L;1X00R`I00</6S@L<1/00R`I00lX62`I<1]47c`OA1ll7C0K:1PX5bPH
;1T`6S`N=1/00d@O14PO1D@O00=47T@MOol01goo0006Ool>A1l01T@N@1d`6R@F;1I07@A47`06?1`T
5B@D91@X52PE0R`F00<l740MA1h00T@N00Hh6bPE:1@X53PJA1h2A1l00c`N;1P/5P05;1H00bPE:1@T
500291<00b@D:1@h6`06A1l01C`L;1LX52PE@1d00T@O0dPO14@O014d6RPG:1LX5cPM@1dl7T@OA1hl
7S`M@1ll7S@J:1HX5c0J00I47`06?1d/5bPG:1HX5c`L0d0O00L`6B`G@1m07c`L;1Tl7@02A1l01C@J
;1P/5b`JA1h00T@O00@l7R`G:1LX5P8X5`8/5`05<1]07d@O?1d/5`02:1H01RPG@1m47d@O?1`/60<X
5`D/5`06:1LX5RPF:1L/63`N0T@O00I87d@OA1m47c`M;1L2:1H012PG<1]47d@O14PO0d@O00Hd6R`I
:1PX62`I=1`2A1l01C`O;1X/6B`I>1d00d@O00E87d@OA1m47c`O00</6`04C1m47c@L;1X3;1T3;1X0
1b`I:1PX62PH;1T/6S`N00A47`A87`E47`03A1i47C`M00Qoo`001Woo4D@O00@l7B`G;1Pl7P=47`03
=1XT5B@C008T5005:1H/5R`F;1Hd6002A1h00d@OA1h`6P02:1D00b`GA1i47`03A1l013`L;1H/5RPE
0RPD0b@D0R@C00<T52PE@1h01d@O00@`6RPE:1D/5P=47`=87`=47`03?1h/62PG008X5`8/5`8/60@/
5`05:1LX5RPF;1Ll7006A1l01C@J;1LX5RPF:1L00b`G00<X5b`G=1`01d@O00@d6R`H;1Pd6P947`06
?1dX5bPG;1Lh7B`H0b`G00P/6T0O@1l/6BPG:1H/5c@L0d@O00@l7R`H:1LX5`LX5P04:1L/5c`LA1l5
B1l01d@O?1l/6RPG:1HX5b`H00947`E87`947`06?1h/6BPH:1P/6C@J0T@O00Hd6b`I:1P/6B`JA1l5
B1l3A1l023`OA1m47d`OA1ld72`J;1T7:1P00b`I;1Xl7P08A1l00dPOA1m47`02A1l2A1d2?1d8Ool0
00Iooa947`04?1h/62@G=1X2A1h023PK91@T4b@D:1@`6D0M<1X2;1H02B`G?1a47T@N@1d/5RPE;1Hd
6P05A1l014@N;1PX5BPD12@D00DT4b@D91@/5T0O00M47`05?1h/5RPE;1Hl7003A1l2B1l4A1l00c`L
;1LX5P03:1H7:1L2:1H00bPG=1a47`05A1l00c`N;1PX5`03:1H3:1L00b`G=1Y47`07A1l02C@J;1P/
63@LA1ld6RPG;1Ll7002A1l01c@J;1P/5b`G;1Ph73@J008X5P03:1Ld74@O00=47`03?1h/62PG00LX
5P04:1L/6T@OA1l5B1l2A1l01c@L;1LX5RPG;1Xl7d@O00I87`03A1ll7b`J00<X6005;1Y47c`O;1X/
6@02:1P00b`J@1m47`06B1l7A1l00c@L;1X/6@06:1P00b`J=1a47`0<A1l2A1d3?1d8Ool000IooaA4
7`06=1PT5c@J@1d`620D0R@C00<X530IA1h00T@N00L/62`F;1H/5c`M@1d`6@02:1D00b`GA1m47`04
A1l01d@N=1X/5RPE:1Hd6B@E008T5003:1@/5d@O00Q47`05<1/X5BPE;1Pl7P09A1l00b`G:1LX5P02
:1H3:1L4;1L01BPG:1HX5b`H?1h01T@O00<l72`G:1H00bPF0bPG00</64@OA1l01d@O00Pd6R`H;1L`
6R`I:1L/5c`N0d@O00<d72`H;1L00RPG00DX62PF:1HX5b`H00E47`0<?1h/6R`G:1LX5R`G=1XX5RPG
;1Ll7T@O0TPO1T@O00Hl7R`I:1LX5R`G?1`2A1l3B1l4A1l02B`J;1TX62PH;1Tl7D@O<1//6@02:1P0
0b`I<1]47`02A1l6B1l2A1l03C`O=1e47d@O=1`/6RPH:1L/6C0J:1P/6C@L00=47`03?1ld6R`H008/
6003=1a47d@O00947`947@@l7@Qoo`001Woo5D@O00Td62@F<1P/5b0C818T4bPD<1X014@O00Xl72`G
;1H/5SPJ?1`X5bPD:1Dl7@M47`05?1`/62`F:1@T5@0291@01bPD;1Q07d@O<1Xd6c`N00A47`05@1lX
5bPD:1D`6P05A1l01S`N=1/l740O>1`X5@8T5@05:1Ll6cPL>1X`6P03?1d01C@L:1HX5RPG<1X01T@O
00=07B`G:1H00RPF00X/6CPK>1`X5b`G?1e47c`N=1//6`E47`03;1LX5bPG008X5`03;1Pl7d@O00=4
7`04?1h/62`G:1L4:1H00b`G@1i47`04A1l013`O=1X/5bPG0RPF0RPG00P/6S`OA1m87d0O=1//6c`N
0d@O00I07b`I:1LX5RPG;1X5A1l01S`O=1e47d@O?1d/6@8X6005;1T`6d0O<1//6@03:1P2;1X00c`O
A1m47`07A1l2=1d2A1l00c`O=1`/6P03:1P00b`I;1Xl7002A1l013@L;1X/5bPG0RPE00</5S@IA1l0
0T@O00=47C`M?1d00c`M27oo0006OolEA1l014@N>1XP520D0R0B00<P4b0D<1X01D@O00I47S@H;1H/
5RPE;1H2:1@00c@JA1i47`06A1l014@N=1P/5RPE0RPD00HX5C@JA1m47c@K;1L3;1H012`H<1X`6C0G
0RPD00DX5B`G<1P/5R`G00</5P05;1L`6C0H:1HX5002:1H012PG;1Lh6c`L0d@O00D`6RPG:1HX5RPG
008/6006=1a47d@O?1ll72`G12PF00@/6C@K:1LX5`8/5`8/6003;1U47d@O00947`03?1h/5bPG008X
5P05:1Ld6T@OA1m87`04A1l00c@L;1LX5`03:1H012PG<1Td73`M1D@O00<d72`H:1L00RPG00</63`N
A1l00TPO00<l7B`H;1P00R`H00@/6R`H@1lX5`<X5P0<:1L`6S@J=1`d6R`I;1/l7d@O?1``6b`I0bPH
00</6C0K<1X00b`I0RPH00D/6B`J;1Xd6c@L00<l7`06=1dd6b`K;1/l7d0O0d@O00H`6b`I:1P/6B`J
?1h2A1l02S@J;1LX5bPF91@X52PE;1Hl740N0T@M13`M0S`K27oo0006Ool00d0OA1m47`0AA1l02S`M
?1`h6R@E818L4Q`A818P53PK1d@O00<l7R`H;1H00bPD00DX5C0J?1`l73`O00I47`06?1`/5RPE:1D/
5S`M0T@O00<l7b`G;1H01BPE12PD12PE12PD00<X5BPD:1@00bPD0RPG00</63@JA1l00T@O00@/62PG
:1HX5P8X5`07;1L/63@KA1ll7B`H;1L00bPF22PG00</5c@JA1l00d@O00L/6b`G:1LX5RPG;1Pl7P02
A1l3B1l3A1l00c@L;1PX5`02:1H2:1L01B`H=1e47d@OB1l00d@O00Dl7R`H;1L/5b`H00947`987`04
>1d/62`G:1L4;1L2:1L00bPF:1LX5`04;1L012`H=1]47c`O0R`J00D/6BPH:1PX62`I008/6P03=1`l
7B`J008/6P</6@P/6P03=1]47d@O00987`06A1ll7R`J;1T/6S@L0T@O00@d72`H:1LX5P<X5005;1Hl
73`MA1i47@04?1d3?1/8Ool000Ioo`03?1m47d@O01947`05;1PT5R0D818L4P0281800b0D?1a47`07
A1l014@N=1P/5RPE0RPD0RPE0R`F00<d6d@OA1l014@O00E47S@L;1H/5c@K00A47`03=1P/5R`F00T/
5PHX5@L/5P03;1L/62`I00=47`03<1X/62`G00@/5`04;1P/6D@O?1d2;1P:;1L2;1P00c@LA1m47`02
A1l00b`K;1P/6003;1P2A1l5B1l3A1l00c`N;1P/6002;1L2;1P00c@KA1m47`02B1l3A1l01C`O;1//
6C@MA1l00dPO00=47b`K;1P01R`H0b`G1R`H00@l7D@O;1//6`L/6P03;1]47d@O00947`03<1//6R`J
00L/6P8/6`03?1m47dPO00=87`04A1ld7B`K;1/3A1l02b`I;1PX5bPE:1@X5B`F?1`l6d@OA1d01C`M
0c`K27oo0006Ool00c`M?1m47`0CA1l00b`H91DP500281<00b0D91Dl700:A1l01D@N;1P/5RPE:1@0
0RPE00</5R`G?1l024@O00<d6d@OA1l014@O00<l72`H=1`00c`L00@d6S`L?1`l708d6P03=1/d6S0K
008/6003?1dl73`L00<l7004=1`l73@L=1/3A1l02D0N?1`l73`L?1hd6R`H=1`l7@02A1l00c`O?1dl
7002?1`00c@K?1hl7P03=1`2?1`00c`M=1dl7`04A1l01C`OA1m47c`M=1/00T@O1TPO14@O00Dl7C`L
?1`l7S`M00947`A87`A47`03?1m47d@O00A87`03A1ll7c`O00947`03=1dl7C`O00Hl7`03A1ll7c@L
00947`Hl7`03=1e47c`O00947`987`947`04?1ld6c`O?1l2=1d00c`OA1m47`02A1l00d0OB1m87`04
B1l00d@O=1e47`04A1l02B`I;1P/62`H=1Xl73`MA1m47@04?1d5?1/8Ool000Ioo`8l7AA47`03?1``
62`F00<T5@03;1Hl74@O00]47`04?1`/5b`F;1L2;1P00c@I?1e47`0ZA1l2B1lKA1l3B1l7A1l9B1l8
A1l7B1l4A1l5B1lBA1l2B1l8A1l6B1l8A1l:B1l3A1l2B1l9A1l014@M?1dl7C`M1c`K27oo0006Ool2
?1d00c`NA1m87`07B1l3A1l6B1l4A1l2B1l00d@N@1e87`0=B1l4A1l00d@NB1m87`0DB1l00d@OB1m8
7`2LB1l01D@OA1i07T0N?1d013`K00@l6S`I?1Tl6@Qoo`001Woo00Hl7D0N@1hl73`K>1X2=1T013@J
?1]47T@O14PO00A47d0M?1]07PE87`03@1hl6cPJ008h6PXd6@03>1Xl74@N00A87`03A1hl73PJ00Dd
6@05?1/l740M?1`l6`06=1T00c`K@1i47`03B1l014@O@1hl6c@I1C@J00@l740OA1m87`947`03?1hh
6c@J008d6P04=1Td6S@I>1`2@1l00c`M>1/d6P04=1X8=1T01SPK?1e07d@O@1ll7@@d6P03>1/l7T@O
00A87`947`A87`03A1m07c`L00Hd6P<d6@@d6P03?1e47d@O00=87`03A1m07cPK00@d6P07>1/l7D0O
A1m07c`O?1d01SPM00<l7D0OA1l00dPO0T@O00<l7c`M>1d01CPM00<l7d0OA1l00dPO0T@O00<l7c@J
<1P00R`F00D`6C@K?1dl7d0O00947`03@1ll7c`N008l7003@1m47d@O00A87`03A1m07T0K00@l70=0
7004?1/l6C`H?1P2>1P8Ool000Ioo`907P03=1PH2`0200@00004108H2c@IA1l2B1l01T@O?1/L3Ab9
91507@=87`03@1dH300000h00003108H2c`L00987`03?1`P3`P400L0000420@@2@`6108600002@01
40Td64@OB1m47c`K60`40P0600001`0150Xd6D@O@1l/5A0700L00004004@21@:20@?00001@P360/P
41@:10801@0000<<1R`E@1l00TPO00Y47c@J=1Y07d@OB1m47c@J60/00@d0000920<P43`NA1m87d@O
<1P@200100D0000620<H2b0@60/810011@0000@00@@260`d6P987`05A1ll7B0A20D40P0600402@P4
60`d6d@OB1m47c@K70l810020002004200001`@240PL3b@C814D2PP500840P0420<@2BPC@1d2B1l0
14@O>1XP3a0710P500P<1Q@9<1E06S`J?1Tl63PH0S0F27oo0006Ool01D0N>1/LJP@QE[H017oo019_
OAR9108`5T@OB1m07Q`=000I1P0080m47TPOA1l`5aBVLih>Ool00fmm56LL3`02A1l012PC000ml7_O
1Woo00A9mA@9;F]S6@Ioo`09MklU2`@2=1Q47d0N80h005:F00Ioo`08LilHb0P5<1P`5`P38@UcW`Io
o`05LidLB10U?O5cW`0>Ool01EJh20D0011TE[P017oo011kgcW@004`64@OB1ld6@aT20<P440OA1ll
7A0848A[G0eoo`03AS<002PC00947`04=1T40Re/Lil3Ool027_oJka2D@PR000<A4HbHaX3Je/036]L
Kga[FdYC124H:T0OA1ll7a@;000el@=BnP0?Ea]K?5/lEa/el@0062/l7d@O:1@40PP3?UASof_o009W
oP05JomSOdZh9@l<A@02108021S:>Q9:]dZh?U@@Y@@Q<1L2A1l01C0G20@48R6>;Q800RWa00T^4RWa
4:L<I3@F@1/l6CPH<1H00S0E27oo0006Ool05T0N>1/@9bg]Oom6dA2T268<Pbg]NolTS`P3=1U47c@I
005F]Woo?U@003@H0T@O00D/53h@IodI1P@Q00P0000>1248HPQR4:@bTg?o40Tl740M80h8@Woo@]88
HP@000075@MKoddoLcm_ob6:12400`0000D48B6[NolD:AP;008l700540PUK7oo<Rh@Y00300002`A1
;U1co`@260dP3`00MomGfAT62680100000L^DEnOCA]oodmF5>D8HP0:00003`@Q8DQ[n7on?1eGGfOo
;RlDi@b2268Uc7oo34HL3P02A1l03B0@5:HUB000918l7S0G001Soe?o7FT@a0QR00@00009124@aA34
3884@AeYNolLbAP;00907`0670h<97oo=[@I212T0PQR00LbKG_oC]PX56?fFol^/`02<Td04BW:9JTQ
J39]Lolo>@P3@1i47bPD001Gmf?oAeLk4T<B=Zl00RX;00a7Dgoo:S4<1C@K;1P003GaKomCnB7:26<2
00007PAS7Ja3LeokJomWndlkIomGoe?hCoMWoeoo000X4T0K>1X811BWGom[of?oEoiCo5?kDoaCg@00
<1A06`8`5P<`5@Qoo`001Woo00Q07S`M80l00FOo;Y<000@Q0P0001`M[G?o20<L3C`K:1840WooD_5o
o`a650Y07T@O=1TM;U_o024@a3X<AXhabQC54@PIJaDY3:H08@@00010@idYl3@H@1h/5000HolIJ`QR
;LXf32VY5@T4@Q3WEom8_Uoo4>L@PbEX8DL@a@R40013WD]O108h6T0M60/@AV_o6F/001T69FP@aPAS
005Woab]40LL3@00Holfm`00120I1Rg::JT@j0bVIom47e_o9Q0000@Q8DLf32g:7Fd56F/02A4814<0
01CUFgAE_DJj<]L8Y002000020A1005Soab]60]07d@O50X2Ool01cea108`5b@B001[objD00800005
4:<UJ2VY6DT<YP03000060A128<006?o68TH3D0O?1h@22e>Hol<a`00124@a0b34<M[od2MDGM[obIB
16<08@80000?0200000RFol]]@P5@1i07aP<58=[obIB3:D8P`QR00<0000<4@QcoaQ940T`6A09?S=[
odND;TdE:@A2100001l8Q1TW9N/Z;1fY6F/EJAeW9PmSod/o000/5D0K810005OoGom3_Clb>ZdZSR7[
<a5Bn`P3<1Hd630F<1D/5002;1<8Ool000Ioo`0Q?1e07SPK50T816Oo=W08HPA34<Lg6G?o5@TD:C0G
50XH36_f000YRD[h000d6D@OA1h@24lOAol003X<Gg@j30@Q0080000K148A22H@;N/M2ATZ;YE[obH@
=1Q47c`K40LPSU?o0012STJ`7BL48P00=aUkocdeAKi?>@00008I1P0/00408RHaLolk>``5?1]47S0G
000o6Voo7L`@XaT60008HcmKNol8IAP;;1@<1Qd9LoloN@@Q4<@f31dW0016ef?o915:ofoo=]L48A2S
=P`UJ0000048Q08<aP0A4<L@iag?9TlZ;4J_HkQF>6?oBoh]ZA2S268g6GOo004T4P02A1l051@;Oom:
lGoo20D@2B@B000o>Foo>eP<HP@Q:JTUJ0004>LkFRkD7>@200009b7_Mol002`EA1m07aP=38Agod_o
38<001CU269Goe_o50XQ:f_oIolc5AdV124000QR000V<G?o000T4T@O@1lL3`A2JomOobkD9FP@PP03
00002DNmCal002@C:1DHQf_oDoUGo`03Hol08E_oDolkF27^6HTE9Q3428<4@@R36DL^2D_gIol811P;
>1/l72QA001Kog?oFoYBdcjE<XhQjdOi:K0/4SPI=1L`5@04;1<8Ool000Ioo`8l7@0[@1hh6a@920=c
?gOi@S][odjo8<h002@A<1H40U6l6@DHi@00JoP<AAP;A1i47c0F0015_cV@248nCASU128j>VOoE_m9
_cRHEAYoodYOC[lPC@`6?1`00T@O00//5000B1l48BVY9FP001C[Fom>_a0W0080000^:98n;@00124<
Hd[oCOlH:P00:1947TPOA1hX4`0140MEgQC5000M2E]o?=P80`P5=1Q07C0G20D40T6LC]T00134000f
632E00000@0056U5_eiJ144<HP00;Bd2DOl0;dgOBKm5_dFoAIdnNfOgFoEooVjn?1dd>6ZnOoY_OdRO
20@@23`LA1m07bB^De<48Cj?ESP0020@70h00@`7AKmb/@@Q4:<002T;B1lX4dHLOoA23Ci?C;lH2a07
?1h00T@O040d6PP520A9_gH`1244@BT/>7T000P41080021]C_aoU3EX00008@QRDK`D2Q08?1i87d@O
=1X<1P01<CIKm7MA>HT48@005@IEgPP450/`61P<AQ5CmeOkCK`H30@200440a@:>KICoeOi?c8UbQ3U
3<@IISI/Cfi[oCCF000/5@907P07=1PD2@0048PYJCUYEiP00Slb00EOo``7>1Td5c0E00</4`03;18/
4Goo00Moo`001Woo0c`M01E07SPK30HH:WooD]lHR@0030HX4c`K<1H007oo001:d2F8:HU2M`00>1X0
0T@O00lP3P01IQl002F8144YKDHo66/000@2004003G;8D/00P@200l`5d@OB1m47cPJ001Qoa2T6>D<
HPb6Ekle<`0050X00S0G00X<1W<O388003iC<C<00109<1M47P=87`0TA1ld610740M6<@QREa/XD@00
80hh6T@OB1m47c`K80h004BN<N/001C4CCl81B`E=1X`5a@:000`65ME000@Y67o50X000P30PP400`8
0`@2000894<?HoI_n`P5000@9V]AMoX230H04CPKA1m47c`L=G0I1P00269oo@P550X`630G50X005VI
?P/00P0000]g_a@:10<004G/OoEoof9l000P44@O00987`0HA1ld6Q0910=]>aCU0011e000810h6c@J
:1<<1P00HJifS`A1000QRE6m000/5D@O0TPO02M47c`L81040PP4M[9^TA34000^CDiO000X52PE005J
mCk@Nol@2@P491</5RPE60d40P01=XaOnd_E<XhfSdogDgQOlUoe>Zl002`E@1h00T0L00`h6RPB40L0
01RTGoE;eU?i?]PH2c@H<1H2;1<2;1800b`A:11oo`07Ool000Ioo`@l7@0f@1h/5@00E_=6MP0060/l
6D@OA1hX4PP4JoX@Xf?E?Vd007om22<L3D@NB1ll6`@2EKdI1Ad6124n<c6I000P3S@I=1PD2P002480
01`=@1e47dPOA1l`5`00Lcl48ASU000YeUNo000P3T0M0T@O00TH3EWO;JT004lf9A081C`KA1l01DPO
00Y47c0G001BMAd6Hil40R0>@1e47`A87`08@1dH2aP;JoP002F9HW`H30=47`0h@1lP3`@3OolDa2F9
HOl002@B?1`l7S`L=1X/5RPC40L001e66FP00108000XiU<A000L3T0OB1m47c@I@Q04@@QR001SU58h
000`5d0O?1`@1ba[MmH000A1Ool00B`F70h006<@Oo]oo`P360/l7P=47`09@1lT4P00OcD@a000?Q8@
2C`N00=47`0;>1/@21QUOe<@a0006@I_o@00=1Y47`03B1l04d@O@1l/5@@2>DQo52E8144bCE]M000X
51`?001SU3k@Hgl002`F00A47`05>1/L3P009LQKnP02FoT026_n>GMG/E_cBg<00AP;?1/2@1`02d0K
?1T/4PPQ58=OmDNDHo`U[b@@<1D00R`C00D/4RP@:10X42@>00Qoo`001Woo13`M02107S`L30HLbD9A
000/4d@MB1i47AP;84m2KBF8JoHf2`00JoHf4`01>1Y87d0N40Td/bV97@H8HT:A;GH40S@H0T@O01<`
5`P3000000`6:1@d6C@H;1@<1QP;Ko/001SU001;Gc7f20<h6P03B1l022PBAEI2S`00Ea8U4108@1h2
B1l3A1l2B1l023`L20E>=B58Mol40BPCA1l2B1l2A1l02DPOA1lX4`@2Ool8HRF9HW`L3P03B1l014@O
=1P007oo0RF900=oo`00=1T00d@O02dh6bad<1P`61P;00400109:1<@2@01=N/40Q@:@1m87d@O:1A2
4@00248004H[Ool811P;?1i07b0@20=omP@Q001oo0@2=1Xh6``66<Eke7om?G4001P=;1D00S0H00TX
50`640Qom`@Q00164R@BA1l00dPO00]47b@B001oDbE70008PW_o000`64@O00E87`0B?1hH2`PROaDn
;@A18FQco`0060`<1R25C_E34U]M000d6T@O0dPO00E47d0M914001TU0093D`096>/80c8;Fo=Glbh;
000P3S@H008h600;=1DL30009@IWoSlbIohDZBPA<1</4P03:1000b@?90hP3@08Ool000Ioo`Dl7@0V
@1hP3`@1BY<002PC?1e07T0K20AIgPb2AXiGDaSV000nKgon004P3dPOA1lH32A@>R`M1Q2S>U4aV@@1
=1U47dPO?1`H2`0048@40P@0000:60]<_aT56>D@a0R4Fol<IQP;@1h3B1l02B`E:8iGD`00=R`e^PP5
@1e87`05A1l02DPOA1hD2THc6>Ego`P491547`06B1l01c0G001ooaC47@IcoA`=00987`0:A1m87c`K
109NNSH<7@Ioo`00<1P2A1l0440O918@IRPD?1dh6bPC:<ld6C`L:1<002e75:H40S@J0T@O01/L3T8B
0004@@Q2DTeooTie000`5d0O;1D007oh268007_m30Hd6D@O:1<005J^Oo]ooc1e20D0100000P@2E2K
@NT000QR@Q0`64@O0dPO00E47c0H001o4CG:00800004Ool40bPDA1l5B1l024@O918007[CCVd8HQ34
Ool220D01`01?@U:/Ck@Fc`003PK00I87`03@1hX4PP30080000710<81A2SDi5Kle_g8D/00P000P01
0P0000`HY5KCBmHbSV_o22</4S0C:10X3b@?90h380d8Ool000Ioo`Dl7@0V@1h/5@00@[<0021=?1a0
7S@H001oo`00Ee=:d0@Q14<U/Goo8@`40d0MA1lP3aP;B]0HiAC4;Lde^`00=1Q47dPOA1lX4`00@W1c
m@Aoo`0EIcXHa@QR=R`<PQd/Fol002PBA1m87d@OB1l`5a08JoP0025XCSh40S`K00987`A47`09B1m4
7a`=AQ8@Y6_i66TH2d0M00I47`07<1L007oo7@HDa7_o60`00TPO00Y47dPO@1d<1TEhC_4@Y7oo20@P
40907`06<1L80`0050Xl7D@O0T0O0T@O02<h6`P468<f3000:1=47c`N40Lid@A1124<A45AOiAoo`PS
40Th6c0G001oUA2T001SmQa=;1I47c`M40P@A7odOo]gnW_O00Aoo`07Kgd]J@00144DiCE`?1`014PO
00E47cPK109^KTh/00800004FoPH:Q`>A1l5B1l01D@O<1L006I?I]000PQR00Y[nbC=000004e;A[0b
KUmM000d6P987`A47`05B1ll7C0F;;8X4`02=1L01PP4=V]KleoeJomOOP9KG@0;GeeOOU]fGoE;Db7:
:Ra_o`00<1</4@0290h380d280/8Ool000Ioo`03?1/l7C`M008l7@0P@1hd6P@2>Q048a16=1Xl7B@A
20EWfA2SGi@ajP006BdjOGooIoX003@IA1lX4Q07Gi@Da1SU:J/ig000<1M47`987`07?1/80c5_OolU
k@R314800P0001@QAdlA@Xh003IKAal003@HA1m87d@OB1lh6P@3Ool001T5Hgl003PJ0TPO14@O00Y8
7d@O:1964PA1@Zl]4`0070dX4`@/500D:18@20@2IQlDa0A1MolH2d@OB1m47dPOA1lH2bB=GkD4@Ene
;1@00@8H3006108<1THc108h6d@O14PO00Y07aP=000abP0150Xl7C@J004m/08Di@0G000YLVZ_Oo]B
dP00:1</5@00O]@I1P00A]0m=2PDB1m47b`E001S3g>F:JT8HP0500001PA18DL@Xbg[9:m07`E87`04
?1d81DeZM[02000014;@?C@@1d0O1DPO00U47c@J109=bgJb4<@48DnD?EH00P0000MF;3h:8JU_oP00
<1M87`03A1l3?1d2@1h4@1`04a`=4:=?L5ocEoA7U2ib7H/IJRG^;[<feAf94>D^KF_o000`52@>008P
3@<P2`0370/L2Woo00Moo`001Woo0S`K0c`M02107S`L40Pek12822<`5S@J40Lm=2g9<NYOU1d614<j
OE_OK[ioo`aU914l7BPC20Acn0QR5<@UJSWM000`5d@O0TPO00I47Q`=20=ooaf=024200002ADYCa=S
mSi]001;Gbfd20Dl6`02B1l034@OB1ll6`00Ool8HQC4Iil003@HA1m87`A47`0:B1m47b`D@Q4000A1
?KTd610720@410<010P450Tl74Eg0P0001I[nAP<A1m87d@OB1m47b0?40MgnP004:A57C@H60`H2c0F
K[ioo@00<1Q47`A87`0TA1l/5@00:BHDYPP3<1L/5@00@Q0]bSH<148@adUcOkAoo``U30HP4001JLlf
2`00=Pa9NB@BB1m47c`M40PHIGoe9DL0000Q0`0000L4@Qd8=P`8HUMd72a47`05B1l02T0O50XPQGlC
144002F9GUX40S`M1TPO018l7@`6;@Mo<QT6000bCEmn20@40EY=@NXDiG?o20@X4T@O@1l5?1d2?1/0
5c`L?1/l6c`L<1D0038ZGo=;CbG[26<0000R4>LV<Bf^8FLA1SlbGo/40R`B80d00R0;0Q`;00<L2Q`9
70T027oo0006Ool3?1/3?1d07d0N80lf2b4>000T4B`D005VW0@QB]1GD`Q25@aGgdjo<1M_nc7`50Tl
72`D005go@004:<UBCgo000`5d@O00=87`0A=1T00DHaEgeOoe>o:O@003H/Fi@UR0Q3Fol<IQP;A1i8
7`02A1l014PO@1`007oo0Q2T00ESW`00<1I47dPO00A47`987`07=1Q24000388003i?ImT01WOm01=S
^37<000I1PaRA]0L;4@OB1m47dPOA1l/50@2Ool000A1000aj`02Fg@01Si]6>EFe0@2;1A47`A87`0K
A1lh6`P556<UJ@0070hH3@@3@Q1:lD[e9Pl002C=K/moneKc000@1`01EFe>S`00:JUIgR0@00987`0A
A1lX5000Je5NlQ343:HZ4TMM?_X@j0006@H007_m60a47`05B1l02T@O81080Wme4:@00134Loh003@J
1TPO01907a@:56=oLbUY000MAg?o50X80f8_@PX8HUoh9<d@2C`M@1h3?1d5?1/2?1P2?1T04Q07249?
L5_B?ZhV2bhcBimSof=o@H/n[RG[DoTb;``5:14P2`8L2`0570XL2AP960TH2008Ool000Ioo`@l6`8l
7@0N@1h/5BV8;M4001@Y60`007oo001GDdJ>000]meoo8:`812d<C_<<1C@I;1@007_O000@Xad6ASl0
02`F0T0O01=47dPOA1hP3P00;DdL3B0>A1l];0b2@Xh@Xb5>Fol002PBA1m87`03A1l02T@M001ooaT5
145go`@R:1=47dPO14@O0TPO00Hh6T8@000]jRVY268800002@R39NiCDbVY8FPXcD0NB1m47`02B1l0
23@H001oo`00=P`b;Q4702430000148?30HP3d@O1DPO00E07aP=000abP@20088100;70hb2e^eIoe?
oa474<MAMgoEOolTjP0200001Sb?Ie<001CUOolL3@987`0AA1ll7Q0856AomF_FGUY47bPD=1U@^aBS
144007oo60]47`05B1l02T@O;1D007mD9BH000@QMol00B`E1TPO01947b0?105oDcg/000<XgOo80l<
15X^BR/48CZ_Ea/00C@J@1h6?1/2?1P2>1P0530F=1PX4P00:LQKleNaHo]9?2P@90hiDFWfDmHbSV?m
7BPH2BP?70/270T260T350L8Ool000Ioo`Dl6`0@?1e07SPK58I6U@@S10440a@:HiHDa6?E<NX004LO
Bel200003P01@Y0@QB@A918007oo000@a1C4ASl002`D@1h2?1d013`O@1ld6``70`0000/L3F<I000Y
Z@00<SY77`01=1Q47dPO00=47`0:A1h40UWN9HT006Om34DP3T@OB1l5A1l01dPO?1/aC`QR;NXf;0A2
00P000092:DZM5ND=R`<PS4^?1a87d@O00987`07?1/007oo000nKCZA3<H00`0000L08DYb78XH34@O
B1m47`03B1l01D@O;1@002U77>P00P0001A=VRVYIoQGof?o=aH002adMa5kmg?l20D003B=Nk@0012T
OolH34@O0TPO00m47b`E001>:gokOohPC`00109<VCH;000@Y7oo60d01TPO00E47c@J001nlSg:0080
0003Ko/<1b0@00I87`0BA1lX5000O_5>;000125SnSAf30E:2e:=124I1Woo004T4D0L0S`K00@l6S`H
?1Pl608h600E<1L`5S0E<1Ld5QA912573UocGoECTP`5104LQV7EDoTfcf_o22<P32@=008L2@8H208D
1`8D1PQoo`001Woo00<d6SPJ?1/00c`K01ll740M40MKDac<1248168m4:A2SEnD4:@@bE_O8E48120@
40L<I37<20<D2@00Ool001SU38=:G`00:1A07P03?1d2@1h03B`F10800000;5AC<`QR6@D48E>o7@d@
240M00987`A47`0940PhmDK?001?4RDA50U47TPO00E47`09B1m47bB>;NX@Y0Q1;De1W3HB00@b4009
<Q4iNSDa124QRCH/000a/3@I00A87`08@1d8156L6@DDa0Q1:98eV@8b4@05<Q=K5A1540U07P06B1l0
6T0M40P88Ci=88`T4FlN7DM_nVOoKomSob7]4:M8g7McOoYN^1`/<3Ao]@@Q001ooA@;A1l3B1l03T0O
81448E9/OoUF]0`V40Qj/ae7000f;4Ui9187B1l0140N30I5:FkA0P0000A2/45F30I47`E87`0EA1ld
6P01HRabSP@Q000bCEYL30HLYF/A48<4@D;@F[X40S@G@1`l6C@H008d5`04<1Ld5S0F;1@2<1@04c@D
=1H`50`638=73U_cFoI2UAbY>@aZedNE>a5_o`00:0dP2a`:00<P2P0470TH1a@650H8Ool000Ioo`05
91<X5S@H>1Xl6P02?1/07T0L60Y2T3YE8>hhV5[G269OU4J>000b>U_O000P43`L=1P40c6/00400001
Ool001SU145:G`0191907P@l7@0?@1hl7A08248P3F8O268I1PQR8DiKo`0091=07dPO00E47`0880lD
2F_h000ajcWL20<l70987`947`=87`03A1lH2e^D00800004Hgl81109<1H2=1P02c0F60`007lD:DHH
a4:>124j4R0?A1h00dPO00]47aP<951?4@004:AQo`0060`X4QP;0080000350Xl74@O00E87`0:A1l/
5000CViood0N9DTfC7_lKol2Mol02E_o16<]e5BlOmEooF[OA1mjT00200000e^e62e47`04B1l023`N
50X88F]AOoUVgT@ONY02000016_i50X`5dPO0d@O0dPO00Y47a`>32=oE123000I1Vko00107@E87`0E
A1ll6``6;@Mo<Q2T000@a7_o30H005Jc=NX8HPbSHoT]L`P4?1Td6S0F00<`5@05;1@/4b`C<1@d5P02
>1D0630E;14<9A2SCe1OlfOkKge^5f:b;Vhk4F?m004X3R@=80X@1@P340DP2RP<80X@1PQoo`001Woo
01800AP=91@d5cPI?1Xl6d0L80hU:U]K?KXiEaT5;JU[mSX/0002Aal04P00<1I07S`M80lUJ1d90008
0g<o144Hi@00Del48R0??1e07P@l7@0=@1hL3AS5@KY6<P009HP003Ym@]l00C0I?1l0140O00Xl7d0O
;1@817om000QJ4ho004l6`987`04A1m87dPOB1l2A1l01aP;No/00000Kol00BPC00947`0RB1m47c@I
005f/ReW7@E?4A34<J`8A3@dA1i47dPOA1lP3a08JoP001C4JYd003@HA1hl6a@824@P3S`KA1i07D0N
A1l3B1l02CPJ1095Zgoo6>/005MDMomco`02Mol02F_o:U4<YTROJ_1omGooC;mI[00200000dK@98i0
7`04B1l024@O<1L00CV8OoEood2L;BL200005Woo108d6T@O@1ld6SPK@1m47dPOA1lX4`00OgDHi@00
269ko`00?1a87d@O0TPO01Q47S@K;1D@1a12Og<U9P00145ko``6000abUmb5@H8HRW:Ool40Q`=>1Td
5c0E;1<2;1801R`A<18d52`D:8/XLP8`4`0F80`002gYFo=Olf_oFSQ6SAfY?e=Gn`@1:0/L2000=Xi?
ocJ>000@220:1088Ool000Ioo`80000D40LP4BPF=1Pl6S`K:18@I7_nBQhf=000Da5[mQd626EG_c8i
10<d6P907P0>=1T@QCj>24@@26HO388Hi@00Hgh8AAa<?1e07P@l7@0@@1hX4aB6Iil@Q1T69HP005=o
8Bl<1S@H>1/d6c@J?1l3@1l02C0H001oo`QR7BISO`00>1Y87`02A1l00dPOA1ll6`02>1X021@9Nml0
0000Kol40bPCA1l3B1l05S`K005f/RU67>AKU2VY;F/00237:A4l=d@N@1hT4@P4Ool000QROol003@I
0T@O00T`5bVc;1@d62PC60`TSC@IA1l00TPO00Ll70`6>@acnP00;NYcnP02Lol047OoNomgoe?o000i
nEBlOmEooF<J::H000@Q:JTXcc`N14PO01=47c`N50X88WNcLoT]B0001448HWoo000l73`N80l<@`P4
70hl7002A1l0130H001o4cEX0P0000=oo`@3=1X00T@O00U07c`L918PZaPZ30H40Glc?LX00P0000mW
naQ;108@Y7?g@Vd@Y0bTCiAG6`00:14h5c0D;1400RP@00L/4C0C<1<P3@0024@/3`02<1405A@9145;
CeocFoQ>cR4V4@I7]CYa104P2@P3GomWodooHomSo@QR14800008Ool000Ioo`<0000C208L3RPE=1Pl
6c0F105oodHL5<HHiF_fGi@8@QT]GolDZA@9?1d00T0N00hl7@`6Ca4YSR1_HQl8HQ34001KG@aW62Xl
740N1C`M010`6000Ool002VY6@H<Q5oo24@H2c@H=;8HY``680ld6P907`03<1T007oo008@Y003Iil0
02`F00=07`07@1hX4P`634@H2``5Ool00P0000AWfA0X80m47`=87`0Q?1/00G[C9BHLi5^D?VdUC@@0
248LaRdZ==Dd5b0>109kg`@Q001oo001;1A47dPO@1dd6C0F60`000Q2000L3D0N00987`07?1/813Ab
CXh8HU_EOod00W?o0W_o00mgof_o7J`UlDPOGOUomEnD344480QR5>DaCcPKA1l00dPO01U47d0O9180
059<Li<Hi@@P268I1V9l004l7B`E000Q9S6=004<1S0H@1l`6001O[55j`0200000e_f62//5P02@1l0
2C`L8100000010481@00O]1J30020000;DMC<Bl40PP3Da1[mB6Y268I1V_mBU<00B`A=1D`4b@A90lX
430C:0l004J_000P2c0?:0lT3@@Q9JQKleNa=XlA1Q3UDoX^3P0030<UjV?K>]0UbS9;FoU?U0020008
Ool000Ioo`D0000A60dT53@H?1X007oo<O8004J>MoU:d000;OMKg`0181107P02?1d03T0N60d]ZV?g
=CE670006@H005/l48PH:S`L@1h5?1d0:c`L001ZoaC44<@4@Ae?Ekl002@B=1PD2P007>P00@P4;1@h
72PC001oo`b2145Wg`PT60`d6cPM<1TD2`00;F/ac@00109oo`00124bCBC>30Hl6d@O00987`0A>1X0
07lD5:<Y9V?fDe<]SQbV000<@QR57<HPa`P430EZo`02000016>h60`@2C@I0T0M00Pd61P;0015/goo
;F`81C`L0T@O00TX4`00D]lHa257JoUooUooLol00g_o00igodOL001=gd0mOmEO=0A148<I1P00@Q0T
4S`N0T@O01E07c`M<1PP3`0168MomBE800048CY]AEH<1T0N80l407NgOomcoa@Y10800R0?01<007X`
EL/000A1;N]=VQP=?1`d6R0?0015RDKa00081000KViR;00200008269IY`000`6369OnDnd6BH8HR5Y
MolPc@01:0l`4b`B:10/430A72THI7ni8B/@1bP=80/X3Q`<00173U_c?Zh26DL03Eom9>P00000AXiN
SCZ_;BPfSU=aFgHE9P0000Qoo`001Woo1P00010D2B@C=1/00F]L9F`Da6ohNoXj;000@_m77`00;1E0
7P8l7@0>@1hX5@@1Ooa>ncfA000QA`00Ea/@RA@Y?1a07PHl7@0350XPK6Of00800007>Ym2g`@2<1Lh
6a@946D00Woo00H/D`0020@00Q09FMh2000083jB;A8000@210<00108HWaoog_o<G@D2UFl000Hi@A1
EWl40Q@:=1U47T@O<1L007oj000ibGOjJoPM9TF_46@2000080@Q000003E;AK<0012S7BI<_`@3004<
1Q0910840eWONoeoocWi20<l6d@O=1T40S2dBoh004/ALo]ooV_oMol3Nol01gOoHol<aS[HA1mIf7mc
008@X`0Z=R`4@Cec000H2b0?918P3aPZ44H<A0@P001Rcf<@124006_i74dH3D0O80l48GodJiASmWoo
<7D40P0030Ef4T5Y120Di@@QJ[h4110710800DUE0Woo0P@203@007ibDLX000QR001_o1@9004002WY
Io]7<Q34248QRGOo<O4001@7:0l`4B`A70T006W@Ko/n[@`4:0dP2R@::0/8@QdVFo=;CbX;:TeWobD9
120HIF=jETdbCBH`>a5Kn6:_6DL000Qoo`001Woo1`0000l<1R`H30IF>1dV>RaongOi8DL8IENo<ST8
0c@J@1h00c`M00dh6P00E_=ko1C7144]jP00C]THcA0W>1]07P05?1d0<T0N<1H007on=R`QBEOO8@h@
1cPK?1dP40@2OomWmV_hLam=FSbJC;lQ9`b28DL003gHEAeA_5FmIQmcnCH/6@Del5]oA5lM1QSUCa4@
a2g:C;l@2@0080ld6R0?20Aom`00E_42Oo`06BF8>Lm9d2dZ34<48A1T>FeR/@@Q9HQ2SPQR<HeR7eWO
AGPXDT2LKalf;0QRDe<n6PP4008l6`0:40TH2f?o000j/W?kOoigog?oMol4Nol03W?o>eP8HddO?3]k
<ReX2892d2gZ;@a0W1PY10450000;0@2004@9Woe?RdYZGoo108X4d@O;1D006]AHa<M1QCUEgA_7f7o
ECi=MP@Q6@HjC@QR9FU<ncSDEKeWWfOIEkEoo``7000H2ghc48<8HRG:4<@<Y6oL64T200002Bh:Io/n
d0c5044I1U_hLolPc00400001TDcNo]7<dne30@X2`8L200B:0XL1`00@/eKldk_BoM_oa@958=5k5oI
?J/MAaUX?O1SndU?0P0027oo0006Ool800003R0?40Q4nA2TDa5oo6?E248I;Eoo5:T@23`L@1h3?1d0
3D0M50TLRWoo36@@a4:>2696LRUE108d640N00Dl7@09@1hl7A@976Yoof?OFol48R0?00907P04=1T0
06KN@Zl2000032EX<N/8HPQR@XiCDbF8000Da25X7BH8HP<0000B36Df2bEW2696d6_hCa48HQT5OomW
n00070h@2B2[GmD@a6oi0Woo021OE@@Q6@EooE;B;BYoo7_k144HiE^DJoQ2SPQR000I1Bg9?XhUR@00
144004K@;98@1c@H80h40Woo144jKG?k0Woo00=kog?oMol017_o029goeoo16=3?4@OHOYBC@00@]1?
Da34=Raoogom@JTPi1RS468000PT40T004X;OoAooF]M000d6TPO?1`@1b3VOo@Ha080000`124DiACU
0008HSY=Cc<jC@@Q8DLjKBF94:@8HSI]LohL3D2`OolUR@00:LY2/39=2688PV_m12400@007DI[ocoL
4@L08@A17DUC]WooHgmOGFNOHo/nlDmI6BLH22@90QP601@P22P:34DDi5_BFo=_oToh104DPdKABe8I
1`QS3<I?6FK?3:@<QP0027oo0006Ool800003@01001V6`00HmEooEMC000]me_o004P3d0L00@l7@0>
@1h/5000Ool@YB57Fi@UR1dXASl001P<>1]07PDl7@08@1hd6001<OEOocIK000`5P907P0??1dH31@Y
Ood000AS0240000Q3<HMcR7_6F/8Y@0Q008000808@0D0004@AT;8Lh4@`RU7LhVDR7`4@P001U[Kol@
J0P620DL3Cmk5BUGo`9Wo`0OGoloO14828AOodF_Cc/^]P006Hdbn3Lj;[HMcQ4828@4H`000248Q0RU
001Soa@Y40TP3`00CYlURAU[Fol00f_o00=WogOoMol01G_o029cocLF26<lVTbnHZl08ADY;[LM[P00
6FaS_f:_ALX]AaRS000D:2PC60/005J^FoT@IA@:@1i87d@O91<006mbG_88HPD00005148<YQ483:H4
@P0400008`c6DoUKGB@?LoECo`QS000<YQ483<L4@P01Fml001`:80`<10A2?miGobH`28<000AS6F/f
eD_mFomSof_oCeX40RP:80P00Q@5018H1R@980P002W8Gg8j/000104003j]@a<4@PA18LYSmR6938D2
0008Ool000Ioo`T0000=4>Q:CQC4NoYoo4J>000ngd]O000`5D0L?1/00c`M00i07SPK20A9e4:_6>EO
eDK@000mf3Ug004<1S0F0T0N13`M01107RPC0044@P0050Xh6d0N?1e07S@I005JNToo4@PA2@8QcP06
7JdIJaU[6F/M[B7>0R7?00TV4BjD@k]?mU_oDWm;O27?6F/00QDY01LM[BIBComco`bX20H81BR<@a8<
aTOoFomOoe_oAolQl000Gom1[dHgHolM[@0Q00H0000?0244@PR43<LM[DooKol00109108TDDNo249?
of_o00=_o`03Lomgog_o00Eko`07MomSoa476@QNKEY=8Lh00`0001400BHAK/mVce8/?JTLa00080hl
6c@I40P000@Q108`640O00907P06?1`D2@aROo]coe_o15Oo00ECodooCom?oe?o009Ko`0AEomKoe_o
Aol810Q3AiikofOoDom;odNoBomCofoo?S4D1`02<1402S0@70/00A36Gom_oe_oComGof_o0WOo00<n
MQ2X10800R@900LH1Q0440@D1A`7:0XLC@0200002`@R70LT2@QS025CoRjC>cUkob7?14<00`0027oo
0006Ool700003TJg@Q1KD`00B]1ong_j;JT08Dmo=Wd40S@H@1`3?1/2?1d03d0N91840Woo0012SU^D
9HP004JbOolY;@02<1I07P04?1d01d0N?1d/5AQ;88hd640N00<l7@07@1hT4P00@_mWodWO?G/00Sej
0cek13ej00PiFC4f;A<HbaT:30HTlSej13ek00/iFBdC24@40Q`>20H/c6M@124nCULd009KE@07@Y0D
a@b2Del]:QbV?W@00UMO1e=O00eGGeKoDOm9_bdC00480`@240MSo`00Comco`0=Nol04W?oBoh08BD5
DRaFC4G:?HPiJ4WZIZmk<W;aIXi=jac4000X4`947`06>1XP3b1];1Dl7D0N0S`M00I07S@H52P000@2
20@420<8108020PT02400A0750P40P00108420<040@2000<1C0@:0dT2b0=:0lX3AP9108000@R26D<
IPQ30P0000H80QP690TP21P650D340@03AP680PX2SU];9<X2R@950D000b47JeKod=n00H000Qoo`00
1Woo1`0000hUjDYb4:@<Y3ZACeDjT@QS4:QKoaC:50Tl6c`L13`K014l740N?1`81CTcHo`00269:N`<
Y000;Neooab]20Dh6d0N00@l7@907P04?1dl740N@1h3?1d01T0N?1dX4``500040`/@1`0740PD2A@9
66TH2a@:40P0110700/@21@980l`5S0G40P/c6m@000Y9SW90091jP09=HP<HQc4EQTLYP@000088Pa3
00L@H`0<34848@0000040Q0720D40UKo0457o7Oo3W_o00Igof_o;Y<0046YK]03Ma402G/ANc9g4G;`
IXiB:aB320<`5`03@1h2?1d00d0N@1`l6`03?1/0240K?1Td5B`C:1@/52`C;1D2;1@02R`E;1</4b`C
<1@`4R`A;10/4B`D0c0C00</4BP?:0l00RP?00H/3b`>:0lX420=60T260P00aP780TT2`02:0X00b@9
80PP200290T2:0X012@970LH1Q@5110400HD1AP680PT2B0850H=0008Ool000Ioo`P0000<7NaSoaU[
024000A26F]KoeOo000T4T0L1c`K00m07S0G000iFEoo5BT08@0028@Ul6?oEol002`E@1h01S`M0T0N
1S`M0T0N0SPK00<h73`M?1d05S`M0T0N00Ll71@9;<agT@007>@Y9P02;FL01BTV3688@G>C;BX03000
00X48@Q246@@Ia0600157Ag<;[=coa1ko`05MomGo`bU7<Ab/@04OcD02glEMa5blF[?CN/<@A0W>1]0
7S`M?1/00S`L0S`K00Pl6S`H>1Pd63@I=1Xh6C@J0c@H13@F00<d5C@C<1<00R`C00L/4B`C;18/4BP@
90hX3P02:0l2:0d280d0120;80XH2AP60Q@600<@1A@550D00QP600<L1b0880P00b0800@L1aP660HD
1@X@10032080000000`000Qoo`001Woo2@0000T4@SKIAom?od_o?mdQk`0074`00S@K00<l6c`J?1X0
1C`K00E0730F008DRUNo00=Wo`06HomCWaT]148`5T0N43`M0d0N33`M140N2C`M00M07S`L40Pdkfm@
000H``02:BH02RU6:BHLi000BT]Zl11S00088Pa2211S00T@I12448@88P00:A4jnQ47Jol00W_o17Oo
2g_o00Igof_o;Y<004V[O_@4OcD027lEOaAnle7/000L3C`K?1`3?1/013`I?1Ph630H0S0G0c0F0S0E
0S0C0R`C00P/4R`A;14T42P@910T3R0=0R@>00<P320;80/00Q`:0QP:00<H2AP860T00QP800<H1Q@6
40@01a041Q@52a0400<<0`P200003`0027oo0006Ool@00001QP8>1/d6S0H<1T`6P<l6P<l6`05?1a0
73@I60`40`05000010`6918l740N;S`M00D<1UG]G/d001S3008U1@0::BH]IbeW488487NDAK@40000
46<946@01a1S1240034[@e`4@V?o009ko`Mgo`Yko`07MomGo`bU68=bSWlbOg<017lc00Mnm4F[000X
4T0L?1/l6003>1P3<1L01C0E;1@/52`D;1<00b`B00@X4BP@:10X3`8T3`@P3@8L2`<L2P0370TH2AP9
008D200350LD1Q@6008D1P8@1AP@100330<40P00010000Qoo`001Woo4@0000L00AP<91<X5S0H<1T`
6P02?1X4?1/2@1`00c`J>1/d6P02=1X013PK?1a07T0N;C`M00M07S`L20AVCdi;000DX`039@D02ReW
=JPYAP@Q8@Aom23700088A1S211T00H88P009>QGN000Dol3Nol9Mol9Nol01Foo=aH003eXO]0017mC
00MoLglb>DL40R`F@1Xd6002<1L0130F<1D`5B`D0R`C0R`B00LX42PA:10T42P?90lT3P0380d280/2
70/270T360T00aP850LD1`0450H340@00a0540@@100F40@00``320840@0B0008Ool000Iooa<00007
40HP3bPC;1L`6C0J=1X00S`J13`K00@l740L@1a070=07Rll7@07@1hl70P4JW1::P005:<00bD500X]
IcFX=JPU9@00BT]Zc`a3000<@PL@I0072480011SBUT8PcLfMol00W_o3GOo1W_o00EgoeOo4<D@HV8/
009o<P=oD`06Oa8Ta106=1Xd5c0F0S0E00</4b`D;1<00R`B00<X42@@90l00R@?1R0=00@P2a`;70/L
2@<H2@<D1`@D1Qh@100430<<0P@2104C0008Ool000Iooa@00007104<1Q`>:1</5S0I<1X00S@J00<l
6S`K?1/02C`K0S`L:c`M00M07SPK005j/S67000H``039@D032U6<FLibCFX7>@484i<H]@@H`00348@
H`@@I00734<001RUAU8E9cZeLol00W_o47Oo1G_o00Egod^K268XaGJ`00=o<`07OgAj/0@Q70dh630D
;1800bPA0R@@00<P42@>80h00R0=0Q`<11`:00DH2QP950PD21@7008D1P0440H@1A0540D>40@<30@7
30<00`P320840@0F0008Ool000IooaL0000340HP42PE008`5`<`6P<l6Q0l6b@l7@07@1hd6P00OoXP
i0@P6<<00bD500`Y9ReW=JQ22SW99BD004Y;J/l<@`00348446@01P@Q68EBd`00Aoago`9koaEgo`9k
o`0?Lolg=P00EL]n/GkAOa=oMEW[004`4b`D914T3b0?00<P3@8P2`8L2`8L2@<H2@0550PD1a@750LD
1P0340D00a0430@@100A30@>30<340<01@`330880P@1000030@01@000`@00P0027oo0006OolH0000
0``6814/5@02<1L00c0I<1X`6P02<1X2?1XC?1/O?1d01d0N=1P007oi4888@AS300<Li0039@DY9SFX
008ib@08;FL@PP@PNkA1T`00100@H`8@I00634848DF_6FXZLG?o0W_o5gOo0W_o00aKoa2T;@EbSg[@
Oc=oUCDW40H`4bP@90h280/370/00a`:70TH2@0260T00aP850LD1P0250H340D00a0430@<100730@J
30<540<00``32044000H1008Ool000IooaT0000430LT4BPD;1H3<1L00c0I<1X`6P04?1XE?1/J?1d0
1d0N<1H007oh3688@ABS00<H``0=7>@U9ReW<JPibC6X8BD482DVOo@Pa`0022400Q1S00D401bV>YT<
aFOo009koaQgo`9ko`0>Lolg=QARFLYZKWlBOg@HHaP:<14T3A`;70XL2@@H2@8D1`<D1P<@1@P<10/<
0`H@0`@<0`D@0`D<0`<@0`0550<@0a0330880002100>2003100520000`@0Oomoo`06Ool000IooaX0
000450XP42PC;1H4<1L0130I<1X`6S0J0c`J5S`K5c`M00M07RPC005on0@Q244DX`046<<00b459DH]
I`02<JP03BeW6<@004Y/J/l<@`0022888@00?S0005?o009koaUgo`=ko`04Fol<HSTVIP`2Oc<01000
90d/420:0QP90Q@70a@61A0500<@10`430@010`43P`30a0300<D0a0340<01Q030Q@30a030``30Q03
11@300<@0P`120000`P030`01PP00P`00PP027oo0006OolK000010`680lT4b`E1S0G0c0J0c`J6S`K
4C`M00I07R0@44EomP@P24445:<011S37>@UAReW0S6X00L]IbE63684@G^d@I<400020000111T:Fhg
=WOo0W_o6gOo00ikog?o7H/LPf7[Oe=nS`01:0dP2aP850LD1Q060a0500<@10`430@01P`44@`30Q03
1Q@3110311@30a030P`30Q030Q@30Q@200<@0@`030000``01A000``00a000P`00PP01P`000<807oo
Ool01Woo0006OolL00001@P270hT4RPD;1H01S0G130J0S`J7C`K0c`L2C`K00I0720>66Mom@002444
5:<011S37>@UAReV0c6X00/]Ib4U000U9Wod9@T48@a3@JlMZV_o01ako`9go`9ko`0;>W<4056YOiE5
B0`6:0`L21@640D@100:30@D30<00a0350<D0`0460<550<460<00a@340<@0`0340<250<250800a01
400@0003300=4005300340023008Ool000Iooad00005108D2b@A:1</5P08<1L2<1X4?1XB?1/2?1X5
?1/<?1X01c`K@1`L31b8Oo<000Q100@DX`046<<Li24U;FH2<JP2>LT02BeW7>@004I;K_0eC46_@Y8Z
400QNol02EMJ001=JGnE448H22@;40H@100730@H30<240<00a@360<H0`0270<260<350<011P370<L
0a`30QP300<D0a0340<00a0300@@0Q@2404D00H@0003500D8A@000H@008D008@00<<008@0004500@
01003008Ool000Iooal0000540HP3b@C;1D/5P09<1L00c0H<1Xh6P0A?1X5?1P3?1X00cPH<1P`6002
<1P7<1L01c@I>1TH2RBZNm8000Q100<DX`076<<Li2459DH]IS6W>LT00ch:00PibAS40011jdFd=L/0
05NK4G_o00=gog?oLol026oo0W?o00]gog_oNomSo`Q1?BMoM00080TL210400<<11d<0`0640<D0aP3
60<L0a`S0Q`31AP300HL0a`S70<L0aP350<240<00``340<<0P0240400a@0524D0005400352464002
52400a@0400<00023002400011@Q500@00`027oo0006OolP000010P560dT4BPC12`D4C0G00<`630G
<1L0330G0c0F0S0G00<`5S0E<1D0130E00P`5S@G>1T@23T<Je00012211cT00PU9ReV>LPn2DY[DXaF
cUl?0V<`00MO3beW000<HP@Q4@MWo`0AOol016?o?iT^e2jB1RYa0RjB00`g5DOmGom[ofOo4>Ha1WmC
104P2QP730@O30<240<011@360<L0a`S0R0S0Q`30aP300TL0b0S82<L8a`360<D0a0330<00P`200<<
0@`040000a@Q110000@D01@Q524D8@D@0005500D8A@Q524@000330001Q00500D8A@Q400<00Qoo`00
1Woo8@0000H40Q@:70lP4B@C91@;;1@6<1D4<1L<<1D4;1@5<1D0130C<1@`53@E0S@F00D40f;?ARX0
01cT008]IP0:<HM22DY;DXaS3fnaNo9olgolOod2Ool00g_oHolMJ`0200000aT5KmIon`0?Oo/00gOg
:R`8PP0;00002`A35@Tbed_n4@P]B7k@208P2Q@630@07``300D@0a@360<H0a`300<P8`0372<L0a`3
00<L0`<P8`0670<H0a@340<<0PP10PP000<<010052400Q@Q00@D0100400@00<D8@03500@010000@@
00<D8@06400<00`0300@01@00Q@Q00<@00`0Ool01goo0006OolS000010`660`L3R0@2B@A00<T4R`B
;1@00R`D22`B0b`D0b`B1b`C0S0C00D`530E=1@/53@F00<`5@08=1@X4A@9005oT`Q17>A:B`9NkP03
Kg5kmgol00Yoo`07KolE20@R38D004k_OoP03goi00IWTa44000080AQ28872:<03@R2128000003:H0
8C9`HJl40B@:60P@1Q04008<11`<0`0440<D0aP370<582<370<011`S82<T8b@S0R0S00TH0a@34088
0@@0200<010052400aPQ00<D8A@050000Q@Q0QPQ00<D8A0040000a000Q@Q00<H8A@Q40000``000H@
01@0524D8A003008Ool000Ioob@0000520@D21P;70dP3P0580l00b0@80lP3`0690l391400bPA910T
3`0490l01B@@;18/4R`C<1@00b`C00//530E<1D`5B`D;1<T41`;50P810010080000650Ph^B4W6>EW
nGOo3goo00QGo`00:Hhj=2E78DI_mG_g3Woh00ISTaTW000080RS4BD44DD25FH02a554BTE2`b624@0
000Q;YIAn00040@00R0:00@L21P740D@108<11@<0`H@0`0450<L0b0S82<292<282<01A`S70<L8b0S
92<00bPT00XT8b0S628@0@`0200<0100524L8@<P8@DL8@04824L8APQ6242524262401Q`Q824L8APQ
524@008<0006400D01@Q524@00`027oo0006OolU00001P@130HD21P;70/H2PDL30@L3@<L308P3@TP
3P0490lX430C<1<2;1<04RPA80hH2P`61080000130DU;CFcASMkofooAi`UkP00=[=go`]oo`Ako`9o
o`088J`Q24kJC_]6V134Dc1gmPeon008NoMWe3H:268000AQ4BDEIP<IY`045FHIYaEV6D/26Bd02QT<
4<X@Z25aOoQ_oacZ00040A@60R0:00HL21P740D@10`430<?20<010`340<@0a030Q@30Q0300<D0a`3
82<012@S00HP8a`S70<L8b0S92<3;2<03bPR928L8A@Q400<0100624L8B@Q:44X@RQ1924P8@059240
1Q`Q624H8APQ724P8@<T8@03824H8A0000<@0005500D8A@Q400<0008Ool000IoobL0000420@@1Q08
40P;50P00aP960XH2P0860X01AP;80d`4bPA30@00P0000T80aA9;FlmmUnOIom3VRG^3<D00P0000D8
Q26/>_AGogOo00Moo`Eko`Ago`9oo`0:Iol0048_B[PnEDkJ7BL]bF?CNoL;OoP02W_gMoIO/T:<8DH4
@@002:<AAAEV0QVW0aEV0ad^00dQCae@5>/DabD5?Pa;ofooHolU;P00104D1P0280X011`850H@10`4
3PP200<@0Q@250800aPR00DD0Q02508L8b@S00<X8`8T8`@L8`0892</@c13<4</@RQ2844H8@8D8@04
624P8BQ1;482<6800baR:44X@@04;4800bQ2944L8@0272400bA1:44/@P02;4800bA1744@80034003
52400a00301oo`07Ool000IoobP000840QL80`0A30@D22@>38@f4F_oDolf/aU926<0000Q8Ja7^U_o
Jomko`08Ool4Nol9Mol03W_oOomoobib8>E>ncHC;K0n=3Xb145>kVoeNoL:OoP05W_gMoI?<B698DH<
HP00164A9AEV6JLEIQVW5FHEB1T]7BhQL25a7E0@b@a21@0000LA24ooIolYC`00004@1@0260P01106
20<40@@12P0100<40@`1624011`R00DD8A01504P8RQ200</@P03:48T@Q`Q008H8@09724P8Ra2<68d
HS1R;68XHR11008H8@04724X@BaR=682>8800c1R:68XHP02<682=6803S1R:44P@Aa1844X@RaR<68d
HS1R;68P@A@Q4202400352400a00301oo`07Ool000Iood@00007025oo`QS7FY?o6OoLol02goo0W_o
3gOo0Woo00a_o`00CP]6^2f_8D/]/4Zi268I1E]aMoH:OoP02W_gKoDfC0R21484@@00144=11EV0QVW
0QEV00PEBAT]6BdQCb5a7Bl@Q@PQ0P0000X8@BE7<_Lg>BHa1488HdOoGolUka000004100@01`Q8442
94404R11644D8A@Q824/@S1R=68`HRaR:44P@APQ524H8B0Q;48dHP8hPP03=88`PRAQ008L@@07944/
HSAR>88lPcb2=8800RaR00HdPSR2>88hPSB2;682844032A1;68dHSR2?88hPS1R964D8A@P400D008D
8@03500@00`000Qoo`001WooA@0000=So`0PFol02Woo0g_o4GOo011kogooOoljmA23C]Xf4b5;7@XQ
Bd:G=Rl003H:JoAkm`Uon003NoM[m2F700@00004164=11556JL35FH26Bd01Ad^8DlQLAC[34800`00
00@481RS@ReKo`9[o`07Folbm`R426=?oeoo7Jd03@0000@801PQ944/HP8`HP07:68L@APQ724X@C1R
>6800SR200@dPRaR844H8@8D8@08724`HSR2?88lXSRR=88XHP8P@@03:48dHSR20090X`06?:<dPRb2
<68lPSbS0T2S010hPRb2964P@BQ2=68hPSbS@:<lXcB2:68H8A@P400D008D8@8@0003301oogoo00Io
o`001WooAP0000=36AeZJol00goo27_o4GOo00QkogooOomOg@Q1AU56UbV>0ad:00L]/4kI<O04@CHZ
ImAkm`08OoP02W_gIo@I1@0P2888XPRS3>@A9AEV0QF600TEIQE76BdI;Ae?8DlMD12W10001@0000@D
PSY?Gmaco`9ko`0:Mom_oe^L?S4bDS[F4>LZ42h@3:D700003P@0300P8BaR<88hPSB2;88PHAa1924/
@SQR@8<2@:<02CR2<88THAa1524@01P0;68hPP03>:803SBR:88PHB11;68hPSbR@:=0`cRR;:8THBaQ
>:83@:<01cRR;88PHAa1944dHSR200=0X`04=:8XHQPQ520540000``0201oo`07Ool000IoodL00003
Jol007?o009oo`EkoaAgo`=oo`053:A=JTkJ;K0Q:`037@X0224[>S=>nc7^000nBfoeNoL6OoP01W_g
MoIO/QC4164A9@8AA@8EIP076JLEIQEV5FHIBaT]7Bh00R5?00<I30a300001P0000D8@BdVDm]cog_o
00Ago`09Nomgoeoo<_L4H`R4DomOoaU[00@00008200H8BQ1<88hXSbS>:8/XP8TH@04;68dHSb2@:<2
@<<02CRR;:8THAa1524@01@0:68dPP02>:800cS2=:8XPP0286402RaR>88hXSbR><8dXRR2764THCRR
0cc300LhXRbR864L@BA1=88hXP03?<<00cBR:88H8@04400230000`P0101oo`07Ool000IoodP00003
Kol<aGoo009oo`=koaAgo`08NomoogooDol484Zi?S@QB`DM2P07;Jm2MdZi6@H@Xe<`KoD01Woh00Ik
mgOfFg4<X`R24DD35FH26JL25FH021556BdI;Ad^8DlM;Q39124800001PQ1;BISogOoNomgo`=co`0Q
Momkog_oLomOoc;g16<8Q5?oGolEBP@0300L8BaR=88hXSc2=:8/XRB1:68dHSR2?:8lXd33?<<dXRRR
864L@A@Q008@0004:44dPSRR>:82=:801BR2864P@BaR=8800cRR00H`XRB1644P@CB2>:82?:<03SRR
;88PHAQ1944`HSRR?:<l`cbS=:8XPQ@P4004300220000`@0Oomoo`06Ool000IoodP00003129;^SJc
009oo`03NomgogOo01Ago`=oo`064>I1RSkH8HdI2ASX0QT90Qd:00PQ:c7AC]XnK`00:JQSdg_g1Goh
00IkmgOfDc08HPRS4DD46JL011EV6JLEIQUY0aT]00@M;QC[48D400T00006348eZE_oMomkogOo17?o
0WOo0W_o019coeoo<_L4H`c6FmiKNbA2<68dPSRR=<8`XRR2964/HSR2?882?:<02cc3><8`XRB2764H
@A@Q320<02A1<88013BR00DXPAaQ844/HSB200<hXP06<:8PHAA1824dPSRR0SbS00LhXRb2764H8BA1
<68hXP03?:<00cBR:68D8002300320031008Ool000IoodT00005<WD<aE_oOomko`0EMol027_oOomo
odNk249ZWBhA5@P24<H26>P02aT97@XM2RE]?SA>nRV[1216[FoeNoL017oh00IkmgOfC^h8HPRS4DD5
6JL00aEV4DDEB@026Bd011d^6@`@b@a22`0000H@HSg:Gomgog_oMol6Lol2Mol2Nol01W?oFolYcR24
L[DdPP8dXP06<:8XXRB1964`HSb20SbS00`hXSS2=<8/XR21764H@A@Q320<0211<883=:803S2R984L
HAa1;68dPSBR>:8dXRbR864D@A`Q=883>:801cBR:88LHAPQ944`HSRR00<lX`04=:8THQ0P30022003
10020008Ool000IoodX00004Kol08GooOolEMol2Ool05G_o26<Q;cjg7D/<YPb53:H@aQC76>PI2Ad:
8B/][dJh@WH<HQT5Fg5gmP04OoP01g_gMoIKL@R3164AAAEV00<IY`<EIP075BXI;AT]6Bd@b@b42240
300000H@HTH:Gomgog_oMol8Lol01gOoNomkgVNL869^/cB2008dXP08;:8TPB1Q964`PSbR?:<l``8d
XP03;:8TXQaQ008H@@05544@80`0844`PP03=:801S2R984LHAa1:68dPP<dXP05;:8PHA11724dPP03
>:801cBR:88H@A@Q844`HSRR00<lX`04<:8THA0P200410030008Ool000Iood/00004DolZ;gooNolC
Mol02W_oOomood=i5:AZ_RF_4>L<Q@R40Pb500d<YQC76>PI2Ad:8D/f4dkj>U0003H:JoAkm`03OoP0
27_gMoIO/QT5000<`a555FH26JL35FH36Bd00aC[4:P<@P0>00001QB3CR]SogOoNomgo`Qco`06Momg
_D:e>FY9[3B20SBR00dXPAb1764THCB2?:<l`cS2=:8/XRRR884HH@035440110P300P@C220cBR00H`
XRB1764L@BQR=883=:801BbR864@@1`Q=8800cRR00LdXRR2744H8B11<68hXP03?:<00c2R864<8002
10060008Ool000Iood/000049O4IBF_oOolCMol3Ool01@QS4>PjTaTZ38D00`R40Pb500d<YQC76@TI
2Ad::Hi2MT[I8DL8@Dk^KoEkm`03OoP01g_gJo@b2P00040=115500@EIP076FTI;AT]5@/@b@b31200
3`0000<HXdI^Iol00W_o1W?o0WOo00EkgU>L76=J]3B200<dXP0=>DPHHAaQ:68dPSbS?<<h`SST>BLa
9bd7:@H00bSV00dLPa0P944haEH?I[ANTT58988Ta2cV>@M19`03ADP01CdW9:<HHRcV@BL00TE800Q9
B4E8>@HPHAa1:48dHSbR0SbS00DhXS2R864<00@000L000Qoo`001WooC00000AGod8@Oomkoa5go`08
NomoogooDol8@WkD:Ll@aPH8Q00=38D<YQSX6>PI2B4[;K1:fD:I144QAUnbMoH00Woh0W_g00MgmU<_
5>@000A14BDEIP026JL01aEV5BXI;AD;4<T@Z0Q1014000056:=BdF_oNomgo`05Lol2Ool01Enm525R
mc1R>8800SRR00`hi6LG668L@BaR>88lXcc3><8m9TG<JcT3Nkd01V/i?J`XaA@Q;:A[60=oo`06Mia5
c34WALecNW^m0W_M00I_64g<;<DPPdh@Nkd2Nmd01gNLFRm1J2AQ944`HSR200<lX`05=:8/PQaQ2004
00070008Ool000Ioodd00004?eQoogooNol@Mol027_oOomooa3V;HY25ae;38D728@06`b54<HDaaSX
7@XUKCXcC_/akP00?T]_mG_gOoQkmgOfKoEKTSH:268000RS4DDEIQVW5FHI;@025@/00a2X2684800A
00001P@P7<ABdG?oNomgo`=co`07Nomoog_o7:E[FRA1<6800cBR0115RW^m9<DL@BQR=88hXSS2=<8/
XR21<FUg_GooMkdeBP<H@@0CBLekgWooMkeF;eH`IaL/aBARCLekgWooMia9J3BR:88H@BASLeX00Woo
00QNDCc3<:8XPRQR<68hPSbS0SRR00@`XRB2664800P000Qoo`001WooC@0000A23GooOomko`mgo`09
Nomoogoo@gT@Q7kE;Ll@i`b500P8Q00F38D<YQC76>PM2RV>@WM:^AT64:=C<6oeNoMgmVoeEk5?3T:<
8DH000A14BD25FH01QEW6BdE2a3926D88A<00004120TiF>INol2Mol00g?oMomoo`02Ool016oN644P
@Bb20S2R00L``f;EOom1cAQ1964`PP02<:8022bR9:8LHAbTLg]oog=k9:@364404eIBOomooejc<<<X
XSUY;>DPHCdWNkeoog=J@>@dXRR2644P@F[f009oo`06CJ/h`S2R:88XHS220cBR00D`XRRR884D@@@0
00P000Qoo`001WooC@0000E>37ooOomoog_o00igo`08NomoogOo000YdRVC6@T@YP8<Q@L8Q08<Q@0:
3:HDaaSX8B/adDkJ?Vl002VXHm<2MoH03enb=VhIIbVX9FL8@P003<<AAAEV5DXI;AD;38H48@0E0000
0`@P=Ra[o`02Nol00gOoNomoo`02Ool03VmK8:@L@BQQ;88/XS45Lg]ooeJd648PHBR2;:82::801b21
564HY6mkOomcNb2T008D@@0D645BDWooOom>3RR2984PPAaQ864m9g^mOomcFScT=:8TPQQQ845VmP9o
o`06CJ/h`S2R:88TPRb20S2R00H/XRRR884HH@a010080008Ool000Iood`00003OoPa1Goo00=oo`03
NomgogOo00]go`9oo`06>_D]ja364>H8H`b40`b520R401X<Q@bV4<HI2B5;?SA>nRg<1216[F_dKoEC
<1eX0204@PbR144000R24BDEIQU;6Bd@Z0Q45`0000<DPdniKol00W_o00QgogooOom[Ne9cANlH@B1Q
0RB100Q1[7ooMkeW>BC5744PHBB10R2100PHHA105:A_NgooLg/LY1110QA101=BDGooOom:3R21764H
HAQ1744i1g^mOomcFScT<:8TPQQQ845VmP02Ool01Df[><8`XRR296400bR200DTPR21664@@0PP00T0
00Qoo`001WooB`0000AoM7j`569nd@=oo`03NomgogOo00Ygo`09Nomoog_o000n]0b51488@`R400D<
Q@L8Q00538D<YQ366@TYSP02AYL01Q2S6>AKLF?CAZd<Y0D00007164=11556BdE2`b61240600000DY
1TniJom[oe/i009oo`0?CW<LYEkF7:@H@AaQ789J]7ooF]E>DSf]544HHAaQ008HH@07444<@1B4Kg]o
og=k7:@00a0P00I>DGooOom5kQQQ564254402aQ1=@Mk_GooLeXdi2bR984HHB11I_H00Woo00E=ZcC2
;:8TPAaQ008PH@06884LHAQQ440<@0@P2@0027oo0006Oom:00001WmCO]4001b3GP]on0=oo`03Nomg
ogOo00Ugo`0:Nomood=i8J/@aPA10244@PQS28@638D628@238D05A367@XadDkJ>Rh002g9Dc0nB`QR
1448PP1000048@R23@@A9aD;4:P8A00I000050@P:@HfTBh@MkeoogNm7>L884H`?Lh@@111:BQg_Goo
AQ0aRf;g58<4440020a0220DPfmkOom_NaR43202220014YAOomood7>0a1000`@81A1<@Mk_GooKeX`
`bR2884D@Aa1I_H2Ool01TVZ<:8XXQb1564D@@8DH@04444<@0PP120:0008Ool000IoodP00009125o
4WlB12088AbS?DQfSgol00=oo`03NomgogOo00Qgo`9oo`0626=;5@bV12808@Q310R410b51`R40Pb5
00`@aR4[?SA>fREX144f2RVX0204PQ4U3<<228801Pc34BDA:13938H48AX000848004<JeoogooF]H2
00001AS5F]H@P`PP>Ld00Woo00@U:A1SG]HLi`D48007100@PfmkOom_Na2312000P@000A:<GooOolm
c@D<800::>Ik_GooKeTTXab1664@@211J_H2Ool01DV:<:8TPAAQ34000`Q00PPP00<4800000002@00
27oo0006Oom800002TfZOg@88@PQ68<PY2cUBHUn/7_o0goo00=kogOoMol01gOo00Qooe?o3:@DaQ27
1488@`QS10R41@b520R400D<QA3W:Hi:^4:I008<HP036>@000RS008AA@8A9@8AA@035@/@Z0Q401`0
0005125cVWooOole[@0300002THa=Jd<HfLiOomg_@aS000icTHa1P0000D<HfmkOom_Na2300<00004
BS5oogoo@Lh242003@`P2208823VMkeoof]I7:8DHAA1440P@F[f009oo`04BJX`XR21564412000`0P
0000000;0008Ool000IoodL00005=BMon112120DHP028:@012C4<@IEbglB0goo00=kogOoMol01WOo
00]kogoo8J`Yc67E7BXDia3W4<H<YPb500D8Q08<Q@T8Q0053:HE237AC_XjD00300000`S35FHEIP04
5FH00aD;38H48@0K00001P@P8@MogWooMkdHaP<0000;6<IJeSF]Nmmoof<H124001cWE[D8@P050000
1@aSKg]oofmk36<00`0000A:<GooOom1cP8D8@08420880@07<Ig_GooJcTDHP8<@003320L@FKf009o
o`05BHX/XR214404800A0008Ool000IoodH00005;@IonAR3000DHP038:@29<@00cTWIReonP03Ool0
0g_oMomgo`05Mol03WooNol08A38>EM2]cjE=S<U[aTZ5@P@aPbV38D428@238D828@02@bV6BXf<dkJ
?Tl8H@103>@EIP036JL011EV4BT@b@Q45`0000L480Q1348DPaRT8<A>D002Ool00ekg4:D000030000
1DHaHaQoogooAS400P0000<8@U[F6<H01@0000D<HfmkOom_N`aS00<0000>BS5oogoo@LhD@A@Q4208
80@06<Eg_GooIcT<@P888003320H8FKf009oo`04AHXXXQb1440B0008Ool000Iood@00006469FT59?
34488AAR1R2T00@Xa3DWHRaoV@=oo`03NomgogOo00=go`9oo`0AGg/002SV<Q4Y[2F>;Q4nUd:H=WD^
<bW`7F`E2A3X4<H<Y@0238D928@02@b55BTb<d[jFgDN:AF75HHIY`025HH00a554>T<QP0;000020@0
1208@@Q1348@HPa2124210003`Q28>DiZTi<E/mG3U<`Ee9KDg_mOomooSY=144E1P0300001Be/Oomo
ofLi36<00`0000<]JeJe2480100000D<HfmkOom_N`aS00<0000?BS5oogooALhH@AA1444<80@05:Eg
_GooKg/DY00000840008521VmgooOolmJAaQ564<818000Qoo`001Woo@`0000H@HWoj8:@001AR68<8
8:@012C4>BMN2goH0goo00EkogOoMomgog_o009oo`04<S04000016<26H`03`c600000148:S<k6CkH
@_Tj]S8c:LlMBa3W4<H<YP0238D628@01PbV5@PYlDKHHo/RJP@EIP045DP@b@b3224800005AB2;DM9
jUZ_FXmS4fLdIc1C44i_>JPY9ReY;V`gD3]`=`l^cBZ;9V]G=@02Ool02f^j?Z/<a0009NT000Q2HaQo
og_N=Jd0100000<48Dic:D/00P0000L48@Q2:DYg_GooKg/<H`03000014XaOomood7>0QA101<@@0PP
100DYGNmOomcW5:D8>L48@@09>MkgWooJeTLXaAQ4408800B0008Ool000Iood800006225oncDV000D
HQR32B2T00DTa2cU?DQ^KWoj00=oo`04NomgogOoNol2Ool061U90004@RHa?ia3gS]J;Y@Qka3W0240
00c68PlfeclK@_Tj]bhA9JhI:QD84<L<YP8<Q@<8Q00638D<YQTZ;S9GVCL_0aVW00XEIQV/7Bh]AbTV
9@DY9RD5488480800004244ejU<DFo42@i82?g403SL_=dm3TWNgIa5ZcfmeGo4c3RY[9RXVBSJ^Mm`2
Ool02f^k<^hR2@0P5DHbK2e[Nmioog_N<H`01@0000Lmkekg36<001cWAS5W>P02Ool01Vmk36<00000
249>L`9oo`0FAPlHPaR358<880005:Eg_GooIcTLieJeBU8icUkgOomogdXa444<@0PP120B0008Ool0
00Iood400006101oncTW000@@QR32b2T00@Ta346BHUn/0Aoo`9ko`9oo`0KKol<P`006H`kFT_oCom;
odOo@mhoW2kF9S4A20A20004@QTY;[Dnf4;i>[H^<RF_7D/E2136008<YP0?38D8Q0b54<HMK3ZFCmHN
2AVW6LPN2BiECaUc<W]d009ST`0KBdiBT3EX4888@ART<X/kD3;]:V/VBRHZ8PTV:RI[<^dkD5O@NgAo
e6of@i4c3S;^@e5[fW_m009koP07MomOo3:e000fDd9CG_L00g>L01EJeQS600000124<HaS67>LCW<H
aP0048@YBUkgNmi_N`aS00048EJeMkd00Woo019_Nf<HHaQBT`Q2000DYGNmOomW>@@Q5:E>Lg>LOomk
gdYB3683120C0008Ool000Iood400005Oo]1J000448HP`0=8:@012C4<@I=ZWlB0goo00UkogooOomo
oe?o024A22kFAol01Too01=;odOo?i`g62Yc6H`<aP002:DQccKG@a]2nCkG=U@Ycae/6@T@i`033:H0
2aD8:LmCVBm>8TTN2B[/DkloLC]@=bl00SL?010g;d>BHkANKBTV9@EFdCma:V/QjAg88NTR2BHZ9V/b
k@8gC`0@</djJSZ;;X`^cBk/<]4fTcZDAcUOogOoDol48@b5124648@010Q2000000Q2112400P8@P00
000000Q2;F/mk`Q20P0000DLigNmOomood8@008DY00>48@000005:Eg_GooIcT48@00124LidHa?Nh8
@AL000Qoo`001Woo@00000Ion6H]00088AAR68<>8:@012C4<@IN2goh1Woo00A>laCV:W=3gPY?o`0B
Bom7od?N?i`ZU148148000AS7F`^eSkj@]Pnf3Ie:O4QSATZ0QD800TMK4Kg@o<_CR[/@mU?oc<>9TX0
0b7Y00TV:Rj]?i5GEF:`>JQJSEoa=`l00Rk=00`bkSL_>e0kL3M?;ZdVC1EX2884@P0Q004200001@01
5BY;og?o7Lh07@0000@e[GNmOole[@D000055:Eg_GooIcT48@0M0008Ool000Iooch000044:DLY7oj
>BL3000010PQ344DHQAR0QR32B2T00DTa2cU>BMV;Gol00Aoo`05Nom=ZRH@?ia;o`0>Col0>4_o?i`g
62Yc5DX8Q000024IBBIB?cXnf4;i>[H^<RW?9JlfM4Oj?o4klEOo=g4VJaVW5FHIIag88PTZS3<>>g0f
kSFZ:BLZBc;]<`hbkC<c:XlQkaV;3<H08P00148<aaU;6HdM[aU;148Qcf_o@ehP00000cW>Nmhe[@05
00001ABUMkeoofLi12407@0027oo0006Ooln00002GkA001oUGojOa9Ejbd64484800200001@@P344@
@Q1256800QR31R2T00@Ta3e8KXmko`Aoo`03C[4c64Oo011?o`08Bom7od?N?i`^eRHa6H`8Y@80000G
3<HZ<c/IAca2nCkH>[M;>dOeAoA?ocL?9V/V:RI::X`bkSL_>g0kD2k=6JP8PP02000214802@0Q0000
00AS3<HIBb7`;YHbf002?e`01BjF6Hd08CJDCol08P0000<8@V<H=Jd01@0000DDYGNmOomW>@@Q01d0
00Qoo`001Woo@00000@4@@008@U>cP9on`05OoMW4E6Z9<@88@04000010PQ344DHQAR11R300@LY3U8
Ne5ko`9oo`04Koi3gT?NBol;Col2Bol2Aol01D?N?i`oW3]J=aP00RjD0RYc01XQkaE:16<000R48Nlo
>DOoBga2nD?iDolkTSL_=`lg;c]`=dlo]S<B9U8Mc@R40004@QDZ0aV=00lQl2Yd<]PoG4>NAmm;odoo
Aklo?2HB26@4@U?o:Q00900000<aS2e[0000100000DDYGNmOomW>@@Q01d000Qoo`001WooA00000`4
8@00148Q[T?nEomkof?LA]@]J1221203000412001`PQ568iB7=dLomgodoo00E;o`M?o`0;Bom3gSnL
=aP^eRYc7JdEBQ483<H8Y@0216<02PRU4@PIS1E:16<000A24BTZLdOo0V?o00Y;gd_iAoY;ndooFomC
ocKH6F/4H`80000516<E2B7`=aY3WP02Aml00d_OCom?o`02Dol02DooBmlo?2HB3<H002jFGol<QP0L
000010Q2:DXYBPQ21P0000@48@00000000848@05;F]kgWooIcT48@0M0008Ool000IoodT0000<0200
013X:W=?oe?oAolbmQg=5<D8@P@P100000H8@CU8FiIKodooBol2Aol00d_oAom;o`04Aol01d?N?i`k
FSJA8HT@i@Q2008000041488Q0cV5BT26DL03a2T12400000248IAbI^<a8gLCoD=m0g[c^CAoQ6g`02
Col01F?oBmlff1f=28@00`0000P8I1D98Q4b^3Kj?ca3OTNo0T_O00Y7_cKj8M08I000;WASobhC000<
Q1`00004=Jekgg_O=Jd:00000bU:AS5_N`02Ool00fLi1240000L0008Ool000Ioodd0000@024001D9
=eQ?oeOoCom3gC;f9S0EB@RU00008BF;Fe<2Ji@2He<3Jc<3K^l03Ej=ET/iR2T5368000PQ7<@ajCK=
;bdcKSN?00<g[`06<fi2k4I=CTe73CN?0S_@0SN_00Hgd3N_?oDom4NoB_l2A]l02D[oA]lnW3/I;W@Z
<b6^4>P8Q002000010A33<HIBaf^0R7@00XIB`0Q16<oG6Oo9K000127<CDI:Q`00004>Lioogoo>Ll:
00001`@Q5:DicV]JOomW>@@Q01d000Qoo`001WooD@0000T4@@0028@QkcoMDomGod_o;]D00PAS00A6
dGL@Ma1^k`9V[PANS@03ET]1bAc4008000096<@ajD;/>k<kd3O@=hlcKS=>00<cC@0A=hlkd5_aLoHk
d3=^;bdZk2[;:^`gSdMm;a0VJc_@Aei2_@03=SP00cjL@[e6g`02B_l04DKO@[hj^BXc8Jh@j0bV0240
000Q5@UCof?o4>P000QS5<H07`0000@@Q4icCW<@Q0d0000436<mkd8@124M0008Ool000IooeD0000=
148080006BY;od?n000beG^eNe5VcUI;BNX00T790U8[00I9jR3T000@XcI:=g82>m0013N?<ddZjbJ[
0bJ:00@VJRJ[://_3@8cCP0?;`dVZbJ:9VXVRRl=>m8jnA564@DVJcohAalf>2Fb008UT@0@9K8Y/bge
<QHf>3YJ@[e6od[oA]m2_SjK@ga;ge_o28EF0008Ool000IooeX00008:PX@ibjeOo]o/f:]@LTPi08<
HP0=:@DiR3V8488001WX=hlg[c==:^`VZbJ:9VX01R9900<VJRJ:9XX00RJ:00LVJR999VXZbc=^>iLE
I`0204401a45=i97GCkL;M@QKb5>008M;@<QCP098FlUTRgD<QHf>3YJ>WXjFRFA05P000Qoo`001Woo
F`0001@QkS]kJomonVJ]7<@000Q1120001B2:@DHX`00:NTg[cN?://VRRIZ3R9900HVJRJ[<ddkd2Yc
0202000020116JPom4/o=WPUdR5>7Bd66@`38Dh28Fl00aT;004@b01G0008Ool000IooeX0000D6F/<
aU?oMomoof:]000I1B999VdQ9P00368HX`009@EC/SN_://VJPlRB@059XX_3CN_=g8<Y`04000020RS
:/m;OT;m;SDU/B5>7Bd96@`011T;38H001e>E`0027oo0006OomJ000054=L001[og_oMomFS000>Zdg
[c_@=g8aZ@00224480a1Ec0kd2[/9VX38TT49VX78TT022IZ://gScN_;SL4@A34144200001`0P4DHk
d4/o>YX]m2Fa008QCPPI30046@/<Q@008I1G0008Ool000IooeX00003IolMcW_o009ko`0?AU0Q9SN_
<hh_;3>?=k4^RacU368U1EMa=m0ZjbIZ008RB@068VTVZRl>://VZRJ:1B9900XVRR[/<hho]S9G14<M
Ic9/6DL4HP80000:024IZ3]fAcln^bhE9K8QT25^7Bd66@`011T;26@002VcE`0027oo0006OomJ0000
4bjeComkogOoNoljDAVY>m0_3BJ:9Z/_3C_@>i=K4FMC>m0cKRJ[00<RB@079XX_;EOcKoHcCBJ[9VX0
0b9900`VJR[/=hloe2hg0284@Ri;@a0fKA46044200002`AR6LTomD/o>YX]m2GB9K4QSb5>7Bd011T<
00@Dj@Q4004I:UL000Qoo`001WooF@0000XQk`12IomkogOoNom6]A4U>m0_3@8VJP089Z/cCC_@HoIo
nS_@;`dVRP<RB@079ZXgSeoeGoDg[b[/9XX00R9900dVJRJ[<dhkd4LN3:L001e7;T`jSSI]7FL4HP03
00001PRS:/m;OTKn=UPYl`8UdP038HlQKQd]00<I30045>T4@`@R6B]G0008Ool000IooeT0000DJol4
HgOoMomcogOoEi`4HSN?<dhVRR999VXZbc=>FoESmS=^://VJP<RB@07://kd7?fOoU3mC==9XX00RIZ
00hVRRl=>m13O1e_000<YBh[;T`QbRG98HP<Y00P0P0000L08156>k=;?cjk;SDYl`029M800b6@8DhM
;@026@`011CY14<48QT[E`0027oo0006OomI00002fOo9S5kogOoLomgof_o000VJSN_:^`00RIZ00LV
RR[/;bdgLRl=9XXVJP038TT01b/<>m1[U7MAFo4gSbJ[008VJP04:^`g[coD<UT2128022WZ?/l^C14V
4@HE1A34044300002PAR9Va7WTLO=WP]m2Wc9M8U/R6?0R5>00DI31CY14<48QU;05L000Qoo`001Woo
F@0000<oNdOoNol00g?o00Ugob7_3<@g[c==9XXRBBIZ9XX00RJ[00<VRRIZ8TT00R9901PVJRm=Dk9>
:dX:Dflg[b[/9XXZbc=^>m137a38000A23k?Ci<fS@RS1688P`bS148400001Q45<fi7GT;M<SHYl`<U
dP8QS`058DhI2`Q4128IB`1G0008Ool000IooeP000045BT<aV?oNol3Lol027_oEol002J;=jlZbbIZ
8TT49VX48TT00bIZ<fiKD@02;DH04DL`>m0_;B[/<dhkd3nm7Fl000A39LY?TeOe?/l8X`1102000PA1
100000L4HQW9?oE;?cZJ;O@Yl`039M801RFa8HlI30Q4128IBeL000Qoo`001WooF00000A?o`00Lomg
o`=co`07Momcob7_3<@g[c==9XX02B9900<VRS>?AXd00PaR010bBc_@=hlc[S_`>i<^5`A3000A23k?
DmAKmTLa4>D080P000062:<Zcd]nA_hfF2Wc12GB00DU/Ae=26@48QU;05L000Qoo`001WooF00000A[
oaV<Momgo`=co`08MomkoeOo000VRcN_://VJPLRB@049VXVZc>>;JT200004246;b`kl3_`?m@fV`bW
0008I26YAe5CmE_fBg8E9`11200000L08156>k=;?cjk;SDUd`049M80125^26@4@QU;E`0027oo0006
OolM0006124>0003124J000:12400d_o?g]ko`05Lol01g_oLolV<@c4=jlcCBJ:00<RB@@VJP0<9XXZ
k3NA8@H000@P000UIb/<;ddjVA3Z0P0000P@aS9<Bi9GmU_hCk<QZ0AR2@0000H4HRI/Aii77cIh:O@4
9M80125^26@8I1C6E`0027oo0006OolL000212442482124;0007124:0003124:0002124324841240
1@Q25@PIBUooNol01W?o00Moof?o000VRcN_:^`VRP039VX39XX05R/;<fe3T0Q1124DX`@P000I1AEX
4>X08P0026@MJ3Z^Bi9CmEOfDmDUb@R32P0000D<i3MbAee2gBhE00@UdP038Fh8I12W05P000Qoo`00
1Woo700000<48@Q224800`aS00<8@P@Q12402P000P@Q0`Q20P@Q2@001P@Q1P000P@Q1PQ200P48@Q2
2488@TMK249_og_o1G?o00QkogooOol^L`c4=jlcCRJ[0RJ:01/Vbb/;:`/[2bm==jlk3`005:<U1AS4
1200000Q0040000Q4<HUbCZ^Bi9CmEOfDmDbKA3U02002@0000H0@AW9?ke;?c9G:O<39M800b5^26@@
b01H0008Ool000Iooa`000031248@PaS008@Q08<H`0324848@0000T000031248@PQ200<8@P848@L0
00031248@PQ200<8@P031240000000<000031248@PQ200<<H`H8@P0336=_oaTY009go`Ico`07Nomo
of?o000VRc_@;dd00b/;0b/<00P_CCO?;Z/001BS:BHPi0aR100000H8I1eX<T`nkd]bDoD2EoH00cJ>
5BL0@@0:00001@RS9Ze3?3jk;O@00bGB00<QKPA34>T0F00027oo0006OolK00000`@Q248<H`0448@0
0`aS24848@0:00000`@Q2488@P0236<224800`@Q0000000400000`@Q248<H`0436<00`Q212400002
00001@@Q248<H`aS48@00PaS0`Q200P48@Q2248<He_o?aQkogOo0g?o00A_ogOoLomgo`9oo`059S48
a3O?=ll_C@04:``02Bm=<fdgkbI9000Pi2TV8>@8H`0300001`0Q4<HUbCJ=?^m;LU?d009GmP03Ae4Q
Z0AR00/000054DI;eTLN;SDYl`029M800ae=14<E2P1H0008Ool000Iooa/000041248@PaS48@35:D0
0a2436<48@0:00000`@Q248<H`0236<224800`@Q0000000400000`Q236<@Q00348@00`aS24848@02
00000`@Q248<H`0348@236<2248312401PQ24:@^<E_oNomgo`9co`03Komgog?o009co`08Nomooe_o
000ZSC_`<hh_C@<[3005;ddcSS_`9JP48@028>@00aBS128000020000300Q4>DUbCJ=?^m7<DoCDoEG
mTMA;P/8X`/0000:168N2TMN=WP]m2GB9K4MC@A26@]H0008Ool000Iooa`00003248<HaBU00<DY@03
48@<H`@Q00X000031248@PaS008<H`88@P031240000000<00003124<Ha2400<DY@0848@<H`Q21240
00@Q248<H`8@Q0045:D@Q0aS248312402`001248@Rha4:E_og_oMomcofooMol017?o00EgogooNolQ
k`S4008gc`0<;dd[32/<:``_CCN_>o0YJ0aR6<@DX`A30P0000l00@@R0000013T9LTbC3k_Bg9CmEOf
Bi<bKA3U02002`0000T8a4=H@]db=RWc9K4MC@@R6BX0F00027oo0006OolK00000`@Q248<H`045:D0
0a2436<48@0:00000`@Q2488@P0324800`@Q000000030002124010aS5:DHaQS60QBU00<@Q0Q21240
0P@Q00<<Ha245:D00QS600@DY@aS24848@@00008125_oaSWMomkogOoLomgo`Ico`08Nomooe_o000R
JC_`<hh_C@8[3007;ddg[c_`:FP48A2226<00P000108QRhGDkmWnck_4<@000QR7FLfSDMADoE?e3J>
5BL0@@/00009020Z/d/o>YX]m2GB6B`001e<05P000Qoo`001Woo5P001@@Q0PQ200<@Q1BU5:D00QBU
00<@Q0Q2124010001P@Q1PQ200<48@0000000P0000<48@Q236<00QBU0QS600<DYA2436<00`Q200<<
Ha246<H00acW00DDYA242488@P@Q00<00004125[oc9cNol2Mol00g?oMomco`05Lol047OoOomoobHa
2:@cSSO_<fd_CB/<;ddcSS_`<RX000@P0P00014<Yd]OJolDj@A28FYSoG?l<V`4@@005@DnkdMA>/lM
I`A100`000087JU7?4;m<SHUdQT/000MCB`00003<01H05P0009H0003E01001`002@000Qoo`001Woo
5P0000<48@Q2248010aS00<@Q1BU5:D00QBU00<@Q0aS1240100000<48@Q224802@Q200P48@000000
00Q236<@Q1BU0aS600<DYA2448@00`aS00@@Q1BU7>LLi`8Q20047>LDY@aS36<3248212401136:S5O
og_o17Oo1W?o00Ugog_oOom_oa474BDgT3O@;hd00Rm=00TcKCO_<bh<Q@0Q26D^<eOO;Q001P0000TI
9d8@<P`^C3H=5BH@i@R20200300000P4HSL@Bei2gBhE6@/001e>;@0000D/07P0O01d02@002H000Qo
o`001Woo5@0000<48@Q236<011BU00<@Q1BU5:D011BU00<<H`Q212400P@Q00<8@PaS36<010aS1PQ2
0`@Q00<<HaBU6<H00acW00<HaQBU48@00a240QBU00<HaQcW9BT00RDY00@Q21cW48@<H`@@Q08<H`03
Jol@YGOo00Igo`9co`03Momkog_o009oo`0@NomGoa3W4:AK4CO_>o0gccN_=nlkl3=B7FlQTE?OEol<
000020A2DoMooe_g:TdA1@RS020:00001a45AiA7?cjk8Fh001e>02d00005401`07`0K00<000V0008
Ool000IooaD000031248@Q2400@HaPLDY@0348@<H`Q20088@P0336<@Q1BU00<@Q00336<8@PQ200L8
@P0748@Lib489BTQ21cW5:D0112400@DYAS67>LQ20@U:@058@PLia2436<@Q0045:D01A24Jol^<G_o
Ool01GOo00ikogooOomoogOoFolb]@A228A7_EooOoQKm3O_0S_`00Hom3^c<[@b>EooGiP@00001`bV
CmI_od_j;a0EI`AR00P00007044N2T=lAalUdP007Dh0;@0000D@0700O01/00`002H000Qoo`001Woo
5@0000@48@aS5:DHaP8Li`036<HDYABU00DDY@0848@<H`Q2248<Ha245:DHaP8DY@0348@<H`Q200H8
@P0436<@Q1S69BT2:DX00bDY7>LDY@0348@011BU6<HLib480RDY00DYBRDY9BTQ21S6008<H`035:DH
aQS6008HaP055:E;gTooOomko`02Mol067_oOomoogooKoloNa3W000^Lf?oDolIAP0Q001JSWoiEk<n
kSZ=@Va2;3IjIomGEQ@000073:I7oVoo@oHRB`cT04401P0000H8a3/BAedfF0008Fh]00001A00L01l
06`030009P0027oo0006OolF00000`aS5:DLi`037>L00aS65:DDY@045:D02Q2436<8@PQ236<@Q1S6
7>LHaQBU0Q241@Q20PaS00l@Q1S68@P]Jc6<;F/U:AS648@<Ha245:DHaQcW8@P012DY0R4800HHaQBU
2488@Q246<H37>L00bV]8DYSo`05Ool027?oDolMS0c6Dom_obiD024600000godNi8U9P02488011dX
Dce6/P0P5`0000LQd6_oIolg/aW92<@0800300001P0P6JQ;eD;M128ULBd00005501`07`0K00<000V
0008Ool000Iooa8000@48@0736<DYAcW8@PLiaS65:D00Q2411BU00`@Q0aS36<<Ha245:DLib487>LH
aQ2436<424804`aS48@DYAS68@P]JcF]<H`]JbDY6<H@Q0aS48@HaQcW8@PU:BU:00<]J`04:DXU:B48
6<H224800`aS6<HLi`027>L035Oo5:EWog_oMomWoclH5<H<YD=kJolUk`X000087<Aon1c4120<HRhG
B_D08@<00003400D01@000/D00032000000000H0000B300mdG=JIeY2TBD5600<000P4@E3U5=M=29D
@dP0700000P01Q@000<@000040001a@00P@01Q@000T@000000040200C01@04@050000P0000d403P0
D00/00P0000@0700O01/010070180002D0000c@030000005000270001T@0D01@0500>00<018000Qo
o`001Woo4@0000H8@Q245:DHaQcW9BT2;F/012DY8@PHaQBU0Q2411BU0Q2400XDYAcW9BTYBS6<;F/U
:AcW5:D@Q0<8@P0636<@Q1BU7>LQ22e[0SW>00@e[Be[8@PDY@8<H`065:DLibDY;F/e[CW>0Sg_00Hi
cSF]=JdaS2e[6<H248@03QS67>LLiacWJolM225[6BT<HbhaHom_ob5X124=00001WofKc162T]ONol<
a0@00004700X03@0L003M0000g00H01P0002H0001VP0M01L04`0;00800D0000=500`06@0N01l8Ga2
HBPYR29;<e5?^75:N0000W`000Ed03`0500/060000=d0006K00T0000800h07000g@000ID02P02008
02P0F003M0001U@0700000@0A01h00=l000EL00/00P0>01h07`0N01D00@0501`07`0N01H0600=00`
03@0J01P030000D0000:F01h07@0D00`03@0G01T03P0100@0008Ool000Iooa40000<36<@Q1cW=Je6
<DicBU8mkbe[8@PHaQBU0Q240aBU0a2400<HaR48=Jd00THa00DicRe[7>LDYA2400<8@P0<48@DYAcW
9BTaS4HaBU92436<:DXLia240PaS00@HaRDY=Je6<@9>L`03AS4mkcF]008aS0<e[@03<H`U:AcW00<L
i`06AcUoog?oKolo6@b40PQ20P@Q3P0000Eom7oo9Jl000@Q00L00005D01l07`0K00X000420001a@0
?01X07`0N01D00`000H0000F:01d07`0M00nDW?oHolkUE:]N21l07P0?00d05`0700001`0L01l07P0
<00300001P@0@01l07`0I00<00@00005401/07`0O00L000200002R@0N01l07`0I00d03`0I01@05P0
0W`000EX03`0900@0700009l0003J00P000000800004801/07@0;004000015P0E00P00@00P0000@<
05`0M00P010000Qoo`001Woo4@000P@Q00L@Q36<DYANmdic=JdQ20027>L00aS65:DDY@035:D03A24
36<8@PQ25:DaS4YBAS4aS2DY7>LHaQ240088@P0636<DYAcW9BTaS4Ha0UJe00m242e[8@PDYA2436<@
Q2DY?NmBU5[FE[E2436<9BT00acW00HQ22DY?Nm>Ld8@:DX27>L014;@:LdDY@aS1P@Q3P0000D@YB4V
124001C600P00004801l07`0H00700001Q00E01l07`0E00<00D00009401`07`0K00400A3288003P0
009l0003J:L40000008H0006000806P0O01h01P0100000@X07`0O01H00H00004G01l07`040020000
1E@0O01l06@050000P0000<@0500M0000W`000Pd0000300@0700O01d02001@0000@h07`0H00@00<0
0003D00D000000@00004<01l05@0100?0008Ool000Iooa<00006124U:DicG_M:DRe[0acW0aS60QcW
00hHaQBU36<48@00248U:D8@?NlYBR487>LHaQ240PaS01P@Q1S68@P]Jd8@E[ES65Je>LhU:AcW48@<
Ha247>LicUJeG_MF]D8@;F/LiaBU48@45:D01CF]F]IBU36<8@P00QcW00<HaQBU24801P@Q6`0000@L
07`0O01P00P00006301L07`0O01@00@0100000D@0700O01/00@000<00004>01l07`0A00500001@P0
J01l07P06000100000@X07`0O01H00H00004G01l07`04002000015P0O01l02`0100000D40500O01l
030000800005501`07`0K00<000500001100L01l03`00`0000<l00000000100000@/07`0M00D00l0
00Qoo`001Woo4`0000L48B48CW=NmdHa:DXLi`028@P031cW6<HLiacW8@PLiaS648@8@P@Q248Q208m
k`05:DXQ21cW6<H@Q00236<01AS68@P]Jcg_DY@00V<H00ABU3F]8@PDY@8<H`0;6<HaS5:DHaQJeTHa
<H`Q21BU48@<H`0248@01QBU48@aS5kgDY@e[@<Q20047>LDY@aS2485124K000011`0O01l06002@00
00D/07P0O01/010000@00005401`07`0K0040003000013P0O01l04@01@0000D806P0O01h01P000@0
0004:01l07`0F006000015`0O01l01000P0000AH07`0O00X00D00004C01l07`0<00200001A00L01l
06`030001@0000D40500O01L00P0008000037000000000<00005201@07`0N00H000?0008Ool000Io
oa<00008124Q24icG_M6<BU:7>LQ20@Li`<Q200C7>LHaQ24248<HbDY@Q0mkbU:8@PLiaS65:D@Q1BU
7>LYBSg_DY@00ekg00Q>Lc6<6<H@Q0aS48@U:DYB0Ukg00E:DSF]:DXLia240088@P0936<@Q1245:Da
S5kgDY@e[B48008Li`036<HDY@aS00D8@P031240000001T00004701l07`0H00900001A@0M01l07`0
<000100000D@0700O01/00@000<00004>01l07`0A00500001@P0J01l07P06000100000@X07`0O01H
00H00004G01l07`04002000013@0O01l03001@0000A<07`0O00`00800005501`07`0K00<00060000
13`0O01d01P01`0000H@04`0O01l0700500?0008Ool000Iooa<00007124Q24icG_M6<BU:7>L00R48
0acW00HQ22DY9BTU:AcW6<H248@01bU:@Q0mkbU:8@PLiaS6008DY@046<HU:CF]CW<2G_L04E:DCW=6
<Be[5:D@Q0aS6<HicUJeHaQF]Cg_;F/U:AcW48@00PQ20PaS0Q2400HaS5kgDY@e[B487>L26<H00aBU
48@<H`0236<22482124J000011`0O01l06002@0000DD07@0O01l03@000@00005401`07`0K0040003
000013P0O01l04@01@0000D806P0O01h01P000@00004:01l07`0F006000015`0O01l01000P0000D<
05`0O01P00P000@00004C01l07`0<00200001A@0L01l06`030001P0000@l07`0O01000D00003100X
06`000=l0003C000000000h000Qoo`001Woo4`0000L48B48CW=NmdHa:DXLi`028@P37>L01R489BTY
BRU:9BTLi`8HaP05;F]243W>9BTLi`046<H01R48<Ha6<E[FG_MBU08mk`03@Q0YBQ24008<H`0=9BU6
<EkgG_M>LcF]:DXU:AcW48@8@P@Q24800PaS00H@Q36<G_MBU36<7>L26<H25:D00a2436<<H`0236<2
24800`@Q0000000H000011`0O01l06002@0000Dd07P0O01l02`000@00005401`07`0K00400030000
13P0O01l04@01@0000D806P0O01h01P000@00004:01l07`0F006000015`0O01l01000`0000D@05P0
M018010000<00004C01l07`0<00200001A00L01l06`030001P0000@l07`0O01400<00004100T05@0
M002O0000gP0F00@000@0008Ool000Ioo`X00003480DX1BP00HDX0077>4YB4icG_M6<BU:7>L00R48
00lLibDV;FDaQSFX>LPibCW8=JL]IQcW;F]243W>9BT00QS60QcW01@Q22e[EZaNm5kgE[DicRU:=Jdm
kbDY48@<Ha24=J]JdF]?LhQ[AV<40UkS00<iaBDS36800P@Q0PQ200H<Hc6<G_MBU36<7>L35:D348@3
36<00`Q212448@0J000011`0O01l0600200000H@06@0O01l06@0300400001A00L01l06`010000`00
00@h07`0O01400D00005201X07`0N00H0004000012P0O01l05P01P0000AL07`0O00@00@00008200/
05P0H00T00`0001<009l0008<0000000501`07`0K00<00H00004?01l07`0A00200000`@0C01d0003
O0000f`0<008000A0008Ool000Ioo`X000047>0Y@3FPLh03Mj0025[0:D0]HBU7CW=NmdHa:DX37>L0
3aS6:D@eXd84Kf5k`7NQEZE21SgV9BPaS48@>LhQ20026<H05AcW8@PYBEJYMj=W=5[F?NlQ21cW<H`m
kbDY48@<HcFSKf=ohG_4KfIBR002BTH01eJTJd5cP5[0?N0@P0@Q0088@P05;F]JeU:D;F/HaP025:D3
48@336<224800`@Q0000000I000011`0O01l06001`0000<H05`0O0000W`000<d000000000`0000D@
0700O01/00@000<00004>01l07`0A00500001@P0J01l07P06000100000@X07`0O01H00H00004G01l
07`0400600001PP0;01D0600>01H009l0008<0000000401`07`0K00<00D00005101D07`0O0140002
00001bP0O01l07`0K01001@001<000Qoo`001Woo3@0000`i`7oPOn1_H1BP00048B48CW=NmdYB;F/3
9BT02AcW5:D@Q1BU?NAk`5kT;FTYBP029BT06c6<@Q0e[AcW5:DLib48;FYFZ7_1On9S4dHa:DX@Q1BU
;F/icRDY48@aPW>0Nl5_K6<DBU4]J`029BT03RDX9BLeXfmPOn1[@1201248@Re[F]IBU2e[5:D348@4
36<22482124K000011`0O01l06001@0000<803@0L0000W`000=/03P020001@0000D@0700O01/00@0
00<00004>01l07`0A00500001@P0J01l07P06000100000@X07`0O01H00H00004G01l07`040030000
0`P030040002000010P0;01407000W`000P`0000000D0700O01/00`01@0000D<06`0O01l03`00080
0005C01l07@0@00D000300000`@00000000?0008Ool000Ioo`d000047>1oh7oPH`0200001`@Q8@Q>
LekgCW<icSF]008icP07=Jd]JacW6<HYAGNPCVD00RU:0RDY01/aS48@=JdLiaBU7>LU:DH[Nl9oh7_2
DXl]JaBU248@Q2e[>LhU:CFRLh1k`5[;G_MJeT8@:DX00b4800<LiaS5Ib000WoP00PLh000124aS5[F
DY@]JaBU1@aS0PQ20`@Q700000DL07`0O01`040000@l0006I01h07`0M01402001`0000D@0700O01/
00@000<00004>01l07`0A00500001@P0J01l07P06000100000@X07`0O01H00H00004G01l07`04002
000010`0G01/04001@0000AD07`0O00`00800005501`07`0K00@000500001B@0M01l070060000P00
00=H07@0<0001@0000<X01@000003`0027oo0006Ool=000011cPOn1oh6<00P0000H48B48CW=Nme:D
=Jd2;F/023F]@Q16<DYB@Q0mjGNPBTD38@P02AcW:DY2436<6<HDYBeYEZQgXP02On004G_1<HL<H`@Q
0008@RU:@PeJ`g_0On1>HdYAG_MJeSW>7>L00QS600@LiaS66<EW809oh0098@4<HaS6?NmBU4ic>LhQ
21BU00<<H`032480000001l00004701l07`0J00580001c00F01h07P0G00X00@000D00006100L07@0
O01/00@00`0000@h07`0O01400@00003100D06`0009l0006B00<0000000<05`00W`000=L03003000
0`0000D@06`0O01h00`000800005401`07`0I004000300004P@0F01l07`0;00000@0801d07`0M01<
02`040040100<01X009l0006A0040000001@04`01@0000<L06P06000400027oo0006Ool=000011cP
On1oh6<00P0000H48B48CW=NmdYB:DX47>L022e[CW=Nme[ANl5>IB487>L26<H01RU:?NlaS1cW8@QS
1`9oh005LhQS1WNP7>848@0200000`Q2;FYJb002On002G>08@E245kgE[DicQS648@DY@026<H03QS5
Ib5oh7oP9B8<HaBU6<HLib489BTQ21S648@236<00`@Q0000000O000011`0O01l06001`0000HP07@0
O01l04@0100300000b00A01/0002O0001V`0100000P03010009l0007C00<00`0200H04P0H0000W`0
00Ih0200D00h03`0J002O00027P0600L05`0A00h04@0L002O0000d0000000002000013P0N01`01`0
0`0000Pd07P0O01T00`0001007000W`000I/01`0F01d06`0M003O0000e@02000000200000bP0G00@
000200001@@0801X07P06000400027oo0006Ool=000011cPOn1oh6<00P0001848B48CW=NmdYB;F/L
ib487>LDYAcW?NmJeV<CNl5BQQcW6<H25:D032U:@Q0e[AcW>LQk`WoPNl=:CTiTLh0DX@<0000336<e
ZG>2009oh008EZ4HaCW>E[EJeSg_6<H<H`8@Q0035:DHaFLQ009oh0069B88@PQ2248<Ha240aBU00@@
Q0aS24848B400004701l07`0H00800001D`0O01l07007000100000D@02`0G01h06`000840003@01T
07@0009l000=L01P0600F00@00`0901D07@0N00H0200J0000g`000Md0400100001`0J01d0003O000
0d@010000003000010@0=01T06@00SP000E007@0M01D020000800009200`05`0N01/00`0401H07P0
009l0003N01D010000@00007101<06@0@00h04`0K0000W0000<H000000003P0027oo0006Ool=0000
11cPOn1oh6<00P0000H48B48CW=NmdHa:DX27>L02QS65:DDYBe[E[EW=7oQEZLLiaBU0Q2400`U:Cg_
=JdU:6LTOn1k`U[;:DU68W>0480300003@aSCVIoh7oPNl0i`Q24;F]>Le[FAS4Li`Q2008<H`0348@D
Y6LQ009oh0069B88@PQ2248<Ha240QBU0Q2400<<H`Q212408@0000@L07`0O01P00P00005701h07`0
O00d0006000020P0;00l00@000080300L002O00014`0400@01000`0000L402@0@00<0000400l0002
A0000a`01000000200001P`0801404@0>00400L00003300d04@000=400039004000000@00007200`
03`0100000P0;0000T@000</00P000001@0000<802P0A0000T@000@`0100800<010000Qoo`001Woo
3@0000@Lh7oPOn1S00800006124Q24icG_M6<BDY0QS600XDYA2448@YBU:DIcAohEJW6<H@Q08<H`0<
8@PmkcF]?NUk`GoQG^`]JQ24?N1cP1200`00018<HeJVOn1oh7>08@<<Hb48@Q1JeTic9BT8@P@Q248<
Ha23Ib02On001BDR2488@PQ236<00QS600HDYA2436<<H`Q2124Q000011`0O01l0600200000DD07@0
O01l03@000d00004;01`07`0A01H0008Ool000Ioo`d000047>1oh7oPH`0200001P@Q8@Q>LekgAS4Q
208DY@<@Q006:DYBU6LdOn5FYaBU0`Q200`Q23g_BTi[AGoPKfLeZaBU248mh7>0480300001ABRKf9o
h7oPEZ400PaS00HDYC6<DYABU36<48@212400`Q2369W8002On0022DR2488@PQ25:DU:B4848@236<0
0`Q21240000Q000011`0O01l0600200000DH07@0O01l030000h00003;01d04@005P000Qoo`001Woo
3@0000hLh7oPOn1W81cP?N1205:4IbU_Je:<:DL<Ha240PaS00LDYC6<E[EW=7_1CVH<H`0312403248
BTm_IWoPMj<ibaBU248003gPLh0@P0<000055:9cPGoPOn1BP@0224802`aS7>M245JeAS4Li`@Q0004
8@aQJd000WoP00PU8PQ22488@RDY=JdYBQ240PQ20P@Q8P0000@L07`0O01P00P00005A01l07`0J00@
000>00000`@0F014001H0008Ool000Ioo`d000047>1oh7oPNl03@P001U:4LhAohG_1Jd4U8@88@P08
36<Licg_F]IS4W_1AR@48@800003124Q26LX009oh003G^LDY@Q200800003>L1cP12000<000055:9c
PGoPOn1BP@0224802P@Q36<U:DHaCW<aS124000482eP0goP00Q68@aR248@Q3F]@Q0U:@Q20P@Q9000
00@L07`0O01P00L00005:01`07`0M00/000@00000b@0?000001G0008Ool000Ioo`d0000F7>1oh7oP
Ib048000124Q25:AJdicPgoPIb0<H@@Q48@]JdicE[E>KWNP?N4300001`Q2ARIk`WoPNl8eY`@Q00<0
0003>L1cP12000<000055:9cPGoPOn1>H00212401000124<HbU:0T8@00@aR3FR?N1W80=cP008Kf0m
h2U17>M:DT8@7>L48BH00005=01l07`0I004000400001`P0>01`07`0L01400@000L00004200X02P0
100600000`P00000001F0008Ool000Ioo`d000047>1oh7oPH`02000040Q2:DY>Le[FDXak`7oP=J4D
YBU:AS5BU4Ha=JQgX3FP0P0000L8@SgTNl5oh7oQEZT@Q00400000cW0Lh0@P00300001AS4Kf9oh7oP
CV001@0000L<HbDY?Nm23S688@@H`P0348001aBP6<4Li36<CW<icQ2402@000840006401X07`0O01h
040012`000EP07P0N01L02P000T00004>01l07`0601O0008Ool000Ioo`d0000J7>1oh7oPH`08@PaS
6<He[DicE[E:DF<4On1[@SFZ>Li243W>8@PLhWNP=J08@Q24;FIgX@9oh003JdDYB0@Q00@00008>L1c
P1200008@PaS8@QNj09oh003F/8<H`0000@0000736<Q23F]>Lh]Jb486<H00QBU00LHaR48:DXicSg_
;F/<H`0T00000bP0>00l000:D0000b`06008000:00001400O01l01`0G`0027oo0006Ool=000021cP
On1oh6LP48<DYAcW;F/2=Jd03c6<@PQoh7oPARLU:AcW5:D8@QBPMj0i`0aS9BES0`02On000fLT;FL<
H`0500003CW0Lh0@P0@Q48@HaR48@PQk`7oPLh0U90@Q00D00004248DYBDY:DX2;F/3:DX00be[:DXL
i`026<H00`Q20000000l000011@0C01800P0G`0027oo0006Ool=00001QcPOn1oh6<02448@PD<H`06
9B=oh7oP>L88@P@Q0P0000DDX7NP=J0@PEJQ009oh003Kf0LhPQ200H0000<>L1cP12000048@Q236<D
Xf]0On1k`241200000<8@PaS5:D00QS60RDV00HDY@aS00048@Q2126Q0008Ool000Ioo`d000047>1o
h7oPH`07000011S0On1oh360100000@DX7NP>L12009oh003Lh0]H00000L00003>L1cP12000D00004
>L1oh7oPBT0<00000`@Q?N0Lh02W0008Ool000Ioo`d000047>1oh7oPH`0700001360On1oh2eP1000
00LDX7NPG^1cP7oPLh0]H00900000cW0Lh0@P00500001A20Jd1oh7>024002`0000<@P6mP7>00Y`00
27oo0006Ool=000011cPOn1oh6<01`0000AFX7oPIb0<H0@000035:1gX7oP009oh003?N00000000P0
0003>L1cP12000H00004:D1cP7oP8@0;00000d80On0Lh02W0008Ool000Ioo`d000047>1oh7oPH`06
000012ePMj1k`2U01@0000HH`7NPOn1oh5[0360:00000cW0Lh0@P00700000bU0Lh1>H00:000012DP
Nl1_H120Y`0027oo0006Ool=00001B40On1oh6LP12000`0000D482ePMj1oh4iP00D00006241BP7oP
On1J`0aP2`0000<mh7>05:00200000@Y@7>0:D0480L00004:D1S07oPF/2X0008Ool000Ioo`X00004
5:0Lh2DPF/02Jd000fLPBT120002@P001EJPDX1:@36024000P0000@@P2DP?N1FX09[@003DX0@P000
00P000096<0Lh360G^1[@4iP9B0Lh12000H000059B1BP5:0AR0i`002<H001cFPAR1:@4iP@P0aP4iP
0:P000Qoo`001Woo?`0000<8@1S06<000QS000<@P00000000P0000<<H0000000YP0027oo0006Ooob
0008Ool000Iooo8000Qoo`001WoolP0027oo0006Ooob0008Ool000Iooo8000Qoo`001WoolP0027oo
0006Ooob0008Ool000Iooo8000Qoo`00ogoo0Goo003oOol1Ool00?moo`5oo`00ogoo0Goo0000\
\>"],
  ImageRangeCache->{{{0, 255}, {191, 0}} -> {-6.57725, -4.80099, 1.05551, 
  1.05551}}]
}, Open  ]],

Cell[TextData[{
  "Here is the example from the \"First Look to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0\" brochure, but now a true bitmap image instead of an integer matrix \
is read into the kernel."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bm = 
      ReadPNGBitmap[Directory[]\ <>\ $PathnameSeparator\ <>"\<cat.png\>"]\)], 
  "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    InterpretationBox[
      \("PNGBitmap"\[InvisibleSpace]"["\[InvisibleSpace]"<<image data>>"\
\[InvisibleSpace]","\[InvisibleSpace]100
        \[InvisibleSpace]","\[InvisibleSpace]100
        \[InvisibleSpace]","\[InvisibleSpace]8\[InvisibleSpace]"]"\),
      SequenceForm[ 
      "PNGBitmap", "[", "<<image data>>", ",", 100, ",", 100, ",", 8, "]"],
      Editable->False]], "Output",
  CellLabel->"Out[14]="]
}, Open  ]],

Cell["Is it a gray level picture or RGB picture?", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ColorImageQ[bm]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[15]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GrayImageQ[bm]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(True\)], "Output",
  CellLabel->"Out[16]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[Graphics[bm], ImageSize -> {100, 100}]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[17]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write PNG images", "Section"],

Cell[TextData[
"The function WritePNGBitmap[] will save the image data. Different formats \
are supported. With out additional options the RGB-\[Alpha] image is saved."],
   "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(WritePNGBitmap["\<written.png\>", bm]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \({"written.png", PNGColorGray}\)], "Output",
  CellLabel->"Out[18]="]
}, Open  ]],

Cell["\<\
Conversion to other formats can be done explicit by transformations \
of the pixel entries.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(WritePNGBitmap["\<written.png\>", 
      bm\  /. \ Pixel[g_] :> \ Pixel[g, g, g, 255]]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \({"written.png", PNGColorRGBA}\)], "Output",
  CellLabel->"Out[19]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(WritePNGBitmap["\<written.png\>", 
      bm\  /. \ Pixel[g_] :> \ Pixel[g, g, g]]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \({"written.png", PNGColorRGB}\)], "Output",
  CellLabel->"Out[20]="]
}, Open  ]],

Cell["\<\
The other way to force the image conversion is to use the \
PNGFormat-option of WritePNGBitmap[]. When a color bitmap is convrted to gray \
scale, the PNGGrayFunction-option can be used to convert the RGB triples to \
one gray value. The PNGGrayFunction must return integers in the range \
[0,255].\
\>", "Text"],

Cell[BoxData[
    \(WritePNGBitmap["\<written.png\>", bm, PNGFormat -> PNGColorRGBA]\)], 
  "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(WritePNGBitmap["\<written.png\>", bm, PNGFormat -> PNGColorRGB]\)], 
  "Input",
  CellLabel->"In[27]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Image Transformations", "Section"],

Cell["\<\
The package offers some manipulation functions for the image data. \
Here are the examples. Notice that Transpose[PNGBitmap[__]] works as \
expected.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[
      GraphicsArray[{{Graphics[bm], Graphics[Transpose[bm]]}, \n
          \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ {Graphics[MirrorVertical[Transpose[bm]]], 
            Graphics[MirrorHorizontal[bm]]}}]]; \)\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
Now the black cat is converted to an white one: this works for \
color images as well as for gray images.\t\
\>", "Text"],

Cell[BoxData[
    \(\(Show[Graphics[InvertBitmap[bm\ ], ImageSize -> {100, 100}]]; \)\)], 
  "Input",
  CellLabel->"In[18]:="],

Cell["Now we look for the number of colors in the image.", "Text"],

Cell[BoxData[
    \(CountColors[bm]\)], "Input",
  CellLabel->"In[19]:="],

Cell["\<\
You can produce images with random RGB colors by using the \
RandomPalette[] function. \
\>", "Text"],

Cell[BoxData[
    \(\(Show[\n\t\ \ 
      GraphicsArray[\n\t\t\ \ \ \ \ \ \ 
        Table[\n\t\t\t\ \ \ \ \ \ \ \ \ Graphics[RandomPalette[bm]], \n
          \t\t\t\ \ \ \ \ \ \ \ {i, 1, 3}, \n\t\t\t\ \ \ \ \ \ \ \ {j, 1, 3}\n
          \t\t\ \ \ \ \ \ \ \ ]\n\t\ \ \ \ \ ]\n\ \ ]; \)\)], "Input",
  CellLabel->"In[32]:="],

Cell["\<\
You can still do some manipulations by hand, like shear the \
image.\
\>", "Text"],

Cell[BoxData[
    \(\(Show[\n\t
      GraphicsArray[\n
        \t\t{{\n\t\t\t
            Graphics[
              bm\  /. \ 
                PNGBitmap[im_, dta__] :> \ 
                  PNGBitmap[MapIndexed[RotateLeft[#1, First[#2]]\ &, im], 
                    dta]], \n\t\t\t
            Graphics[
              bm\  /. \ 
                PNGBitmap[im_, dta__] :> \ 
                  PNGBitmap[
                    Transpose[
                      MapIndexed[RotateRight[#1, First[#2]]\ &, 
                        Transpose[im]]], dta]]}, {\n\t\t\t
            Graphics[
              bm\  /. \ 
                PNGBitmap[im_, dta__] :> \ 
                  PNGBitmap[MapIndexed[RotateRight[#1, First[#2]]\ &, im], 
                    dta]], \n\t\t\t
            Graphics[
              bm\  /. \ 
                PNGBitmap[im_, dta__] :> \ 
                  PNGBitmap[
                    Transpose[
                      MapIndexed[RotateLeft[#1, First[#2]]\ &, 
                        Transpose[im]]], dta]]\n\t\t\t}}\n\t]\n]; \)\)], 
  "Input",
  CellLabel->"In[22]:="],

Cell["\<\
The best method to convert color images to gray scale pictures is \
to use some popular image processing software like ImageMagic or XV.\
\>", 
  "Text"],

Cell[BoxData[
    \(bm = ReadPNGBitmap["\<tigerg.png\>"]; \n
    Show[Graphics[bm], ImageSize -> {192, 192}]; \)], "Input",
  CellLabel->"In[4]:="],

Cell[TextData[{
  "For some applications it is usefull to convert the ",
  StyleBox["PNGBitmap[]",
    FontFamily->"Courier"],
  " data to an ordinary matrix of integers. The ",
  StyleBox["PNGBitmapToData[]",
    FontFamily->"Courier"],
  " functions uses also the ",
  StyleBox["PNGGrayFunction",
    FontFamily->"Courier"],
  "-option to convert RGB images."
}], "Text"],

Cell[BoxData[
    \(\(imdata = PNGBitmapToData[bm]; \)\)], "Input",
  CellLabel->"In[7]:="],

Cell[TextData[{
  "Now the standard ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions for density,  contour, and 3d plots can be used."
}], "Text"],

Cell[BoxData[
    \(\(ListDensityPlot[imdata, Mesh \[Rule] False, FrameTicks \[Rule] None, 
      ColorFunction \[Rule] Hue, ImageSize -> Dimensions[imdata]]; \)\)], 
  "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(\(ListContourPlot[imdata, ContourShading \[Rule] False, 
      Contours \[Rule] 6, FrameTicks \[Rule] None]; \)\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(\(ListPlot3D[Sqrt[imdata], ColorFunction \[Rule] Hue, \n\t\ 
      Mesh \[Rule] False, ViewPoint\  \[Rule] \ {0.2, \(-2\), 5}]; \)\)], 
  "Input",
  CellLabel->"In[10]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Three Dimensional Textures", "Section"],

Cell["Here is the tiger again, but now in color.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bm = 
      ReadPNGBitmap[Directory[]\ <>\ $PathnameSeparator\ <>"\<tiger.png\>"]
        \)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    InterpretationBox[
      \("PNGBitmap"\[InvisibleSpace]"["\[InvisibleSpace]"<<image data>>"\
\[InvisibleSpace]","\[InvisibleSpace]192
        \[InvisibleSpace]","\[InvisibleSpace]192
        \[InvisibleSpace]","\[InvisibleSpace]8\[InvisibleSpace]"]"\),
      SequenceForm[ 
      "PNGBitmap", "[", "<<image data>>", ",", 192, ",", 192, ",", 8, "]"],
      Editable->False]], "Output",
  CellLabel->"Out[21]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[Graphics[bm], ImageSize -> {192, 192}]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[22]="]
}, Open  ]],

Cell["Let's put the tiger on a 3d surface.", "Text"],

Cell[BoxData[
    \(\(plt = 
      PNGTexturePlot3D[bm, 
        y*Exp[\(-x^2\) - y^2], {x, \(-2\), 2}, {y, \(-2\), 2}, 
        ViewPoint -> {1.484, \ \(-2.740\), \ 1.319}, AxesLabel -> {x, y, z}]; 
    \)\)], "Input",
  CellLabel->"In[23]:="],

Cell["The final picture can be saved as PNG file with ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<tigersurf.png\>", plt, "\<PNG\>"]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[24]="]
}, Open  ]],

Cell["\<\
The second possibility for mapping the images onto surfaces is to \
use a three dimensional parametric surface.\
\>", "Text"],

Cell[BoxData[
    \(bm = ReadPNGBitmap["\<mathlogoh.png\>"]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(\(Show[Graphics[bm]]; \)\)], "Input",
  CellLabel->"In[42]:="],

Cell["\<\
The function PNGTextureParametricPlot3D[] provides this kind of \
mapping. \
\>", "Text"],

Cell[BoxData[
    \(\(PNGTextureParametricPlot3D[\n\t\ \ bm, \n\t\ {Cos[u], Sin[u], v}, \n
      \t\ {u, \(-Pi\), Pi}, {v, 0, 1}\ , \n\t\ \ Axes -> True, \n\t\ \ 
      BoxRatios -> Automatic, \n\t\ \ 
      ViewPoint -> \n\t\t\ \ \ {1.453, \ \(-2.682\), \ 1.465}, \n\t\ \ 
      LightSources -> \n
        \t\t\ \ {\n\t\t\t\ \ {{1. , 0. , 1. }, RGBColor[1. , 0, 0]}, \n
          \t\t\t\ \ {{1. , 1. , 1. }, RGBColor[0, 1. , 0]}, \n
          \t\t\t\ \ {{0. , 1. , 1. }, RGBColor[0, 0, 1. ]}, \n
          \t\t\t\ \ {{0, 3, 3}, RGBColor[1, 1, 1]}\n\t\t\ \ }, \n\t\ 
      RenderAll -> False\n\ ]; \)\)], "Input",
  CellLabel->"In[43]:="],

Cell["\<\
The polygon outlines are omittet by default. With the DrawEdges \
option you can get the edges back in your picture. Instead of True or False \
you may give a vaild EdgeForm[]-primitive as value for this option.\
\>", 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bm = 
      ReadPNGBitmap[Directory[]\ <>\ $PathnameSeparator\ <>"\<checker.png\>"]
        \)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    InterpretationBox[
      \("PNGBitmap"\[InvisibleSpace]"["\[InvisibleSpace]"<<image data>>"\
\[InvisibleSpace]","\[InvisibleSpace]30\[InvisibleSpace]","\[InvisibleSpace]10
        \[InvisibleSpace]","\[InvisibleSpace]8\[InvisibleSpace]"]"\),
      SequenceForm[ 
      "PNGBitmap", "[", "<<image data>>", ",", 30, ",", 10, ",", 8, "]"],
      Editable->False]], "Output",
  CellLabel->"Out[25]="]
}, Open  ]],

Cell[BoxData[
    \(\(PNGTextureParametricPlot3D[\n\t\ \ bm, \n
      \t\ {\((1 + u/Pi)\)*Cos[u], \((1 + u/Pi)\)*Sin[\(-u\)], v*u + u}, \n
      \t\ {u, 0, 3*Pi}, {v, 0, 1}\ , \n\t
      ViewPoint -> {\(-2.047\), \ 1.805, \ 2.000}, \n\tDrawEdges -> True, \n
      \t\ Axes -> True, \n\t\ RenderAll -> False\n\ ]; \)\)], "Input",
  CellLabel->"In[26]:="],

Cell["\<\
Here are two more complicated examples of how to use texture \
mapping.\
\>", "Text"],

Cell[BoxData[
    \(bm = 
      ReadPNGBitmap[
        Directory[]\ <>\ $PathnameSeparator\ <>"\<mathlogoh.png\>"]\)], 
  "Input"],

Cell[BoxData[
    \(r1 = 1; \nr2 = 3; \n
    PNGTextureParametricPlot3D[\n\t\ \ Transpose[MirrorVertical[bm]], \n
      \t{\((r2 + r1*Cos[u])\)*Sin[v], \((r2 + r1*Cos[u])\)*Cos[v], 
        r1*Sin[u]}, \n\t\ \ {u, \(-Pi\) + 1, Pi + 1}, {v, 0, 2  Pi}\ , \n
      \t\ \ Boxed -> False, \n\t\ \ BoxRatios -> Automatic, \n\t\ \ 
      ViewPoint -> {0.937, \ \(-1.730\), \ 2.753}, \n\t\ \ 
      LightSources -> \n
        \t\t\ \ {\n\t\t\t\ \ {{1. , 0. , 1. }, RGBColor[1. , 0, 0]}, \n
          \t\t\t\ \ {{1. , 1. , 1. }, RGBColor[0, 1. , 0]}, \n
          \t\t\t\ \ {{0. , 1. , 1. }, RGBColor[0, 0, 1. ]}, \n
          \t\t\t\ \ {{0, 3, 3}, RGBColor[0.25, 0.25, 0.25]}\n\t\t\ \ }, 
      RenderAll -> False\n\ ]; \)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(bm = ReadPNGBitmap["\<apple2h.png\>"]\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \(\(Show[Graphics[bm, ImageSize -> Reverse[Dimensions[bm[\([1]\)]]]]]; 
    \)\)], "Input",
  CellLabel->"In[4]:="],

Cell[BoxData[
    \(\(\n\(PNGTextureParametricPlot3D[bm
      \ , {Cos[u]*Sin[v], Sin[u]*Sin[v], Cos[v]}, {u, 0, 2*Pi}, {v, 0, Pi}, \n
      \t\tBoxed -> False, \n\t\t
      ViewPoint -> {\(-2.825\), \ \(-1.582\), \ 0.982}, \n\t\t
      LightSources -> {\n\t\t\t\ \ \ {{1, 1, 1}, RGBColor[1, 1, 1]}, \n
          \t\t\t\t{{\(-1\), \(-1\), 1}, RGBColor[1, 1, 1]}\n\t\t\t}, \n\t\ \ 
      RenderAll -> False\n\ \ \ \ ]; \)\)\)], "Input",
  CellLabel->"In[11]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Exporting Bitmaps createad by ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "and an interface to POVray"
}], "Section"],

Cell[TextData[{
  "The ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " programm offers the creation of bitmaps. One very popular algorithm that \
yields bitmaps are the Mandelbrot pictures. The PNGBitmap package comes with \
a built-in function to create these."
}], "Text"],

Cell[BoxData[
    \(Options[PNGMandelbrot]\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \(zarea = {{\(-2\), 0.6}, {\(-1.2\), 1.2}}; \n
    img = PNGMandelbrot[zarea, PNGMandelPotential -> False]\)], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \(Show[Graphics[RandomPalette[img]]]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(\(imdat = PNGBitmapToData[img]; \)\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(\(ListPlot3D[imdat, Mesh -> False]; \)\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(\(ListDensityPlot[imdat, ColorFunction -> Hue, MeshRange -> zarea, 
      Mesh -> False]\n\t\t\t\t\)\)], "Input",
  CellLabel->"In[56]:="],

Cell["\<\
If more than 255 iterations are given, PNGMandelbrot[] will create \
images with 16 bit depth.\
\>", "Text"],

Cell[TextData[{
  "The package can create continuous potential pictures of Mandelbrot and \
Julia sets. This is the default behaviour of the option ",
  StyleBox["PNGMandelPotential",
    FontFamily->"Courier"],
  ". At first we will try a continuous potential picture of the Mandelbrot \
set."
}], "Text"],

Cell[BoxData[
    \(zarea = {{\(-2\), 0.75}, {\(-1.25\), 1.25}}; \n
    mandelPot = 
      PNGMandelbrot[\n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        zarea, \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        PNGMandelSize -> {512, 512}, \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        PNGMandelScale -> \((1 - Sqrt[#]\ &)\)\n
        \t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \)], "Input",
  CellLabel->"In[19]:="],

Cell[TextData[{
  "We can render the image with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell[BoxData[
    \(\(mandeldata = PNGBitmapToData[mandelPot]; \)\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(ListPlot3D[mandeldata, Mesh -> False, MeshRange -> zarea]; \)\)], 
  "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(WritePNGBitmap["\<mandel1.png\>", mandelPot]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(\(mandelMap = 
      PNGMandelbrot[zarea, PNGMandelPotential -> False, 
        PNGMandelSize -> {512, 512}]; \)\)], "Input",
  CellLabel->"In[28]:="],

Cell["\<\
Now the image map for the surface is generated. A picture with 8 \
bit bit depth will be more than enough.\
\>", "Text"],

Cell[BoxData[
    \(WritePNGBitmap["\<mandel2.png\>", 
      InvertBitmap[mandelMap]\  /. {Pixel[0] -> \ Pixel[255, 0, 0], 
          Pixel[g_\  /; \ g < 230]\  -> Pixel[0, 0, 255], \n\t\t
          Pixel[g_\  /; \ g < \ 240]\  -> Pixel[0, 255, 0], 
          Pixel[g_] :> \ Pixel[255, 255, 255]}]\)], "Input",
  CellLabel->"In[44]:="],

Cell["Runing POVray with:", "Text"],

Cell[TextData[StyleBox["jpk@max>x-povray +D +FN8 +A -Imandel3d.pov",
  CellFrame->True,
  FontFamily->"Courier",
  Background->None]], "Text",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell["on the script", "Text"],

Cell[BoxData[
    \(\(!! mandel3d.pov\)\)], "Input",
  CellLabel->"In[49]:="],

Cell["yields this nice picture", "Text"],

Cell[BoxData[
    \(\(Show[
      Graphics[ReadPNGBitmap["\<mandel3d.png\>"], ImageSize -> {320, 240}]]; 
    \)\)], "Input",
  CellLabel->"In[52]:="],

Cell["We can do the same with a Julia fractal.", "Text"],

Cell[BoxData[
    \(zarea = {{\(-1.75\), 1.75}, {\(-1.75\), 1.75}}; \n
    juliaPot = 
      PNGMandelbrot[\n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        zarea, \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        PNGMakeJulia -> True, \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        PNGMandelSize -> {512, 512}, \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        PNGMandelScale -> \((1 - Sqrt[#]\ &)\)\n
        \t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \)], "Input",
  CellLabel->"In[4]:="],

Cell[BoxData[
    \(WritePNGBitmap["\<julia1.png\>", juliaPot]\)], "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    \(\(juliaMap = 
      PNGMandelbrot[zarea, PNGMandelPotential -> False, PNGMakeJulia -> True, 
        PNGMandelSize -> {512, 512}]; \)\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(WritePNGBitmap["\<julia2.png\>", 
      juliaMap /. {Pixel[g_ /; \ g < 25] -> \ Pixel[255, 255, 255], 
          Pixel[g_\  /; \ g < 39]\  -> Pixel[0, 0, 255], \n\t\t
          Pixel[g_\  /; \ g < \ 53]\  -> Pixel[0, 255, 0], 
          Pixel[g_] :> \ Pixel[255, 0, 0]}]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(Show[
      Graphics[ReadPNGBitmap["\<julia3d.png\>"], ImageSize -> {320, 240}]]; 
    \)\)], "Input",
  CellLabel->"In[22]:="],

Cell[TextData[{
  "The function ",
  StyleBox["DataToPNGBitmap[]",
    FontFamily->"Courier"],
  " can be used to export a matrix of numbers as a PNG bitmap. The following \
code uses the ",
  StyleBox["fBmRA[]",
    FontFamily->"Courier"],
  " function from R. M\[ADoubleDot]ders \"The Mathematica Programmer II\" \
book to create a fractal landscape. The code will only work if you have the \
packages from this book installed."
}], "Text"],

Cell[BoxData[
    \(Needs["\<MathProg`fBm`\>"]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \(\(landscape0 = fBmRA[2, 256, 0.75]; \)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(\(bm0 = DataToPNGBitmap[landscape0, PNGBitDepth -> 16]; \)\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(\(Show[Graphics[bm0]]; \)\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
After writing the PNG bitmap, it can be used as a natural landscape \
in POVray scripts or as a texture for POVray.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Closing", "Section"],

Cell[TextData[{
  "Keep in mind that image processing has huge memory requirements as does ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " alone. So if you are working with larger pictures (> 512 \[Times]512 \
pixel) you will need a bit more RAM than otherwise. This memory usage is the \
result of the calculation of a 2048 \[Times] 2048 Mandelbrot image with 16 \
bit gray levels."
}], "Text"],

Cell[BoxData[
    \(Share[]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(MaxMemoryUsed[]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(MemoryInUse[]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(Links[]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(Uninstall[Last[Links[]]]\)], "Input",
  CellLabel->"In[87]:="]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 832}, {0, 604}},
WindowToolbars->{},
WindowSize->{732, 556},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
StyleDefinitions -> "ArticleClassic.nb",
MacintoshSystemPageSetup->"\<\
00<0001804P000000]P2:?oQon82n@960dL5:0?l0080001804P000000]P2:001
0000I00000400`<300000BL?00400@0000000000000006P801T1T00000000000
00000000000000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 123, 5, 107, "Title"],
Cell[1857, 58, 33, 0, 40, "Author"],

Cell[CellGroupData[{
Cell[1915, 62, 31, 0, 57, "Section"],
Cell[1949, 64, 696, 13, 84, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2682, 82, 43, 0, 57, "Section"],
Cell[2728, 84, 291, 7, 39, "Text"],
Cell[3022, 93, 114, 2, 28, "Input"],
Cell[3139, 97, 37, 0, 24, "Text"],

Cell[CellGroupData[{
Cell[3201, 101, 78, 2, 28, "Input"],
Cell[3282, 105, 159, 4, 23, "Print"]
}, Open  ]],
Cell[3456, 112, 55, 0, 24, "Text"],
Cell[3514, 114, 64, 2, 28, "Input"],
Cell[3581, 118, 76, 0, 24, "Text"],
Cell[3660, 120, 73, 2, 28, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3770, 127, 32, 0, 57, "Section"],
Cell[3805, 129, 333, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[4163, 140, 114, 2, 28, "Input"],
Cell[4280, 144, 112, 2, 28, "Output"]
}, Open  ]],
Cell[4407, 149, 211, 5, 44, "Input"],
Cell[4621, 156, 32, 0, 24, "Text"],
Cell[4656, 158, 101, 2, 28, "Input"],
Cell[4760, 162, 45, 0, 24, "Text"],
Cell[4808, 164, 230, 5, 44, "Input"],
Cell[5041, 171, 428, 9, 124, "Input"],
Cell[5472, 182, 34, 0, 24, "Text"],
Cell[5509, 184, 171, 4, 28, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5717, 193, 33, 0, 57, "Section"],
Cell[5753, 195, 110, 3, 24, "Text"],

Cell[CellGroupData[{
Cell[5888, 202, 151, 4, 28, "Input"],
Cell[6042, 208, 427, 9, 28, "Output"]
}, Open  ]],
Cell[6484, 220, 358, 5, 52, "Text"],

Cell[CellGroupData[{
Cell[6867, 229, 74, 2, 28, "Input"],
Cell[6944, 233, 119, 3, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7100, 241, 132, 3, 28, "Input"],
Cell[7235, 246, 370293, 5121, 200, 302065, 4275, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[377543, 5370, 235, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[377803, 5380, 140, 4, 28, "Input"],
Cell[377946, 5386, 427, 9, 28, "Output"]
}, Open  ]],
Cell[378388, 5398, 58, 0, 24, "Text"],

Cell[CellGroupData[{
Cell[378471, 5402, 73, 2, 28, "Input"],
Cell[378547, 5406, 64, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[378648, 5413, 72, 2, 28, "Input"],
Cell[378723, 5417, 63, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[378823, 5424, 101, 2, 28, "Input"],
Cell[378927, 5428, 155, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[379131, 5438, 35, 0, 57, "Section"],
Cell[379169, 5440, 181, 3, 38, "Text"],

Cell[CellGroupData[{
Cell[379375, 5447, 95, 2, 28, "Input"],
Cell[379473, 5451, 88, 2, 28, "Output"]
}, Open  ]],
Cell[379576, 5456, 115, 3, 24, "Text"],

Cell[CellGroupData[{
Cell[379716, 5463, 144, 3, 28, "Input"],
Cell[379863, 5468, 88, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[379988, 5475, 139, 3, 28, "Input"],
Cell[380130, 5480, 87, 2, 28, "Output"]
}, Open  ]],
Cell[380232, 5485, 322, 6, 52, "Text"],
Cell[380557, 5493, 125, 3, 28, "Input"],
Cell[380685, 5498, 124, 3, 28, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[380846, 5506, 40, 0, 57, "Section"],
Cell[380889, 5508, 173, 4, 38, "Text"],
Cell[381065, 5514, 320, 6, 60, "Input"],
Cell[381388, 5522, 131, 3, 24, "Text"],
Cell[381522, 5527, 126, 3, 28, "Input"],
Cell[381651, 5532, 66, 0, 24, "Text"],
Cell[381720, 5534, 73, 2, 28, "Input"],
Cell[381796, 5538, 111, 3, 24, "Text"],
Cell[381910, 5543, 323, 6, 156, "Input"],
Cell[382236, 5551, 92, 3, 24, "Text"],
Cell[382331, 5556, 1082, 29, 252, "Input"],
Cell[383416, 5587, 163, 4, 24, "Text"],
Cell[383582, 5593, 147, 3, 44, "Input"],
Cell[383732, 5598, 373, 11, 40, "Text"],
Cell[384108, 5611, 91, 2, 28, "Input"],
Cell[384202, 5615, 166, 5, 26, "Text"],
Cell[384371, 5622, 201, 4, 44, "Input"],
Cell[384575, 5628, 168, 3, 28, "Input"],
Cell[384746, 5633, 191, 4, 44, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[384974, 5642, 45, 0, 57, "Section"],
Cell[385022, 5644, 58, 0, 24, "Text"],

Cell[CellGroupData[{
Cell[385105, 5648, 148, 4, 28, "Input"],
Cell[385256, 5654, 427, 9, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[385720, 5668, 101, 2, 28, "Input"],
Cell[385824, 5672, 155, 4, 28, "Output"]
}, Open  ]],
Cell[385994, 5679, 52, 0, 24, "Text"],
Cell[386049, 5681, 244, 6, 44, "Input"],
Cell[386296, 5689, 64, 0, 24, "Text"],

Cell[CellGroupData[{
Cell[386385, 5693, 102, 2, 28, "Input"],
Cell[386490, 5697, 162, 4, 28, "Output"]
}, Open  ]],
Cell[386667, 5704, 135, 3, 24, "Text"],
Cell[386805, 5709, 97, 2, 28, "Input"],
Cell[386905, 5713, 82, 2, 28, "Input"],
Cell[386990, 5717, 99, 3, 24, "Text"],
Cell[387092, 5722, 638, 11, 284, "Input"],
Cell[387733, 5735, 238, 5, 38, "Text"],

Cell[CellGroupData[{
Cell[387996, 5744, 150, 4, 28, "Input"],
Cell[388149, 5750, 414, 8, 28, "Output"]
}, Open  ]],
Cell[388578, 5761, 353, 6, 156, "Input"],
Cell[388934, 5769, 95, 3, 24, "Text"],
Cell[389032, 5774, 130, 4, 28, "Input"],
Cell[389165, 5780, 750, 13, 284, "Input"],
Cell[389918, 5795, 94, 2, 28, "Input"],
Cell[390015, 5799, 133, 3, 28, "Input"],
Cell[390151, 5804, 460, 8, 188, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[390648, 5817, 150, 5, 57, "Section"],
Cell[390801, 5824, 282, 7, 40, "Text"],
Cell[391086, 5833, 79, 2, 28, "Input"],
Cell[391168, 5837, 161, 3, 44, "Input"],
Cell[391332, 5842, 91, 2, 28, "Input"],
Cell[391426, 5846, 92, 2, 28, "Input"],
Cell[391521, 5850, 96, 2, 28, "Input"],
Cell[391620, 5854, 158, 3, 44, "Input"],
Cell[391781, 5859, 118, 3, 24, "Text"],
Cell[391902, 5864, 306, 7, 40, "Text"],
Cell[392211, 5873, 471, 9, 108, "Input"],
Cell[392685, 5884, 120, 5, 26, "Text"],
Cell[392808, 5891, 103, 2, 28, "Input"],
Cell[392914, 5895, 124, 3, 28, "Input"],
Cell[393041, 5900, 102, 2, 28, "Input"],
Cell[393146, 5904, 170, 4, 44, "Input"],
Cell[393319, 5910, 130, 3, 24, "Text"],
Cell[393452, 5915, 335, 6, 60, "Input"],
Cell[393790, 5923, 35, 0, 24, "Text"],
Cell[393828, 5925, 196, 5, 41, "Text"],
Cell[394027, 5932, 29, 0, 24, "Text"],
Cell[394059, 5934, 77, 2, 28, "Input"],
Cell[394139, 5938, 40, 0, 24, "Text"],
Cell[394182, 5940, 150, 4, 28, "Input"],
Cell[394335, 5946, 56, 0, 24, "Text"],
Cell[394394, 5948, 564, 11, 124, "Input"],
Cell[394961, 5961, 99, 2, 28, "Input"],
Cell[395063, 5965, 190, 4, 44, "Input"],
Cell[395256, 5971, 328, 6, 76, "Input"],
Cell[395587, 5979, 149, 4, 28, "Input"],
Cell[395739, 5985, 442, 11, 53, "Text"],
Cell[396184, 5998, 84, 2, 28, "Input"],
Cell[396271, 6002, 96, 2, 28, "Input"],
Cell[396370, 6006, 116, 2, 28, "Input"],
Cell[396489, 6010, 83, 2, 28, "Input"],
Cell[396575, 6014, 139, 3, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[396751, 6022, 26, 0, 57, "Section"],
Cell[396780, 6024, 406, 8, 54, "Text"],
Cell[397189, 6034, 65, 2, 28, "Input"],
Cell[397257, 6038, 73, 2, 28, "Input"],
Cell[397333, 6042, 71, 2, 28, "Input"],
Cell[397407, 6046, 65, 2, 28, "Input"],
Cell[397475, 6050, 82, 2, 28, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

